***********************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Attention Deficit/     *;
*                                              Hyperactivity Disorder *;
***********************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %add3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro add3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ADD3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ADD3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data add3r;
      set &dsn.;

      attrib at3ra1-at3ra14 at3rax at3ra at3rb at3rdx at3rsk length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************;
            * often fidgets/squirms *;
            *************************;

               if at15 not in (1,5) then at3ra1=.;
               else at3ra1=(at15=5);

            *******************************;
            * difficulty remaining seated *;
            *******************************;

               if at16 not in (1,5) then at3ra2=.;
               else at3ra2=(at16=5);

            *******************************************;
            * easily distracted by extraneous stimuli *;
            *******************************************;

               if at8 not in (1,5) then at3ra3=.;
               else at3ra3=(at8=5);

            ****************************;
            * difficulty awaiting turn *;
            ****************************;

               if at12 not in (1,5) then at3ra4=.;
               else at3ra4=(at12=5);

            ************************************************;
            * blurts out answers before question completed *;
            ************************************************;

               if at11 not in (1,5) then at3ra5=.;
               else at3ra5=(at11=5);

            ************************************************;
            * difficulty following through on instructions *;
            ************************************************;

               if at4 not in (1,5) then at3ra6=.;
               else at3ra6=(at4=5);

            ***********************************;
            * difficulty sustaining attention *;
            ***********************************;

               if sum(at2a not in (1,5),
                      at2b not in (1,5))=2 then at3ra7=.;
               else at3ra7=(at2a=5 or at2b=5);

            *********************************************;
            * often shifts among uncompleted activities *;
            *********************************************;

               if at10 not in (1,5) then at3ra8=.;
               else at3ra8=(at10=5);

            ******************************;
            * difficulty playing queitly *;
            ******************************;

               if at18 not in (1,5) then at3ra9=.;
               else at3ra9=(at18=5);

            ***************************;
            * often talks excessively *;
            ***************************;

               if at20 not in (1,5) then at3ra10=.;
               else at3ra10=(at20=5);

            *******************************;
            * often interupts or intrudes *;
            *******************************;

               if at13 not in (1,5) then at3ra11=.;
               else at3ra11=(at13=5);

            *****************************;
            * often seems not to listen *;
            *****************************;

               if at3 not in (1,5) then at3ra12=.;
               else at3ra12=(at3=5);

            *****************************************************;
            * often loses things necessary for tasks/activities *;
            *****************************************************;

               if at7 not in (1,5) then at3ra13=.;
               else at3ra13=(at7=5);

            ******************************************************;
            * often engages in unconsidered dangerous activities *;
            ******************************************************;

               if at14 not in (1,5) then at3ra14=.;
               else at3ra14=(at14=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of at3ra1-at3ra14)=0 then at3rax=.;
               else at3rax=sum(of at3ra1-at3ra14);

            *************************************;
            * behavior lasted at least 6 months *;
            *************************************;

               if at3rax=. then at3ra=.;
               else if at3rax<8 then at3ra=0;
               else do;
                  if (at21c not in (1,5)) then at3ra=.;
                  else at3ra=((at3rax>7) and (at21c=5));
               end;

      *********************;
      * test for skip out *;
      *********************;

         if at3rax=. then at3rsk=1;
         else do;
            if at3rax>3 then at3rsk=0;
            else at3rsk=1;
         end;

         if at3rsk=1 then do;
            array skp at3ra at3rb at3rdx;
            do over skp;
               skp=.S;
            end;
         end;

         if at3rsk=1 then go to skipout;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if at21a_1<0 then at3rb=.;
            else at3rb=(0<at21a_1<7);

      **************************************;
      * Criterion C -- Skip in this module *;
      **************************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(at3ra,at3rb)=0 then at3rdx=.;
            else at3rdx=(at3ra=1 and at3rb=1);

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         array one at3rsk at3ra1-at3ra14 at3ra at3rb at3rdx;
         do over one;
            if one=1 then one=5;
            if one=0 then one=1;
         end;

      *******************;
      * variable labels *;
      *******************;

         label at3rsk  = "DSM3R ADHD Skip Out"
               at3ra1  = "DSM3R ADHD (Lifetime) Crit. A1"
               at3ra2  = "DSM3R ADHD (Lifetime) Crit. A2"
               at3ra3  = "DSM3R ADHD (Lifetime) Crit. A3"
               at3ra4  = "DSM3R ADHD (Lifetime) Crit. A4"
               at3ra5  = "DSM3R ADHD (Lifetime) Crit. A5"
               at3ra6  = "DSM3R ADHD (Lifetime) Crit. A6"
               at3ra7  = "DSM3R ADHD (Lifetime) Crit. A7"
               at3ra8  = "DSM3R ADHD (Lifetime) Crit. A8"
               at3ra9  = "DSM3R ADHD (Lifetime) Crit. A9"
               at3ra10 = "DSM3R ADHD (Lifetime) Crit. A10"
               at3ra11 = "DSM3R ADHD (Lifetime) Crit. A11"
               at3ra12 = "DSM3R ADHD (Lifetime) Crit. A12"
               at3ra13 = "DSM3R ADHD (Lifetime) Crit. A13"
               at3ra14 = "DSM3R ADHD (Lifetime) Crit. A14"
               at3rax  = "DSM3R ADHD (Lifetime) Sum of As"
               at3ra   = "DSM3R ADHD (Lifetime) 8+ in 6Mo"
               at3rb   = "DSM3R ADHD (Lifetime) Crit. B"
               at3rdx  = "DSM3R ADHD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id at3rsk 
                    at3ra1-at3ra14 at3rax at3ra at3rb at3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_add_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(add3r,at15 at3ra1);
                     %multi(add3r,at16 at3ra2);
                     %multi(add3r,at8 at3ra3);
                     %multi(add3r,at12 at3ra4);
                     %multi(add3r,at11 at3ra5);
                     %multi(add3r,at4 at3ra6);
                     %multi(add3r,at2a at2b at3ra7);
                     %multi(add3r,at10 at3ra8);
                     %multi(add3r,at18 at3ra9);
                     %multi(add3r,at20 at3ra10);
                     %multi(add3r,at13 at3ra11);
                     %multi(add3r,at3 at3ra12);
                     %multi(add3r,at7 at3ra13);
                     %multi(add3r,at14 at3ra14);
                     %multi(add3r,at3ra1-at3ra14 at3rax);

                  ************;
                  * skip out *;
                  ************;
   
                     %multi(add3r,at3rax at3rsk);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(add3r,at3rsk at3rax at21c at3ra);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(add3r,at3rsk at21a_1 at3rb);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(add3r,at3rsk at3ra at3rb at3rdx);

            ******************;
            * keep variables *;
            ******************;

               data add3r;
               set add3r;
               keep ind_id at3rsk
                    at3ra1-at3ra14 at3rax at3ra at3rb at3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=add3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend add3r;

********;
* test *;
********;
/*
   %add3r(coga.csaga2,1);
*/
