*****************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Alcohol Abuse and Dependence *;
*****************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %alc3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro alc3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ALC3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ALC3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data alc3r;
      set &dsn.;

      attrib al3rsk al3da1-al3da9 al3dax al3da al3db al3rdp
             al3aa1 al3aa2 al3aa al3ab al3ac al3rab length=3;

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(al1a not in (1,5),
                   al1f not in (1,5),
                   al1g not in (1,5),
                   al11a<0)=4 then al3rsk=5;
            else do;
               if (al1a=5 and (al1f=5 or al1g=5) and al11a>2) then al3rsk=1;
               else al3rsk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(al1c not in (1,5),
                   al1f not in (1,5),
                   al11a<0)=3 then al3rsk=5;
            else do;
               if ((al1c=5 or al1f=5) and al11a>2) then al3rsk=1;
               else al3rsk=5;
            end;
         %end;

         if al3rsk=5 then do;
            array skp al3da1-al3da9 al3dax al3da al3db al3rdp
                      al3aa1 al3aa2 al3aa al3ab al3ac al3rab;
            do over skp;
               skp=.S;
            end;
         end;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &study.=coga %then %do;
            %if &who.=p %then %do;
               if al3rsk=1 then do;
                  array nas al3da1-al3da3 al3da5-al3da9 al3dax al3da 
                            al3db al3rdp;
                  do over nas;
                     nas=.N;
                  end;
               end;
               if al3rsk=5 then go to skipout;
               else go to par_hop1;
            %end;
         %end;

      if al3rsk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            *********************************************************;
            * used larger amount or for longer period than intended *;
            *********************************************************;

               if sum(al15c not in (1,5),
                      al16c not in (1,5))=2 then al3da1=.;
               else al3da1=(4*(al15c=5 or al16c=5))+1;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            ***************************************************************;

               if sum(al13a not in (1,5),
                      al13c_1 not in (1,5))=2 then al3da2=.;
               else al3da2=(4*(al13a=5 or al13c_1=1))+1;

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            ****************************************************************;

               if al18b not in (1,5) then al3da3=.;
               else al3da3=(4*(al18b=5))+1;

            **************************************;
            * jump here if coga parent interview *;
            **************************************;

               par_hop1:

            *******************************************************;
            * intoxication/withdrawal interferes with obligations *;
            *******************************************************;

               if sum(al20d not in (1,5),
                      al25b not in (1,5),
                      al26f not in (1,5),
                      al26i not in (1,5),
                      al29c not in (1,5),
                      al30c not in (1,5),
                      al31g not in (1,5),
                      al35c not in (1,5))=8 then al3da4=.;
               else al3da4=(4*(sum(al20d=5,al25b=5,al26f=4,al26i=5,
                                   al29c=5,al30c=5,al31g=5,al35c=5)>0))+1;

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if sum(al19d not in (1,5),
                      al31c not in (1,5))=2 then al3da5=.;
               else al3da5=(4*(al19d=5 or al31c=5))+1;

            **********************************;
            * test for coga parent interview *;
            **********************************;

               %if &study.=coga %then %do;
                  %if &who.=p %then %do;
                     go to par_hop2;
                  %end;
               %end;

            **********************************************************;
            * use despite use-derived social/pysch/physical problems *;
            **********************************************************;

               if sum(al32b not in (1,2,3,4),
                      sum(al33a_3 not in (1,5),
                          al33d_3 not in (1,5),
                          al33e_3 not in (1,5),
                          al33h not in (1,5))=4,
                      al34c not in (1,5),
                      al36a not in (1,5),
                      al37e not in (1,5))=5 then al3da6=.;
               else al3da6=(4*(al32b=2 or 
                               (sum(al33a_3=5,al33d_3=5,al33e_3=5)>0 and
                                al33h=5) or
                               al34c=5 or al36a=5 or al37e=5))+1;

            ********************;
            * marked tolerance *;
            ********************;

               if sum(al12e not in (1,5),
                      al12f not in (1,5))=2 then al3da7=.;
               else al3da7=(4*(al12e=5 or al12f=5))+1;

            **************************************;
            * characteristic withdrawal symptoms *;
            **************************************;

               if 0<dm3a<13 then do;
                  al3da8=.N;
               end;
               else do;
                  if sum(al46a_2 not in (1,5),
                         sum(al46b_2 not in (1,5),al46c_2 not in (1,5),
                             al46d_2 not in (1,5),al46e_2 not in (1,5),
                             al46f_2 not in (1,5),al46g_2 not in (1,5),
                             al46h_2 not in (1,5),al46i_2 not in (1,5),
                             al46j_2 not in (1,5))=9)=2 then al3da8=.;
                  else al3da8=(4*(al46a_2=5 and 
                                  sum(al46b_2=5,al46c_2=5,al46d_2=5,al46e_2=5,
                                      al46f_2=5,al46g_2=5,al46h_2=5,al46i_2=5,
                                      al46j_2=5)>0))+1;
               end;

            ********************************************;
            * use to relieve/avoid withdrawal symptoms *;
            ********************************************;

               if 0<dm3a<13 then do;
                  al3da9=.N;
               end;
               else do;
                  if al46t not in (1,5) then al3da9=.;
                  else al3da9=(4*(al46t=5))+1;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of al3da1-al3da9)=0 then al3dax=.;
               else do;
                  al3dax=0;
                  array sxs al3da1-al3da9;
                  do over sxs;
                     if sxs=5 then al3dax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if al3dax=. then al3da=.;
               else al3da=(4*(al3dax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if sum(al47b not in (1,5),
                   sum(al3da1 not in (1,5),
                       sum(al13a not in (1,5),
                           al13c_2 not in (1,5))=2,
                       al3da3 not in (1,5),
                       al3da4 not in (1,5),
                       al3da5 not in (1,5),
                       al3da6 not in (1,5),
                       al3da7 not in (1,5),
                       sum(al46a_2 not in (1,5),
                           sum(al46b_2 not in (1,5),al46c_2 not in (1,5),
                               al46d_2 not in (1,5),al46e_2 not in (1,5),
                               al46f_2 not in (1,5),al46g_2 not in (1,5),
                               al46h_2 not in (1,5),al46i_2 not in (1,5),
                               al46j_2 not in (1,5))=9,
                           sum(al46n<0,
                               al46q1 not in (1,5))=2)=3,
                       al3da9 not in (1,5))=9)=2 then al3db=.;
            else al3db=(4*((al47b=5) or 
                           sum(al3da1=5,
                               sum(al13a=5,al13c_2=5)>0,
                               al3da3=5,
                               al3da4=5,
                               al3da5=5,
                               al3da6=5,
                               al3da7=5,
                               sum(al46a_2=5,
                                   sum(al46b_2=5,al46c_2=5,
                                       al46d_2=5,al46e_2=5,
                                       al46f_2=5,al46g_2=5,
                                       al46h_2=5,al46i_2=5,
                                       al46j_2=5)>0,
                                   sum(al46n ge 30,al46q1=5)>0)>0,
                               al3da9=5)>2))+1;
                   
         *************;
         * Diagnosis *;
         *************;

            if n(al3da,al3db)=0 then al3rdp=.;
            else al3rdp=(4*(al3da=5 and al3db=5))+1;

      **************************************;
      * jump here if coga parent interview *;
      **************************************;

         par_hop2:

      *********;
      * Abuse *;
      *********;

         ***************;
         * Criterion A *;
         ***************;

            **********************************************************;
            * use despite use-derived social/pysch/physical problems *;
            **********************************************************;

               if sum(al32b not in (1,2,3,4),
                      sum(al33a_3 not in (1,5),
                          al33d_3 not in (1,5),
                          al33e_3 not in (1,5),
                          al33h not in (1,5))=4,
                      al34c not in (1,5),
                      al36a not in (1,5),
                      al37e not in (1,5))=5 then al3aa1=.;
               else al3aa1=(4*(al32b=2 or 
                               (sum(al33a_3=5,al33d_3=5,al33e_3=5)>0 and 
                                al33h=5) or
                               al34c=5 or al36a=5 or al37e=5))+1;

            ****************************************************;
            * recurrent use in physically dangerous situations *;
            ****************************************************;

               if sum(al25b not in (1,5),
                      al26f not in (1,5),
                      al26i not in (1,5),
                      al35c not in (1,5))=4 then al3aa2=.;
               else al3aa2=(4*(sum(al25b=5,al26f=5,al26i=5,al35c=5)>0))+1;

            *********************;
            * at least one item *;
            *********************;

               if n(al3aa1,al3aa2)=0 then al3aa=.;
               else al3aa=(4*(al3aa1=5 or al3aa2=5))+1;

         ***************;
         * Criterion B *;
         ***************;

            if sum(al25b not in (1,5),
                   al26f not in (1,5),
                   al26i not in (1,5),
                   al32b not in (1,2,3,4),
                   al33a_3 not in (1,5),
                   al33c_3 not in (1,5),
                   al33d_3 not in (1,5),
                   al33e_3 not in (1,5),
                   al34c not in (1,5),
                   al35c not in (1,5),
                   al36a not in (1,5),
                   al37e not in (1,5))=12 then al3ab=.;
            else al3ab=(4*(sum(al25b=5,al26f=5,al26i=5,al32b=2,al33a_3=5,
                               al33c_3=5,al33d_3=5,al33e_3=5,al34c=5,
                               al35c=5,al36a=5,al37e=5)>1))+1;

         ***************;
         * Criterion C *;
         ***************;

            if al3rdp=. then al3ac=.;
            else al3ac=al3rdp;

         *************;
         * Diagnosis *;
         *************;

            if n(al3aa,al3ab,al3ac)=0 then al3rab=.;
            %if &who.=p %then %do;
               else al3rab=(4*(al3aa=5 and al3ab=5))+1;
            %end;
            %else %do;
               else al3rab=(4*(al3aa=5 and al3ab=5 and al3ac=1))+1;
            %end;

      *************************;
      * jump here if skip out *;
      *************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label al3rsk = "DSM3R ALC Skip Out"
               al3da1 = "DSM3R ALC Dependence Crit. A1"
               al3da2 = "DSM3R ALC Dependence Crit. A2"
               al3da3 = "DSM3R ALC Dependence Crit. A3"
               al3da4 = "DSM3R ALC Dependence Crit. A4" 
               al3da5 = "DSM3R ALC Dependence Crit. A5" 
               al3da6 = "DSM3R ALC Dependence Crit. A6" 
               al3da7 = "DSM3R ALC Dependence Crit. A7" 
               al3da8 = "DSM3R ALC Dependence Crit. A8" 
               al3da9 = "DSM3R ALC Dependence Crit. A9" 
               al3dax = "DSM3R ALC Dependence Sum of As" 
               al3da  = "DSM3R ALC Dependence 3+ As" 
               al3db  = "DSM3R ALC Dependence Crit. B" 
               al3rdp = "DSM3R ALC Dependence Diagnosis"
               al3aa1 = "DSM3R ALC Abuse Crit. A1"
               al3aa2 = "DSM3R ALC Abuse Crit. A2"
               al3aa  = "DSM3R ALC Abuse A1 or A2"
               al3ab  = "DSM3R ALC Abuse Crit. B"
               al3ac  = "DSM3R ALC Abuse Crit. C"
               al3rab = "DSM3R ALC Abuse Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id al3rsk
                    al3da1-al3da9 al3dax al3da al3db al3rdp
                    al3aa1 al3aa2 al3aa al3ab al3ac al3rab;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_alc_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(alc3r,al1a al1f al1g al11a al3rsk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(alc3r,al1c al1f al11a al3rsk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(alc3r,al3rsk al15c al16c al3da1);
                     %multi(alc3r,al3rsk al13a al13c_1 al3da2);
                     %multi(alc3r,al3rsk al18b al3da3);
                     %multi(alc3r,al3rsk al20d al25b al26f al26i al29c 
                                  al30c al31g al35c al3da4);
                     %multi(alc3r,al3rsk al19d al31c al3da5);
                     %multi(alc3r,al3rsk al32b al33a_3 al33d_3 al33e_3 
                                  al34c al36a al37e al3da6);
                     %multi(alc3r,al3rsk al12e al12f al3da7);
                     %multi(alc3r,al3rsk al46a_2 al46b_2 al46c_2 al46d_2 al46e_2
                                  al46f_2 al46g_2 al46h_2 al46i_2 al46j_2
                                  al3da8);
                     %multi(alc3r,al3rsk al46t al3da9);
                     %multi(alc3r,al3rsk al3da1-al3da9 al3dax);
                     %multi(alc3r,al3rsk al3dax al3da);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(alc3r,al3rsk al47b al3da1 al13a al13c_2 
                                  al3da3-al3da7
                                  al46a_2 al46b_2 al46c_2 al46d_2 al46e_2
                                  al46f_2 al46g_2 al46h_2 al46i_2 al46j_2
                                  al46n al46q1 al3da9 al3db);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(alc3r,al3rsk al3da al3db al3rdp);

               *********;
               * Abuse *;
               *********;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(alc3r,al3rsk al32b al33a_3 al33d_3 al33e_3 
                                  al34c al36a al37e al3aa1);
                     %multi(alc3r,al3rsk al25b al26f al26i al35c al3aa2);
                     %multi(alc3r,al3rsk al3aa1 al3aa2 al3aa);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(alc3r,al3rsk al25b al26f al26i al32b al33a_3 al33c_3
                                  al33d_3 al33e_3 al34c al35c al36a al37e
                                  al3ab);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(alc3r,al3rsk al3rdp al3ac);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(alc3r,al3rsk al3aa al3ab al3ac al3rab);

            ******************;
            * keep variables *;
            ******************;

               data alc3r;
               set alc3r;
               keep ind_id al3rsk 
                    al3da1-al3da9 al3dax al3da al3db al3rdp
                    al3aa1 al3aa2 al3aa al3ab al3ac al3rab;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=alc3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend alc3r;

********;
* test *;
********;
/*
   %alc3r(coga.csaga2,1)
*/
