*****************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Anorexia Nervosa *;
*****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %anx3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro anx3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ANX3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ANX3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data anx3r;
      set &dsn.;

      attrib an3ra an3rb an3rc an3rd an3rdx an3rsk length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas an3ra an3rb an3rc an3rd an3rdx an3rsk;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to skipout;

      *********************;
      * test for skip out *;
      *********************;

         if sum(an1 not in (1,3,5),
                an4d not in (1,3,5))=2 then an3rsk=5;
         else do;
            if (an1 in (3,5) and an4d=5) then an3rsk=1;
            else an3rsk=5;
         end;

         if an3rsk=5 then do;
             array skp an3ra an3rb an3rc an3rd an3rdx;
             do over skp;
                skp=.S;
             end;
         end;

         if an3rsk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ***********************************;
            * refusal to maintain body weight *;
            ***********************************;

               if sum(an1 not in (1,3,5),
                      an4d not in (1,5))=2 then an3ra=.;
               else an3ra=(4*(sum(an1=5,an4d=5)=2))+
                          (2*(sum(an1=3,an4d=5)=2))+1;

      ***************;
      * Criterion B *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *******************************;
            * intense fear of weight gain *;
            *******************************;

               if an5 not in (1,3,5) then an3rb=.;
               else an3rb=(4*(an5=5))+(2*(an5=3))+1;

      ***************;
      * Criterion C *;
      ***************;

         ************;
         * lifetime *;
         ************;

            **************************;
            * body image disturbance *;
            **************************;

               if sum(an2 not in (1,3,5),
                      an3 not in (1,3,5))=2 then an3rc=.;
               else do;
                  an3rc=1;
                  if sum(an2=3,an3=3)>0 then an3rc=3;
                  else if sum(an2=3,an3=3)=0 and
                          sum(an2=5,an3=5)>0 then an3rc=5;
               end;

      ***************;
      * Criterion D *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ***********************;
            * cessation of menses *;
            ***********************;

               if sum(dm1 not in (1,2),
                      an8 not in (1,5),
                      an9 not in (1,5))=3 then an3rd=.;
               else if dm1=2 then an3rd=(4*(sum(an8=5,an9=5)=2))+1;
                     
      *************;
      * Diagnosis *;
      *************;

         ************;
         * lifetime *;
         ************;

            if n(an3ra,an3rb,an3rc,an3rd)=0 then an3rdx=.;
            else do;
               an3rdx=1;
               if (sum(an3ra in (3,5),an3rb in (3,5),an3rc in (3,5))=3 and
                   sum(an3ra=3,an3rb=3,an3rc=3) and 
                   sum(dm1=1,dm1=2 and an3rd=5)>0) then an3rdx=3;
               else if (sum(an3ra=3,an3rb=3,an3rc=3)=0 and
                        sum(an3ra=5,an3rb=5,an3rc=5)=3 and
                        sum(dm1=1,dm1=2 and an3rd=5)>0) then an3rdx=5;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label an3rsk = "DSM3R ANOREX Skip Out"
               an3ra  = "DSM3R ANOREX (Lifetime) Crit. A"
               an3rb  = "DSM3R ANOREX (Lifetime) Crit. B"
               an3rc  = "DSM3R ANOREX (Lifetime) Crit. C"
               an3rd  = "DSM3R ANOREX (Lifetime) Crit. D"
               an3rdx = "DSM3R ANOREX (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id an3rsk an3ra an3rb an3rc an3rd an3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_anx_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(anx3r,an1 an4d an3rsk);

               ***************;
               * Criterion A *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anx3r,an3rsk an1 an4d an3ra);

               ***************;
               * Criterion B *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anx3r,an3rsk an5 an3rb);

               ***************;
               * Criterion C *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anx3r,an3rsk an2 an3 an3rc);

               ***************;
               * Criterion D *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anx3r,an3rsk dm1 an8 an9 an3rd);

               *************;
               * Diagnosis *;
               *************;

                  ************;
                  * lifetime *;
                  ************;
 
                     %multi(anx3r,an3rsk an3ra an3rb an3rc an3rd an3rdx);

            ******************;
            * keep variables *;
            ******************;

               data anx3r;
               set anx3r;
               keep ind_id an3rsk an3ra an3rb an3rc an3rd an3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=anx3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend anx3r;

********;
* test *;
********;
/*
   %anx3r(coga.csaga2,1);
*/
