****************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Bulimia Nervosa *;
****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %bul3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro bul3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in BUL3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in BUL3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data bul3r;
      set &dsn.;

      attrib bu3rsk1 bu3rsk2 bu3rsk
             bu3ra bu3rb bu3rc bu3rd bu3re bu3rdx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas bu3rsk1 bu3rsk2 bu3rsk
                      bu3ra bu3rb bu3rc bu3rd bu3re bu3rdx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      ***************;
      * Criterion A *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *********************;
            * recurrent binging *;
            *********************;

               if sum(bu1a not in (1,3,5),
                      sum(bu2a not in (1,3,5),
                          bu2b not in (1,5))=2)=2 then bu3ra=.;
               else do;
                  bu3ra=1;
                  if sum(bu1a=3,bu2a=3)>0 then bu3ra=3;
                  else if sum(bu1a=5,sum(bu2a=5,bu2b=5)>0)>0 then bu3ra=5;
               end;

      *********************;
      * test for skip out *;
      *********************;

         if sum(bu1a not in (1,3,5),
                bu2b not in (1,5))=2 then bu3rsk1=5;
         else do;
            if (bu1a in (3,5) and bu2b ne 1) then bu3rsk1=1;
            else bu3rsk1=5;
         end;

         if bu3rsk1=5 then do;

             array skp1 bu3rb bu3rc bu3rd bu3re bu3rdx;
             do over skp1;
                skp1=.S;
             end;

         end;

         if bu3rsk1=5 then go to skipout;

      ***************;
      * Criterion D *;
      ***************;

         ************;
         * lifetime *;
         ************;

            **************************************************;
            * at least two binges/week for at least 3 months *;
            **************************************************;

               if bu3 not in (1,5) then bu3rd=.;
               else bu3rd=(4*(bu3=5))+1;

      *********************;
      * test for skip out *;
      *********************;

         if (bu3 not in (1,5)) then bu3rsk2=5;
         else do;
            if (bu3 ne 1) then bu3rsk2=1;
            else bu3rsk2=5;
         end;

         if bu3rsk2=5 then do;

             array skp2 bu3rb bu3rc bu3re bu3rdx;
             do over skp2;
                skp2=.S;
             end;

         end;

         if bu3rsk2=5 then go to skipout;

      ***************;
      * Criterion B *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ******************************************;
            * feeling of lack of control over eating *;
            ******************************************;

               if bu7 not in (1,3,5) then bu3rb=.;
               else bu3rb=(4*(bu7=5))+(2*(bu7=3))+1;

      ***************;
      * Criterion C *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *******************************;
            * regular self-induced purges *;
            *******************************;

               if sum(bu5 not in (1,3,5),
                      bu6 not in (1,3,5))=2 then bu3rc=.;
               else do;
                  bu3rc=1;
                  if sum(bu5=3,bu6=3)>0 then bu3rc=3;
                  else if sum(bu5=3,bu6=3)=0 and
                          sum(bu5=5,bu6=5)>0 then bu3rc=5;
               end;

      ***************;
      * Criterion E *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *********************************************;
            * persistent concern with body shape/weight *;
            *********************************************;

               if sum(bu4a not in (1,3,5),
                      bu4b not in (1,3,5))=2 then bu3re=.;
               else do;
                  bu3re=1;
                  if sum(bu4a=3,bu4b=3)>0 then bu3re=3;
                  else if sum(bu4a=5,bu4b=5)>0 then bu3re=5;
               end;
                     
      *************;
      * Diagnosis *;
      *************;

         ************;
         * lifetime *;
         ************;

            if n(bu3ra,bu3rb,bu3rc,bu3rd,bu3re)=0 then bu3rdx=.;
            else do;
               bu3rdx=1;
               if (sum(bu3ra in (3,5),bu3rb in (3,5),
                       bu3rc in (3,5),bu3re in (3,5))=4 and
                   sum(bu3ra=3,bu3rb=3,bu3rc=3,bu3re=3)>0 and 
                   bu3rd=5) then bu3rdx=3;
               else if (sum(bu3ra=3,bu3rb=3,bu3rc=3,bu3re=3)=0 and
                        sum(bu3ra=5,bu3rb=5,bu3rc=5,bu3rd=5,bu3re=5)=5) then
                  bu3rdx=5;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         bu3rsk=max(bu3rsk1,bu3rsk2);

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label bu3rsk1 = "DSM3R BULIMIA Skip Out #1"
               bu3rsk2 = "DSM3R BULIMIA Skip Out #2"
               bu3rsk  = "DSM3R BULIMIA Skip Out"
               bu3ra   = "DSM3R BULIMIA (Lifetime) Crit. A"
               bu3rb   = "DSM3R BULIMIA (Lifetime) Crit. B"
               bu3rc   = "DSM3R BULIMIA (Lifetime) Crit. C"
               bu3rd   = "DSM3R BULIMIA (Lifetime) Crit. D"
               bu3re   = "DSM3R BULIMIA (Lifetime) Crit. E"
               bu3rdx  = "DSM3R BULIMIA (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id bu3rsk1 bu3rsk2 bu3rsk 
                    bu3ra bu3rb bu3rc bu3rd bu3re bu3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_bul_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(bul3r,bu1a bu2a bu2b bu3ra);

               ************;
               * skip out *;
               ************;

                  %multi(bul3r,bu1a bu2b bu3rsk1);

               ***************;
               * Criterion D *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(bul3r,bu3rsk1 bu3 bu3rd);

               ************;
               * skip out *;
               ************;

                  %multi(bul3r,bu3rsk1 bu3 bu3rsk2);

               ***************;
               * Criterion B *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(bul3r,bu3rsk1 bu3rsk2 bu7 bu3rb);

               ***************;
               * Criterion C *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(bul3r,bu3rsk1 bu3rsk2 bu5 bu6 bu3rc);

               ***************;
               * Criterion E *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(bul3r,bu3rsk1 bu3rsk2 bu4a bu4b bu3re);

               *************;
               * Diagnosis *;
               *************;

                  ************;
                  * lifetime *;
                  ************;
 
                     %multi(bul3r,bu3rsk1 bu3rsk2
                                  bu3ra bu3rb bu3rc bu3rd bu3re bu3rdx);

            ******************;
            * keep variables *;
            ******************;

               data bul3r;
               set bul3r;
               keep ind_id bu3rsk1 bu3rsk2 bu3rsk 
                    bu3ra bu3rb bu3rc bu3rd bu3re bu3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=bul3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend bul3r;

********;
* test *;
********;
/*
   %bul3r(coga.csaga2,1);
*/
