******************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Cannabis Abuse and Dependence *;
******************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %can3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro can3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in CAN3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in CAN3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data can3r;
      set &dsn.;

      attrib mj3rsk
             mj3da1-mj3da9 mj3dax mj3da mj3db mj3rdp
             mj3aa1 mj3aa2 mj3aa mj3ab mj3ac mj3rab length=3;

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(mj1a not in (1,5),
                   mj1c not in (1,5),
                   mj7a<0,
                   mj7b not in (0,1,2,3,4,5))=4 then mj3rsk=5;
            else do;
               if (mj1a=5 and mj1c=5 and (mj7a>20 or mj7b in (1,2,3,4,5))) then
                  mj3rsk=1;
               else mj3rsk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(mj1c not in (1,5),
                   mj1c_1 not in (1,5),
                   mj7a<0,
                   mj7b not in (0,1,2,3,4,5))=4 then mj3rsk=5;
            else do;
               if ((mj1c=5 or mj1c_1=5) and 
                   (mj7a>20 or mj7b in (1,2,3,4,5))) then mj3rsk=1;
               else mj3rsk=5;
            end;
         %end;

         if mj3rsk=5 then do;
             array skp mj3da1-mj3da9 mj3dax mj3da mj3db mj3rdp
                       mj3aa1 mj3aa2 mj3aa mj3ab mj3ac mj3rab;
             do over skp;
                skp=.S;
             end;
         end;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &study.=coga %then %do;
            %if &who.=p %then %do;
               if mj3rsk=1 then do;
                  array nas mj3da1-mj3da3 mj3da6-mj3da9 mj3dax mj3da mj3db 
                            mj3rdp;   
                  do over nas;   
                     nas=.N;
                  end;
               end;
               if mj3rsk=5 then go to skipout;
               else go to par_hop1;
            %end;
         %end;

      if mj3rsk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            *********************************************************;
            * used larger amount or for longer period than intended *;
            *********************************************************;

               if sum(mj10b not in (1,5),
                      mj10d not in (1,5))=2 then mj3da1=.;
               else mj3da1=(4*(mj10b=5 or mj10d=5))+1;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            ***************************************************************;

               if sum(mj9b not in (1,5),
                      mj9c not in (1,5))=2 then mj3da2=.;
               else mj3da2=(4*(mj9b=5 or mj9c=1))+1;

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            ****************************************************************;

               if mj11 not in (1,5) then mj3da3=.;
               else mj3da3=(4*(mj11=5))+1;

            **************************************;
            * jump here if coga parent interview *;
            **************************************;

               par_hop1:

            *******************************************************;
            * intoxication/withdrawal interferes with obligations *;
            *******************************************************;

               if sum(mj15d not in (1,5),
                      mj15f not in (1,5),
                      mj20f not in (1,5),
                      mj23b not in (1,5))=4 then mj3da4=.;
               else mj3da4=(4*(sum(mj15d=5,mj15f=5,mj20f=4,mj23b=5)>0))+1;

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if mj12c not in (1,5) then mj3da5=.;
               else mj3da5=(4*(mj12c=5))+1;

            **********************************;
            * test for coga parent interview *;
            **********************************;

               %if &study.=coga %then %do;
                  %if &who.=p %then %do;
                     go to par_hop2;
                  %end;
               %end;

            **********************************************************;
            * use despite use-derived social/pysch/physical problems *;
            **********************************************************;

               if sum(sum(mj21a_3 not in (1,5),
                          mj21b_3 not in (1,5),
                          mj21c_3 not in (1,5),
                          mj21e not in (1,5))=4,
                      mj22b not in (1,5),
                      mj24e not in (1,5))=3 then mj3da6=.;
               else mj3da6=(4*(sum(mj21a_3=5,mj21b_3=5,mj21c_3=5,mj21e=5)>0 or
                               mj22b=5 or mj24e=5))+1;

            ********************;
            * marked tolerance *;
            ********************;

               if sum(mj8a not in (1,5),
                      mj8b not in (1,5))=2 then mj3da7=.;
               else mj3da7=(4*(mj8a=5 or mj8b=5))+1;

            **************************************;
            * characteristic withdrawal symptoms *;
            **************************************;

               if 0<dm3a<13 then do;
                  mj3da8=.N;
               end;
               else do;
                  if mj29d not in (1,5) then mj3da8=.;
                  else mj3da8=(4*(mj29d=5))+1;
               end;

            ********************************************;
            * use to relieve/avoid withdrawal symptoms *;
            ********************************************;

               if 0<dm3a<13 then do;
                  mj3da9=.N;
               end;
               else do;
                  if mj29c not in (1,5) then mj3da9=.;
                  else mj3da9=(4*(mj29c=5))+1;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of mj3da1-mj3da9)=0 then mj3dax=.;
               else do;
                  mj3dax=0;
                  array sxs mj3da1-mj3da9;
                  do over sxs;
                     if sxs=5 then mj3dax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if mj3dax=. then mj3da=.;
               else mj3da=(4*(mj3dax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if sum(mj30b not in (1,5),
                   sum(mj3da1 not in (1,5),
                       sum(mj9b not in (1,5),
                           mj9d not in (1,5))=2,
                       mj3da3 not in (1,5),
                       mj3da4 not in (1,5),
                       mj3da5 not in (1,5),
                       mj3da6 not in (1,5),
                       mj3da7 not in (1,5),
                       sum(mj29f<0,mj29g<0)=2,
                       mj3da9 not in (1,5))=9)=2 then mj3db=.;
            else mj3db=(4*((mj30b=5) or 
                           sum(mj3da1=5,
                               sum(mj9b=5,mj9d=5)>0,
                               mj3da3=5,
                               mj3da4=5,
                               mj3da5=5,
                               mj3da6=5,
                               mj3da7=5,
                               sum(mj29f>2,mj29g ge 30)>0,
                               mj3da9=5)>2))+1;
                   
         *************;
         * Diagnosis *;
         *************;

            if n(mj3da,mj3db)=0 then mj3dp=.;
            else mj3rdp=(4*(mj3da=5 and mj3db=5))+1;

      **************************************;
      * jump here if coga parent interview *;
      **************************************;

         par_hop2:

      *********;
      * Abuse *;
      *********;

         ***************;
         * Criterion A *;
         ***************;

            **********************************************************;
            * use despite use-derived social/pysch/physical problems *;
            **********************************************************;

               if sum(sum(mj21a_3 not in (1,5),
                          mj21b_3 not in (1,5),
                          mj21c_3 not in (1,5),
                          mj21e not in (1,5))=4,
                      mj22b not in (1,5),
                      mj24e not in (1,5))=3 then mj3aa1=.;
               else mj3aa1=(4*((sum(mj21a_3=5,mj21b_3=5,mj21c_3=5)>0 and
                                mj21e=5) or mj22b=5 or mj24e=5))+1;

            ****************************************************;
            * recurrent use in physically dangerous situations *;
            ****************************************************;

               if sum(mj15d not in (1,5),
                      mj15f not in (1,5),
                      mj23b not in (1,5))=3 then mj3aa2=.;
               else mj3aa2=(4*(sum(mj15d=5,mj15f=5,mj23b=5)>0))+1;

            *********************;
            * at least one item *;
            *********************;

               if n(mj3aa1,mj3aa2)=0 then mj3aa=.;
               else mj3aa=(4*(mj3aa1=5 or mj3aa2=5))+1;

         ***************;
         * Criterion B *;
         ***************;

            if sum(mj15d not in (1,5),
                   mj15f not in (1,5),
                   mj21a_3 not in (1,5),
                   mj21b_3 not in (1,5),
                   mj21c_3 not in (1,5),
                   mj22b not in (1,5),
                   mj23b not in (1,5))=7 then mj3ab=.;
            else mj3ab=(4*(sum(mj15d=5,mj15f=5,mj21a_3=5,mj21b_3=5,
                               mj21c_3=5,mj22b=5,mj23b=5)>1))+1;

         ***************;
         * Criterion C *;
         ***************;

            if mj3rdp=. then mj3ac=.;
            else mj3ac=mj3rdp;

         *************;
         * Diagnosis *;
         *************;

            if n(mj3aa,mj3ab,mj3ac)=0 then mj3rab=.;
            %if &who.=p %then %do;
               else mj3rab=(4*(mj3aa=5 and mj3ab=5))+1;
            %end;
            %else %do;
               else mj3rab=(4*(mj3aa=5 and mj3ab=5 and mj3ac=1))+1;
            %end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label mj3rsk = "DSM3R CAN Skip Out"
               mj3da1 = "DSM3R CAN Dependence Crit. A1"
               mj3da2 = "DSM3R CAN Dependence Crit. A2"
               mj3da3 = "DSM3R CAN Dependence Crit. A3"
               mj3da4 = "DSM3R CAN Dependence Crit. A4" 
               mj3da5 = "DSM3R CAN Dependence Crit. A5" 
               mj3da6 = "DSM3R CAN Dependence Crit. A6" 
               mj3da7 = "DSM3R CAN Dependence Crit. A7" 
               mj3da8 = "DSM3R CAN Dependence Crit. A8" 
               mj3da9 = "DSM3R CAN Dependence Crit. A9" 
               mj3dax = "DSM3R CAN Dependence Sum of As" 
               mj3da  = "DSM3R CAN Dependence 3+ As" 
               mj3db  = "DSM3R CAN Dependence Crit. B" 
               mj3rdp = "DSM3R CAN Dependence Diagnosis"
               mj3aa1 = "DSM3R CAN Abuse Crit. A1"
               mj3aa2 = "DSM3R CAN Abuse Crit. A2"
               mj3aa  = "DSM3R CAN Abuse A1 or A2"
               mj3ab  = "DSM3R CAN Abuse Crit. B"
               mj3ac  = "DSM3R CAN Abuse Crit. C"
               mj3rab = "DSM3R CAN Abuse Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id mj3rsk
                    mj3da1-mj3da9 mj3dax mj3da mj3db mj3rdp
                    mj3aa1 mj3aa2 mj3aa mj3ab mj3ac mj3rab;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_can_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(can3r,mj1a mj1c mj7a mj7b mj3rsk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(can3r,mj1c mj1c_1 mj7a mj7b mj3rsk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(can3r,mj3rsk mj10b mj10d mj3da1);
                     %multi(can3r,mj3rsk mj9b mj9c mj3da2);
                     %multi(can3r,mj3rsk mj11 mj3da3);
                     %multi(can3r,mj3rsk mj15d mj15f mj20f mj23b mj3da4);
                     %multi(can3r,mj3rsk mj12c mj3da5);
                     %multi(can3r,mj3rsk mj21a_3 mj21b_3 mj21c_3 mj21e
                                  mj22b mj24e mj3da6);
                     %multi(can3r,mj3rsk mj8a mj8b mj3da7);
                     %multi(can3r,mj3rsk mj29d mj3da8);
                     %multi(can3r,mj3rsk mj29c mj3da9);
                     %multi(can3r,mj3rsk mj3da1-mj3da9 mj3dax);
                     %multi(can3r,mj3rsk mj3dax mj3da);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(can3r,mj3rsk mj30b mj3da1 mj9b mj9d mj3da3-mj3da7
                                  mj29f mj29g mj3da9 mj3db);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(can3r,mj3rsk mj3da mj3db mj3rdp);

               *********;
               * Abuse *;
               *********;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(can3r,mj3rsk mj21a_3 mj21b_3 mj21c_3 mj21e mj3aa1);
                     %multi(can3r,mj3rsk mj15d mj15f mj23b mj3aa2);
                     %multi(can3r,mj3rsk mj3aa1 mj3aa2 mj3aa);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(can3r,mj3rsk mj15d mj15f mj21a_3 mj21b_3 mj21c_3 
                                  mj22b mj23b mj24e mj3ab);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(can3r,mj3rsk mj3rdp mj3ac);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(can3r,mj3rsk mj3aa mj3ab mj3ac mj3rab);

            ******************;
            * keep variables *;
            ******************;

               data can3r;
               set can3r;
               keep ind_id mj3rsk
                    mj3da1-mj3da9 mj3dax mj3da mj3db mj3rdp
                    mj3aa1 mj3aa2 mj3aa mj3ab mj3ac mj3rab;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=can3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend can3r;

********;
* test *;
********;
/*
   %can3r(coga.csaga2,1);
*/
