*****************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Conduct Disorder *;
*****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %cd3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro cd3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in CD3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in CD3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data cd3r;
      set &dsn.;

      attrib cd3ra1-cd3ra14 cd3ra3x cd3ra5x cd3ra cd3rb cd3rdx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************;
            * nonconfrontational, nontrivial theft *;
            ****************************************;

               if sum(sum(cd14a not in (1,3,5),cd14b not in (1,3,5))=2,
                      sum(cd14c<0,cd14c1 not in (1,2,3,4,5))=2,
                      cd14d not in (1,3,5),
                      sum(cd14e<0,cd14e1 not in (1,2,3,4,5))=2)=4 then 
                  cd3ra1=.;
               else do;
                  cd3ra1=1;
                  if (((cd14a=3 or cd14b=3) and 
                       (cd14c>1 or cd14c1 in (2,3,4,5))) or
                      (cd14d=3 and (cd14e>1 or cd14e1 in (2,3,4,5))) or
                      sum(cd14a=3,cd14b=3,cd14d=3)>1) then cd3ra1=3;
                  else if (((cd14a=5 or cd14b=5) and 
                            (cd14c>1 or cd14c1 in (2,3,4,5))) or
                           (cd14d=5 and (cd14e>1 or cd14e1 in (2,3,4,5))) or
                           sum(cd14a=5,cd14b=5,cd14d=5)>1) then cd3ra1=5;
               end;

            **********************;
            * run away from home *;
            **********************;

               if sum(cd7a not in (1,3,5),
                      cd7b not in (1,2,3),
                      sum(cd7c<0,cd7c1 not in (1,2,3,4,5))=2,
                      cd7d<0)=4 then cd3ra2=.;
               else do;
                  cd3ra2=1;
                  if ((cd7a=3 and cd7b=3) and 
                      (cd7c>1 or cd17c1 in (2,3,4,5) or cd7d ge 30)) then
                     cd3ra2=3;
                  else if ((cd7a=5 and cd7b=3) and 
                           (cd7c>1 or cd17c1 in (2,3,4,5) or cd7d ge 30)) then 
                     cd3ra2=5;
               end;

            **************;
            * often lies *;
            **************;

               if sum(sum(cd8a not in (1,3,5),cd8b not in (1,3,5))=2,
                      sum(cd8c_1 not in (1,5),cd8c_2 not in (1,5),
                          cd8c_3 not in (1,5))=3,
                      cd9a not in (1,3,5),
                      sum(cd9b<0,cd9b1 not in (1,2,3,4,5))=2,
                      cd10a not in (1,3,5))=5 then cd3ra3=.;
               else do;
                  cd3ra3=1;
                  if (((cd8a=3 or cd8b=3) and max(of cd8c_1-cd8c_3)=5) or
                      (cd9a=3 and (cd9b>2 or cd9b1 in (3,4,5))) or
                      (cd10a=3)) then cd3ra3=3;
                  else if (((cd8a=5 or cd8b=5) and max(of cd8c_1-cd8c_3)=5) or
                           (cd9a=5 and (cd9b>2 or cd9b1 in (3,4,5))) or
                           (cd10a=5)) then cd3ra3=5;
               end;

            *********************;
            * firesetting/arson *;
            *********************;

               if cd17a not in (1,3,5) then cd3ra4=.;
               else cd3ra4=(4*(cd17a=5))+(2*(cd17a=3))+1;

            ********************;
            * frequent truancy *;
            ********************;

               if sum(cd3a not in (1,3,5),
                      sum(cd3b<0,cd3b1 not in (1,2,3,4,5))=2,
                      cd4a not in (1,3,5),
                      sum(cd4b<0,cd4b1 not in (1,2,3,4,5))=2)=4 then cd3ra5=.;
               else do;
                  cd3ra5=1;
                  if ((cd3a=3 and (cd3b>2 or cd3b1 in (3,4,5))) or
                      (cd4a=3 and (cd4b>2 or cd4b1 in (3,4,5))) or
                      ((cd3a=3 and cd4a=3) and 
                       (sum(cd3b,cd4b)>2 or sum(cd3b1,cd4b1)>2))) then cd3ra5=3;
                  else if ((cd3a=5 and (cd3b>2 or cd3b1 in (3,4,5))) or
                           (cd4a=5 and (cd4b>2 or cd4b1 in (3,4,5))) or
                           ((cd3a=5 and cd4a=5) and 
                            (sum(cd3b,cd4b)>2 or sum(cd3b1,cd4b1)>2))) then 
                     cd3ra5=5;
               end;

            ***********************************;
            * committed breaking and entering *;
            ***********************************;

               if cd16a not in (1,3,5) then cd3ra6=.;
               else cd3ra6=(4*(cd16a=5))+(2*(cd16a=3))+1;

            *****************************;
            * non-firesetting vandalism *;
            *****************************;

               if cd18a not in (1,3,5) then cd3ra7=.;
               else cd3ra7=(4*(cd18a=5))+(2*(cd18a=3))+1;

            ****************************************;
            * exhibits physical cruelty to animals *;
            ****************************************;

               if cd19a not in (1,3,5) then cd3ra8=.;
               else cd3ra8=(4*(cd19a=5))+(2*(cd19a=3))+1;

            ****************************;
            * committed sexual assault *;
            ****************************;

               if cd24 not in (1,3,5) then cd3ra9=.;
               else cd3ra9=(4*(cd24=5))+(2*(cd24=3))+1;

            ********************************;
            * used a weapon more than once *;
            ********************************;

               if sum(cd22a not in (1,3,5),
                      sum(cd22b<0,cd22b1 not in (1,2,3,4,5))=2)=2 then 
                  cd3ra10=.;
               else do;
                  cd3ra10=1;
                  if (cd22a=3 and (cd22b>1 or cd22b1 in (2,3,4,5))) then
                     cd3ra10=3;
                  else if (cd22a=5 and (cd22b>1 or cd22b1 in (2,3,4,5))) then
                     cd3ra10=5;
               end;

            ***********************************;
            * often initiates physical fights *;
            ***********************************;

               if sum(sum(cd21a not in (1,3,5),
                          cd21b<0,
                          cd21b1 not in (1,2,3,4,5))=3,
                      sum(cd21f not in (1,3,5),
                          cd21g<0,
                          cd21g1 not in (1,2,3,4,5))=3)=2 then cd3ra11=.;
               else do;
                  cd3ra11=1;
                  if (cd21a=3 and (cd21b>2 or cd21b1 in (3,4,5))) or
                     (cd21f=3 and (cd21g>2 or cd21g1 in (3,4,5))) then
                     cd3ra11=3;
                  else if (cd21a=5 and (cd21b>2 or cd21b1 in (3,4,5))) or
                          (cd21f=5 and (cd21g>2 or cd21g1 in (3,4,5))) then
                     cd3ra11=5;
               end;

            ***********************************;
            * committed confrontational crime *;
            ***********************************;

               if cd15a not in (1,3,5) then cd3ra12=.;
               else cd3ra12=(4*(cd15a=5))+(2*(cd15a=3))+1;

            ***************************************;
            * exhibits physical cruelty to others *;
            ***************************************;

               if sum(cd23a not in (1,5),
                      cd23a1 not in (1,3,5))=2 then cd3ra13=.;
               else cd3ra13=(4*(cd23a=5 and cd23a1=1))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of cd3ra1-cd3ra13)=0 then do;
                  cd3ra3x=.;
                  cd3ra5x=.;
               end;
               else do;
                  cd3ra3x=0;
                  cd3ra5x=0;
                  array one cd3ra1-cd3ra13;
                  do over one;
                     if one=3 then cd3ra3x+1;
                     if one=5 then cd3ra5x+1;
                  end;
               end;

            *************************************;
            * behavior lasted at least 6 months *;
            *************************************;

               if cd26 not in (1,5) then cd3ra14=.;
               else cd3ra14=(4*(cd26=5))+1;
 
            *****************************************************;
            * at least three items and lasted at least 6 months *;
            *****************************************************;

               if n(cd3ra3x,cd3ra5x,cd3ra14)=0 then cd3ra=.;
               else do;
                  cd3ra=1;
                  if (cd3ra3x>0 and sum(cd3ra3x,cd3ra5x)>2 and 
                      cd3ra14=5) then cd3ra=3;
                  else if (cd3ra3x<1 and cd3ra5x>2 and cd3ra14=5) then 
                     cd3ra=5;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if dm3a<0 then cd3rb=.;
            else cd3rb=(4*(dm3a>17))+1;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(cd3ra,cd3rb)=0 then cd3rdx=.;
            else do;
               cd3rdx=1;
               if (cd3ra=3 and cd3rb=1) then cd3rdx=3;
               else if (cd3ra=5 and cd3rb=1) then cd3rdx=5;
            end;

      *******************;
      * variable labels *;
      *******************;

         label cd3ra1  = "DSM3R CD (Lifetime) Crit. A1"
               cd3ra2  = "DSM3R CD (Lifetime) Crit. A2"
               cd3ra3  = "DSM3R CD (Lifetime) Crit. A3"
               cd3ra4  = "DSM3R CD (Lifetime) Crit. A4"
               cd3ra5  = "DSM3R CD (Lifetime) Crit. A5"
               cd3ra6  = "DSM3R CD (Lifetime) Crit. A6"
               cd3ra7  = "DSM3R CD (Lifetime) Crit. A7"
               cd3ra8  = "DSM3R CD (Lifetime) Crit. A8"
               cd3ra9  = "DSM3R CD (Lifetime) Crit. A9"
               cd3ra10 = "DSM3R CD (Lifetime) Crit. A10"
               cd3ra11 = "DSM3R CD (Lifetime) Crit. A11"
               cd3ra12 = "DSM3R CD (Lifetime) Crit. A12"
               cd3ra13 = "DSM3R CD (Lifetime) Crit. A13"
               cd3ra3x = "DSM3R CD (Lifetime) Sum of As=3"
               cd3ra5x = "DSM3R CD (Lifetime) Sum of As=5"
               cd3ra14 = "DSM3R CD (Lifetime) Crit. A14"
               cd3ra   = "DSM3R CD (Lifetime) Crit. A"
               cd3rb   = "DSM3R CD (Lifetime) Crit. B"
               cd3rdx  = "DSM3R CD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    cd3ra1-cd3ra14 cd3ra3x cd3ra5x cd3ra cd3rb cd3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_cd_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cd3r,cd14a cd14b cd14c cd14c1 cd14d cd14e cd14e1
                                 cd3ra1);
                     %multi(cd3r,cd7a cd7b cd7c cd7c1 cd7d cd3ra2);
                     %multi(cd3r,cd8a cd8b cd8c_1-cd8c_3
                                 cd9a cd9b cd9b1 cd10a cd3ra3);
                     %multi(cd3r,cd17a cd3ra4);
                     %multi(cd3r,cd3a cd3b cd3b1 cd4a cd4b cd4b1 cd3ra5);
                     %multi(cd3r,cd16a cd3ra6);
                     %multi(cd3r,cd18a cd3ra7);
                     %multi(cd3r,cd19a cd3ra8);
                     %multi(cd3r,cd24 cd3ra9);
                     %multi(cd3r,cd22a cd22b cd22b1 cd3ra10);
                     %multi(cd3r,cd21a cd21b cd21b1 cd21f cd21g cd21g1 cd3ra11);
                     %multi(cd3r,cd15a cd3ra12);
                     %multi(cd3r,cd23a cd23a1 cd3ra13);
                     %multi(cd3r,cd3ra1-cd3ra13 cd3ra3x cd3ra5x);
                     %multi(cd3r,cd26 cd3ra14);
                     %multi(cd3r,cd3ra3x cd3ra5x cd3ra14 cd3ra);
                     

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cd3r,dm3a cd3rb);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cd3r,cd3ra cd3rb cd3rdx);

            ******************;
            * keep variables *;
            ******************;

               data cd3r;
               set cd3r;
               keep ind_id  
                    cd3ra1-cd3ra14 cd3ra3x cd3ra5x cd3ra cd3rb cd3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=cd3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend cd3r;

********;
* test *;
********;
/*
   %cd3r(coga.csaga2,1);
*/
