*******************************************************************;
* CSSAGA-II Diagnostic Algorithm -- DSM-III-R -- Major Depression *;
*******************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %dep3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro dep3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DEP3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DEP3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data dep3r;
      set &dsn.;

      attrib dc3rsk dp3rsk md3rsk
             dc3ra1-dc3ra9 dc3rax dc3ra
             dp3ra1-dp3ra9 dp3rax dp3ra
             dc3rb1-dc3rb3 dc3rb
             dp3rb1-dp3rb3 dp3rb
             dc3rdx dp3rdx 
             md3ra md3rb md3rdx length=3;

      *********************;
      * test for skip out *;
      *********************;

         *******************;
         * current episode *;
         *******************;

            if sum(dp1c_1 not in (1,5),
                   dp1c_2 not in (1,5),
                   dp1c_3 not in (1,5),
                   dp1c_4 not in (1,5),
                   dp1c_5 not in (1,5))=5 then dc3rsk=5;
            else do;
               if sum(dp1c_1=5,dp1c_2=5,dp1c_3=5,dp1c_4=5,dp1c_5=5)>0 then 
                  dc3rsk=1;
               else dc3rsk=5;
            end;

            if dc3rsk=5 then do;
                array skp1 dc3ra1-dc3ra9 dc3rax dc3ra
                           dc3rb1-dc3rb3 dc3rb dc3rdx;
                do over skp1;
                  skp1=.S;
                end;
            end;

         ****************************;
         * most severe past episode *;
         ****************************;

            if sum(dp7c_1 not in (1,5),
                   dp7c_2 not in (1,5),
                   dp7c_3 not in (1,5),
                   dp7c_4 not in (1,5),
                   dp7c_5 not in (1,5))=5 then dp3rsk=5;
            else do;
               if sum(dp7c_1=5,dp7c_2=5,dp7c_3=5,dp7c_4=5,dp7c_5=5)>0 then 
                  dp3rsk=1;
               else dp3rsk=5;
            end;

            if dp3rsk=5 then do;
                array skp2 dp3ra1-dp3ra9 dp3rax dp3ra
                           dp3rb1-dp3rb3 dp3rb dp3rdx;
                do over skp2;
                  skp2=.S;
                end;
            end;

         *****************;
         * lifetime ever *;
         *****************;

            if (dc3rsk=5 and dp3rsk=5) then md3rsk=5;
            else md3rsk=1;

            if md3rsk=5 then do;
               md3ra=.S;
               md3rb=.S;
               md3rdx=.S;
            end;

            if md3rsk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         if dc3rsk=1 then do;

         *******************;
         * current episode *;
         *******************;

            *******************************;
            * depressed mood/irritability *;
            *******************************;

               if sum((dp1c_1 not in (1,5)),
                      (dp1c_2 not in (1,5)),
                      (dp1c_5 not in (1,5)))=3 then dc3ra1=.;
               else dc3ra1=sum((dp1c_1=5),(dp1c_2=5),(dp1c_5=5))>0;

            *************;
            * anhedonia *;
            *************;

               if sum((dp1c_3 not in (1,5)),
                      (dp1c_4 not in (1,5)))=2 then dc3ra2=.;
               else dc3ra2=sum((dp1c_3=5),(dp1c_4=5))>0;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if sum((dp19a_1 not in (1,5)),
                      (dp19a1_1 not in (1,5)),
                      (dp20a_1 not in (1,5)),
                      (dp20a1_1 not in (1,5)))=4 then dc3ra3=.;
               else dc3ra3=sum((dp19a_1=5),(dp19a1_1=5),
                               (dp20a_1=5),(dp20a1_1=5))>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dp21a_1 not in (1,5)),
                      (dp21b_1 not in (1,5)),
                      (dp21c_1 not in (1,5)),
                      (dp21d_1 not in (1,5)))=4 then dc3ra4=.;
               else dc3ra4=sum((dp21a_1=5),(dp21b_1=5),
                               (dp21c_1=5),(dp21d_1=5))>0;

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;

               if sum((dp22_1 not in (1,5)),
                      (dp23_1 not in (1,5)))=2 then dc3ra5=.;
               else dc3ra5=sum((dp22_1=5),(dp23_1=5))>0;
   
            **************************;
            * fatigue/loss of energy *;
            **************************;

               if (dp1c_6 not in (1,5)) then dc3ra6=.;
               else dc3ra6=(dp1c_6=5);

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if sum((dp24_1 not in (1,5)),
                      (dp25_1 not in (1,5)),
                      (dp25a_1 not in (1,5)))=3 then dc3ra7=.;
               else dc3ra7=sum((dp24_1=5),(dp25_1=5),(dp25a_1=5))>0;

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dp26a_1 not in (1,5)),
                      (dp26b_1 not in (1,5)),
                      (dp27a_1 not in (1,5)),
                      (dp27b_1 not in (1,5)))=4 then dc3ra8=.;
               else dc3ra8=sum((dp26a_1=5),(dp26b_1=5),
                               (dp27a_1=5),(dp27b_1=5))>0;

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if sum((dp28a_1 not in (1,5)),
                      (dp28b_1 not in (1,5)),
                      (dp28c_1 not in (1,5)),
                      (dp28d_1 not in (1,5)))=4 then dc3ra9=.;
               else dc3ra9=sum((dp28a_1=5),(dp28b_1=5),
                               (dp28c_1=5),(dp28d_1=5))>0;

            ***********************************************************;
            * at least 5 from a1..a9 including at least 1 from a1..a2 *;
            ***********************************************************;

               if n(of dc3ra1-dc3ra9)=0 then dc3rax=.;
               else dc3rax=(n(of dc3ra1-dc3ra9)>0 and 
                            (dc3ra1 or dc3ra2) and 
                            sum(of dc3ra1-dc3ra9)>=5);

            ***************************************;
            * sxs occured in same two-week period *;
            ***************************************;

               if (dp30a_1 not in (1,5)) then dc3ra=.;
               else if (dp30a_1=1) then dc3ra=0;
               else if (dp30a_1=5) then do;
                  if sum((dc3rax not in (0,1)),
                         (dp30a1_1 not in (1,5)),
                         (dp30b_1 not in (1,5)))>0 then dc3ra=.;
                  else dc3ra=((dc3rax=1) and
                              (sum((dp30a1_1=5),(dp30b_1=5))=2));
               end;

         end;
         if dp3rsk=1 then do;

         ****************************;
         * most severe past episode *;
         ****************************;

            *******************************;
            * depressed mood/irritability *;
            *******************************;

               if sum((dp7c_1 not in (1,5)),
                      (dp7c_2 not in (1,5)),
                      (dp7c_5 not in (1,5)),
                      (dp14b not in (1,5)))=4 then dp3ra1=.;
               else dp3ra1=sum((dp7c_1=5),(dp7c_2=5),(dp7c_5=5),(dp14b=5))>0;

            *************;
            * anhedonia *;
            *************;

               if sum((dp7c_3 not in (1,5)),
                      (dp7c_4 not in (1,5)),
                      (dp14c not in (1,5)))=3 then dp3ra2=.;
               else dp3ra2=sum((dp7c_3=5),(dp7c_4=5),(dp14c=5))>0;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;
  
               if sum((dp19a_2 not in (1,5)),
                      (dp19a1_2 not in (1,5)),
                      (dp20a_2 not in (1,5)),
                      (dp20a1_2 not in (1,5)))=4 then dp3ra3=.;
               else dp3ra3=sum((dp19a_2=5),(dp19a1_2=5),
                               (dp20a_2=5),(dp20a1_2=5))>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dp21a_2 not in (1,5)),
                      (dp21b_2 not in (1,5)),
                      (dp21c_2 not in (1,5)),
                      (dp21d_2 not in (1,5)))=4 then dp3ra4=.;
               else dp3ra4=sum((dp21a_2=5),(dp21b_2=5),
                               (dp21c_2=5),(dp21d_2=5))>0;

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;
  
               if sum((dp22_2 not in (1,5)),
                      (dp23_2 not in (1,5)))=2 then dp3ra5=.;
               else dp3ra5=sum((dp22_2=5),(dp23_2=5))>0;

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if sum((dp7c_6 not in (1,5)),
                      (dp14d not in (1,5)))=2 then dp3ra6=.;
               else dp3ra6=sum((dp7c_6=5),(dp14d=5))>0;

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if sum((dp24_2 not in (1,5)),
                      (dp25_2 not in (1,5)),
                      (dp25a_2 not in (1,5)))=3 then dp3ra7=.;
               else dp3ra7=sum((dp24_2=5),(dp25_2=5),(dp25a_2=5))>0;

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dp26a_2 not in (1,5)),
                      (dp26b_2 not in (1,5)),
                      (dp27a_2 not in (1,5)),
                      (dp27b_2 not in (1,5)))=4 then dp3ra8=.;
               else dp3ra8=sum((dp26a_2=5),(dp26b_2=5),
                               (dp27a_2=5),(dp27b_2=5))>0;

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if sum((dp28a_2 not in (1,5)),
                      (dp28b_2 not in (1,5)),
                      (dp28c_2 not in (1,5)),
                      (dp28d_2 not in (1,5)))=4 then dp3ra9=.;
               else dp3ra9=sum((dp28a_2=5),(dp28b_2=5),
                               (dp28c_2=5),(dp28d_2=5))>0;

            ***********************************************************;
            * at least 5 from a1..a9 including at least 1 from a1..a2 *;
            ***********************************************************;

               if n(of dp3ra1-dp3ra9)=0 then dp3rax=.;
               else dp3rax=(n(of dp3ra1-dp3ra9)>0 and 
                            (dp3ra1 or dp3ra2) and 
                            sum(of dp3ra1-dp3ra9)>=5);

            ***************************************;
            * sxs occured in same two-week period *;
            ***************************************;

               if (dp30a_2 not in (1,5)) then dp3ra=.;
               else if (dp30a_2=1) then dp3ra=0;
               else if (dp30a_2=5) then do;
                  if sum((dp3rax not in (0,1)),
                         (dp30a1_2 not in (1,5)),
                         (dp30b_2 not in (1,5)))>0 then dp3ra=.;
                  else dp3ra=((dp3rax=1) and 
                              (sum((dp30a1_2=5),(dp30b_2=5))=2));
               end;

         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dc3ra,dp3ra)=0 then md3ra=.;
            else md3ra=(dc3ra or dp3ra);

         ***************************;
         * rescale from 0,1 to 1,5 *;
         ***************************;

            array cea dc3ra1-dc3ra9 dc3rax dc3ra;
            array msa dp3ra1-dp3ra9 dp3rax dp3ra;
            do over cea;
               if (cea ne . and cea ne .S) then cea=((4*cea)+1);
               if (msa ne . and msa ne .S) then msa=((4*msa)+1);
            end;
            md3ra=((4*md3ra)+1);

      ***************;
      * Criterion B *;
      ***************;

         if dc3rsk=1 then do;

         *******************;
         * current episode *;
         *******************;

            ***************;
            * alc/drg/med *;
            ***************;

               if sum((dp4d not in (1,5)),(dp4g not in (1,5)),
                      (sum((dp5a_1 not in (1,5)),(dp5a_2 not in (1,5)),
                           (dp5a_3 not in (1,5)),(dp5a_4 not in (1,5)),
                           (dp5a_5 not in (1,5)),(dp5a_6 not in (1,5)))=6 or
                       sum((dp5b_1 not in (1,5)),(dp5b_2 not in (1,5)),
                           (dp5b_3 not in (1,5)),(dp5b_4 not in (1,5)), 
                           (dp5b_5 not in (1,5)),(dp5b_6 not in (1,5)))=6),
                       (dp6a not in (1,5)),(dp31c_1 not in (1,5)))=5 then 
                  dc3rb1=.;
               else dc3rb1=((dp4d=5 or dp4g=5) or
                            (sum((dp5a_1=5),(dp5a_2=5),(dp5a_3=5),
                                 (dp5a_4=5),(dp5a_5=5),(dp5a_6=5))>0 or
                             sum((dp5b_1=5),(dp5b_2=5),(dp5b_3=5),
                                 (dp5b_4=5),(dp5b_5=5),(dp5b_6=5))>0) or
                            (dp6a=5) or (dp31c_1=5));

            ***************;
            * bereavement *;
            ***************;

               if sum((dp31a_1 not in (1,5)),
                      (dp31a1_1 not in (1,5)))=2 then dc3rb2=.;
               else dc3rb2=((dp31a_1=5) or (dp31a1_1=5));
/*
               if ((dp31a_1=5) and (0<dp30d_1<9)) then dc3rb2=0;
*/
               if ((dp31a_1=5) and (dp30d_1>12)) then dc3rb2=0;

            ******************;
            * illness/injury *;
            ******************;

               if dp31b_1 not in (1,5) then dc3rb3=.;
               else dc3rb3=(dp31b_1=5);

            if n(of dc3rb1-dc3rb3)=0 then dc3rb=.;
            else dc3rb=max(of dc3rb1-dc3rb3)=0;

         end;
         if dp3rsk=1 then do;

         ****************************;
         * most severe past episode *;
         ****************************;

            ***************;
            * alc/drg/med *;
            ***************;

               if sum((dp11d not in (1,5)),(dp11g not in (1,5)),
                      (sum((dp12a_1 not in (1,5)),(dp12a_2 not in (1,5)),
                           (dp12a_3 not in (1,5)),(dp12a_4 not in (1,5)),
                           (dp12a_5 not in (1,5)),(dp12a_6 not in (1,5)))=6 or
                       sum((dp12b_1 not in (1,5)),(dp12b_2 not in (1,5)),
                           (dp12b_3 not in (1,5)),(dp12b_4 not in (1,5)), 
                           (dp12b_5 not in (1,5)),(dp12b_6 not in (1,5)))=6),
                       (dp13a not in (1,5)),(dp31c_2 not in (1,5)))=5 then
                  dp3rb1=.;
               else dp3rb1=((dp11d=5 or dp11g=5) or
                            (sum((dp12a_1=5),(dp12a_2=5),(dp12a_3=5),
                                 (dp12a_4=5),(dp12a_5=5),(dp12a_6=5))>0 or
                             sum((dp12b_1=5),(dp12b_2=5),(dp12b_3=5),
                                 (dp12b_4=5),(dp12b_5=5),(dp12b_6=5))>0) or
                            (dp13a=5) or (dp31c_2=5));
               if (dp14a=5 and dp14a1=5 and 
                   dp14a2=5 and dp31c_2=1) then dp3rb1=0;

            ***************;
            * bereavement *;
            ***************;

               if sum((dp31a_2 not in (1,5)),
                      (dp31a1_2 not in (1,5)))=2 then dp3rb2=.;
               else dp3rb2=((dp31a_2=5) or (dp31a1_2=5));
/*
               if ((dp31a_2=5) and (0<dp30d_2<9)) then dp3rb2=0;
*/
               if ((dp31a_2=5) and (dp30d_2>12)) then dp3rb2=0;

            ******************;
            * illness/injury *;
            ******************;

               if dp31b_2 not in (1,5) then dp3rb3=.;
               else dp3rb3=(dp31b_2=5);

            if n(of dp3rb1-dp3rb3)=0 then dp3rb=.;
            else dp3rb=max(of dp3rb1-dp3rb3)=0;
  
         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dc3rb,dp3rb)=0 then md3rb=.;
            else md3rb=(dc3rb or dp3rb);

         ***************************;
         * rescale from 0,1 to 1,5 *;
         ***************************;

            array ceb dc3rb1-dc3rb3 dc3rb;
            array msb dp3rb1-dp3rb3 dp3rb;
            do over ceb;
               if (ceb ne . and ceb ne .S) then ceb=((4*ceb)+1);
               if (msb ne . and msb ne .S) then msb=((4*msb)+1);
            end;
            md3rb=((4*md3rb)+1);

      *******************************;
      * Criterion C -- Not assessed *;
      *******************************;

      **************************************;
      * Criterion D -- Skip in this module *;
      **************************************;

      *************;
      * Diagnosis *;
      *************;

         if dc3rsk=1 then do;

            *******************;
            * current episode *;
            *******************;

               if (dc3ra not in (1,5)) then dc3rdx=.;
               else do;
                  if (dc3rb not in (1,5)) then dc3rdx=.;
                  else if dc3ra=1 then dc3rdx=1;
                  else if dc3ra=5 then do;
                     if dc3rb=5 then dc3rdx=5;
                     else if dc3rb=1 then do;
                        if dc3rb1=5 then dc3rdx=4;
                        else if dc3rb2=5 then dc3rdx=3;
                        else if dc3rb3=5 then dc3rdx=2;
                     end;
                  end;
               end;

         end;
         if dp3rsk=1 then do;

            ****************************;
            * most severe past episode *;
            ****************************;

               if (dp3ra not in (1,5)) then dp3rdx=.;
               else do;
                  if (dp3rb not in (1,5)) then dp3rdx=.;
                  else if dp3ra=1 then dp3rdx=1;
                  else if dp3ra=5 then do;
                     if dp3rb=5 then dp3rdx=5;
                     else if dp3rb=1 then do;
                        if dp3rb1=5 then dp3rdx=4;
                        else if dp3rb2=5 then dp3rdx=3;
                        else if dp3rb3=5 then dp3rdx=2;
                     end;
                  end;
               end;

         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if ((md3ra not in (1,5)) and (md3rb not in (1,5))) then md3rdx=.;
            else do;
               if ((md3ra=5) and (md3rb=5)) then md3rdx=5;
               else md3rdx=1;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label dc3rsk = "DSM3R MDD (Current Episode) Skip Out"
               dp3rsk = "DSM3R MDD (Worst Episode) Skip Out"
               md3rsk = "DSM3R MDD (Lifetime) Skip Out"
               dc3ra1 = "DSM3R MDD (Current Episode) Crit. A1"
               dc3ra2 = "DSM3R MDD (Current Episode) Crit. A2"
               dc3ra3 = "DSM3R MDD (Current Episode) Crit. A3"
               dc3ra4 = "DSM3R MDD (Current Episode) Crit. A4"
               dc3ra5 = "DSM3R MDD (Current Episode) Crit. A5"
               dc3ra6 = "DSM3R MDD (Current Episode) Crit. A6"
               dc3ra7 = "DSM3R MDD (Current Episode) Crit. A7"
               dc3ra8 = "DSM3R MDD (Current Episode) Crit. A8"
               dc3ra9 = "DSM3R MDD (Current Episode) Crit. A9"
               dc3rax = "DSM3R MDD (Current) 5+ A (inc A1 or A2)"
               dc3ra  = "DSM3R MDD (Current Episode) Criterion A"
               dp3ra1 = "DSM3R MDD (Worst Past Episode) Crit. A1"
               dp3ra2 = "DSM3R MDD (Worst Past Episode) Crit. A2"
               dp3ra3 = "DSM3R MDD (Worst Past Episode) Crit. A3"
               dp3ra4 = "DSM3R MDD (Worst Past Episode) Crit. A4"
               dp3ra5 = "DSM3R MDD (Worst Past Episode) Crit. A5"
               dp3ra6 = "DSM3R MDD (Worst Past Episode) Crit. A6"
               dp3ra7 = "DSM3R MDD (Worst Past Episode) Crit. A7"
               dp3ra8 = "DSM3R MDD (Worst Past Episode) Crit. A8"
               dp3ra9 = "DSM3R MDD (Worst Past Episode) Crit. A9"
               dp3rax = "DSM3R MDD (Wst Past) 5+ A (inc A1 or A2)"
               dp3ra  = "DSM3R MDD (Wst Past) Criterion A"
               dc3rb1 = "DSM3R MDD (Current) Crit B - alc/drg/med"
               dc3rb2 = "DSM3R MDD (Current) Crit B - bereavement"
               dc3rb3 = "DSM3R MDD (Current) Crit B - illness"
               dc3rb  = "DSM3R MDD (Current) Criterion B (clean)"
               dp3rb1 = "DSM3R MDD (Wst Pst) Crit B - alc/drg/med"
               dp3rb2 = "DSM3R MDD (Wst Pst) Crit B - bereavement"
               dp3rb3 = "DSM3R MDD (Wst Past) Crit B - illness"
               dp3rb  = "DSM3R MDD (Wst Past) Criterion B (clean)"
               dc3rdx = "DSM3R MDD (Current Episode) Diagnosis"
               dp3rdx = "DSM3R MDD (Worst Past Episode) Diagnosis"
               md3ra  = "DSM3R MDD (Lifetime) Criterion A"
               md3rb  = "DSM3R MDD (Lifetime) Criterion B (clean)"
               md3rdx = "DSM3R MDD (Lifetime) Diagnosis (clean)";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    dc3rsk dp3rsk md3rsk
                    dc3ra1-dc3ra9 dc3rax dc3ra
                    dp3ra1-dp3ra9 dp3rax dp3ra
                    dc3rb1-dc3rb3 dc3rb
                    dp3rb1-dp3rb3 dp3rb
                    dc3rdx dp3rdx 
                    md3ra md3rb md3rdx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_dep_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(dep3r,dp1c_1 dp1c_2 dp1c_3 dp1c_4 dp1c_5
                                  dc3rsk);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                    %multi(dep3r,dp7c_1 dp7c_2 dp7c_3 dp7c_4 dp7c_5
                                 dp3rsk);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dep3r,dc3rsk dp3rsk md3rsk);

               ***************;
               * Criterion A *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(dep3r,dc3rsk dp1c_1 dp1c_2 dp1c_5 dc3ra1);
                     %multi(dep3r,dc3rsk dp1c_3 dp1c_4 dc3ra2);
                     %multi(dep3r,dc3rsk dp19a_1 dp19a1_1 dp20a_1 dp20a1_1 
                                  dc3ra3);
                     %multi(dep3r,dc3rsk dp21a_1 dp21b_1 dp21c_1 dp21d_1 dc3ra4);
                     %multi(dep3r,dc3rsk dp22_1 dp23_1 dc3ra5);
                     %multi(dep3r,dc3rsk dp1c_6 dc3ra6);
                     %multi(dep3r,dc3rsk dp24_1 dp25_1 dp25a_1 dc3ra7);
                     %multi(dep3r,dc3rsk dp26a_1 dp26b_1 dp27a_1 dp27b_1 dc3ra8);
                     %multi(dep3r,dc3rsk dp28a_1 dp28b_1 dp28c_1 dp28d_1 dc3ra9);
                     %multi(dep3r,dc3rsk dc3ra1-dc3ra9 dc3rax);
                     %multi(dep3r,dc3rsk dc3rax dp30a_1 dp30a1_1 dp30b_1 dc3ra);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(dep3r,dp3rsk dp7c_1 dp7c_2 dp7c_5 dp14b dp3ra1);
                     %multi(dep3r,dp3rsk dp7c_3 dp7c_4 dp14c dp3ra2);
                     %multi(dep3r,dp3rsk dp19a_2 dp19a1_2 dp20a_2 dp20a1_2 
                                  dp3ra3);
                     %multi(dep3r,dp3rsk dp21a_2 dp21b_2 dp21c_2 dp21d_2 dp3ra4);
                     %multi(dep3r,dp3rsk dp22_2 dp23_2 dp3ra5);
                     %multi(dep3r,dp3rsk dp7c_6 dp14d dp3ra6);
                     %multi(dep3r,dp3rsk dp24_2 dp25_2 dp25a_2 dp3ra7);
                     %multi(dep3r,dp3rsk dp26a_2 dp26b_2 dp27a_2 dp27b_2 dp3ra8);
                     %multi(dep3r,dp3rsk dp28a_2 dp28b_2 dp28c_2 dp28d_2 dp3ra9);
                     %multi(dep3r,dp3rsk dp3ra1-dp3ra9 dp3rax);
                     %multi(dep3r,dp3rsk dp3rax dp30a_2 dp30a1_2 dp30b_2 dp3ra);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dep3r,md3rsk dc3ra dp3ra md3ra);

               ***************;
               * Criterion B *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(dep3r,dc3rsk dp4d dp4g dp5a_1-dp5a_6 dp5b_1-dp5b_6
                                  dp6a dp31c_1 dc3rb1);
                     %multi(dep3r,dc3rsk dp31a_1 dp31a1_1 dp30d_1 dc3rb2);
                     %multi(dep3r,dc3rsk dp31b_1 dc3rb3);
                     %multi(dep3r,dc3rsk dc3rb1 dc3rb2 dc3rb3 dc3rb);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(dep3r,dp3rsk dp11d dp11g dp12a_1-dp12a_6 dp12b_1-dp12b_6 
                                  dp13a dp31c_2 dp14a dp14a1 dp14a2 dp3rb1);
                     %multi(dep3r,dp3rsk dp31a_2 dp31a1_2 dp30d_2 dp3rb2);
                     %multi(dep3r,dp3rsk dp31b_2 dp3rb3);
                     %multi(dep3r,dp3rsk dp3rb1 dp3rb2 dp3rb3 dp3rb);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dep3r,md3rsk dc3rb dp3rb md3rb);

               *************;
               * Diagnosis *;
               *************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(dep3r,dc3rsk dc3ra dc3rb dc3rdx);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(dep3r,dp3rsk dp3ra dp3rb dp3rdx);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dep3r,md3rsk md3ra md3rb md3rdx);

            ******************;
            * keep variables *;
            ******************;

               data dep3r;
               set dep3r;
               keep ind_id  
                    dc3rsk dp3rsk md3rsk
                    dc3ra1-dc3ra9 dc3rax dc3ra
                    dp3ra1-dp3ra9 dp3rax dp3ra
                    dc3rb1-dc3rb3 dc3rb
                    dp3rb1-dp3rb3 dp3rb
                    dc3rdx dp3rdx 
                    md3ra md3rb md3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=dep3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend dep3r;

********;
* test *;
********;
/*
   %dep3r(coga.csaga2,1);
*/
