*******************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Substance Abuse and Dependence *;
*******************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %drg3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro drg3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DRG3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DRG3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      %let drglst=co st op ha se ot;
      %let wrdlst=COC AMP OPI HAL SED OTH;

      data drg3r;
      set &dsn.;

      %do i=1 %to 6;

         %let drg=%scan(&drglst.,&i.);
         %let wrd=%scan(&wrdlst.,&i.);

         attrib &drg.3rsk
                &drg.3da1-&drg.3da9 &drg.3dax &drg.3da &drg.3db &drg.3rdp
                &drg.3aa1 &drg.3aa2 &drg.3aa &drg.3ab &drg.3ac &drg.3rab 
                length=3;

         *********************;
         * test for skip out *;
         *********************;

            %if &study.=irpg or &who.=c %then %do;
               if sum(dr1a_&i. not in (1,5),
                      dr1e1_&i. not in (1,5),
                      dr5a_&i.<0,
                      dr5a1_&i. not in (0,1,2,3,4))=4 then &drg.3rsk=5;
               else do;
                  if (dr1a_&i.=5 and dr1e1_&i.=5 and 
                      (dr5a_&i.>6 or dr5a1_&i. in (1,2,3,4))) then &drg.3rsk=1;
                  else &drg.3rsk=5;
               end;
            %end;
            %if &study.=coga and &who.=p %then %do;
               if sum(dr1e1_&i. not in (1,5),
                      dr5a_&i.<0,
                      dr5a1_&i. not in (0,1,2,3,4))=4 then &drg.3rsk=5;
               else do;
                  if (dr1e1_&i.=5 and 
                      (dr5a_&i.>6 or dr5a1_&i. in (1,2,3,4))) then &drg.3rsk=1;
                  else &drg.3rsk=5;
               end;
            %end;

            if &drg.3rsk=5 then do;
                array skp&i. &drg.3da1-&drg.3da9 &drg.3dax &drg.3da &drg.3db 
                             &drg.3rdp &drg.3aa1 &drg.3aa2 &drg.3aa &drg.3ab 
                             &drg.3ac &drg.3rab;
                do over skp&i.;
                   skp&i.=.S;
                end;
            end;

         **********************************;
         * test for coga parent interview *;
         **********************************;
   
            %if &study.=coga %then %do;
               %if &who.=p %then %do;
                  if &drg.3rsk=1 then do;
                     array nas&i. &drg.3da1-&drg.3da3 &drg.3da7-&drg.3da9;
                     do over nas&i.;
                        nas&i.=.N;
                     end;
                  end;
                  if &drg.3rsk=5 then go to skipout&i.;
                  else go to parhop1&i.;
               %end;
            %end;

         if &drg.3rsk=5 then go to skipout&i.;

         **************;
         * Dependence *;
         **************;

            ***************;
            * Criterion A *;
            ***************;

               *********************************************************;
               * used larger amount or for longer period than intended *;
               *********************************************************;

                  if sum(dr13b_&i. not in (1,5),
                         dr13d_&i. not in (1,5))=2 then &drg.3da1=.;
                  else &drg.3da1=(4*(dr13b_&i.=5 or dr13d_&i.=5))+1;
   
               ***************************************************************;
               * persistent desire or 1+ unsuccessful efforts to control use *;
               ***************************************************************;

                  if sum(dr12a_&i. not in (1,5),
                         dr12b_&i. not in (1,5))=2 then &drg.3da2=.;
                  else &drg.3da2=(4*(dr12a_&i.=5 or dr12b_&i.=1))+1;

               ****************************************************************;
               * great deal of time spent obtaining/using/recovering from use *;
               ****************************************************************;

                  if dr14_&i. not in (1,5) then &drg.3da3=.;
                  else &drg.3da3=(4*(dr14_&i.=5))+1;

               **************************************;
               * jump here if coga parent interview *;
               **************************************;

                  parhop1&i.:

               *******************************************************;
               * intoxication/withdrawal interferes with obligations *;
               *******************************************************;

                  if sum(dr18d_&i. not in (1,5),
                         dr18f_&i. not in (1,5),
                         dr24f_&i. not in (1,5),
                         dr27b_&i. not in (1,5))=4 then &drg.3da4=.;
                  else &drg.3da4=(4*(sum(dr18d_&i.=5,dr18f_&i.=5,
                                         dr24f_&i.=4,dr27b_&i.=5)>0))+1;

               ***********************************************************;
               * use reduced social/occupational/recreational activities *;
               ***********************************************************;

                  if dr15c_&i. not in (1,5) then &drg.3da5=.;
                  else &drg.3da5=(4*(dr15c_&i.=5))+1;

               **********************************************************;
               * use despite use-derived social/pysch/physical problems *;
               **********************************************************;

                  if sum(dr25e_&i. not in (1,5),
                         dr26b_&i. not in (1,5),
                         dr28e_&i. not in (1,5))=3 then &drg.3da6=.;
                  else &drg.3da6=(4*(sum(dr25e_&i.=5,dr26b_&i.=5,
                                         dr28e_&i.=5)>0))+1;

               **********************************;
               * test for coga parent interview *;
               **********************************;

                  %if &study.=coga %then %do;
                     %if &who.=p %then %do;
                        go to parhop2&i.;
                     %end;
                  %end;

               ********************;
               * marked tolerance *;
               ********************;

                  if sum(dr11a_&i. not in (1,5),
                         dr11b_&i. not in (1,5))=2 then &drg.3da7=.;
                  else &drg.3da7=(4*(dr11a_&i.=5 or dr11b_&i.=5))+1;

               **************************************;
               * characteristic withdrawal symptoms *;
               **************************************;

                  if 0<dm3a<13 then do;
                     &drg.3da8=.N;
                  end;
                  else do;
                     if dr33b1_&i. not in (1,5) then &drg.3da8=.;
                     else &drg.3da8=(4*(dr33b1_&i.=5))+1;
                  end;

               ********************************************;
               * use to relieve/avoid withdrawal symptoms *;
               ********************************************;

                  if 0<dm3a<13 then do;
                     &drg.3da8=.N;
                  end;
                  else do;
                     if dr33d2_&i. not in (1,5) then &drg.3da9=.;
                     else &drg.3da9=(4*(dr33d2_&i.=5))+1;
                  end;

               **************************************;
               * jump here if coga parent interview *;
               **************************************;

                  parhop2&i.:

               ****************;
               * sum of items *;
               ****************;

                  if n(of &drg.3da1-&drg.3da9)=0 then &drg.3dax=.;
                  else do;
                     &drg.3dax=0;
                     array sxs&i. &drg.3da1-&drg.3da9;
                     do over sxs&i.;
                        if sxs&i.=5 then &drg.3dax+1;
                     end;
                  end;

               ************************;
               * at least three items *;
               ************************;

                  if &drg.3dax=. then &drg.3da=.;
                  else &drg.3da=(4*(&drg.3dax>2))+1;

            ***************;
            * Criterion B *;
            ***************;

               if sum(dr34b_&i. not in (1,5),
                      sum(&drg.3da1 not in (1,5),
                          sum(dr12a_&i. not in (1,5),
                              dr12c_&i. not in (1,5))=2,
                          &drg.3da3 not in (1,5),
                          &drg.3da4 not in (1,5),
                          &drg.3da5 not in (1,5),
                          &drg.3da6 not in (1,5),
                          &drg.3da7 not in (1,5),
                          sum(dr33b1_&i. not in (1,5),
                              dr33c_&i. not in (1,5))>0,
                          &drg.3da9 not in (1,5))=9)=2 then &drg.3db=.;
               else &drg.3db=(4*((dr34b_&i.=5) or 
                                 sum(&drg.3da1=5,
                                     sum(dr12a_&i.=5,dr12c_&i.=5)>0,
                                     &drg.3da3=5,
                                     &drg.3da4=5,
                                     &drg.3da5=5,
                                     &drg.3da6=5,
                                     &drg.3da7=5,
                                     sum(dr33b1_&i.=5,dr33c_&i.=5)=2,
                                     &drg.3da9=5)>2))+1;
                      
            *************;
            * Diagnosis *;
            *************;
   
               if n(&drg.3da,&drg.3db)=0 then &drg.3dp=.;
               else &drg.3rdp=(4*(&drg.3da=5 and &drg.3db=5))+1;

         *********;
         * Abuse *;
         *********;

            ***************;
            * Criterion A *;
            ***************;

               **********************************************************;
               * use despite use-derived social/pysch/physical problems *;
               **********************************************************;

                  if sum(dr25e_&i. not in (1,5),
                         dr26b_&i. not in (1,5),
                         dr28e_&i. not in (1,5))=3 then &drg.3aa1=.;
                  else &drg.3aa1=(4*(sum(dr25e_&i.=5,dr26b_&i.=5,
                                         dr28e_&i.=5)>0))+1;

               ****************************************************;
               * recurrent use in physically dangerous situations *;
               ****************************************************;

                  if sum(dr18d_&i. not in (1,5),
                         dr18f_&i. not in (1,5),
                         dr27b_&i. not in (1,5))=3 then &drg.3aa2=.;
                  else &drg.3aa2=(4*(sum(dr18d_&i.=5,dr18f_&i.=5,
                                         dr27b_&i.=5)>0))+1;

               *********************;
               * at least one item *;
               *********************;

                  if n(&drg.3aa1,&drg.3aa2)=0 then &drg.3aa=.;
                  else &drg.3aa=(4*(&drg.3aa1=5 or &drg.3aa2=5))+1;

            ***************;
            * Criterion B *;
            ***************;
   
               if sum(dr18d_&i. not in (1,5),
                      dr18f_&i. not in (1,5),
                      dr25e_&i. not in (1,5),
                      dr26b_&i. not in (1,5),
                      dr27b_&i. not in (1,5),
                      dr28e_&i. not in (1,5))=6 then &drg.3ab=.;
               else &drg.3ab=(4*(sum(dr18d_&i.=5,dr18f_&i.=5,dr25e_&i.=5,
                                     dr26b_&i.=5,dr27b_&i.=5,dr28e_&i.=5)>1))+1;

            ***************;
            * Criterion C *;
            ***************;

               if &drg.3rdp=. then &drg.3ac=.;
               else &drg.3ac=&drg.3rdp;

            *************;
            * Diagnosis *;
            *************;

               if n(&drg.3aa,&drg.3ab,&drg.3ac)=0 then &drg.3rab=.;
               %if &who.=p %then %do;
                  else &drg.3rab=(4*(&drg.3aa=5 and &drg.3ab=5))+1;
               %end;
               %else %do;
                  else &drg.3rab=(4*(&drg.3aa=5 and &drg.3ab=5 and 
                                     &drg.3ac=1))+1;
               %end;

         ****************************;
         * jump to here if skip out *;
         ****************************;

            skipout&i.:

         *******************;
         * variable labels *;
         *******************;

            label &drg.3rsk = "DSM3R &wrd. Skip Out"
                  &drg.3da1 = "DSM3R &wrd. Dependence Crit. A1"
                  &drg.3da2 = "DSM3R &wrd. Dependence Crit. A2"
                  &drg.3da3 = "DSM3R &wrd. Dependence Crit. A3"
                  &drg.3da4 = "DSM3R &wrd. Dependence Crit. A4" 
                  &drg.3da5 = "DSM3R &wrd. Dependence Crit. A5" 
                  &drg.3da6 = "DSM3R &wrd. Dependence Crit. A6" 
                  &drg.3da7 = "DSM3R &wrd. Dependence Crit. A7" 
                  &drg.3da8 = "DSM3R &wrd. Dependence Crit. A8" 
                  &drg.3da9 = "DSM3R &wrd. Dependence Crit. A9" 
                  &drg.3dax = "DSM3R &wrd. Dependence Sum of As" 
                  &drg.3da  = "DSM3R &wrd. Dependence 3+ As" 
                  &drg.3db  = "DSM3R &wrd. Dependence Crit. B" 
                  &drg.3rdp = "DSM3R &wrd. Dependence Diagnosis"
                  &drg.3aa1 = "DSM3R &wrd. Abuse Crit. A1"
                  &drg.3aa2 = "DSM3R &wrd. Abuse Crit. A2"
                  &drg.3aa  = "DSM3R &wrd. Abuse A1 or A2"
                  &drg.3ab  = "DSM3R &wrd. Abuse Crit. B"
                  &drg.3ac  = "DSM3R &wrd. Abuse Crit. C"
                  &drg.3rab = "DSM3R &wrd. Abuse Diagnosis";

      %end;

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  &drg.3rsk
                  &drg.3da1-&drg.3da9 &drg.3dax &drg.3da &drg.3db &drg.3rdp
                  &drg.3aa1 &drg.3aa2 &drg.3aa &drg.3ab &drg.3ac &drg.3rab 

               %end;
               ;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_drg_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  ************;
                  * skip out *;
                  ************;

                     %if &study.=irpg or &who.=c %then %do;
                        %multi(drg3r,dr1a_&i. dr1e1_&i. dr5a_&i. dr5a1_&i.
                                     &drg.3rsk);
                     %end;
                     %if &study.=coga and &who.=p %then %do;
                        %multi(drg3r,dr1e1_&i. dr5a_&i. dr5a1_&i. &drg.3rsk);
                     %end;

                  **************;
                  * Dependence *;
                  **************;

                     ***************;
                     * Criterion A *;
                     ***************;

                        %multi(drg3r,&drg.3rsk dr13b_&i. dr13d_&i. &drg.3da1);
                        %multi(drg3r,&drg.3rsk dr12a_&i. dr12b_&i. &drg.3da2);
                        %multi(drg3r,&drg.3rsk dr14_&i. &drg.3da3);
                        %multi(drg3r,&drg.3rsk dr18d_&i. dr18f_&i. dr24f_&i. 
                                     dr27b_&i. &drg.3da4);
                        %multi(drg3r,&drg.3rsk dr15c_&i. &drg.3da5);
                        %multi(drg3r,&drg.3rsk dr25e_&i. dr26b_&i. dr28e_&i. 
                                     &drg.3da6);
                        %multi(drg3r,&drg.3rsk dr11a_&i. dr11b_&i. &drg.3da7);
                        %multi(drg3r,&drg.3rsk dr33b1_&i. &drg.3da8);
                        %multi(drg3r,&drg.3rsk dr33d2_&i. &drg.3da9);
                        %multi(drg3r,&drg.3rsk &drg.3da1-&drg.3da9 &drg.3dax);
                        %multi(drg3r,&drg.3rsk &drg.3dax &drg.3da);

                     ***************;
                     * Criterion B *;
                     ***************;

                        %multi(drg3r,&drg.3rsk dr34b_&i. &drg.3da1 dr12a_&i. 
                                     dr12c_&i. &drg.3da3-&drg.3da7 dr33b1_&i.
                                     dr33c_&i. &drg.3da9 &drg.3db);

                     *************;
                     * Diagnosis *;
                     *************;

                        %multi(drg3r,&drg.3rsk &drg.3da &drg.3db &drg.3rdp);

                  *********;
                  * Abuse *;
                  *********;

                     ***************;
                     * Criterion A *;
                     ***************;

                        %multi(drg3r,&drg.3rsk dr25e_&i. dr26b_&i. dr28e_&i. 
                                     &drg.3aa1);
                        %multi(drg3r,&drg.3rsk dr18d_&i. dr18f_&i. dr27b_&i. 
                                     &drg.3aa2);
                        %multi(drg3r,&drg.3rsk &drg.3aa1 &drg.3aa2 &drg.3aa);

                     ***************;
                     * Criterion B *;
                     ***************;

                        %multi(drg3r,&drg.3rsk dr18d_&i. dr18f_&i. dr25e_&i. 
                                     dr26b_&i. dr27b_&i. &drg.3ab);

                     ***************;
                     * Criterion C *;
                     ***************;

                        %multi(drg3r,&drg.3rsk &drg.3rdp &drg.3ac);

                     *************;
                     * Diagnosis *;
                     *************;

                        %multi(drg3r,&drg.3rsk &drg.3aa &drg.3ab &drg.3ac 
                                     &drg.3rab);

               %end;

            ******************;
            * keep variables *;
            ******************;

               data drg3r;
               set drg3r;
               keep ind_id  
               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  &drg.3rsk
                  &drg.3da1-&drg.3da9 &drg.3dax &drg.3da &drg.3db &drg.3rdp
                  &drg.3aa1 &drg.3aa2 &drg.3aa &drg.3ab &drg.3ac &drg.3rab 

               %end;
               ;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=drg3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend drg3r;

********;
* test *;
********;
/*
   %drg3r(coga.csaga2,1);

   proc freq data=drg3r;
   tables co3rdp st3rdp op3rdp ha3rdp se3rdp ot3rdp / list missing;
   tables co3rab st3rab op3rab ha3rab se3rab ot3rab / list missing;
   run;
*/
