************************************************************;
* CSSAGA-II Diagnostic Algorithm -- DSM-III-R -- Dysthymia *;
************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*****************************************;
* uncomment the following four lines to *;
* run this module in standalone mode    *;
*****************************************;
/*
   %include "./dsm3r_dep.sas" / nosource2;
   %include "./dsm3r_man.sas" / nosource2;

   %dep3r(coga.csaga2,0);
   %man3r(coga.csaga2,0);
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %dys3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro dys3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DYS3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DYS3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      proc contents data=&dsn. noprint out=test;
      run;

      data test;
      set test;
      if upcase(substr(name,1,2))="DY";
      run;

      data test;
      set test nobs=count;
      call symput("var"||left(_n_),name);
      call symput("nvar",left(put(count,8.)));
      run;

      data dys3r;
      merge &dsn.(keep=ind_id dm3a dm3b_1-dm3b_3 
                       dp2b_2 dp9_2 dp16_2
                       dp30c1_1 dp30c1_2 dp30c2_1 dp30c2_2 dp30d_1 dp30d_2
                       %do i=1 %to &nvar.;
                          &&var&i.
                       %end;
                       ) 
            dep3r(keep=ind_id dc3rsk dp3rsk dc3rdx dp3rdx)
            man3r(keep=ind_id mn3rdx);
      by ind_id;

      attrib dy3rsk1 dy3rsk2 dy3rsk
             dy3ra dy3rb1-dy3rb6 dy3rbx dy3rb
             dy3rc dy3rd dy3re dy3rf dy3rg dy3rdx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas dy3rsk1 dy3rsk2 dy3rsk
                      dy3ra dy3rb1-dy3rb6 dy3rbx dy3rb
                      dy3rc dy3rd dy3re dy3rf dy3rg dy3rdx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(dc3rsk not in (1,5),
                dp3rsk not in (1,5),
                sum(dp2b_2 not in (1,2,3,4),
                    dp9_2 not in (1,2,3,4),
                    dp16_2 not in (1,2,3,4))=3)=3 then dy3rsk1=5;
         else do;
            if sum((dc3rsk=1 or dp3rsk=1),
                   sum(dp2b_2=4,dp9_2=4,dp16_2=4)=0)=2 then dy3rsk1=1;
            else dy3rsk1=5;
         end;

         if dy3rsk1=5 then do;
            array skp1 dy3ra dy3rb1-dy3rb6 dy3rbx dy3rb
                       dy3rc dy3rd dy3re dy3rf dy3rg dy3rdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if dy3rsk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum((dy1 not in (1,5)),
                   (dy3 not in (1,5)))=2 then dy3ra=.;
            else dy3ra=sum(dy1=5,dy3=5)>0;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(dy1 not in (1,5),
                dy2 not in (1,5),
                dy3 not in (1,5))=3 then dy3rsk2=5;
         else do;
            if sum(dy1=5,dy2=5,dy3=5)>0 then dy3rsk2=1;
            else dy3rsk2=5;
         end;

         if dy3rsk2=5 then do;
            array skp2 dy3rb1-dy3rb6 dy3rbx dy3rb
                       dy3rc dy3rd dy3re dy3rf dy3rg dy3rdx;
            do over skp2;
              skp2=.S;
            end;
         end;

         if dy3rsk2=5 then go to skipout;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if sum((dy5a not in (1,5)),(dy5b not in (1,5)))=2 then dy3rb1=.;
               else dy3rb1=sum(dy5a=5,dy5b=5)>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dy4a not in (1,5)),(dy4b not in (1,5)),
                      (dy4c not in (1,5)),(dy4d not in (1,5)))=4 then dy3rb2=.;
               else dy3rb2=sum(dy4a=5,dy4b=5,dy4c=5,dy4d=5)>0;

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if (dy6 not in (1,5)) then dy3rb3=.;
               else dy3rb3=(dy6=5);

            *******************;
            * low self-esteem *;
            *******************;

               if (dy7 not in (1,5)) then dy3rb4=.;
               else dy3rb4=(dy7=5);

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dy9a not in (1,5)),(dy9b not in (1,5)))=2 then dy3rb5=.;
               else dy3rb5=sum(dy9a=5,dy9b=5)>0;

            ****************************;
            * feelings of hopelessness *;
            ****************************;

               if (dy8 not in (1,5)) then dy3rb6=.;
               else dy3rb6=(dy8=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of dy3rb1-dy3rb6)=0 then dy3rbx=.;
               else dy3rbx=sum(of dy3rb1-dy3rb6);

            ************************;
            * at least two present *;
            ************************;

               if dy3rbx=. then dy3rb=.;
               else dy3rb=(dy3rbx>1);

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(dy12a not in (1,5),dy12b not in (1,2,3,4))=2 then dy3rc=.;
            else dy3rc= sum((dy12a=1),(dy12a=5 and dy12b in (1,2,3)))>0;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum((n(dc3rdx,dp3rdx)=0),
                   (dm3a<0 and min(of dm3b_1-dm3b_3)<0),
                   (sum(dp30c1_1<0,dp30c1_2<0,dp30c2_1<0,dp30c2_2<0)=4 and
                    sum(dp30d_1<0,dp30d_2<0)=2),
                   (n(dy11a_1,dy11a_2)=0))=4 then dy3rd=.;
            else do;
               if max(dc3rdx,dp3rdx)=1 then dy3rd=1;
               else do;
                  if n(dy11a_1,dy11a_2)=0 then dy3rd=.;
                  else do;
                     if sum(dp30c1_1<0,dp30c2_1<0,
                            dp30c1_2<0,dp30c2_2<0)>0 then dy3rd=.;
                     else do;
                        if min(of dm3b_1-dm3b_3)<0 then dy3rd=.;
                        else do;
                           if (dp30c2_2<1 and dp30c2_1>0) then 
                              dy3rd=(int((mdy(dp30c1_1,1,dp30c2_1)-
                                           mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                      dy11a_1)>1 or
                                     (dy11a_1>int(((mdy(dp30c1_1,1,dp30c2_1)-
                                                    mdy(dm3b_1,dm3b_2,dm3b_3))+
                                                   (dp30d_1*7))/365.25));       
                           else if (dp30c2_2>0 and dp30c2_1<1) then
                              dy3rd=(int((mdy(dp30c1_2,1,dp30c2_2)-
                                           mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                      dy11a_1)>1 or
                                     (dy11a_1>int(((mdy(dp30c1_2,1,dp30c2_2)-
                                                    mdy(dm3b_1,dm3b_2,dm3b_3))+
                                                   (dp30d_1*7))/365.25));
                           else do;
                              dy3rd=(int((mdy(dp30c1_2,1,dp30c2_2)-
                                           mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                      dy11a_1)>1 or
                                     ((dy11a_1>int(((mdy(dp30c1_2,1,dp30c2_2)-
                                                     mdy(dm3b_1,dm3b_2,dm3b_3))+
                                                    (dp30d_1*7))/365.25)) and
                                      (int((mdy(dp30c1_1,1,dp30c2_1)-
                                            mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                       dy11a_1)>1);
                           end;
                        end;
                     end;
                  end;
               end;
            end;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            dy3re=(mn3rdx<3);

      ***************;
      * Criterion F *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            dy3rf=.;

      ***************;
      * Criterion G *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum((dy10a not in (1,5)),
                   (dy10b not in (1,5)))=2 then dy3rg=.;
            else dy3rg=max(dy10a,dy10b)=1;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

           if n(dy3ra,dy3rb,dy3rc,dy3rd,dy3re,dy3rf,dy3rg)=0 then
              dy3rdx=.;
           else do;
              dy3rdx=1;
              if sum(dy3ra,dy3rb,dy3rc,dy3rd,dy3re)=5 then do;
                 if dy3rg=0 then dy3rdx=3;
                 else if dy3rg=1 then dy3rdx=5;
              end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         dy3rsk=max(dy3rsk1,dy3rsk2);

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         if dy3rsk1=5 then do;
            array one dy3ra dy3rb1-dy3rb6 dy3rb
                      dy3rc dy3rd dy3re dy3rf dy3rg;
            do over one;
               if one=1 then one=5;
               if one=0 then one=1;
            end;
         end;
         else do;
            if dy3ra=1 then dy3ra=5;
            if dy3ra=0 then dy3ra=1;
         end;

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label dy3rsk1 = "DSM3R Dysthymia Skip Out #1"
               dy3rsk2 = "DSM3R Dsythymia Skip Out #2"
               dy3rsk = "DSM3R Dsythymia Skip Out"
               dy3ra  = "DSM3R Dysthymia Crit. A"
               dy3rb1 = "DSM3R Dysthymia Crit. B1"
               dy3rb2 = "DSM3R Dysthymia Crit. B2"
               dy3rb3 = "DSM3R Dysthymia Crit. B3"
               dy3rb4 = "DSM3R Dysthymia Crit. B4"
               dy3rb5 = "DSM3R Dysthymia Crit. B5"
               dy3rb6 = "DSM3R Dysthymia Crit. B6"
               dy3rbx = "DSM3R Dysthymia Sum + Bs"
               dy3rb  = "DSM3R Dysthymia Crit. B"
               dy3rc  = "DSM3R Dysthymia Crit. C"
               dy3rd  = "DSM3R Dysthymia Crit. D"
               dy3re  = "DSM3R Dysthymia Crit. E"
               dy3rf  = "DSM3R Dysthymia Crit. F"
               dy3rg  = "DSM3R Dysthymia Crit. G"
               dy3rdx = "DSM3R Dysthymia Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id 
                    dy3rsk1 dy3rsk2 dy3rsk 
                    dy3ra dy3rb1-dy3rb6 dy3rbx dy3rb
                    dy3rc dy3rd dy3re dy3rf dy3rg dy3rdx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_dys_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(dys3r,dc3rsk dp3rsk dp2b_2 dp9_2 dp16_2 dy3rsk1);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy1 dy3 dy3ra);

               ************;
               * skip out *;
               ************;

                  %multi(dys3r,dy3rsk1 dy1 dy2 dy3 dy3rsk2);
                  %multi(dys3r,dy3rsk1 dy3rsk2 dy3rsk);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy3rsk2 dy5a dy5b dy3rb1);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy4a dy4b dy4c dy4d dy3rb2);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy6 dy3rb3);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy7 dy3rb4);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy9a dy9b dy3rb5);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy8 dy3rb6);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy3rb1-dy3rb6 dy3rbx);
                     %multi(dys3r,dy3rsk1 dy3rsk2 dy3rbx dy3rb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy3rsk2 dy12a dy12b dy3rc);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy3rsk2 dc3rdx dp3rdx dy11a_1 dy11a_2
                                  dp30c1_1 dp30c2_1 dp30c1_2 dp30c2_2 
                                  dm3a dm3b_1 dm3b_2 dm3b_3
                                  dy3rd);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy3rsk2 mn3rdx dy3re);

               ***************;
               * Criterion G *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy3rsk2 dy10a dy10b dy3rg);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dys3r,dy3rsk1 dy3rsk2 dy3ra dy3rb dy3rc dy3rd 
                                  dy3re dy3rg dy3rdx);

            ******************;
            * keep variables *;
            ******************;

               data dys3r;
               set dys3r;
               keep ind_id  
                    dy3rsk1 dy3rsk2 dy3rsk
                    dy3ra dy3rb1-dy3rb6 dy3rbx dy3rb
                    dy3rc dy3rd dy3re dy3rf dy3rg dy3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=dys3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend dys3r;

********;
* test *;
********;
/*
   %dys3r(coga.csaga2,1);
*/
