********************************************************;
* CSSAGA-II Diagnostic Algorithm -- DSM-III-R -- Mania *;
********************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %man3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro man3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in MAN3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in MAN3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data man3r;
      set &dsn.;

      attrib mn3rsk1 mn3rsk2 mn3rsk
             mn3ra 
             mn3rb1-mn3rb7 mn3rbx mn3rb
             mn3rc mn3rf mn3rdx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas mn3rsk1 mn3rsk2 mn3rsk mn3ra 
                      mn3rb1-mn3rb7 mn3rbx mn3rb
                      mn3rc mn3rf mn3rdx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(mn1 not in (1,5),
                mn3a not in (1,5))=2 then mn3rsk1=5;
         else do;
            if sum(mn1=5,mn3a=5)=2 then mn3rsk1=1;
            else mn3rsk1=5;
         end;

         if mn3rsk1=5 then do;
            array skp1 mn3ra mn3rb1-mn3rb7 mn3rbx mn3rb
                       mn3rc mn3re mn3rf mn3rdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if mn3rsk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(mn2a not in (1,5),
                   mn3a not in (1,5),
                   mn18a not in (1,2,3,4,5,6,7))=3 then mn3ra=.;
            else mn3ra=((mn2a=5 or mn3a=5) and mn18a in (2,3,4,5,6,7));

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***************************************;
            * inflated self-esteem or grandiosity *;
            ***************************************;

               if sum(mn9a not in (1,5),
                      mn9b not in (1,5),
                      mn9c not in (1,5))=3 then mn3rb1=.;
               else mn3rb1=max(mn9a,mn9b,mn9c)=5;

            ****************************;
            * decreased need for sleep *;
            ****************************;

               if sum(mn8a not in (1,5),
                      mn8b not in (1,5))=2 then mn3rb2=.;
               else mn3rb2=(mn8a=5 and mn8b=5);

            ********************************;
            * talkative/pressure of speech *;
            ********************************;

               if (mn12 not in (1,5)) then mn3rb3=.;
               else mn3rb3=(mn12=5);

            ***********************************;
            * flight of ideas/racing thoughts *;
            ***********************************;

               if (mn13 not in (1,5)) then mn3rb4=.;
               else mn3rb4=(mn13=5);

            *******************;
            * distractibility *;
            *******************;

               if (mn14 not in (1,5)) then mn3rb5=.;
               else mn3rb5=(mn14=5);

            **************************************;
            * increase in goal-directed activity *;
            **************************************;

               if sum(mn10a not in (1,5),
                      mn10b not in (1,5),
                      mn11 not in (1,5))=3 then mn3rb6=.;
               else mn3rb6=max(mn10a,mn10b,mn11)=5;

            **********************************************;
            * excessive involvement in hi-risk activities*;
            **********************************************;

               if sum(mn15a not in (1,5),
                      mn15b not in (1,5),
                      mn15c not in (1,5))=3 then mn3rb7=.;
               else mn3rb7=max(mn15a,mn15b,mn15c)=5;

            ****************;
            * sum of items *;
            ****************;

               if n(of mn3rb1-mn3rb7)=0 then mn3rbx=.;
               else mn3rbx=sum(of mn3rb1-mn3rb7);
 
            **************************;
            * at least three present *;
            **************************;

               if mn3rbx=. then mn3rb=.;
               else mn3rb=(mn3rbx>2);

      *********************;
      * test for skip out *;
      *********************;

         if max(mn3rbx,mn16a=5,mn16b=5)=0 then do;
            mn3rsk2=5;
            array skp2 mn3rc mn3rf mn3rdx;
            do over skp2;
               skp2=.S;
            end;
         end;
         else mn3rsk2=1;

         if mn3rsk2=5 then go to skipout;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(mn17f not in (1,5),
                   mn20b_1 not in (1,2,3),mn20b_2 not in (1,2,3),
                   mn20b_3 not in (1,2,3),mn20b_4 not in (1,2,3),
                   mn20b_5 not in (1,2,3),mn20b_6 not in (1,2,3),
                   mn20b_7 not in (1,2,3),mn20b_8 not in (1,2,3))=9 then
               mn3rc=.;
            else do;
               sum2s=0;
               sum3s=0;
               array one mn20b_1-mn20b_8;
               do over one;
                  if one=2 then sum2s+1;
                  else if one=3 then sum3s+1;
               end;
               mn3rc=(mn17f=5 or sum2s>3 or sum3s>1 or 
                     (sum3s>0 and sum2s>1));
            end;

      *******************************;
      * Criterion D -- Not Assessed *;
      *******************************;

      **************************************;
      * Criterion E -- Skip in this module *;
      **************************************;

      ***************;
      * Criterion F *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(mn5c<0,mn5e not in (1,2,3,4,5,6,7),
                   mn6a not in (1,5),mn6b not in (1,5),mn6e not in (1,5),
                   mn7a not in (1,5),mn7c not in (1,5),mn7e not in (1,5),
                   mn18a not in (1,2,3,4,5,6,7))=9 then mn3rf=.;
            else do;
               mn3rf=((mn5c<3 and mn5e<2) and (mn6a=1 or mn6b=1) and mn6e=1) or
                     ((mn7a=5 or mn7c=5 or mn7e=5) and mn18a in (2,3,4,5,6,7));
            end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(mn3ra,mn3rb,mn3rc,mn3rf)=0 then mn3rdx=.;
            else do;
               if (mn7a=1 and mn7c=1 and mn7e=5 and 
                   mn18a in (2,3,4,5,6,7)) then do;
                  mn3rdx=(5*(mn3ra and mn3rbx>3 and mn3rc and mn3rf))+
                         (3*(mn3ra and mn3rbx>3 and mn3rc and mn3rf=0));
                  if mn3rdx=0 then mn3rdx=1;
               end;
               else do;
                  mn3rdx=(5*(mn3ra and mn3rb and mn3rc and mn3rf))+
                         (3*(mn3ra and mn3rb and mn3rc and mn3rf=0));
                  if mn3rdx=0 then mn3rdx=1;
               end;
            end;

      ***************************;
      * rescale from 0,1 to 1,5 *;
      ***************************;

         array two mn3ra mn3rb1-mn3rb7 mn3rb mn3rc mn3rf;
         do over two;
            if two=1 then two=5;
            if two=0 then two=1;
         end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         mn3rsk=max(mn3rsk1,mn3rsk2);

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label mn3rsk1 = "DSM3R Mania Skip Out #1"
               mn3rsk2 = "DSM3R Mania Skip Out #2"
               mn3rsk  = "DSM3R Mania Skip Out"
               mn3ra  = "DSM3R Mania (Lifetime) Crit. A"
               mn3rb1 = "DSM3R Mania (Lifetime) Crit. B1"
               mn3rb2 = "DSM3R Mania (Lifetime) Crit. B2"
               mn3rb3 = "DSM3R Mania (Lifetime) Crit. B3"
               mn3rb4 = "DSM3R Mania (Lifetime) Crit. B4"
               mn3rb5 = "DSM3R Mania (Lifetime) Crit. B5"
               mn3rb6 = "DSM3R Mania (Lifetime) Crit. B6"
               mn3rb7 = "DSM3R Mania (Lifetime) Crit. B7"
               mn3rbx = "DSM3R Mania (Lifetime) Sum of Bs"
               mn3rb  = "DSM3R Mania (Lifetime) Crit. B"
               mn3rc  = "DSM3R Mania (Lifetime) Crit. C"
               mn3re  = "DSM3R Mania (Lifetime) Crit. E"
               mn3rf  = "DSM3R Mania (Lifetime) Crit. F"
               mn3rdx = "DSM3R Mania (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    mn3rsk1 mn3rsk2 mn3rsk
                    mn3ra 
                    mn3rb1-mn3rb7 mn3rbx mn3rb
                    mn3rc mn3re mn3rf mn3rdx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_man_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(man3r,mn1 mn3a mn3rsk1);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(man3r,mn3rsk1 mn2a mn3a mn18a mn3ra);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(man3r,mn3rsk1 mn9a mn9b mn9c mn3rb1);
                     %multi(man3r,mn3rsk1 mn8a mn8b mn3rb2);
                     %multi(man3r,mn3rsk1 mn12 mn3rb3);
                     %multi(man3r,mn3rsk1 mn13 mn3rb4);
                     %multi(man3r,mn3rsk1 mn14 mn3rb5);
                     %multi(man3r,mn3rsk1 mn10a mn10b mn11 mn3rb6);
                     %multi(man3r,mn3rsk1 mn15a mn15b mn15c mn3rb7);
                     %multi(man3r,mn3rsk1 mn3rb1-mn3rb7 mn3rbx);
                     %multi(man3r,mn3rsk1 mn3rbx mn3rb);

               ************;
               * skip out *;
               ************;

                  %multi(man3r,mn3rsk1 mn3rbx mn16a mn16b mn3rsk2);
                  %multi(man3r,mn3rsk1 mn3rsk2 mn3rsk);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(man3r,mn3rsk1 mn3rsk2 mn17f mn20b_1-mn20b_8 mn3rc);

               ***************;
               * Criterion F *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(man3r,mn3rsk1 mn3rsk2 mn5c mn5e mn6a mn6b mn6e 
                                  mn7a mn7c mn7e mn18a mn3rf);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(man3r,mn3rsk1 mn3rsk2 
                                  mn7a mn7c mn7e mn18a mn3rbx 
                                  mn3ra mn3rb mn3rc mn3rf mn3rdx);

            ******************;
            * keep variables *;
            ******************;

               data man3r;
               set man3r;
               keep ind_id
                    mn3rsk1 mn3rsk2 mn3rsk  
                    mn3ra 
                    mn3rb1-mn3rb7 mn3rbx mn3rb
                    mn3rc mn3rf mn3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=man3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend man3r;

********;
* test *;
********;
/*
   %man3r(coga.csaga2,1);
*/
