******************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Obsessive/Compulsive Disorder *;
******************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %ocd3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro ocd3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in OCD3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in OCD3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data ocd3r;
      set &dsn.;

      attrib ob3rsk cp3rsk oc3rsk
             ob3ra1 ob3ra2 ob3ra3 ob3ra3x ob3ra5x ob3rax ob3ra ob3rb ob3rdx
             cp3ra1 cp3ra2 cp3ra cp3rb cp3rdx oc3rdx length=3;

      **************;
      * Obsessions *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            *************************************************;
            * recurrent, persistent ideas/thoughts/impulses *;
            *************************************************;

               if sum(ob1 not in (1,3,5),
                      ob2b not in (1,3,5),
                      ob3c not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob5c not in (1,3,5),
                      ob6b not in (1,3,5))=6 then ob3ra1=.;
               else do;
                  ob3ra1=1;
                  if sum(ob1=3,ob2b=3,ob3c=3,ob4b=3,ob5c=3,ob6b=3)>0 then 
                     ob3ra1=3;
                  else if sum(ob1=5,ob2b=5,ob3c=5,ob4b=5,ob5c=5,ob6b=5)>0 then
                     ob3ra1=5;
               end;

            *************************************************;
            * attempts to ignore/suppress thoughts/impulses *;
            *************************************************;

               if sum(ob2b not in (1,3,5),
                      ob3b not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob7 not in (1,3,5))=4 then ob3ra2=.;
               else do;
                  ob3ra2=1;
                  if sum(ob2b=3,ob3b=3,ob4b=3,ob7=3)>0 then 
                     ob3ra2=3;
                  else if sum(ob2b=5,ob3b=5,ob4b=5,ob7=5)>0 then
                     ob3ra2=5;
               end;

            *********************;
            * test for skip out *;
            *********************;

               if sum(ob1 not in (1,3,5),
                      ob2b not in (1,3,5),
                      ob3c not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob5c not in (1,3,5),
                      ob6b not in (1,3,5))=6 then ob3rsk=5;
               else do;
                  if sum(ob1 in (3,5),
                         ob2b in (3,5),
                         ob3c in (3,5),
                         ob4b in (3,5),
                         ob5c in (3,5),
                         ob6b in (3,5))>0 then ob3rsk=1;
                  else ob3rsk=5;
               end;

               if ob3rsk=5 then do;
                  array skp1 ob3ra3 ob3ra3x ob3ra5x ob3rax ob3ra ob3rb ob3rdx;
                  do over skp1;
                     skp1=.S;
                  end;
               end;

               if ob3rsk=5 then go to compuls;

            ***************************************************;
            * obsession recognized as from within not without *;
            ***************************************************;

               if ob8a not in (1,5) then ob3ra3=.;
               else ob3ra3=(4*(ob8a=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of ob3ra1-ob3ra3)=0 then ob3rax=.;
               else do;
                  ob3ra3x=0;
                  ob3ra5x=0;
                  array sxs ob3ra1-ob3ra3;
                  do over sxs;
                     if sxs=3 then ob3ra3x=ob3ra3x+1;
                     if sxs=5 then ob3ra5x=ob3ra5x+1;
                  end;
                  ob3rax=sum(ob3ra3x,ob3ra5x);
                  ob3ra=(4*(ob3ra3x=0 and ob3ra5x=3))+
                        (2*(ob3ra3x>0 and ob3rax=3))+1;
               end;

         ***************;
         * Criterion B *;
         ***************;

            ************************************;
            * obsessions cause marked distress *;
            ************************************;

               if sum(ob9b not in (1,3,5),
                      sum(ob14a_1a not in (1,3,5),
                          ob14a_2a not in (1,3,5),
                          ob14a_3a not in (1,3,5),
                          ob14a_4a not in (1,3,5),
                          ob14a_5a not in (1,3,5),
                          ob14a_6a not in (1,3,5))=6)=2 then ob3rb=.;
               else do;
                  ob3rb=1;
                  sum3as=0;
                  sum5as=0;
                  sum243=0;
                  sum343=0;
                  sum245=0;
                  sum345=0;
                  array oxa ob14a_1a ob14a_2a ob14a_3a 
                            ob14a_4a ob14a_5a ob14a_6a;
                  array oxb ob14a_1b ob14a_2b ob14a_3b 
                            ob14a_4b ob14a_5b ob14a_6b;
                  do over oxa;
                     if oxa=3 then do;
                        sum3as=sum3as+1;
                        if oxb=2 then sum243=sum243+1;
                        if oxb=3 then sum343=sum343+1;
                     end;
                     if oxa=5 then do;
                        sum5as=sum5as+1;
                        if oxb=2 then sum245=sum245+1;
                        if oxb=3 then sum345=sum345+1;
                     end;
                  end;
                  if sum3as>0 then do;
                     if ((sum3as>0 and sum343>0) or
                         (sum3as>1 and sum243>1)) then ob3rb=3;
                  end;
                  else if sum5as>0 then do;
                     if sum(((sum5as>0 and sum345>0) or
                             (sum5as>1 and sum245>1)),
                            ob9b=5)=2 then ob3rb=5;
                  end;
               end;

         *************;
         * Diagnosis *;
         *************;

            if n(ob3ra1,ob3ra2,ob3ra3,ob3rb)=0 then ob3rdx=.;
            else do;
               ob3rdx=1;
               if (ob3ra3=5 and 
                   sum(ob3ra1 in (3,5),ob3ra2 in (3,5),ob3rb in (3,5))=3 and
                   sum(ob3ra1=3,ob3ra2=3,ob3rb=3)>0) then ob3rdx=3;
               else if (sum(ob3ra1=3,ob3ra2=3,ob3rb=3)=0 and
                        sum(ob3ra1=5,ob3ra2=5,ob3ra3=5,ob3rb=5)=4) then 
                  ob3rdx=5;
            end;

      *************************;
      * jump here if skip out *;
      *************************;

         compuls:

      ***************;
      * Compulsions *;
      ***************;

         ***************;
         * Criterion A *;
         ***************;

            **************************************************************;
            * repititive, intentional behaviors in response to obsession *;
            **************************************************************;

               if sum(cp1a_1 not in (1,5),
                      cp1a_2 not in (1,5),
                      cp1a_3 not in (1,5),
                      cp1a_4 not in (1,5),
                      cp1a_5 not in (1,5),
                      cp2a_1 not in (1,5),
                      cp2a_2 not in (1,5),
                      cp2a_3 not in (1,5),
                      cp3a not in (1,5))=9 then cp3ra1=.;
               else cp3ra1=(4*(sum(cp1a_1=5,cp1a_2=5,cp1a_3=5,
                                   cp1a_4=5,cp1a_5=5,cp2a_1=5,
                                   cp2a_2=5,cp2a_3=5,cp3a=5)>0))+1;

            *********************;
            * test for skip out *;
            *********************;

               if sum(cp1a_1 not in (1,5),
                      cp1a_2 not in (1,5),
                      cp1a_3 not in (1,5),
                      cp1a_4 not in (1,5),
                      cp1a_5 not in (1,5),
                      cp1b not in (1,3,5),
                      cp2a_1 not in (1,5),
                      cp2a_2 not in (1,5),
                      cp2a_3 not in (1,5),
                      cp2b not in (1,3,5),
                      cp3a not in (1,5),
                      cp3b not in (1,3,5))=12 then cp3rsk=5;
               else do;
                  if sum(cp1a_1=5,
                         cp1a_2=5,
                         cp1a_3=5,
                         cp1a_4=5,
                         cp1a_5=5,
                         cp1b in (3,5),
                         cp2a_1=5,
                         cp2a_2=5,
                         cp2a_3=5,
                         cp2b in (3,5),
                         cp3a=5,
                         cp3b in (3,5))>0 then cp3rsk=1;
                  else cp3rsk=5;
               end;

               if cp3rsk=5 then do;
                  array skp2 cp3ra2 cp3ra cp3rb cp3rdx;
                  do over skp2;
                     skp2=.S;
                  end;
               end;

               if cp3rsk=5 then go to skipout;

            ***********************************************************;
            * behavior intended to prevent discomfort/event/situation *;
            ***********************************************************;

               if sum(sum(cp4a not in (1,3,5),
                          cp4b not in (1,3,5))=2,
                      cp4c not in (1,3,5))=2 then cp3ra2=.;
               else do;
                  cp3ra2=1;
                  if sum(sum(cp4a in (3,5),cp4b in (3,5))=2,
                         sum(cp4a=3,cp4b=3)>0,
                         cp4c in (3,5))=3 then cp3ra2=3;
                  else if sum(sum(cp4a=3,cp4b=3)=0,
                              sum(cp4a=5,cp4b=5)>0,
                              cp4c=5)=3 then cp3ra2=5;
               end;

            **************;
            * both items *;
            **************;

               if n(cp3ra1,cp3ra2)=0 then cp3ra=.;
               else do;
                  if sum(cp3ra1=5,cp3ra2 in (3,5))=2 then cp3ra=cp3ra2;
                  else cp3ra=1;
               end;

         ***************;
         * Criterion B *;
         ***************;

            *************************************;
            * compulsions cause marked distress *;
            *************************************;

               if sum(cp1b not in (1,3,5),
                      cp2b not in (1,3,5),
                      cp3b not in (1,3,5),
                      sum(cp7b not in (1,5),
                          cp9b not in (1,5))=2,
                      sum(cp11a_1a not in (1,3,5),
                          cp11a_2a not in (1,3,5),
                          cp11a_3a not in (1,3,5),
                          cp11a_4a not in (1,3,5),
                          cp11a_5a not in (1,3,5),
                          cp11a_6a not in (1,3,5))=6)=5 then cp3rb=.;
               else do;
                  cp3rb=1;
                  sum3as=0;
                  sum5as=0;
                  sum243=0;
                  sum343=0;
                  sum245=0;
                  sum345=0;
                  array cxa cp11a_1a cp11a_2a cp11a_3a 
                            cp11a_4a cp11a_5a cp11a_6a;
                  array cxb cp11a_1b cp11a_2b cp11a_3b 
                            cp11a_4b cp11a_5b cp11a_6b;
                  do over cxa;
                     if cxa=3 then do;
                        sum3as=sum3as+1;
                        if cxb=2 then sum243=sum243+1;
                        if cxb=3 then sum343=sum343+1;
                     end;
                     if cxa=5 then do;
                        sum5as=sum5as+1;
                        if cxb=2 then sum245=sum245+1;
                        if cxb=3 then sum345=sum345+1;
                     end;
                  end;
                  if (sum(cp1b=3,cp2b=3,cp3b=3,cp9b=5)>0 or
                      (sum3as>0 and sum343>0) or
                      (sum3as>1 and sum243>1)) then cp3rb=3;
                  else do;
                     if ((sum(cp1b=3,cp2b=3,cp3b=3)=0 and sum3as=0) and
                         (sum(cp1b=5,cp2b=5,cp3b=5,cp7b=5)>0 or
                          (sum5as>0 and sum345>0) or
                          (sum5as>1 and sum245>1))) then cp3rb=5;
                  end;
               end;

         *************;
         * Diagnosis *;
         *************;

            if n(cp3ra1,cp3ra2,cp3rb)=0 then cp3rdx=.;
            else do;
               cp3rdx=1;
               if (cp3ra1=5 and 
                   sum(cp3ra2 in (3,5),cp3rb in (3,5))=2 and
                   sum(cp3ra2=3,cp3rb=3)>0) then cp3rdx=3;
               else if sum(cp3ra1=5,cp3ra2=5,cp3rb=5)=3 then cp3rdx=5;
            end;

      *************************;
      * jump here if skip out *;
      *************************;

         skipout:

         oc3rsk=(4*(sum(ob3rsk=5,cp3rsk=5)=2))+1;

      *************;
      * Diagnosis *;
      *************;

         if oc3rsk=1 then do;
            if n(ob3rdx,cp3rdx)=0 then oc3rdx=.;
            else do;
               oc3rdx=1;
               if sum(ob3rdx=3,cp3rdx=3) then oc3rdx=3;
               else if sum(ob3rdx=3,cp3rdx=3)=0 and 
                       sum(ob3rdx=5,cp3rdx=5)>0 then oc3rdx=5;
            end;
         end;
         else do;
            oc3rdx=.S;
         end;

      *******************;
      * variable labels *;
      *******************;

         label ob3rsk  = "DSM3R OBSESV Skip Out"
               cp3rsk  = "DSM3R CMPLSV Skip Out"
               oc3rsk  = "DSM3R OCD Skip Out"
               ob3ra1  = "DSM3R OBSESV (Lifetime) Crit. A1"
               ob3ra2  = "DSM3R OBSESV (Lifetime) Crit. A2"
               ob3ra3  = "DSM3R OBSESV (Lifetime) Crit. A3"
               ob3ra3x = "DSM3R OBSESV (Lifetime) Sum of 3s"
               ob3ra5x = "DSM3R OBSESV (Lifetime) Sum of 5s"
               ob3rax  = "DSM3R OBSESV (Lifetime) Sum of As"
               ob3ra   = "DSM3R OBSESV (Lifetime) Crit. A"
               ob3rb   = "DSM3R OBSESV (Lifetime) Crit. B"
               ob3rdx  = "DSM3R OBSESV (Lifetime) Diagnosis"
               cp3ra1  = "DSM3R CMPLSV (Lifetime) Crit. A1"
               cp3ra2  = "DSM3R CMPLSV (Lifetime) Crit. A2"
               cp3ra   = "DSM3R CMPLSV (Lifetime) Crit. A"
               cp3rb   = "DSM3R CMPLSV (Lifetime) Crit. B"
               cp3rdx  = "DSM3R CMPLSV (Lifetime) Diagnosis"
               oc3rdx  = "DSM3R OCD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    ob3rsk cp3rsk oc3rsk
                    ob3ra1 ob3ra2 ob3ra3 ob3ra3x ob3ra5x ob3rax ob3ra 
                    ob3rb ob3rdx
                    cp3ra1 cp3ra2 cp3ra cp3rb cp3rdx oc3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_ocd_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               **************;
               * Obsessions *;
               **************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(ocd3r,ob1 ob2b ob3c ob4b ob5c ob6b ob3rsk);

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(ocd3r,ob1 ob2b ob3c ob4b ob5c ob6b ob3ra1);
                     %multi(ocd3r,ob3rsk ob2b ob3b ob4b ob7 ob3ra2);
                     %multi(ocd3r,ob3rsk ob8a ob3ra3);
                     %multi(ocd3r,ob3rsk ob3ra1-ob3ra3 ob3ra3x ob3ra5x);
                     %multi(ocd3r,ob3rsk ob3ra3x ob3ra5x ob3rax);
                     %multi(ocd3r,ob3rsk ob3ra3x ob3ra5x ob3rax ob3ra);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(ocd3r,ob3rsk 
                                  ob14a_1a ob14a_1b ob14a_2a ob14a_2b 
                                  ob14a_3a ob14a_3b ob14a_4a ob14a_4b 
                                  ob14a_5a ob14a_5b ob14a_6a ob14a_6b
                                  ob9b ob3rb);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(ocd3r,ob3rsk ob3ra1 ob3ra2 ob3ra3 ob3rb ob3rdx);

               ***************;
               * Compulsions *;
               ***************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(ocd3r,cp1a_1 cp1a_2 cp1a_3 cp1a_4 cp1a_5 cp1b
                                  cp2a_1 cp2a_2 cp2a_3 cp2b cp3a cp3b 
                                  cp3rsk);

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(ocd3r,cp1a_1-cp1a_5 cp2a_1-cp2a_3 cp3a 
                                  cp3ra1);
                     %multi(ocd3r,cp3rsk cp4a cp4b cp4c cp3ra2);
                     %multi(ocd3r,cp3rsk cp3ra1 cp3ra2 cp3ra);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(ocd3r,cp3rsk cp1b cp2b cp3b cp7b cp9b
                                  cp11a_1a cp11a_1b cp11a_2a cp11a_2b 
                                  cp11a_3a cp11a_3b cp11a_4a cp11a_4b 
                                  cp11a_5a cp11a_5b cp11a_6a cp11a_6b
                                  cp3rb);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(ocd3r,cp3rsk cp3ra1 cp3ra2 cp3rb cp3rdx);

               *************;
               * Diagnosis *;
               *************;

                  %multi(ocd3r,ob3rsk cp3rsk oc3rsk);
                  %multi(ocd3r,oc3rsk ob3rdx cp3rdx oc3rdx);

            ******************;
            * keep variables *;
            ******************;

               data ocd3r;
               set ocd3r;
               keep ind_id  
                    ob3rsk cp3rsk oc3rsk
                    ob3ra1 ob3ra2 ob3ra3 ob3ra3x ob3ra5x ob3rax ob3ra 
                    ob3rb ob3rdx
                    cp3ra1 cp3ra2 cp3ra cp3rb cp3rdx oc3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=ocd3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend ocd3r;

********;
* test *;
********;
/*
   %ocd3r(coga.csaga2,1);
*/
