******************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Oppositional Defiant Disorder *;
******************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
**************************************************************;
* to run as standalone module, uncomment the following lines *;
**************************************************************;
/*
   %include "./dsm3r_cd.sas" / nosource2;

   %cd3r(coga.csaga2,0);
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %odd3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro odd3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ODD3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ODD3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      proc contents data=&dsn. noprint out=test;
      run;

      data test;
      set test;
      if upcase(substr(name,1,2))="OP";
      run;

      data test;
      set test nobs=count;
      call symput("var"||left(_n_),name);
      call symput("nvar",left(put(count,8.)));
      run;

      data odd3r;
      merge &dsn.(keep=ind_id cd11a 
                       %do i=1 %to &nvar.;
                          &&var&i.
                       %end;
                       ) 
            cd3r(keep=ind_id cd3rdx);
      by ind_id;

      attrib od3ra1-od3ra9 od3rax od3ra od3rb od3rdx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **********************;
            * often loses temper *;
            **********************;

               if sum(op1 not in (1,5),
                      cd11a not in (1,3,5))=2 then od3ra1=.;
               else od3ra1=(op1=5 or cd11a=5);

            ****************************;
            * often argues with adults *;
            ****************************;

               if (op2a not in (1,5)) then od3ra2=.;
               else od3ra2=(op2a=5);

            *******************************;
            * often defies requests/rules *;
            *******************************;

               if (op3a not in (1,5)) then od3ra3=.;
               else od3ra3=(op3a=5);

            *******************************;
            * often deliberately annoying *;
            *******************************;

               if (op4 not in (1,5)) then od3ra4=.;
               else od3ra4=(op4=5);

            ***********************;
            * often blames others *;
            ***********************;

               if (op5 not in (1,2,3,4)) then od3ra5=.;
               else od3ra5=(op5=2);

            **********************************;
            * often touchy or easily annoyed *;
            **********************************;

               if (op6 not in (1,5)) then od3ra6=.;
               else od3ra6=(op6=5);

            ****************************;
            * often angry or resentful *;
            ****************************;

               if (op7 not in (1,5)) then od3ra7=.;
               else od3ra7=(op7=5);

            *****************************;
            * often spiteful/vindictive *;
            *****************************;

               if (op8a not in (1,5)) then od3ra8=.;
               else od3ra8=(op8a=5);

            *********************************;
            * often swears/uses obscenities *;
            *********************************;

               if (op9 not in (1,5)) then od3ra9=.;
               else od3ra9=(op9=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of od3ra1-od3ra9)=0 then od3rax=.;
               else od3rax=sum(of od3ra1-od3ra9);

            *************************************;
            * behavior lasted at least 6 months *;
            *************************************;

               if od3rax=. then od3ra=.;
               else if od3rax<5 then od3ra=0;
               else do;
                  if (op10b not in (1,5) or op10c not in (1,5)) then od3ra=.;
                  else od3ra=((od3rax>4) and (op10b=5 and op10c=5));
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************;
            * does not meet criteria for conduct disorder *;
            ***********************************************;

               od3rb=1;
               if cd3rdx=5 then od3rb=0;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(od3rax,od3ra,od3rb)=0 then od3rdx=.;
            else od3rdx=(od3rax>4 and od3ra=1 and od3rb=1);

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         array one od3ra1-od3ra9 od3ra od3rb1 od3rb od3rdx;
         do over one;
            if one=1 then one=5;
            if one=0 then one=1;
         end;

      *******************;
      * variable labels *;
      *******************;

         label od3ra1 = "DSM3R ODD (Lifetime) Crit. A1"
               od3ra2 = "DSM3R ODD (Lifetime) Crit. A2"
               od3ra3 = "DSM3R ODD (Lifetime) Crit. A3"
               od3ra4 = "DSM3R ODD (Lifetime) Crit. A4"
               od3ra5 = "DSM3R ODD (Lifetime) Crit. A5"
               od3ra6 = "DSM3R ODD (Lifetime) Crit. A6"
               od3ra7 = "DSM3R ODD (Lifetime) Crit. A7"
               od3ra8 = "DSM3R ODD (Lifetime) Crit. A8"
               od3ra9 = "DSM3R ODD (Lifetime) Crit. A9"
               od3rax = "DSM3R ODD (Lifetime) Sum of As"
               od3ra  = "DSM3R ODD (Lifetime) 5+ in 6Mo"
               od3rb  = "DSM3R ODD (Lifetime) Crit. B"
               od3rdx = "DSM3R ODD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    od3ra1-od3ra9 od3rax od3ra od3rb od3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_odd_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(odd3r,op1 cd11a od3ra1);
                     %multi(odd3r,op2a od3ra2);
                     %multi(odd3r,op3a od3ra3);
                     %multi(odd3r,op4 od3ra4);
                     %multi(odd3r,op5 od3ra5);
                     %multi(odd3r,op6 od3ra6);
                     %multi(odd3r,op7 od3ra7);
                     %multi(odd3r,op8a od3ra8);
                     %multi(odd3r,op9 od3ra9);
                     %multi(odd3r,od3ra1-od3ra9 od3rax);
                     %multi(odd3r,od3rax op10b op10c od3ra);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(odd3r,cd3rdx od3rb);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(odd3r,od3rax od3ra od3rb od3rdx);

            ******************;
            * keep variables *;
            ******************;

               data odd3r;
               set odd3r;
               keep ind_id  
                    od3ra1-od3ra9 od3rax od3ra od3rb od3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=odd3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend odd3r;

********;
* test *;
********;
/*
   %odd3r(coga.csaga2,1);
*/
