*********************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Overanxious Disorder *;
*********************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %ovx3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro ovx3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in OVX3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in OVX3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data ovx3r;
      set &dsn.;

      attrib ov3rsk
             ov3ra1-ov3ra8 ov3ra3x ov3ra5x ov3rax ov3ra ov3rdx length=3;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(ov2a not in (1,5),
                ov2b not in (1,3,5),
                ov3a not in (1,5),
                ov3b not in (1,3,5))=4 then ov3rsk=5;
         else do;
            if sum((ov2a ne 5 and ov3a ne 5),
                   (ov2b not in (3,5) and ov3b not in (3,5)))=2 then ov3rsk=5;
            else ov3rsk=1;
         end;

         if ov3rsk=5 then do;
            array skp1 ov3ra1-ov3ra8 ov3ra3x ov3ra5x ov3rax ov3ra ov3rdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if ov3rsk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************************;
            * excessive, unrealistic worry about future events *;
            ****************************************************;

               if ov2b not in (1,3,5) then ov3ra1=.;
               else ov3ra1=(4*(ov2b=5))+(2*(ov2b=3))+1;

            **************************************************;
            * excessive, unrealistic worry about past events *;
            **************************************************;

               if ov3b not in (1,3,5) then ov3ra2=.;
               else ov3ra2=(4*(ov3b=5))+(2*(ov3b=3))+1;
 
            *************************************************;
            * excessive, unrealistic worry about competence *;
            *************************************************;

               if sum(ov4a not in (1,3,5),
                      ov4b not in (1,3,5))=2 then ov3ra3=.;
               else do;
                  ov3ra3=1;
                  if (ov4a=3 or ov4b=3) then ov3ra3=3;
                  else if (ov4a=5 or ov4b=5) then ov3ra3=5;
               end;

            *********************************************;
            * somatic complaints without physical basis *;
            *********************************************;

               if ov5 not in (1,3,5) then ov3ra4=.;
               else ov3ra4=(4*(ov5=5))+(2*(ov5=3))+1;

            *****************************;
            * marked self-consciousness *;
            *****************************;

               if ov6b not in (1,3,5) then ov3ra5=.;
               else ov3ra5=(4*(ov6b=5))+(2*(ov6b=3))+1;

            **********************************;
            * excessive need for reassurance *;
            **********************************;

               if sum(ov7a not in (1,3,5),
                      ov7b not in (1,3,5),
                      ov7c not in (1,3,5),
                      ov7d not in (1,3,5))=4 then ov3ra6=.;
               else do;
                  ov3ra6=1;
                  sum3s=0;
                  sum5s=0;
                  array sxs ov7a ov7b ov7c ov7d;
                  do over sxs;
                     if sxs=3 then sum3s+1;
                     if sxs=5 then sum5s+1;
                  end;
                  if sum3s>1 then ov3ra6=3;
                  else if sum5s>1 then ov3ra6=5;
               end;

            ****************************************;
            * marked tension or inability to relax *;
            ****************************************;

               if ov8 not in (1,3,5) then ov3ra7=.;
               else ov3ra7=(4*(ov8=5))+(2*(ov8=3))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of ov3ra1-ov3ra7)=0 then ov3rax=.;
               else do;
                  ov3ra3x=0;
                  ov3ra5x=0;
                  array sxx ov3ra1-ov3ra7;
                  do over sxx;
                     if sxx=3 then ov3ra3x=ov3ra3x+1;
                     if sxx=5 then ov3ra5x=ov3ra5x+1;
                  end;
                  ov3rax=sum(ov3ra3x,ov3ra5x);
                  ov3ra=(4*(ov3rax>2))+1;
               end;

            *************************************;
            * behavior lasted at least 6 months *;
            *************************************;

               if sum(ov17a not in (1,5),
                      ov17b not in (1,5))=2 then ov3ra8=.;
               else ov3ra8=(4*(sum(ov17a=5,ov17b=5)>0))+1;

      **************************************;
      * Criterion B -- Skip in this module *;
      **************************************;

      *********************************;
      * Criterion C -- Not applicable *;
      *********************************;

      **************************************;
      * Criterion D -- Skip in this module *;
      **************************************;
 
      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if ov3ra=. then ov3rdx=.;
            else do;
               ov3rdx=1;
               if (ov3ra3x>0 and ov3rax>3) then ov3rdx=3;
               else if (ov3ra3x=0 and ov3rax>3) then ov3rdx=5;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label ov3rsk  = "DSM3R OVRANX Skip Out"
               ov3ra1  = "DSM3R OVRANX (Lifetime) Crit. A1"
               ov3ra2  = "DSM3R OVRANX (Lifetime) Crit. A2"
               ov3ra3  = "DSM3R OVRANX (Lifetime) Crit. A3"
               ov3ra4  = "DSM3R OVRANX (Lifetime) Crit. A4"
               ov3ra5  = "DSM3R OVRANX (Lifetime) Crit. A5"
               ov3ra6  = "DSM3R OVRANX (Lifetime) Crit. A6"
               ov3ra7  = "DSM3R OVRANX (Lifetime) Crit. A7"
               ov3ra3x = "DSM3R OVRANX (Lifetime) Sum of 3s"
               ov3ra5x = "DSM3R OVRANX (Lifetime) Sum of 5s"
               ov3rax  = "DSM3R OVRANX (Lifetime) Sum of As"
               ov3ra8  = "DSM3R OVRANX (Lifetime) Lasted 6Mo+"
               ov3ra   = "DSM3R OVRANX (Lifetime) Crit. A"
               ov3rdx  = "DSM3R OVRANX (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    ov3rsk
                    ov3ra1-ov3ra8 ov3ra3x ov3ra5x ov3rax ov3ra ov3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_ovx_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(ovx3r,ov2a ov2b ov3a ov3b ov3rsk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(ovx3r,ov3rsk ov2b ov3ra1);
                     %multi(ovx3r,ov3rsk ov3b ov3ra2);
                     %multi(ovx3r,ov3rsk ov4a ov4b ov3ra3);
                     %multi(ovx3r,ov3rsk ov5 ov3ra4);
                     %multi(ovx3r,ov3rsk ov6b ov3ra5);
                     %multi(ovx3r,ov3rsk ov7a ov7b ov7c ov7d ov3ra6);
                     %multi(ovx3r,ov3rsk ov8 ov3ra7);
                     %multi(ovx3r,ov3rsk ov3ra1-ov3ra6 ov3ra3x ov3ra5x);
                     %multi(ovx3r,ov3rsk ov3ra3x ov3ra5x ov3rax);
                     %multi(ovx3r,ov3rsk ov17a ov17b ov3ra8);
                     %multi(ovx3r,ov3rsk ov3ra3x ov3ra5x ov3ra8 ov3ra);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(ovx3r,ov3rsk ov3ra3x ov3ra ov3rdx);

            ******************;
            * keep variables *;
            ******************;

               data ovx3r;
               set ovx3r;
               keep ind_id
                    ov3rsk  
                    ov3ra1-ov3ra8 ov3ra3x ov3ra5x ov3rax ov3ra ov3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=ovx3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend ovx3r;

********;
* test *;
********;
/*
   %ovx3r(coga.csaga2,1);
*/
