***************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Panic Disorder *;
***************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %pan3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro pan3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in PAN3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in PAN3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data pan3r;
      set &dsn.;

      attrib pn3rsk1 pn3rsk2 pn3rsk
             pn3ra pn3rb pn3rc1-pn3rc13 pn3rcx pn3rc pn3rd pn3re pn3rdx 
             length=3;

      *********************;
      * test for skip out *;
      *********************;
 
         if pn1a not in (1,2,3,4,5) then pn3rsk1=5;
         else do;
            if pn1a in (2,3,4,5) then pn3rsk1=1;
            else pn3rsk1=5;
         end;

         if pn3rsk1=5 then do;
            array skp1 pn3ra pn3rb pn3rc1-pn3rc13 pn3rcx pn3rc pn3rd pn3re 
                       pn3rdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if pn3rsk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************;
            * unexpected panic attacks *;
            ****************************;

               if pn1a not in (1,2,3,4,5) then pn3ra=.;
               else pn3ra=(4*(pn1a in (3,4,5)))+1;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(pn2a_1 not in (1,5),pn2a_2 not in (1,5),
                pn2a_3 not in (1,5),pn2a_4 not in (1,5),
                pn2a_5 not in (1,5),pn2a_6 not in (1,5),
                pn2a_7 not in (1,5),pn2a_8 not in (1,5),
                pn2a_9 not in (1,5),pn2a_10 not in (1,5),
                pn2a_11 not in (1,5),pn2a_12 not in (1,5),
                pn2a_13 not in (1,5),pn2a_14 not in (1,5),
                pn2a_15 not in (1,5))=15 then pn3rsk2=5;
         else do;
            if sum(pn2a_1=5,pn2a_2=5,pn2a_3=5,pn2a_4=5,
                   pn2a_5=5,pn2a_6=5,pn2a_7=5,pn2a_8=5,
                   pn2a_9=5,pn2a_10=5,pn2a_11=5,pn2a_12=5,
                   pn2a_13=5,pn2a_14=5,pn2a_15=5)>3 then pn3rsk2=1;
            else pn3rsk2=5;
         end;

         if pn3rsk2=5 then do;
            array skp2 pn3rb pn3rc1-pn3rc13 pn3rcx pn3rc pn3rd pn3re 
                       pn3rdx;
            do over skp2;
              skp2=.S;
            end;
         end;

         if pn3rsk2=5 then go to skipout;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************;
            * unexpected panic attacks *;
            ****************************;

               if sum(pn4b_2 not in (1,5),
                      pn6c not in (1,2,5))=2 then pn3rb=.;
               else pn3rb=(4*(pn4b_2=5 or pn6c=5))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************;
            * shortness of breath *;
            ***********************;

               if pn2a_4 not in (1,5) then pn3rc1=.;
               else pn3rc1=(4*(pn2a_4=5))+1;

            *****************;
            * feeling dizzy *;
            *****************;

               if pn2a_8 not in (1,5) then pn3rc2=.;
               else pn3rc2=(4*(pn2a_8=5))+1;

            ****************;
            * palpitations *;
            ****************;

               if pn2a_1 not in (1,5) then pn3rc3=.;
               else pn3rc3=(4*(pn2a_1=5))+1;

            ************************;
            * trembling or shaking *;
            ************************;

               if pn2a_3 not in (1,5) then pn3rc4=.;
               else pn3rc4=(4*(pn2a_3=5))+1;

            ************;
            * sweating *;
            ************;

               if pn2a_2 not in (1,5) then pn3rc5=.;
               else pn3rc5=(4*(pn2a_2=5))+1;

            **********************;
            * feeling of choking *;
            **********************;

               if pn2a_5 not in (1,5) then pn3rc6=.;
               else pn3rc6=(4*(pn2a_5=5))+1;

            ********************************;
            * nausea or abdominal distress *;
            ********************************;

               if pn2a_7 not in (1,5) then pn3rc7=.;
               else pn3rc7=(4*(pn2a_7=5))+1;

            ***********************************;
            * derealization/depersonalization *;
            ***********************************;

               if pn2a_9 not in (1,5) then pn3rc8=.;
               else pn3rc8=(4*(pn2a_9=5))+1;

            ****************;
            * parasthesias *;
            ****************;

               if pn2a_11 not in (1,5) then pn3rc9=.;
               else pn3rc9=(4*(pn2a_11=5))+1;

            *************************;
            * chills or hot flashes *;
            *************************;

               if sum(pn2a_12 not in (1,5),
                      pn2a_13 not in (1,5))=2 then pn3rc10=.;
               else pn3rc10=(4*(pn2a_12=5 or pn2a_13=5))+1;

            ****************************;
            * chest pain or discomfort *;
            ****************************;

               if pn2a_6 not in (1,5) then pn3rc11=.;
               else pn3rc11=(4*(pn2a_6=5))+1;

            *****************;
            * fear of dying *;
            *****************;

               if pn2a_15 not in (1,5) then pn3rc12=.;
               else pn3rc12=(4*(pn2a_15=5))+1;

            **************************;
            * fear of losing control *;
            **************************;

               if pn2a_10 not in (1,5) then pn3rc13=.;
               else pn3rc13=(4*(pn2a_10=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of pn3rc1-pn3rc13)=0 then pn3rcx=.;
               else do;
                  pn3rcx=0;
                  array sxs pn3rc1-pn3rc13;
                  do over sxs;
                     if sxs=5 then pn3rcx=pn3rcx+1;
                  end;
                  pn3rc=(4*(pn3rcx>3))+1;
               end;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *******************************;
            * symptoms developed suddenly *;
            *******************************;

               if pn3 not in (1,5) then pn3rd=.;
               else pn3rd=(4*(pn3=5))+1;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ********************;
            * organic etiology *;
            ********************;

               if pn1a not in (1,2,3,4,5) then pn3re=.;
               else do;
                  if pn1a in (1,5) then pn3re=1;
                  if pn1a in (3,4) then pn3re=pn1a;
               end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(pn3ra,pn3rb,pn3rc,pn3rd,pn3re)<5 then pn3rdx=.;
            else do;
               pn3rdx=1;
               if (pn3ra=5 and pn3rb=5 and pn3rc=5 and pn3rd=5) then do;
                  if pn3re=1 then pn3rdx=5;
                  if pn3re=3 then pn3rdx=3;
                  if pn3re=4 then pn3rdx=4;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         pn3rsk=max(pn3rsk1,pn3rsk2);

      *******************;
      * variable labels *;
      *******************;

         label pn3rsk1 = "DSM3R PANIC Skip Out #1"
               pn3rsk2 = "DSM3R PANIC Skip Out #2"
               pn3rsk  = "DSM3R PANIC Skip Out"
               pn3ra   = "DSM3R PANIC (Lifetime) Crit. A"
               pn3rb   = "DSM3R PANIC (Lifetime) Crit. B"
               pn3rc1  = "DSM3R PANIC (Lifetime) Crit. C1"
               pn3rc2  = "DSM3R PANIC (Lifetime) Crit. C2"
               pn3rc3  = "DSM3R PANIC (Lifetime) Crit. C3"
               pn3rc4  = "DSM3R PANIC (Lifetime) Crit. C4"
               pn3rc5  = "DSM3R PANIC (Lifetime) Crit. C5"
               pn3rc6  = "DSM3R PANIC (Lifetime) Crit. C6"
               pn3rc7  = "DSM3R PANIC (Lifetime) Crit. C7"
               pn3rc8  = "DSM3R PANIC (Lifetime) Crit. C8"
               pn3rc9  = "DSM3R PANIC (Lifetime) Crit. C9"
               pn3rc10 = "DSM3R PANIC (Lifetime) Crit. C10"
               pn3rc11 = "DSM3R PANIC (Lifetime) Crit. C11"
               pn3rc12 = "DSM3R PANIC (Lifetime) Crit. C12"
               pn3rc13 = "DSM3R PANIC (Lifetime) Crit. C13"
               pn3rcx  = "DSM3R PANIC (Lifetime) Sum of Cs"
               pn3rc   = "DSM3R PANIC (Lifetime) Crit. C"
               pn3rd   = "DSM3R PANIC (Lifetime) Crit. D"
               pn3re   = "DSM3R PANIC (Lifetime) Crit. E"
               pn3rdx  = "DSM3R PANIC (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    pn3rsk1 pn3rsk2 pn3rsk
                    pn3ra pn3rb pn3rc1-pn3rc13 pn3rcx pn3rc pn3rd pn3re 
                    pn3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_pan_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(pan3r,pn1a pn3rsk1);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(pan3r,pn3rsk1 pn1a pn3ra);

               ************;
               * skip out *;
               ************;

                  %multi(pan3r,pn3rsk1 
                               pn2a_1 pn2a_2 pn2a_3 
                               pn2a_4 pn2a_5 pn2a_6 
                               pn2a_7 pn2a_8 pn2a_9 
                               pn2a_10 pn2a_11 pn2a_12 
                               pn2a_13 pn2a_14 pn2a_15 pn3rsk2); 

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(pan3r,pn3rsk1 pn3rsk2 pn4b_2 pn6c pn3rb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_4 pn3rc1);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_8 pn3rc2);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_1 pn3rc3);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_3 pn3rc4);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_2 pn3rc5);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_5 pn3rc6);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_7 pn3rc7);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_9 pn3rc8);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_11 pn3rc9);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_12 pn2a_13 pn3rc10);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_6 pn3rc11);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_15 pn3rc12);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn2a_10 pn3rc13);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn3rc1-pn3rc13 pn3rcx);
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn3rcx pn3rc);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(pan3r,pn3rsk1 pn3rsk2 pn3 pn3rd);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(pan3r,pn3rsk1 pn3rsk2 pn1a pn3re);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(pan3r,pn3rsk1 pn3rsk2 pn3ra pn3rb pn3rc pn3rd
                                  pn3re pn3rdx);

            ******************;
            * keep variables *;
            ******************;

               data pan3r;
               set pan3r;
               keep ind_id  
                    pn3rsk1 pn3rsk2 pn3rsk
                    pn3ra pn3rb pn3rc1-pn3rc13 pn3rcx pn3rc pn3rd pn3re 
                    pn3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=pan3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend pan3r;

********;
* test *;
********;
/*
   %pan3r(coga.csaga2,1);
*/
