*******************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Psychotic Symptoms *;
*******************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %psy3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro psy3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in PSY3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in PSY3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data psy3r;
      set &dsn.;

      attrib ps3ra1a ps3ra1b1 ps3ra1b2 ps3ra1b3 ps3ra2 ps3rsx length=3;

      ***************************;
      * rescale interview items *;
      ***************************;

         array sxs ps1c ps2c ps3 ps4 ps5 ps6c ps7c ps8 ps9 ps10 ps11 ps12;
         do over sxs;
            if sxs=9 then sxs=.;
         end;

      *************;
      * Delusions *;
      *************;

         if sum(ps8 not in (1,2,3,4,5),
                ps9 not in (1,2,3,4,5))=2 then ps3ra1a=.;
         else do;
            ps3ra1a=1;
            if ps8 in (3,4) then ps3ra1a=ps8;
            else if sum(ps8=5,ps9=5)>0 then ps3ra1a=5;
            else if sum(ps8 in (1,2),ps9 in (1,2))>0 then ps3ra1a=max(ps8,ps9);
         end;

      ****************************;
      * Prominent hallucinations *;
      ****************************;

         *************************;
         * visual hallucinations *;
         *************************;

            if ps1c not in (1,2,3,4,5) then ps3ra1b1=.;
            else ps3ra1b1=ps1c;

         ***************************;
         * auditory hallucinations *;
         ***************************;

            if sum(ps2c not in (1,2,3,4,5),
                   ps6c not in (1,2,3,4,5))=2 then ps3ra1b2=.;
            else do;
               ps3ra1b2=1;
               if sum(ps2c=3,ps6c=3)>0 then ps3ra1b2=3;
               else if sum(ps2c=4,ps6c=4)>0 then ps3ra1b2=4;
               else if sum(ps2c=5,ps6c=5)>0 then ps3ra1b2=5;
               else if sum(ps2c in (1,2),ps6c in (1,2))>0 then 
                  ps3ra1b2=max(ps2c,ps6c);
            end;

         ****************************;
         * olfactory hallucinations *;
         ****************************;

            if ps7c not in (1,2,3,4,5) then ps3ra1b3=.;
            else ps3ra1b3=ps7c;

      *********************;
      * Bizarre delusions *;
      *********************;

         if sum(ps3 not in (1,2,3,4,5),
                ps4 not in (1,2,3,4,5),
                ps5 not in (1,2,3,4,5),
                ps10 not in (1,2,3,4,5),
                ps11 not in (1,2,3,4,5),
                ps12 not in (1,2,3,4,5))=6 then ps3ra2=.;
         else do;
            if sum(ps3=3,ps4=3,ps5=3,ps10=3,ps11=3,ps12=3)>0 then ps3ra2=3;
            else if sum(ps3=4,ps4=4,ps5=4,ps10=4,ps11=4,ps12=4)>0 then ps3ra2=4;
            else if sum(ps3=5,ps4=5,ps5=5,ps10=5,ps11=5,ps12=5)>0 then ps3ra2=5;
            else if sum(ps3 in (1,2),ps4 in (1,2),ps5 in (1,2),
                        ps10 in (1,2),ps11 in (1,2),ps12 in (1,2))>0 then 
               ps3ra2=max(ps3,ps4,ps5,ps10,ps11,ps12);
         end;

      ******************;
      * Symptomatology *;
      ******************;

         if n(ps3ra1a,ps3ra1b1,ps3ra1b2,ps3ra1b3,ps3ra2)=0 then ps3rsx=.;
         else do;
            ps3rsx=1;
            if sum(ps3ra1a=3,ps3ra1b1=3,ps3ra1b2=3,
                   ps3ra1b3=3,ps3ra2=3)>0 then ps3rsx=3;
            else if sum(ps3ra1a=4,ps3ra1b1=4,ps3ra1b2=4,
                        ps3ra1b3=4,ps3ra2=4)>0 then ps3rsx=4;
            else if sum(ps3ra1a=5,ps3ra1b1=5,ps3ra1b2=5,
                        ps3ra1b3=5,ps3ra2=5)>0 then ps3rsx=5;
            else if sum(ps3ra1a in (1,2),ps3ra1b1 in (1,2),ps3ra1b2 in (1,2),
                        ps3ra1b3 in (1,2),ps3ra2 in (1,2))>0 then 
               ps3rsx=max(ps3ra1a,ps3ra1b1,ps3ra1b2,ps3ra1b3,ps3ra2);
         end;

      *******************;
      * variable labels *;
      *******************;

         label ps3ra1a  = "DSM3R PSYSXS Delusions"
               ps3ra1b1 = "DSM3R PSYSXS Visual Hallucinations"
               ps3ra1b2 = "DSM3R PSYSXS Auditory Hallucinations"
               ps3ra1b3 = "DSM3R PSYSXS Olfactory Hallucinations"
               ps3ra2   = "DSM3R PSYSXS Bizarre Delusions" 
               ps3rsx   = "DSM3R PSYSXS Psychotic Symptomatology";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id ps3ra1a ps3ra1b1 ps3ra1b2 ps3ra1b3 ps3ra2 ps3rsx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_psy_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               *************;
               * Delusions *;
               *************;

                  %multi(psy3r,ps8 ps9 ps3ra1a);

               ****************************;
               * Prominent hallucinations *;
               ****************************;

                  %multi(psy3r,ps1c ps3ra1b1);
                  %multi(psy3r,ps2c ps6c ps3ra1b2);
                  %multi(psy3r,ps7c ps3ra1b3);

               *********************;
               * Bizarre delusions *;
               *********************;

                  %multi(psy3r,ps3 ps4 ps5 ps10 ps11 ps12 ps3ra2);

               ******************;
               * Symptomatology *;
               ******************;

                  %multi(psy3r,ps3ra1a ps3ra1b1 ps3ra1b2 ps3ra1b3 ps3ra2
                               ps3rsx);

            ******************;
            * keep variables *;
            ******************;

               data psy3r;
               set psy3r;
               keep ind_id ps3ra1a ps3ra1b1 ps3ra1b2 ps3ra1b3 ps3ra2 ps3rsx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=psy3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend psy3r;

********;
* test *;
********;
/*
   %psy3r(coga.csaga2,1);
*/
