****************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Separation Anxiety Disorder *;
****************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %sep3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro sep3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SEP3r: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SEP3r: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data sep3r;
      set &dsn.;

      attrib sa3rsk
             sa3ra1-sa3ra9 sa3ra3x sa3ra5x sa3rax sa3ra 
             sa3rb sa3rc sa3rdx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************************************************;
            * persistent worry about harm to/loss of attachment figures *;
            *************************************************************;

               if sa1a not in (1,5) then sa3ra1=.;
               else sa3ra1=(4*(sa1a=5))+1;

            ********************************************;
            * persistent worry about calamitous events *;
            ********************************************;

               if sa2a not in (1,5) then sa3ra2=.;
               else sa3ra2=(4*(sa2a=5))+1;

            **************************************;
            * persistent refusal to go to school *;
            **************************************;

               if sum(sa1b not in (1,3,5),
                      sa2b not in (1,3,5),
                      sa3a not in (1,5))=3 then sa3ra3=.;
               else do;
                  sa3ra3=1;
                  if (sa1b=3 or sa2b=3) then sa3ra3=3;
                  else if (sa1b=5 or sa2b=5 or sa3a=5) then sa3ra3=5;
               end;

            *********************;
            * test for skip out *;
            *********************;
 
               if sum(sa3a not in (1,5),
                      sa3b not in (1,3,5),
                      sa3c not in (0,1))=3 then sa3rsk=5;
               else do;
                  if sum(sum(sa3a=5,sa3b=5)>0,sa3c>1)>0 then sa3rsk=1;
                  else sa3rsk=5;
               end;

               if sa3rsk=5 then do;
                  array skp1 sa3ra4-sa3ra9 sa3ra3x sa3ra5x sa3rax sa3ra 
                             sa3rb sa3rc sa3rdx;
                  do over skp1;
                     skp1=.S;
                  end;
               end;

              if sa3rsk=5 then go to skipout;

            *************************************;
            * persistent refusal to go to sleep *;
            *************************************;

               if sum(sa4a not in (1,3,5),
                      sa5a not in (1,3,5),
                      sa5b not in (1,3,5))=3 then sa3ra4=.;
               else do;
                  sa3ra4=1;
                  if (sa4a=3 or sa5a=3 or sa5b=3) then sa3ra4=3;
                  else if (sa4a=5 or sa5a=5 or sa5b=5) then sa3ra4=5;
               end;

            ***************************************;
            * persistent avoidance of being alone *;
            ***************************************;

               if sum(sa6a not in (1,3,5),
                      sa6b not in (1,3,5))=2 then sa3ra5=.;
               else do;
                  sa3ra5=1;
                  if (sa6a=3 or sa6b=3) then sa3ra5=3;
                  else if (sa6a=5 or sa6b=5) then sa3ra5=5;
               end;

            ******************************************;
            * repeated separation-related nightmares *;
            ******************************************;

               if sa9 not in (1,3,5) then sa3ra6=.;
               else sa3ra6=sa9;

            ****************************************************************;
            * complaints of physical symptoms when anticipating separation *;
            ****************************************************************;

               if sa10 not in (1,3,5) then sa3ra7=.;
               else sa3ra7=sa10;

            *************************************************************;
            * recurrent excessive distress when anticipating separation *;
            *************************************************************;

               if sum(sa11a not in (1,3,5),
                      sa11b not in (1,3,5))=2 then sa3ra8=.;
               else do;
                  sa3ra8=1;
                  if (sa11a=3 or sa11b=3) then sa3ra8=3;
                  else if (sa11a=5 or sa11b=5) then sa3ra8=5;
               end;

            ***********************************************;
            * recurrent excessive distress when separated *;
            ***********************************************;

               if sum(sa7 not in (1,3,5),
                      sa8a not in (1,3,5),
                      sa8b not in (1,3,5))=3 then sa3ra9=.;
               else do;
                  sa3ra9=1;
                  if (sa7=3 or sa8a=3 or sa8b=3) then sa3ra9=3;
                  else if (sa7=5 or sa8a=5 or sa8b=5) then sa3ra9=5;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of sa3ra1-sa3ra9)=0 then sa3rax=.;
               else do;
                  sa3ra3x=0;
                  sa3ra5x=0;
                  array sxs sa3ra1-sa3ra9;
                  do over sxs;
                     if sxs=3 then sa3ra3x+1;
                     if sxs=5 then sa3ra5x+1;
                  end;
                  sa3rax=sum(sa3ra3x,sa3ra5x);
                  sa3ra=(4*(sa3rax>2))+1;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ************************************;
            * behavior lasted at least 2 weeks *;
            ************************************;

               if sum(sa12c<0,
                      sa12c_1<0,
                      sa13c<0,
                      sa13c_1<0)=4 then sa3rb=.;
               else sa3rb=(4*(sum(sa12c>1,sa12c_1>1,
                                  sa13c>1,sa13c_1>1)>0))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************;
            * onset before age 18 *;
            ***********************;

               if sum(sa12a_1<0,sa13a_1<0)=2 then sa3rc=.;
               else do;
                  if (sa12a_1 ne . and sa12a_1>0) and 
                     (sa13a_1=. or sa13a_1<0) then
                     sa3rc=(4*(sa12a_1<18))+1;
                  if (sa12a_1 ne . and sa12a_1>0) and 
                     (sa13a_1 ne . and sa13a_1>0) then
                     sa3rc=(4*(sum(sa12a_1<18,sa13a_1<18)>0))+1;
                  if (sa13a_1 ne . and sa13a_1>0) and 
                     (sa12a_1=. or sa12a_1<0) then
                     sa3rc=(4*(sa13a_1<18))+1;
               end;

      **************************************;
      * Criterion D -- Skip in this module *;
      **************************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(sa3ra,sa3rb,sa3rc)=0 then sa3rdx=.;
            else do;
               sa3rdx=1;
               if (sa3rax>2 and sa3rb=5 and sa3rc=5) then do;
                  if sa3ra3x>0 then sa3rdx=3;
                  else if sa3ra3x=0 then sa3rdx=5;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label sa3rsk  = "DSM3R SEPANX Skip Out"
               sa3ra1  = "DSM3R SEPANX (Lifetime) Crit. A1"
               sa3ra2  = "DSM3R SEPANX (Lifetime) Crit. A2"
               sa3ra3  = "DSM3R SEPANX (Lifetime) Crit. A3"
               sa3ra4  = "DSM3R SEPANX (Lifetime) Crit. A4"
               sa3ra5  = "DSM3R SEPANX (Lifetime) Crit. A5"
               sa3ra6  = "DSM3R SEPANX (Lifetime) Crit. A6"
               sa3ra7  = "DSM3R SEPANX (Lifetime) Crit. A7"
               sa3ra8  = "DSM3R SEPANX (Lifetime) Crit. A8"
               sa3ra9  = "DSM3R SEPANX (Lifetime) Crit. A9"
               sa3ra3x = "DSM3R SEPANX (Lifetime) Sum of 3s"
               sa3ra5x = "DSM3R SEPANX (Lifetime) Sum of 5s"
               sa3rax  = "DSM3R SEPANX (Lifetime) Sum of As"
               sa3ra   = "DSM3R SEPANX (Lifetime) 8+ in 6Mo"
               sa3rb   = "DSM3R SEPANX (Lifetime) Crit. B"
               sa3rc   = "DSM3R SEPANX (Lifetime) Crit. C"
               sa3rdx  = "DSM3R SEPANX (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    sa3rsk 
                    sa3ra1-sa3ra9 sa3ra3x sa3ra5x sa3rax sa3ra 
                    sa3rb sa3rc sa3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_sep_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(sep3r,sa3a sa3b sa3c sa3rsk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sep3r,sa1a sa3ra1);
                     %multi(sep3r,sa2a sa3ra2);
                     %multi(sep3r,sa1b sa2b sa3a sa3ra3);
                     %multi(sep3r,sa3rsk sa4a sa5a sa5b sa3ra4);
                     %multi(sep3r,sa3rsk sa6a sa6b sa3ra5);
                     %multi(sep3r,sa3rsk sa9 sa3ra6);
                     %multi(sep3r,sa3rsk sa10 sa3ra7);
                     %multi(sep3r,sa3rsk sa11a sa11b sa3ra8);
                     %multi(sep3r,sa3rsk sa7 sa8a sa8b sa3ra9);
                     %multi(sep3r,sa3rsk sa3ra1-sa3ra9 sa3ra3x sa3ra5x);
                     %multi(sep3r,sa3rsk sa3ra3x sa3ra5x sa3rax sa3ra);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sep3r,sa3rsk sa12c sa12c_1 sa13c sa13c_1 sa3rb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sep3r,sa3rsk sa12a_1 sa13a_1 sa3rc);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sep3r,sa3rsk sa3ra3x sa3ra5x sa3rb sa3rc sa3rdx);

            ******************;
            * keep variables *;
            ******************;

               data sep3r;
               set sep3r;
               keep ind_id  
                    sa3rsk
                    sa3ra1-sa3ra9 sa3ra3x sa3ra5x sa3rax sa3ra
                    sa3rb sa3rc sa3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=sep3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend sep3r;

********;
* test *;
********;
/*
   %sep3r(coga.csaga2,1);
*/
