***********************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Social Phobia Disorder *;
***********************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %soc3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro soc3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SOC3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SOC3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data soc3r;
      set &dsn.;

      attrib sp3rsk1 sp3rsk2 sp3rsk
             sp3ra sp3rc sp3rd sp3re sp3rf sp3rdx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas sp3rsk1 sp3rsk2 sp3rsk 
                      sp3ra sp3rc sp3rd sp3re sp3rf sp3rdx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      *********************;
      * test for skip out *;
      *********************;
 
         if sp2 not in (1,3,5) then sp3rsk1=5;
         else do;
            if sp2 in (3,5) then sp3rsk1=1;
            else sp3rsk1=5;
         end;

         if sp3rsk1=5 then do;
            array skp1 sp3ra sp3rc sp3rd sp3re sp3rf sp3rdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if sp3rsk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************;
            * persistent fear of possible scrutiny *;
            ****************************************;

               if sp2 not in (1,3,5) then sp3ra=.;
               else sp3ra=(4*(sp2=5))+(2*(sp2=3))+1;

      *********************;
      * test for skip out *;
      *********************;

         if sum(sum(sp3a_1 not in (1,5),
                    sp3a_2 not in (1,5),
                    sp3a_3 not in (1,5),
                    sp3a_4 not in (1,5),
                    sp3a_5 not in (1,5),
                    sp3a_6 not in (1,5),
                    sp3a_7 not in (1,5),
                    sp3a_8 not in (1,5),
                    sp3a_9 not in (1,5),
                    sp3a_10 not in (1,5),
                    sp3a_11 not in (1,5),
                    sp3a_12 not in (1,5),
                    sp3a_13 not in (1,5))=13,
                sum(sp3b_1 not in (1,5),
                    sp3b_2 not in (1,5),
                    sp3b_3 not in (1,5),
                    sp3b_4 not in (1,5),
                    sp3b_5 not in (1,5),
                    sp3b_6 not in (1,5),
                    sp3b_7 not in (1,5),
                    sp3b_8 not in (1,5),
                    sp3b_9 not in (1,5),
                    sp3b_10 not in (1,5),
                    sp3b_11 not in (1,5),
                    sp3b_12 not in (1,5),
                    sp3b_13 not in (1,5))=13,
                sum(sp3c_1 not in (1,5),
                    sp3c_2 not in (1,5),
                    sp3c_3 not in (1,5),
                    sp3c_4 not in (1,5),
                    sp3c_5 not in (1,5),
                    sp3c_6 not in (1,5),
                    sp3c_7 not in (1,5),
                    sp3c_8 not in (1,5),
                    sp3c_9 not in (1,5),
                    sp3c_10 not in (1,5),
                    sp3c_11 not in (1,5),
                    sp3c_12 not in (1,5),
                    sp3c_13 not in (1,5))=13)=3 then sp3rsk2=5;
         else do;
            if sum(sum(sp3a_1=5,sp3a_2=5,sp3a_3=5,
                       sp3a_4=5,sp3a_5=5,sp3a_6=5,
                       sp3a_7=5,sp3a_8=5,sp3a_9=5,
                       sp3a_10=5,sp3a_11=5,sp3a_12=5,
                       sp3a_13=5)>0,
                   sum(sp3b_1=5,sp3b_2=5,sp3b_3=5,
                       sp3b_4=5,sp3b_5=5,sp3b_6=5,
                       sp3b_7=5,sp3b_8=5,sp3b_9=5,
                       sp3b_10=5,sp3b_11=5,sp3b_12=5,
                       sp3b_13=5)>0,
                   sum(sp3c_1=5,sp3c_2=5,sp3c_3=5,
                       sp3c_4=5,sp3c_5=5,sp3c_6=5,
                       sp3c_7=5,sp3c_8=5,sp3c_9=5,
                       sp3c_10=5,sp3c_11=5,sp3c_12=5,
                       sp3c_13=5)) then sp3rsk2=1;
            else sp3rsk2=5;
         end;

         if sp3rsk2=5 then do;
            array skp2 sp3rc sp3rd sp3re sp3rf sp3rdx;
            do over skp2;
              skp2=.S;
            end;
         end;

         if sp3rsk2=5 then go to skipout;

      **************************************;
      * Criterion B -- Skip in this module *;
      **************************************;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************;
            * phobic stimulus invariable provokes anxiety *;
            ***********************************************;

               if sp6 not in (1,5) then sp3rc=.;
               else sp3rc=(4*(sp6=5))+1;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************************************;
            * phobic situations avoided or indured w/stress *;
            *************************************************;

               if sp5 not in (1,5) then sp3rd=.;
               else sp3rd=(4*(sp5=5))+1;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************;
            * avoidant behavior interferes w/ functioning *;
            ***********************************************;

               if sum(sp7 not in (1,5),
                      sp7a not in (1,5),
                      sp7b not in (1,5),
                      sp7c not in (1,5),
                      sp7d not in (1,5))=5 then sp3re=.;
               else sp3re=(4*(sum(sp7=5,sp7a=5,sp7b=5,sp7c=5,sp7d=5)>0))+1;

      ***************;
      * Criterion F *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * fear recognized as unreasonable *;
            ***********************************;

               if sum(sp3c_1 not in (1,5),
                      sp3c_2 not in (1,5),
                      sp3c_3 not in (1,5),
                      sp3c_4 not in (1,5),
                      sp3c_5 not in (1,5),
                      sp3c_6 not in (1,5),
                      sp3c_7 not in (1,5),
                      sp3c_8 not in (1,5),
                      sp3c_9 not in (1,5),
                      sp3c_10 not in (1,5),
                      sp3c_11 not in (1,5),
                      sp3c_12 not in (1,5),
                      sp3c_13 not in (1,5))=13 then sp3rf=.;
               else sp3rf=(4*(sum(sp3c_1=5,sp3c_2=5,sp3c_3=5,
                                  sp3c_4=5,sp3c_5=5,sp3c_6=5,
                                  sp3c_7=5,sp3c_8=5,sp3c_9=5,
                                  sp3c_10=5,sp3c_11=5,sp3c_12=5,
                                  sp3c_13=5)>0))+1;

      *******************************;
      * Criterion G -- Not assessed *;
      *******************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(sp3ra,sp3rc,sp3rd,sp3re,sp3rf)=0 then sp3rdx=.;
            else do;
               sp3rdx=1;
               if (sp3rc=5 and sp3rd=5 and sp3re=5 and sp3rf=5) then 
                  sp3rdx=sp3ra;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         sp3rsk=max(sp3rsk1,sp3rsk2);

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label sp3rsk1 = "DSM3R SOCPHB Skip Out #1"
               sp3rsk2 = "DSM3R SOCPHB Skip Out #2"
               sp3rsk  = "DSM3R SOCPHB Skip Out"
               sp3ra   = "DSM3R SOCPHB (Lifetime) Crit. A"
               sp3rc   = "DSM3R SOCPHB (Lifetime) Crit. C"
               sp3rd   = "DSM3R SOCPHB (Lifetime) Crit. D"
               sp3re   = "DSM3R SOCPHB (Lifetime) Crit. E"
               sp3rf   = "DSM3R SOCPHB (Lifetime) Crit. F"
               sp3rdx  = "DSM3R SOCPHB (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    sp3rsk1 sp3rsk2 sp3rsk
                    sp3ra sp3rc sp3rd sp3re sp3rf sp3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_soc_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(soc3r,sp2 sp3rsk1);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(soc3r,sp3rsk1 sp2 sp3ra);

               ************;
               * skip out *;
               ************;

                  %multi(soc3r,sp3rsk1 
                               sp3a_1-sp3a_13 sp3b_1-sp3b_13 sp3c_1-sp3c_13
                               sp3rsk2);
                  %multi(soc3r,sp3rsk1 sp3rsk2 sp3rsk);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(soc3r,sp3rsk1 sp3rsk2 sp6 sp3rc);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(soc3r,sp3rsk1 sp3rsk2 sp5 sp3rd);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(soc3r,sp3rsk1 sp3rsk2 
                                  sp7 sp7a sp7b sp7c sp7d sp3re);

               ***************;
               * Criterion F *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(soc3r,sp3rsk1 sp3rsk2 sp3c_1-sp3c_13 sp3rf);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(soc3r,sp3rsk1 sp3rsk2 
                                  sp3ra sp3rc sp3rd sp3re sp3rf sp3rdx);

            ******************;
            * keep variables *;
            ******************;

               data soc3r;
               set soc3r;
               keep ind_id 
                    sp3rsk1 sp3rsk2 sp3rsk
                    sp3ra sp3rc sp3rd sp3re sp3rf sp3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=soc3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend soc3r;

********;
* test *;
********;
/*
   %soc3r(coga.csaga2,1);
*/
