**********************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-III-R - Somatization Disorder *;
***********************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %som3r(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro som3r(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SOM3R: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SOM3R: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data som3r;
      set &dsn.;

      attrib sm3rsk
             sm3ra 
             sm3rb3 sm3rb01-sm3rb06 sm3rb07-sm3rb11 sm3rb12-sm3rb15
             sm3rb16-sm3rb27 sm3rb32-sm3rb34 sm3rb3x sm3rb4x sm3rb5x sm3rbx 
             sm3rb sm3rdx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas sm3rsk sm3ra 
                      sm3rb3 sm3rb01-sm3rb06 sm3rb07-sm3rb11 sm3rb12-sm3rb15
                      sm3rb16-sm3rb27 sm3rb32-sm3rb34 sm3rb3x sm3rb4x sm3rb5x 
                      sm3rbx sm3rb sm3rdx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to skipout;

      ******************************;
      * rescale somatization items *;
      ******************************;

         array itm sm1 sm2 sm3 sm4a sm4b sm4c sm4d sm4e
                   sm5 sm6a sm6b sm6c sm6d sm6e sm7 
                   sm8a sm8b sm8c sm9 sm10 sm11a sm11b
                   sm11c sm11d sm11e sm11f sm11g sm11h
                   sm11i sm11j sm12a sm12b sm12c;
         do over itm;
            if itm<1 or itm>5 then itm=.;
            if itm=2 then itm=1;
         end;

      *********************;
      * test for skip out *;
      *********************;

         if sum(sum(sm4a not in (1,3,4,5),
                    sm4b not in (1,3,4,5),
                    sm4c not in (1,3,4,5),
                    sm4d not in (1,3,4,5),
                    sm4e not in (1,3,4,5))=5,
                sum(sm6a not in (1,3,4,5),
                    sm6b not in (1,3,4,5),
                    sm6c not in (1,3,4,5),
                    sm6d not in (1,3,4,5),
                    sm6e not in (1,3,4,5))=5)=2 then sm3rsk=5;
         else do;
            if sum(max(sm4a,sm4b,sm4c,sm4d,sm4e)=5,
                   sum(sm6a=5,sm6b=5,sm6c=5,sm6d=5,sm6e=5)>3)=2 then 
               sm3rsk=1;
            else sm3rsk=5;
         end;

         if sm3rsk=5 then do;
            array skp sm3ra 
                      sm3rb3 sm3rb01-sm3rb06 sm3rb07-sm3rb11 sm3rb12-sm3rb15
                      sm3rb16-sm3rb27 sm3rb32-sm3rb34 sm3rb3x sm3rb4x sm3rb5x 
                      sm3rbx sm3rb sm3rdx;
            do over skp;
               skp=.S;
            end;
         end;

         if sm3rsk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         sm3ra=sm1;

      ***************;
      * Criterion B *;
      ***************;

         *****************************************;
         * sought treatment or altered lifestyle *;
         *****************************************;

            sm3rb3=sm2;

         ************;
         * symptoms *;
         ************;

            if n(sm3,sm4e,sm4a,sm4b,sm4c,sm4d,
                 sm5,sm6a,sm6d,sm6b,sm6e,
                 sm7,sm8a,sm8b,sm8c, 
                 sm9,sm10,sm11a,sm11b,sm11c,sm11d,sm11e,
                 sm11f,sm11g,sm11h,sm11i,sm11j,
                 sm12a,sm12b,sm12c)=0 then do;
               sm3rb3x=.;
               sm3rb4x=.;
               sm3rb5x=.;
               sm3rbx=.;
            end;
            else do;
               sm3rb3x=0;
               sm3rb4x=0;
               sm3rb5x=0;
               array sxa sm3rb01-sm3rb06
                         sm3rb07-sm3rb11
                         sm3rb12-sm3rb15
                         sm3rb16-sm3rb27
                         sm3rb32-sm3rb34;
               array sxb sm3 sm4e sm4a sm4b sm4c sm4d
                         sm5 sm6a sm6d sm6b sm6e 
                         sm7 sm8a sm8b sm8c 
                         sm9 sm10 sm11a sm11b sm11c sm11d sm11e 
                         sm11f sm11g sm11h sm11i sm11j
                         sm12a sm12b sm12c;
               do over sxa;
                  sxa=sxb;
                  if sxb=3 then sm3rb3x=sm3rb3x+1;
                  if sxb=4 then sm3rb4x=sm3rb4x+1;
                  if sxb=5 then sm3rb5x=sm3rb5x+1;
               end;
               sm3rbx=sum(sm3rb3x,sm3rb4x,sm3rb5x);
            end;

         **********************************************;
         * NOTE:  5 of the 35 sxs listed in DSM3R are *;
         *        not assessed on the CSSAGA-II.  The *;
         *        number of sxs to meet criteria is   *;
         *        adjusted from 13 to 11 accordingly. *;
         **********************************************;

            if n(sm3rb3,sm3rb3x,sm3rb4x,sm3rb5x)=0 then sm3rb=.;
            else do;
               sm3rb=1;
               if sm3rb3 in (3,5) and sm3rb3x>10 then sm3rb=3;
               if sm3rb3 in (4,5) and sm3rb4x>10 then sm3rb=4;
               if sm3rb3=5 and sm3rb5x>10 then sm3rb=5;
            end;

      *************;
      * Diagnosis *;
      *************;

         if n(sm3ra,sm3rb)=0 then sm3rdx=.;
         else do;
            sm3rdx=1;
            if sm3ra in (3,5) and sm3rb=3 then sm3rdx=3;
            if sm3ra in (4,5) and sm3rb=4 then sm3rdx=4;
            if sm3ra=5 and sm3rb=5 then sm3rdx=5;
         end;

      *************************;
      * jump here if skip out *;
      *************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label sm3rsk  = "DSM3R SOM Skip Out"
               sm3ra   = "DSM3R SOM (Lifetime) Crit. A"
               sm3rb3  = "DSM3R SOM (Lifetime) Crit. B"
               sm3rb01 = "DSM3R SOM (Lifetime) SX 1"
               sm3rb02 = "DSM3R SOM (Lifetime) SX 2"
               sm3rb03 = "DSM3R SOM (Lifetime) SX 3"
               sm3rb04 = "DSM3R SOM (Lifetime) SX 4"
               sm3rb05 = "DSM3R SOM (Lifetime) SX 5"
               sm3rb06 = "DSM3R SOM (Lifetime) SX 6"
               sm3rb07 = "DSM3R SOM (Lifetime) SX 7"
               sm3rb08 = "DSM3R SOM (Lifetime) SX 8"
               sm3rb09 = "DSM3R SOM (Lifetime) SX 9"
               sm3rb10 = "DSM3R SOM (Lifetime) SX 10"
               sm3rb11 = "DSM3R SOM (Lifetime) SX 11"
               sm3rb12 = "DSM3R SOM (Lifetime) SX 12"
               sm3rb13 = "DSM3R SOM (Lifetime) SX 13"
               sm3rb14 = "DSM3R SOM (Lifetime) SX 14"
               sm3rb15 = "DSM3R SOM (Lifetime) SX 15"
               sm3rb16 = "DSM3R SOM (Lifetime) SX 16"
               sm3rb17 = "DSM3R SOM (Lifetime) SX 17"
               sm3rb18 = "DSM3R SOM (Lifetime) SX 18"
               sm3rb19 = "DSM3R SOM (Lifetime) SX 19"
               sm3rb20 = "DSM3R SOM (Lifetime) SX 20"
               sm3rb21 = "DSM3R SOM (Lifetime) SX 21"
               sm3rb22 = "DSM3R SOM (Lifetime) SX 22"
               sm3rb23 = "DSM3R SOM (Lifetime) SX 23"
               sm3rb24 = "DSM3R SOM (Lifetime) SX 24"
               sm3rb25 = "DSM3R SOM (Lifetime) SX 25"
               sm3rb26 = "DSM3R SOM (Lifetime) SX 26"
               sm3rb27 = "DSM3R SOM (Lifetime) SX 27"
               sm3rb32 = "DSM3R SOM (Lifetime) SX 32"
               sm3rb33 = "DSM3R SOM (Lifetime) SX 33"
               sm3rb34 = "DSM3R SOM (Lifetime) SX 34"
               sm3rb3x = "DSM3R SOM (LT) Sum SXs=3"
               sm3rb4x = "DSM3R SOM (LT) Sum SXs=4"
               sm3rb5x = "DSM3R SOM (LT) Sum SXs=5"
               sm3rbx  = "DSM3R SOM (LT) Sum SXs=3,4,5"
               sm3rb   = "DSM3R SOM (Lifetime) Crit. B"
               sm3rdx  = "DSM3R SOM (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    sm3rsk
                    sm3ra 
                    sm3rb3 sm3rb01-sm3rb06 sm3rb07-sm3rb11 sm3rb12-sm3rb15
                    sm3rb16-sm3rb27 sm3rb32-sm3rb34 sm3rb3x sm3rb4x sm3rb5x 
                    sm3rbx sm3rb sm3rdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm3r_som_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(som3r,sm4a sm4b sm4c sm4d sm4e
                               sm6a sm6b sm6c sm6d sm6e sm3rsk);

               ***************;
               * Criterion A *;
               ***************;

                  %multi(som3r,sm3rsk sm1 sm3ra);

               ***************;
               * Criterion B *;
               ***************;

                  %multi(som3r,sm3rsk sm2 sm3rb3);
                  %multi(som3r,sm3rsk sm3 sm3rb01);
                  %multi(som3r,sm3rsk sm4e sm3rb02);
                  %multi(som3r,sm3rsk sm4a sm3rb03);
                  %multi(som3r,sm3rsk sm4b sm3rb04);
                  %multi(som3r,sm3rsk sm4c sm3rb05);
                  %multi(som3r,sm3rsk sm4d sm3rb06);
                  %multi(som3r,sm3rsk sm5 sm3rb07);
                  %multi(som3r,sm3rsk sm6a sm3rb08);
                  %multi(som3r,sm3rsk sm6d sm3rb09);
                  %multi(som3r,sm3rsk sm6b sm3rb10);
                  %multi(som3r,sm3rsk sm6e sm3rb11);
                  %multi(som3r,sm3rsk sm7 sm3rb12);
                  %multi(som3r,sm3rsk sm8a sm3rb13);
                  %multi(som3r,sm3rsk sm8b sm3rb14);
                  %multi(som3r,sm3rsk sm8c sm3rb15);
                  %multi(som3r,sm3rsk sm9 sm3rb16);
                  %multi(som3r,sm3rsk sm10 sm3rb17);
                  %multi(som3r,sm3rsk sm11a sm3rb18); 
                  %multi(som3r,sm3rsk sm11b sm3rb19);
                  %multi(som3r,sm3rsk sm11c sm3rb20);
                  %multi(som3r,sm3rsk sm11d sm3rb21);
                  %multi(som3r,sm3rsk sm11e sm3rb22);
                  %multi(som3r,sm3rsk sm11f sm3rb23);
                  %multi(som3r,sm3rsk sm11g sm3rb24);
                  %multi(som3r,sm3rsk sm11h sm3rb25);
                  %multi(som3r,sm3rsk sm11i sm3rb26);
                  %multi(som3r,sm3rsk sm11j sm3rb27);
                  %multi(som3r,sm3rsk sm12a sm3rb32);
                  %multi(som3r,sm3rsk sm12b sm3rb33);
                  %multi(som3r,sm3rsk sm12c sm3rb34);
                  %multi(som3r,sm3rsk sm3rb01-sm3rb06 sm3rb07-sm3rb11 
                               sm3rb12-sm3rb15 sm3rb16-sm3rb27 
                               sm3rb32-sm3rb34 sm3rb3x sm3rb4x sm3rb5x);
                  %multi(som3r,sm3rsk sm3rb3x sm3rb4x sm3rb5x sm3rbx);
                  %multi(som3r,sm3rsk sm3rb3 sm3rb3x sm3rb4x sm3rb5x sm3rb);

               *************;
               * Diagnosis *;
               *************;

                  %multi(som3r,sm3rsk sm3ra sm3rb sm3rdx);

            ******************;
            * keep variables *;
            ******************;

               data som3r;
               set som3r;
               keep ind_id  
                    sm3rsk
                    sm3ra 
                    sm3rb3 sm3rb01-sm3rb06 sm3rb07-sm3rb11 sm3rb12-sm3rb15
                    sm3rb16-sm3rb27 sm3rb32-sm3rb34 sm3rb3x sm3rb4x sm3rb5x 
                    sm3rbx sm3rb sm3rdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=som3r;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend som3r;

********;
* test *;
********;
/*
   %som3r(coga.csaga2,1);
*/
