********************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Attention Deficit/     *;
*                                           Hyperactivity Disorder *;
********************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %addd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro addd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ADDD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ADDD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data addd4;
      set &dsn.;

      attrib atd4a1a atd4a1b atd4a1c atd4a1d atd4a1e 
             atd4a1f atd4a1g atd4a1h atd4a1i atd4a1x 
             atd4a2a atd4a2b atd4a2c atd4a2d atd4a2e 
             atd4a2f atd4a2g atd4a2h atd4a2i atd4a2x 
             atd4a1 atd4a2 atd4a atd4a3 atd4b atd4c 
             atd4d atd4dx atd4sk length=3;

      ***************;
      * Criterion A *;
      ***************;

         ****************;
         * Criterion A1 *;
         ****************;

            *****************;
            * lifetime/ever *;
            *****************;

               *****************************************************;
               * often poor attention to details/careless mistakes *;
               *****************************************************;

                  if sum(at1a not in (1,5),
                         at1b not in (1,5))=2 then atd4a1a=.;
                  else atd4a1a=(at1a=5 or at1b=5);

               ***********************************;
               * difficulty sustaining attention *;
               ***********************************;

                  if sum(at2a not in (1,5),
                         at2b not in (1,5))=2 then atd4a1b=.;
                  else atd4a1b=(at2a=5 or at2b=5);

               *****************************;
               * often seems not to listen *;
               *****************************;

                  if at3 not in (1,5) then atd4a1c=.;
                  else atd4a1c=(at3=5);

               ************************************************;
               * difficulty following through on instructions *;
               ************************************************;

                  if at4 not in (1,5) then atd4a1d=.;
                  else atd4a1d=(at4=5);

               **********************************************;
               * difficulty organizing tasks and activities *;
               **********************************************;

                  if at5 not in (1,5) then atd4a1e=.;
                  else atd4a1e=(at5=5);

               ***********************************************************;
               * often avoids tasks that require sustained mental effort *;
               ***********************************************************;

                  if at6 not in (1,5) then atd4a1f=.;
                  else atd4a1f=(at6=5);

               *****************************************************;
               * often loses things necessary for tasks/activities *;
               *****************************************************;

                  if at7 not in (1,5) then atd4a1g=.;
                  else atd4a1g=(at7=5); 

               *******************************************;
               * easily distracted by extraneous stimuli *;
               *******************************************;

                  if at8 not in (1,5) then atd4a1h=.;
                  else atd4a1h=(at8=5);

               ***************************************;
               * often forgetful in daily activities *;
               ***************************************;

                  if at9 not in (1,5) then atd4a1i=.;
                  else atd4a1i=(at9=5);

               ****************;
               * sum of items *;
               ****************;

                  if n(atd4a1a,atd4a1b,atd4a1c,atd4a1d,atd4a1e, 
                       atd4a1f,atd4a1g,atd4a1h,atd4a1i)=0 then atd4a1x=.;
                  else atd4a1x=sum(atd4a1a,atd4a1b,atd4a1c,atd4a1d,
                                   atd4a1e,atd4a1f,atd4a1g,atd4a1h,
                                   atd4a1i);

               **********************;
               * at least 6 present *;
               **********************;

                  if atd4a1x=. then atd4a1=.;
                  else atd4a1=(atd4a1x>5);

         ****************;
         * Criterion A2 *;
         ****************;

            *****************;
            * lifetime/ever *;
            *****************;

               *************************;
               * often fidgets/squirms *;
               *************************;

                  if at15 not in (1,5) then atd4a2a=.;
                  else atd4a2a=(at15=5);

               *******************************;
               * difficulty remaining seated *;
               *******************************;

                  if at16 not in (1,5) then atd4a2b=.;
                  else atd4a2b=(at16=5);

               *********************************************;
               * often runs/climbs about or feels restless *;
               *********************************************;

                  if at17 not in (1,5) then atd4a2c=.;
                  else atd4a2c=(at17=5);

               ******************************;
               * difficulty playing queitly *;
               ******************************;

                  if at18 not in (1,5) then atd4a2d=.;
                  else atd4a2d=(at18=5);

               ***********************************;
               * often appears driven by a motor *;
               ***********************************;

                  if at19 not in (1,5) then atd4a2e=.;
                  else atd4a2e=(at19=5);

               ***************************;
               * often talks excessively *;
               ***************************;

                  if at20 not in (1,5) then atd4a2f=.;
                  else atd4a2f=(at20=5);

               ************************************************;
               * blurts out answers before question completed *;
               ************************************************;

                  if at11 not in (1,5) then atd4a2g=.;
                  else atd4a2g=(at11=5);

               ****************************;
               * difficulty awaiting turn *;
               ****************************;

                  if at12 not in (1,5) then atd4a2h=.;
                  else atd4a2h=(at12=5);

               *******************************;
               * often interupts or intrudes *;
               *******************************;

                  if at13 not in (1,5) then atd4a2i=.;
                  else atd4a2i=(at13=5);

               ****************;
               * sum of items *;
               ****************;

                  if n(atd4a2a,atd4a2b,atd4a2c,atd4a2d,atd4a2e, 
                       atd4a2f,atd4a2g,atd4a2h,atd4a2i)=0 then atd4a2x=.;
                  else atd4a2x=sum(atd4a2a,atd4a2b,atd4a2c,atd4a2d,
                                   atd4a2e,atd4a2f,atd4a2g,atd4a2h,
                                   atd4a2i);

               **********************;
               * at least 6 present *;
               **********************;

                  if atd4a2x=. then atd4a2=.;
                  else atd4a2=(atd4a2x>5);

         *********************;
         * test for skip out *;
         *********************;

            if sum(atd4a1x<0,atd4a2x<0,
                   at10 not in (1,5), 
                   at14 not in (1,5))=4 then atd4sk=1;
            else do;
               if sum(atd4a1x,atd4a2x,at10=5,at14=5)>3 then atd4sk=0;
               else atd4sk=1;
            end;

            if atd4sk=1 then do;
               array skp atd4a atd4a3 atd4b atd4c atd4d atd4dx;
               do over skp;
                  skp=.S;
               end;
            end;

            if atd4sk=1 then go to skipout;

         ************;
         * A1 or A2 *;
         ************;

            *****************;
            * lifetime/ever *;
            *****************;

               if n(atd4a1,atd4a2)=0 then atd4a=.;
               else atd4a=max(atd4a1,atd4a2)>0;
        
         ****************;
         * Criterion A3 *;
         ****************;

            *****************;
            * lifetime/ever *;
            *****************;

               if atd4a=. then atd4a3=.;
               else do;
                  if (at21c not in (1,5)) then atd4a3=.;
                  else atd4a3=(at21c=5);
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if at21a_1<0 then atd4b=.;
            else atd4b=(0<at21a_1<7);

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(at22a_1a not in (1,5),
                   at22a_2a not in (1,5),
                   at22a_3a not in (1,5),
                   at22a_4a not in (1,5),
                   at22a_5a not in (1,5))=5 then atd4c=.;
            else atd4c=sum((at22a_1a=5 or at22a_2a=5),
                           at22a_3a=5,
                           (at22a_4a=5 or at22a_5a=5))>1;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(at22a_1b not in (1,5),
                   at22a_2b not in (1,5),
                   at22a_3b not in (1,5),
                   at22a_4b not in (1,5),
                   at22a_5b not in (1,5))=5 then atd4d=.;
            else do;
               sum2s=0;
               sum3s=0;
               array one at22a_1b at22a_2b at22a_3b at22a_4b at22a_5b;
               do over one;
                  if one=3 then sum3s+1;
                  if one=2 then sum2s+1;
               end;
               atd4d=(sum3s>0 or sum2s>1);
            end;

      **************************************;
      * Criterion E -- Skip in this module *;
      **************************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(atd4a,atd4a3,atd4b,atd4c,atd4d)=0 then atd4dx=.;
            else atd4dx=sum(atd4a,atd4a3,atd4b,atd4c,atd4d)=5;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         array two atd4a1a atd4a1b atd4a1c atd4a1d atd4a1e 
                   atd4a1f atd4a1g atd4a1h atd4a1i 
                   atd4a2a atd4a2b atd4a2c atd4a2d atd4a2e 
                   atd4a2f atd4a2g atd4a2h atd4a2i 
                   atd4a1 atd4a2 atd4a atd4a3 atd4b atd4c 
                   atd4d atd4dx atd4sk;
         do over two;
            if two=1 then two=5;
            if two=0 then two=1;
         end;

      *******************;
      * variable labels *;
      *******************;

         label atd4sk  = "DSM4 ADHD Skip Out"
               atd4a1a = "DSM4 ADHD (Lifetime) Crit. A1a"
               atd4a1b = "DSM4 ADHD (Lifetime) Crit. A1b"
               atd4a1c = "DSM4 ADHD (Lifetime) Crit. A1c"
               atd4a1d = "DSM4 ADHD (Lifetime) Crit. A1d"
               atd4a1e = "DSM4 ADHD (Lifetime) Crit. A1e"
               atd4a1f = "DSM4 ADHD (Lifetime) Crit. A1f"
               atd4a1g = "DSM4 ADHD (Lifetime) Crit. A1g"
               atd4a1h = "DSM4 ADHD (Lifetime) Crit. A1h"
               atd4a1i = "DSM4 ADHD (Lifetime) Crit. A1i"
               atd4a2a = "DSM4 ADHD (Lifetime) Crit. A2a"
               atd4a2b = "DSM4 ADHD (Lifetime) Crit. A2b"
               atd4a2c = "DSM4 ADHD (Lifetime) Crit. A2c"
               atd4a2d = "DSM4 ADHD (Lifetime) Crit. A2d"
               atd4a2e = "DSM4 ADHD (Lifetime) Crit. A2e"
               atd4a2f = "DSM4 ADHD (Lifetime) Crit. A2f"
               atd4a2g = "DSM4 ADHD (Lifetime) Crit. A2g"
               atd4a2h = "DSM4 ADHD (Lifetime) Crit. A2h"
               atd4a2i = "DSM4 ADHD (Lifetime) Crit. A2i"
               atd4a1x = "DSM4 ADHD (Lifetime) Sum of A1s"
               atd4a2x = "DSM4 ADHD (Lifetime) Sum of A2s"
               atd4a1  = "DSM4 ADHD (Lifetime) Crit. A1"
               atd4a2  = "DSM4 ADHD (Lifetime) Crit. A2"
               atd4a   = "DSM4 ADHD (Lifetime) A1 or A2"
               atd4a3  = "DSM4 ADHD (Lifetime) Crit. A3"
               atd4b   = "DSM4 ADHD (Lifetime) Crit. B"
               atd4c   = "DSM4 ADHD (Lifetime) Crit. C"
               atd4d   = "DSM4 ADHD (Lifetime) Crit. D"
               atd4dx  = "DSM4 ADHD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id atd4sk
                    atd4a1a atd4a1b atd4a1c atd4a1d atd4a1e 
                    atd4a1f atd4a1g atd4a1h atd4a1i atd4a1x 
                    atd4a2a atd4a2b atd4a2c atd4a2d atd4a2e 
                    atd4a2f atd4a2g atd4a2h atd4a2i atd4a2x 
                    atd4a1 atd4a2 atd4a atd4a3 atd4b atd4c 
                    atd4d atd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_add_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  ****************;
                  * Criterion A1 *;
                  ****************;

                     *****************;
                     * lifetime/ever *;
                     *****************;

                        %multi(addd4,at1a at1b atd4a1a);
                        %multi(addd4,at2a at2b atd4a1b);
                        %multi(addd4,at3 atd4a1c);
                        %multi(addd4,at4 atd4a1d);
                        %multi(addd4,at5 atd4a1e);
                        %multi(addd4,at6 atd4a1f);
                        %multi(addd4,at7 atd4a1g);
                        %multi(addd4,at8 atd4a1h);
                        %multi(addd4,at9 atd4a1i);
                        %multi(addd4,atd4a1a atd4a1b atd4a1c atd4a1d 
                                     atd4a1e atd4a1f atd4a1g atd4a1h 
                                     atd4a1i atd4a1x);
                        %multi(addd4,atd4a1x atd4a1);

                  ****************;
                  * Criterion A2 *;
                  ****************;

                     *****************;
                     * lifetime/ever *;
                     *****************;

                        %multi(addd4,at15 atd4a2a);
                        %multi(addd4,at16 atd4a2b);
                        %multi(addd4,at17 atd4a2c);
                        %multi(addd4,at18 atd4a2d);
                        %multi(addd4,at19 atd4a2e);
                        %multi(addd4,at20 atd4a2f);
                        %multi(addd4,at11 atd4a2g);
                        %multi(addd4,at12 atd4a2h);
                        %multi(addd4,at13 atd4a2i);
                        %multi(addd4,atd4a2a atd4a2b atd4a2c atd4a2d 
                                     atd4a2e atd4a2f atd4a2g atd4a2h 
                                     atd4a2i atd4a2x);
                        %multi(addd4,atd4a2x atd4a2);

                  ************;
                  * skip out *;
                  ************;

                     %multi(addd4,atd4a1x atd4a2x at10 at14 atd4sk);

                  ************;
                  * A1 or A2 *;
                  ************;

                     *****************;
                     * lifetime/ever *;
                     *****************;

                        %multi(addd4,atd4sk atd4a1 atd4a2 atd4a);

                  ****************;
                  * Criterion A3 *;
                  ****************;

                     *****************;
                     * lifetime/ever *;
                     *****************;

                        %multi(addd4,atd4sk at21c atd4a3);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addd4,atd4sk at21a_1 atd4b);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addd4,atd4sk at22a_1a at22a_2a at22a_3a 
                                  at22a_4a at22a_5a atd4c);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addd4,atd4sk at22a_1b at22a_2b at22a_3b 
                                  at22a_4b at22a_5b atd4d);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addd4,atd4sk atd4a atd4a3 atd4b atd4c atd4d atd4dx);

            ******************;
            * keep variables *;
            ******************;

               data addd4;
               set addd4;
               keep ind_id atd4sk 
                    atd4a1a atd4a1b atd4a1c atd4a1d atd4a1e 
                    atd4a1f atd4a1g atd4a1h atd4a1i atd4a1x 
                    atd4a2a atd4a2b atd4a2c atd4a2d atd4a2e 
                    atd4a2f atd4a2g atd4a2h atd4a2i atd4a2x 
                    atd4a1 atd4a2 atd4a atd4a3 atd4b atd4c 
                    atd4d atd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=addd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend addd4;

********;
* test *;
********;
/*
   %addd4(coga.csaga2,1);
*/
