**************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Alcohol Abuse and Dependence *;
**************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %alcd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro alcd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ALCD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ALCD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data alcd4;
      set &dsn.;

      attrib al4da1-al4da7 al4dax al4da al4db ald4dp
             al4aa1-al4aa4 al4aax al4aa al4ab ald4ab ald4sk length=3;

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(al1a not in (1,5),
                   al1f not in (1,5),
                   al1g not in (1,5),
                   al11a<0)=4 then ald4sk=5;
            else do;
               if (al1a=5 and (al1f=5 or al1g=5) and al11a>2) then ald4sk=1;
               else ald4sk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(al1c not in (1,5),
                   al1f not in (1,5),
                   al11a<0)=3 then ald4sk=5;
            else do;
               if ((al1c=5 or al1f=5) and al11a>2) then ald4sk=1;
               else ald4sk=5;
            end;
         %end;

         if ald4sk=5 then do;
             array skp al4da1-al4da7 al4dax al4da al4db ald4dp
                       al4aa1-al4aa4 al4aax al4aa al4ab ald4ab;
             do over skp;
                skp=.S;
             end;
         end;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &study.=coga %then %do;
            %if &who.=p %then %do;
               if ald4sk=1 then do;
                  array nas al4da1-al4da5 al4da7 al4dax al4da al4db ald4dp;
                  do over nas;
                     nas=.N;
                  end;
               end;
               if ald4sk=5 then go to skipout;
               else go to par_hop1;
            %end;
         %end;

      if ald4sk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            *************;
            * tolerance *;
            *************;

               if sum(al12e not in (1,5),
                      al12f not in (1,5))=2 then al4da1=.;
               else al4da1=(4*(al12e=5 or al12f=5))+1;

            **************;
            * withdrawal *;
            **************;

               if 0<dm3a<13 then do;
                  al4da2=.N;
               end;
               else do;
                  if sum(sum(al46a_3 not in (1,5),al46b_3 not in (1,5),
                             al46d_3 not in (1,5),al46e_3 not in (1,5),
                             al46f_3 not in (1,5),al46g_3 not in (1,5),
                             al46j_3 not in (1,5),al46k_3 not in (1,5),
                             al46l_3 not in (1,5))=9,
                         al46t not in (1,5))=2 then al4da2=.;
                  else al4da2=(4*(sum(al46a_3=5,al46b_3=5,al46d_3=5,al46e_3=5,
                                      al46f_3=5,al46g_3=5,al46j_3=5,al46k_3=5,
                                      al46l_3=5)>1 or al46t=5))+1;
               end;

            *********************************************************;
            * used larger amount or for longer period than intended *;
            *********************************************************;

               if sum(al15c not in (1,5),
                      al16c not in (1,5))=2 then al4da3=.;
               else al4da3=(4*(al15c=5 or al16c=5))+1;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            ***************************************************************;

               if sum(al13a not in (1,5),
                      al13c_2 not in (1,5))=2 then al4da4=.;
               else al4da4=(4*(al13a=5 or al13c_2=5))+1;

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            ****************************************************************;

               if sum(al18b not in (1,5),
                      al20d not in (1,5))=2 then al4da5=.;
               else al4da5=(4*(al18b=5 or al20d=5))+1;

            **************************************;
            * jump here if coga parent interview *;
            **************************************;

               par_hop1:

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if al19d not in (1,5) then al4da6=.;
               else al4da6=(4*(al19d=5))+1;

            **********************************;
            * test for coga parent interview *;
            **********************************;

               %if &study.=coga %then %do;
                  %if &who.=p %then %do;
                     go to par_hop2;
                  %end;
               %end;

            **********************************************************;
            * use despite use-derived social/pysch/physical problems *;
            **********************************************************;

               if sum(al36a not in (1,5),
                      al37e not in (1,5))=2 then al4da7=.;
               else al4da7=(4*(al36a=5 or al37e=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of al4da1-al4da7)=0 then al4dax=.;
               else do;
                  al4dax=0;
                  array sx1 al4da1-al4da7;
                  do over sx1;
                     if sx1=5 then al4dax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if al4dax=. then al4da=.;
               else al4da=(4*(al4dax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if 0<dm3a<13 then do;
               al4db=.N;
            end;
            else do;
               if al48a not in (1,5) then al4db=.;
               else al4db=(4*(al48a=5))+1;
            end;
                   
         *************;
         * Diagnosis *;
         *************;

            if 0<dm3a<13 then do;
               if al4da=. then ald4dp=.;
               else ald4dp=(4*(al4da=5))+1;
            end;
            else do;
               if n(al4da,al4db)=0 then ald4dp=.;
               else ald4dp=(4*(al4da=5 and al4db=5))+1;
            end;

      **************************************;
      * jump here if coga parent interview *;
      **************************************;

         par_hop2:

      *********;
      * Abuse *;
      *********;

         ***************;
         * Criterion A *;
         ***************;

            ****************************************************;
            * use results in failure of major-role obligations *;
            ****************************************************;

               if sum(al20e not in (1,5),
                      al31g_1 not in (1,5))=2 then al4aa1=.;
               else al4aa1=(4*(al20e=5 or al31g_1=5))+1;

            ****************************************************;
            * recurrent use in physically dangerous situations *;
            ****************************************************;

               if sum(al25d not in (1,5),
                      al26f_1 not in (1,5),
                      al26i_1 not in (1,5),
                      al35d not in (1,5))=4 then al4aa2=.;
               else al4aa2=(4*(sum(al25d=5,al26f_1=5,al26i_1=5,al35d=5)>0))+1;

            ****************************************;
            * recurrent use-related legal problems *;
            ****************************************;

               if 0<dm3a<13 then do;
                  al4aa3=.N;
               end;
               else do;
                  if al34c_1 not in (1,5) then al4aa3=.;
                  else al4aa3=(4*(al34c_1=5))+1;
               end;

            *****************************************************;
            * use despite recurrent use-related social problems *;
            *****************************************************;

               %if &who.=p %then %do;
                  al4aa4=.N;
               %end;
               %else %do;
                  if al33i not in (1,5) then al4aa4=.;
                  else al4aa4=(4*(al33i=5))+1;
               %end;

            ****************;
            * sum of items *;
            ****************;

               if n(of al4aa1-al4aa4)=0 then al4aax=.;
               else do;
                  al4aax=0;
                  array sx2 al4aa1-al4aa4;
                  do over sx2;
                     if sx2=5 then al4aax+1;
                  end;
               end;

            *********************;
            * at least one item *;
            *********************;

               if al4aax=. then al4aa=.;
               else al4aa=(4*(al4aax>0))+1;

         ***************;
         * Criterion B *;
         ***************;

            if ald4dp=. then al4ab=.;
            else al4ab=ald4dp;

         *************;
         * Diagnosis *;
         *************;

            if n(al4aa,al4ab)=0 then ald4ab=.;
            %if &who.=p %then %do;
                else ald4ab=(4*(al4aa=5))+1;
            %end;
            %else %do;
               else ald4ab=(4*(al4aa=5 and al4ab=1))+1;
            %end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label ald4sk = "DSM4 ALC Skip Out"
               al4da1 = "DSM4 ALC Dependence Crit. A1"
               al4da2 = "DSM4 ALC Dependence Crit. A2"
               al4da3 = "DSM4 ALC Dependence Crit. A3"
               al4da4 = "DSM4 ALC Dependence Crit. A4" 
               al4da5 = "DSM4 ALC Dependence Crit. A5" 
               al4da6 = "DSM4 ALC Dependence Crit. A6" 
               al4da7 = "DSM4 ALC Dependence Crit. A7" 
               al4dax = "DSM4 ALC Dependence Sum of As" 
               al4da  = "DSM4 ALC Dependence 3+ As" 
               al4db  = "DSM4 ALC Dependence Crit. B" 
               ald4dp = "DSM4 ALC Dependence Diagnosis"
               al4aa1 = "DSM4 ALC Abuse Crit. A1"
               al4aa2 = "DSM4 ALC Abuse Crit. A2"
               al4aa3 = "DSM4 ALC Abuse Crit. A3"
               al4aa4 = "DSM4 ALC Abuse Crit. A4"
               al4aa  = "DSM4 ALC Abuse 1+ As"
               al4ab  = "DSM4 ALC Abuse Crit. B"
               ald4ab = "DSM4 ALC Abuse Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id ald4sk
                    al4da1-al4da7 al4dax al4da al4db ald4dp
                    al4aa1-al4aa4 al4aax al4aa al4ab ald4ab;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_alc_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(alcd4,al1a al1f al1g al11a ald4sk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(alcd4,al1c al1f al11a ald4sk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(alcd4,ald4sk al12e al12f al4da1);
                     %multi(alcd4,ald4sk al46a_3 al46b_3 al46d_3 al46e_3 al46f_3
                                  al46g_3 al46j_3 al46k_3 al46l_3 al46t
                                  al4da2);
                     %multi(alcd4,ald4sk al15c al16c al4da3);
                     %multi(alcd4,ald4sk al13a al13c_2 al4da4);
                     %multi(alcd4,ald4sk al18b al20d al4da5);
                     %multi(alcd4,ald4sk al19d al4da6);
                     %multi(alcd4,ald4sk al36a al37e al4da7);
                     %multi(alcd4,ald4sk al4da1-al4da7 al4dax);
                     %multi(alcd4,ald4sk al4dax al4da);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(alcd4,ald4sk al48a al4db);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(alcd4,ald4sk al4da al4db ald4dp);

               *********;
               * Abuse *;
               *********;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(alcd4,ald4sk al20e al31g_1 al4aa1);
                     %multi(alcd4,ald4sk al25d al26f_1 al26i_1 al35d al4aa2);
                     %multi(alcd4,ald4sk al34c_1 al4aa3);
                     %multi(alcd4,ald4sk al33i al4aa4);
                     %multi(alcd4,ald4sk al4aa1-al4aa4 al4aax);
                     %multi(alcd4,ald4sk al4aax al4aa);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(alcd4,ald4sk ald4dp al4ab);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(alcd4,ald4sk al4aa al4ab ald4ab);

            ******************;
            * keep variables *;
            ******************;

               data alcd4;
               set alcd4;
               keep ind_id ald4sk
                    al4da1-al4da7 al4dax al4da al4db ald4dp
                    al4aa1-al4aa4 al4aax al4aa al4ab ald4ab;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=alcd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend alcd4;

********;
* test *;
********;
/*
   %alcd4(coga.csaga2,1);
*/
