**************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Anorexia Nervosa *;
**************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %anxd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro anxd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ANXD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ANXD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data anxd4;
      set &dsn.;

      attrib and4a and4b and4c and4d and4dx and4sk length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas and4a and4b and4c and4d and4dx and4sk;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to skipout;

      *********************;
      * test for skip out *;
      *********************;

         if sum(an1 not in (1,3,5),
                an4d not in (1,3,5))=2 then and4sk=5;
         else do;
            if (an1 in (3,5) and an4d=5) then and4sk=1;
            else and4sk=5;
         end;

         if and4sk=5 then do;
             array skp and4a and4b and4c and4d and4dx;
             do over skp;
                skp=.S;
             end;
         end;

         if and4sk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ***********************************;
            * refusal to maintain body weight *;
            ***********************************;

               if sum(an1 not in (1,3,5),
                      an4d not in (1,5))=2 then and4a=.;
               else and4a=(4*(sum(an1=5,an4d=5)=2))+
                          (2*(sum(an1=3,an4d=5)=2))+1;

      ***************;
      * Criterion B *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *******************************;
            * intense fear of weight gain *;
            *******************************;

               if an5 not in (1,3,5) then and4b=.;
               else and4b=(4*(an5=5))+(2*(an5=3))+1;

      ***************;
      * Criterion C *;
      ***************;

         ************;
         * lifetime *;
         ************;

            **************************;
            * body image disturbance *;
            **************************;

               if sum(an2 not in (1,3,5),
                      an3 not in (1,3,5))=2 then and4c=.;
               else do;
                  and4c=1;
                  if sum(an2=3,an3=3)>0 then and4c=3;
                  else if sum(an2=3,an3=3)=0 and
                          sum(an2=5,an3=5)>0 then and4c=5;
               end;

      ***************;
      * Criterion D *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ***********************;
            * cessation of menses *;
            ***********************;

               if sum(dm1 not in (1,2),
                      an8 not in (1,5),
                      an9 not in (1,5))=3 then and4d=.;
               else if dm1=2 then and4d=(4*(sum(an8=5,an9=5)=2))+1;
                     
      *************;
      * Diagnosis *;
      *************;

         ************;
         * lifetime *;
         ************;

            if n(and4a,and4b,and4c,and4d)=0 then and4dx=.;
            else do;
               and4dx=1;
               if (sum(and4a in (3,5),and4b in (3,5),and4c in (3,5))=3 and
                   sum(and4a=3,and4b=3,and4c=3) and 
                   sum(dm1=1,dm1=2 and and4d=5)>0) then and4dx=3;
               else if (sum(and4a=3,and4b=3,and4c=3)=0 and
                        sum(and4a=5,and4b=5,and4c=5)=3 and
                        sum(dm1=1,dm1=2 and and4d=5)>0) then and4dx=5;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label and4sk = "DSM4 ANOREX Skip Out"
               and4a  = "DSM4 ANOREX (Lifetime) Crit. A"
               and4b  = "DSM4 ANOREX (Lifetime) Crit. B"
               and4c  = "DSM4 ANOREX (Lifetime) Crit. C"
               and4d  = "DSM4 ANOREX (Lifetime) Crit. D"
               and4dx = "DSM4 ANOREX (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id and4sk and4a and4b and4c and4d and4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_anx_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(anxd4,an1 an4d and4sk);

               ***************;
               * Criterion A *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anxd4,and4sk an1 an4d and4a);

               ***************;
               * Criterion B *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anxd4,and4sk an5 and4b);

               ***************;
               * Criterion C *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anxd4,and4sk an2 an3 and4c);

               ***************;
               * Criterion D *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(anxd4,and4sk dm1 an8 an9 and4d);

               *************;
               * Diagnosis *;
               *************;

                  ************;
                  * lifetime *;
                  ************;
 
                     %multi(anxd4,and4sk and4a and4b and4c and4d and4dx);

            ******************;
            * keep variables *;
            ******************;

               data anxd4;
               set anxd4;
               keep ind_id and4sk and4a and4b and4c and4d and4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=anxd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend anxd4;

********;
* test *;
********;
/*
   %anxd4(coga.csaga2,1);
*/
