*************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Bulimia Nervosa *;
*************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %buld4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro buld4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in BULD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in BULD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data buld4;
      set &dsn.;

      attrib bud4sk1-bud4sk3 bud4sk 
             bud4a1 bud4a2 bud4a bud4b bud4c bud4d bud4e bud4dx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas bud4sk1-bud4sk3 bud4sk 
                      bud4a1 bud4a2 bud4a bud4b bud4c bud4d bud4e bud4dx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      ****************;
      * Criterion A1 *;
      ****************;

         ************;
         * lifetime *;
         ************;

            *********************;
            * recurrent binging *;
            *********************;

               if bu1a not in (1,3,5) then bud4a1=.;
               else bud4a1=(4*(bu1a=5))+(2*(bu1a=3))+1;

      *********************;
      * test for skip out *;
      *********************;

         if sum(bu1a not in (1,3,5),
                bu2b not in (1,5))=2 then bud4sk1=5;
         else do;
            if (bu1a in (3,5) and bu2b ne 1) then bud4sk1=1;
            else bud4sk1=5;
         end;

         if bud4sk1=5 then do;
             array skp1 bud4a2 bud4a bud4b bud4c bud4d bud4e bud4dx;
             do over skp1;
                skp1=.S;
             end;
         end;

         if bud4sk1=5 then go to skipout;

      ***************;
      * Criterion C *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ***********************************************************;
            * at least two binges/purges a week for at least 3 months *;
            ***********************************************************;

               if sum(bu3 not in (1,5),
                      sum(bu2a not in (1,3,5),
                          bu2b not in (1,5))=2)=2 then bud4c=.;
               else do;
                  bud4c=1;
                  if sum(bu3=5,bu2a=3)=2 then bud4c=3;
                  else if sum(bu3=5,sum(bu2a=5,bu2b=5)>0)=2 then bud4c=5;
               end;

      *********************;
      * test for skip out *;
      *********************;

         if (bu3 not in (1,5)) then bud4sk2=5;
         else do;
            if (bu3 ne 1) then bud4sk2=1;
            else bud4sk2=5;
         end;

         if bud4sk2=5 then do;

             array skp2 bud4a2 bud4a bud4b bud4d bud4e bud4dx;
             do over skp2;
                skp2=.S;
             end;

         end;

         if bud4sk2=5 then go to skipout;

      ****************;
      * Criterion A2 *;
      ****************;

         ************;
         * lifetime *;
         ************;

            ******************************************;
            * feeling of lack of control over eating *;
            ******************************************;

               if bu7 not in (1,3,5) then bud4a2=.;
               else bud4a2=(4*(bu7=5))+(2*(bu7=3))+1;

      ***************;
      * Criterion A *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *************;
            * A1 and A2 *;
            *************;

               if n(bud4a1,bud4a2)=0 then bud4a=.;
               else do;
                  bud4a=1;
                  if sum(bud4a1 in (3,5),bud4a2 in (3,5))=2 and
                     sum(bud4a1=3,bud4a2=3) then bud4a=3;
                  else if sum(bud4a1=3,bud4a2=3)=0 and
                          sum(bud4a1=5,bud4a2=5)=2 then bud4a=5;
               end;

      ***************;
      * Criterion B *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *******************************;
            * regular self-induced purges *;
            *******************************;

               if sum(bu5 not in (1,3,5),
                      bu6 not in (1,3,5))=2 then bud4b=.;
               else do;
                  bud4b=1;
                  if sum(bu5=3,bu6=3)>0 then bud4b=3;
                  else if sum(bu5=3,bu6=3)=0 and
                          sum(bu5=5,bu6=5)>0 then bud4b=5;
               end;

      ***************;
      * Criterion D *;
      ***************;

         ************;
         * lifetime *;
         ************;

            *********************************************;
            * persistent concern with body shape/weight *;
            *********************************************;

               if sum(bu4a not in (1,3,5),
                      bu4b not in (1,3,5))=2 then bud4d=.;
               else do;
                  bud4d=1;
                  if sum(bu4a=3,bu4b=3)>0 then bud4d=3;
                  else if sum(bu4a=5,bu4b=5)>0 then bud4d=5;
               end;

      *********************;
      * test for skip out *;
      *********************;

         if (an4d not in (1,5)) then bud4sk3=5;
         else do;
            if (an4d=5) then bud4sk3=1;
            else bud4sk3=5;
         end;

         if bud4sk3=5 then do;

             array skp3 bud4e bud4dx;
             do over skp3;
                skp3=.S;
             end;

         end;

         if bud4sk3=5 then go to skipout;

      ***************;
      * Criterion E *;
      ***************;

         ************;
         * lifetime *;
         ************;

            ************************************************;
            * not exclusive to epiodes of anorexia nervosa *;
            ************************************************;

               if bu9 not in (1,3,5) then bud4e=.;
               else bud4e=(4*(bu9=1))+1;

      *************;
      * Diagnosis *;
      *************;

         ************;
         * lifetime *;
         ************;

            if n(bud4a,bud4b,bud4c,bud4d,bud4e)=0 then bud4dx=.;
            else do;
               bud4dx=1;
               if sum(bud4a in (3,5),bud4b in (3,5),
                      bud4c in (3,5),bud4d in (3,5),bud4e=5)=5 then do;
                  if sum(bud4a=3,bud4b=3,bud4c=3,bud4d=3)>0 then bud4dx=3;
                  else bud4dx=5;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         bud4sk=max(bud4sk1,bud4sk2,bud4sk);

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label bud4sk1 = "DSM4 BULIMIA Skip Out #1"
               bud4sk2 = "DSM4 BULIMIA Skip Out #2"
               bud4sk3 = "DSM4 BULIMIA Skip Out #3"
               bud4sk  = "DSM4 BULIMIA Skip Out"
               bud4a   = "DSM4 BULIMIA (Lifetime) Crit. A"
               bud4b   = "DSM4 BULIMIA (Lifetime) Crit. B"
               bud4c   = "DSM4 BULIMIA (Lifetime) Crit. C"
               bud4d   = "DSM4 BULIMIA (Lifetime) Crit. D"
               bud4e   = "DSM4 BULIMIA (Lifetime) Crit. E"
               bud4dx  = "DSM4 BULIMIA (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id bud4sk1-bud4sk3 bud4sk
                    bud4a1 bud4a2 bud4a bud4b bud4c bud4d bud4e bud4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_bul_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ****************;
               * Criterion A1 *;
               ****************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bu1a bud4a1);

               ************;
               * skip out *;
               ************;

                  %multi(buld4,bu1a bu2a bu2b bud4sk1);

               ***************;
               * Criterion C *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bud4sk1 bu3 bu2a bu2b bud4c);

               ************;
               * skip out *;
               ************;

                  %multi(buld4,bud4sk1 bu3 bud4sk2);

               ****************;
               * Criterion A2 *;
               ****************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bud4sk1 bud4sk2 bu7 bud4a2);

               ***************;
               * Criterion A *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bud4sk1 bud4sk2 bud4a1 bud4a2 bud4a);

               ***************;
               * Criterion B *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bud4sk1 bud4sk2 bu5 bu6 bud4b);


               ***************;
               * Criterion D *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bud4sk1 bud4sk2 bu4a bu4b bud4d);

               ************;
               * skip out *;
               ************;

                  %multi(buld4,bud4sk1 bud4sk2 an4d bud4sk3);

               ***************;
               * Criterion E *;
               ***************;

                  ************;
                  * lifetime *;
                  ************;

                     %multi(buld4,bud4sk1 bud4sk2 bud4sk3 bu9 bud4e);

               *************;
               * Diagnosis *;
               *************;

                  ************;
                  * lifetime *;
                  ************;
 
                     %multi(buld4,bud4sk1 bud4sk2 bud4sk3 
                                  bud4a bud4b bud4c bud4d bud4e bud4dx);

            ******************;
            * keep variables *;
            ******************;

               data buld4;
               set buld4;
               keep ind_id bud4sk1-bud4sk3 bud4sk
                    bud4a1 bud4a2 bud4a bud4b bud4c bud4d bud4e bud4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=buld4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend buld4;

********;
* test *;
********;
/*
   %buld4(coga.csaga2,1);
*/
