***************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Cannabis Abuse and Dependence *;
***************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %cand4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro cand4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in CAND4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in CAND4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data cand4;
      set &dsn.;

      attrib mjd4sk
             mj4da1-mj4da7 mj4dax mj4da mj4db mjd4dp
             mj4aa1-mj4aa4 mj4aax mj4aa mj4ab mjd4ab length=3;

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(mj1a not in (1,5),
                   mj1c not in (1,5),
                   mj7a<0,
                   mj7b not in (0,1,2,3,4,5))=4 then mjd4sk=5;
            else do;
               if (mj1a=5 and mj1c=5 and (mj7a>20 or mj7b in (1,2,3,4,5))) then
                  mjd4sk=1;
               else mjd4sk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(mj1c not in (1,5),
                   mj1c_1 not in (1,5),
                   mj7a<0,
                   mj7b not in (0,1,2,3,4,5))=4 then mjd4sk=5;
            else do;
               if ((mj1c=5 or mj1c_1=5) and 
                   (mj7a>20 or mj7b in (1,2,3,4,5))) then mjd4sk=1;
               else mjd4sk=5;
            end;
         %end;

         if mjd4sk=5 then do;
             array skp mj4da1-mj4da7 mj4dax mj4da mj4db mjd4dp
                       mj4aa1-mj4aa4 mj4aax mj4aa mj4ab mjd4ab;
             do over skp;
                skp=.S;
             end;
         end;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &study.=coga %then %do;
            %if &who.=p %then %do;
               if mjd4sk=1 then do;
                  array nas mj4da1-mj4da5 mj4da7 mj4dax mj4da mj4db mjd4dp;
                  do over nas;
                     nas=.N;
                  end;
               end;
               if mjd4sk=5 then go to skipout;
               else go to par_hop1;
            %end;
         %end;

      if mjd4sk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            ********************;
            * marked tolerance *;
            ********************;

               if sum(mj8a not in (1,5),
                      mj8b not in (1,5))=2 then mj4da1=.;
               else mj4da1=(4*(mj8a=5 or mj8b=5))+1;

            ********************************************;
            * characteristic withdrawal symptoms or    *;
            * use to relieve/avoid withdrawal symptoms *;
            ********************************************;

               if 0<dm3a<13 then do;
                  mj4da2=.N;
               end;
               else do;
                  if sum(mj29c not in (1,5),
                         mj29d not in (1,5))=2 then mj4da2=.;
                  else mj4da2=(4*(mj29c=5 or mj29d=5))+1;
               end;

            *********************************************************;
            * used larger amount or for longer period than intended *;
            *********************************************************;

               if sum(mj10b not in (1,5),
                      mj10d not in (1,5))=2 then mj4da3=.;
               else mj4da3=(4*(mj10b=5 or mj10d=5))+1;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            ***************************************************************;

               if sum(mj9b not in (1,5),
                      mj9d not in (1,5))=2 then mj4da4=.;
               else mj4da4=(4*(mj9b=5 or mj9d=5))+1;

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            ****************************************************************;

               if mj11 not in (1,5) then mj4da5=.;
               else mj4da5=(4*(mj11=5))+1;

            **************************************;
            * jump here if coga parent interview *;
            **************************************;

               par_hop1:

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if mj12c not in (1,5) then mj4da6=.;
               else mj4da6=(4*(mj12c=5))+1;

            **********************************;
            * test for coga parent interview *;
            **********************************;

               %if &study.=coga %then %do;
                  %if &who.=p %then %do;
                     go to par_hop2;
                  %end;
               %end;

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if 0<dm3a<13 then do;
                  mj4da7=.N;
               end;
               else do;
                  if mj24e not in (1,5) then mj4da7=.;
                  else mj4da7=(4*(mj24e=5))+1;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of mj4da1-mj4da7)=0 then mj4dax=.;
               else do;
                  mj4dax=0;
                  array sxs mj4da1-mj4da7;
                  do over sxs;
                     if sxs=5 then mj4dax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if mj4dax=. then mj4da=.;
               else mj4da=(4*(mj4dax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if 0<dm3a<13 then do;
               mj4db=.N;
            end;
            else do;
               if mj31a not in (1,5) then mj4db=.;
               else mj4db=(4*(mj31a=5))+1;
            end;
                                 
         *************;
         * Diagnosis *;
         *************;

            if n(mj4da,mj4db)=0 then mj4dp=.;
            else do;
               if 0<dm3a<13 then mjd4dp=(4*(mj4da=5))+1;
               else mjd4dp=(4*(mj4da=5 and mj4db=5))+1;
            end;

      **************************************;
      * jump here if coga parent interview *;
      **************************************;

         par_hop2:

      *********;
      * Abuse *;
      *********;

         ***************;
         * Criterion A *;
         ***************;

            ********************************************************;
            * use despite use-related failure in major obligations *;
            ********************************************************;

               if mj20g not in (1,5) then mj4aa1=.;
               else mj4aa1=(4*(mj20g=5))+1;

            ****************************************************;
            * recurrent use in physically dangerous situations *;
            ****************************************************;

               if sum(mj15d_1 not in (1,5),
                      mj15f_1 not in (1,5),
                      mj23c not in (1,5))=3 then mj4aa2=.;
               else mj4aa2=(4*(sum(mj15d_1=5,mj15f_1=5,mj23c=5)>0))+1;

            ****************************************;
            * recurrent use-related legal problems *;
            ****************************************;

               if mj22b_1 not in (1,5) then mj4aa3=.;
               else mj4aa3=(4*(mj22b_1=5))+1;

            ****************************************************;
            * use despite use-derived social/personal problems *;
            ****************************************************;

               %if &who.=p %then %do;
                  mj4aa4=.N;
               %end;
               %else %do;
                  if mj21f not in (1,5) then mj4aa4=.;
                  else mj4aa4=(4*(mj21f=5))+1;
               %end;

            ****************;
            * sum of items *;
            ****************;

               if n(of mj4aa1-mj4aa4)=0 then mj4aax=.;
               else do;
                  mj4aax=0;
                  array axs mj4aa1-mj4aa4;
                  do over axs;
                     if axs=5 then mj4aax+1;
                  end;
               end;

            *********************;
            * at least one item *;
            *********************;

               if mj4aax=. then mj4aa=.;
               else mj4aa=(4*(mj4aax>0))+1;

         ***************;
         * Criterion B *;
         ***************;

            if mjd4dp=. then mj4ab=.;
            else mj4ab=mjd4dp;

         *************;
         * Diagnosis *;
         *************;

            if n(mj4aa,mj4ab)=0 then mjd4ab=.;
            %if &who.=p %then %do;
               else mjd4ab=(4*(mj4aa=5))+1;
            %end;
            %else %do;
               else mjd4ab=(4*(mj4aa=5 and mj4ab=1))+1;
            %end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label mjd4sk = "DSM4 CAN Skip Out"
               mj4da1 = "DSM4 CAN Dependence Crit. A1"
               mj4da2 = "DSM4 CAN Dependence Crit. A2"
               mj4da3 = "DSM4 CAN Dependence Crit. A3"
               mj4da4 = "DSM4 CAN Dependence Crit. A4" 
               mj4da5 = "DSM4 CAN Dependence Crit. A5" 
               mj4da6 = "DSM4 CAN Dependence Crit. A6" 
               mj4da7 = "DSM4 CAN Dependence Crit. A7" 
               mj4dax = "DSM4 CAN Dependence Sum of As" 
               mj4da  = "DSM4 CAN Dependence 3+ As" 
               mj4db  = "DSM4 CAN Dependence Crit. B" 
               mjd4dp = "DSM4 CAN Dependence Diagnosis"
               mj4aa1 = "DSM4 CAN Abuse Crit. A1"
               mj4aa2 = "DSM4 CAN Abuse Crit. A2"
               mj4aa3 = "DSM4 CAN Abuse Crit. A3"
               mj4aa4 = "DSM4 CAN Abuse Crit. A4"
               mj4aax = "DSM4 CAN Abuse Sum of As"
               mj4aa  = "DSM4 CAN Abuse 1+ As"
               mj4ab  = "DSM4 CAN Abuse Crit. B"
               mjd4ab = "DSM4 CAN Abuse Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id mjd4sk 
                    mj4da1-mj4da7 mj4dax mj4da mj4db mjd4dp
                    mj4aa1-mj4aa4 mj4aax mj4aa mj4ab mjd4ab;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_can_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(cand4,mj1a mj1c mj7a mj7b mjd4sk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(cand4,mj1c mj1c_1 mj7a mj7b mjd4sk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(cand4,mjd4sk mj8a mj8b mj4da1);
                     %multi(cand4,mjd4sk mj29c mj29d mj4da2);
                     %multi(cand4,mjd4sk mj10b mj10d mj4da3);
                     %multi(cand4,mjd4sk mj9b mj9c mj4da4);
                     %multi(cand4,mjd4sk mj11 mj4da5);
                     %multi(cand4,mjd4sk mj12c mj4da6);
                     %multi(cand4,mjd4sk mj24e mj4da7);
                     %multi(cand4,mjd4sk mj4da1-mj4da7 mj4dax);
                     %multi(cand4,mjd4sk mj4dax mj4da);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(cand4,mjd4sk mj31a mj4db);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(cand4,mjd4sk mj4da mj4db mjd4dp);

               *********;
               * Abuse *;
               *********;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(cand4,mjd4sk mj20g mj4aa1);
                     %multi(cand4,mjd4sk mj15d_1 mj15f_1 mj23c mj4aa2);
                     %multi(cand4,mjd4sk mj22b_1 mj4aa3);
                     %multi(cand4,mjd4sk mj21f mj4aa4);
                     %multi(cand4,mjd4sk mj4aa1 mj4aa2 mj4aa3 mj4aa4 mj4aax);
                     %multi(cand4,mjd4sk mj4aax mj4aa);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(cand4,mjd4sk mjd4dp mj4ab);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(cand4,mjd4sk mj4aa mj4ab mjd4ab);

            ******************;
            * keep variables *;
            ******************;

               data cand4;
               set cand4;
               keep ind_id mjd4sk
                    mj4da1-mj4da7 mj4dax mj4da mj4db mjd4dp
                    mj4aa1-mj4aa4 mj4aax mj4aa mj4ab mjd4ab;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=cand4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend cand4;

********;
* test *;
********;
/*
   %cand4(coga.csaga2,1);
*/
