**************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Conduct Disorder *;
**************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %cdd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro cdd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in CDD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in CDD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data cdd4;
      set &dsn.;

      attrib cdd4a1-cdd4a17 cdd4a3x cdd4a5x cdd4a cdd4b cdd4c cdd4dx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***************************************;
            * often bullies/threatens/intimidates *;
            ***************************************;

               if sum(cd20a not in (1,3,5),
                      cd20b not in (1,3,5))=2 then cdd4a1=.;
               else do;
                  cdd4a1=1;
                  if (cd20a=3 or cd20b=3) then cdd4a1=3;
                  else if (cd20a=5 or cd20b=5) then cdd4a1=5;
               end;

            ***********************************;
            * often initiates physical fights *;
            ***********************************;

               if sum(sum(cd21a not in (1,3,5),
                          cd21b<0,
                          cd21b1 not in (1,2,3,4,5))=3,
                      sum(cd21f not in (1,3,5),
                          cd21g<0,
                          cd21g1 not in (1,2,3,4,5))=3)=2 then cdd4a2=.;
               else do;
                  cdd4a2=1;
                  if (cd21a=3 and (cd21b>2 or cd21b1 in (3,4,5))) or
                     (cd21f=3 and (cd21g>2 or cd21g1 in (3,4,5))) then
                     cdd4a2=3;
                  else if (cd21a=5 and (cd21b>2 or cd21b1 in (3,4,5))) or
                          (cd21f=5 and (cd21g>2 or cd21g1 in (3,4,5))) then
                     cdd4a2=5;
               end;

            *****************;
            * used a weapon *;
            *****************;

               if cd22a not in (1,3,5) then cdd4a3=.;
               else cdd4a3=(4*(cd22a=5))+(2*(cd22a=3))+1;

            ***************************************;
            * exhibits physical cruelty to others *;
            ***************************************;

               if sum(cd23a not in (1,5),
                      cd23a1 not in (1,3,5))=2 then cdd4a4=.;
               else cdd4a4=(4*(cd23a=5 and cd23a1=1))+1;

            ****************************************;
            * exhibits physical cruelty to animals *;
            ****************************************;

               if cd19a not in (1,3,5) then cdd4a5=.;
               else cdd4a5=(4*(cd19a=5))+(2*(cd19a=3))+1;

            ***********************************;
            * committed confrontational crime *;
            ***********************************;

               if cd15a not in (1,3,5) then cdd4a6=.;
               else cdd4a6=(4*(cd15a=5))+(2*(cd15a=3))+1;

            ****************************;
            * committed sexual assault *;
            ****************************;

               if cd24 not in (1,3,5) then cdd4a7=.;
               else cdd4a7=(4*(cd24=5))+(2*(cd24=3))+1;

            *********************;
            * firesetting/arson *;
            *********************;

               if sum(cd17a not in (1,3,5),
                      cd17b not in (1,5))=2 then cdd4a8=.;
               else cdd4a8=(4*(cd17a=5 and cd17b=5))+
                           (2*(cd17a=3 and cd17b=5))+1;

            *****************************;
            * non-firesetting vandalism *;
            *****************************;

               if cd18a not in (1,3,5) then cdd4a9=.;
               else cdd4a9=(4*(cd18a=5))+(2*(cd18a=3))+1;

            ***********************************;
            * committed breaking and entering *;
            ***********************************;

               if cd16a not in (1,3,5) then cdd4a10=.;
               else cdd4a10=(4*(cd16a=5))+(2*(cd16a=3))+1;

            ******************************************;
            * often lies/cons to obtain goods/favors *;
            ******************************************;

               if sum(sum(cd8a not in (1,3,5),cd8b not in (1,3,5))=2,
                      sum(cd8c_1 not in (1,5),cd8c_2 not in (1,5),
                          cd8c_3 not in (1,5))=3,
                      cd9a not in (1,3,5),
                      sum(cd9b<0,cd9b1 not in (1,2,3,4,5))=2,
                      cd10a not in (1,3,5))=5 then cdd4a11=.;
               else do;
                  cdd4a11=1;
                  if (((cd8a=3 or cd8b=3) and max(of cd8c_1-cd8c_3)=5) or
                      (cd9a=3 and (cd9b>2 or cd9b1 in (3,4,5))) or
                      (cd10a=3)) then cdd4a11=3;
                  else if (((cd8a=5 or cd8b=5) and max(of cd8c_1-cd8c_3)=5) or
                           (cd9a=5 and (cd9b>2 or cd9b1 in (3,4,5))) or
                           (cd10a=5)) then cdd4a11=5;
               end;

            ****************************************;
            * nonconfrontational, nontrivial theft *;
            ****************************************;

               if sum(sum(cd14a not in (1,3,5),cd14b not in (1,3,5))=2,
                      sum(cd14c<0,cd14c1 not in (1,2,3,4,5))=2,
                      cd14d not in (1,3,5),
                      sum(cd14e<0,cd14e1 not in (1,2,3,4,5))=2)=4 then 
                  cdd4a12=.;
               else do;
                  cdd4a12=1;
                  if (((cd14a=3 or cd14b=3) and 
                       (cd14c>1 or cd14c1 in (2,3,4,5))) or
                      (cd14d=3 and (cd14e>1 or cd14e1 in (2,3,4,5))) or
                      sum(cd14a=3,cd14b=3,cd14d=3)>1) then cdd4a12=3;
                  else if (((cd14a=5 or cd14b=5) and 
                            (cd14c>1 or cd14c1 in (2,3,4,5))) or
                           (cd14d=5 and (cd14e>1 or cd14e1 in (2,3,4,5))) or
                           sum(cd14a=5,cd14b=5,cd14d=5)>1) then cdd4a12=5;
               end;

            ******************************;
            * often stays out after dark *;
            ******************************;

               ************************************;
               * note:  review of data indicate   *;
               *        that not all interviews   *; 
               *        completed with 24hr clock *;
               *        so this coding may not be *;
               *        entirely appropriate      *;
               ************************************;

                  if (cd6b_1 ge 0 and cd6c_1 ge 0) then do;
                     time1=(cd6c_1+(cd6c_2/60));
                     time2=cd6b_1;
                     if time2=24 then time2=0;
                     time2=(time2+(cd6b_2/60));
                     if time2<time1 then time2=time2+24;
                     timed=(time2-time1);
                  end;

               if sum(sum(cd5a not in (1,3,5),
                          sum(cd5b<0,cd5b1 not in (1,2,3,4,5))=2,
                          cd5c_1<0)=3,
                      sum(sum(cd6a not in (1,3,5),timed)=2,
                          sum(cd6d<0,cd6d1 not in (1,2,3,4,5))=2,
                          cd6e_1<0)=3)=2 then cdd4a13=.;
               else do;                  
                  cdd4a13=1;
                  if ((cd5a=3 and (cd5b>2 or cd5b1 in (3,4,5)) and
                       (0<cd5c_1<13)) or
                      (cd6a=3 and timed ge 2 and 
                       (cd6d>2 or cd6d1 in (3,4,5)) and 
                       (0<cd6e_1<13))) then cdd4a13=3;
                  else if ((cd5a=5 and (cd5b>2 or cd5b1 in (3,4,5)) and
                            (0<cd5c_1<13)) or
                           (cd6a=5 and timed ge 2 and 
                            (cd6d>2 or cd6d1 in (3,4,5)) and 
                            (0<cd6e_1<13))) then cdd4a13=5;
               end;

            **********************;
            * run away from home *;
            **********************;

               if sum(cd7a not in (1,3,5),
                      cd7b not in (1,2,3),
                      sum(cd7c<0,cd7c1 not in (1,2,3,4,5))=2,
                      cd7d<0)=4 then cdd4a14=.;
               else do;
                  cdd4a14=1;
                  if ((cd7a=3 and cd7b=3) and 
                      ((cd7c>1 or cd17c1 in (2,3,4,5)) or cd7d ge 7)) then
                     cdd4a14=3;
                  else if ((cd7a=5 and cd7b=3) and 
                           ((cd7c>1 or cd17c1 in (2,3,4,5)) or cd7d ge 7)) then
                     cdd4a14=5;
               end;

            ********************;
            * frequent truancy *;
            ********************;

               if sum(cd3a not in (1,3,5),
                      sum(cd3b<0,cd3b1 not in (1,2,3,4,5))=2,
                      cd3c_1<0,
                      cd4a not in (1,3,5),
                      sum(cd4b<0,cd4b1 not in (1,2,3,4,5))=2,
                      cd4c_1<0)=6 then cdd4a15=.;
               else do;
                  cdd4a15=1;
                  if ((cd3a=3 and (cd3b>2 or cd3b1 in (3,4,5)) and
                       (0<cd3c_1<13)) or
                      (cd4a=3 and (cd4b>2 or cd4b1 in (3,4,5)) and
                       (0<cd4c_1<13)) or
                      ((cd3a=3 and cd4a=3) and 
                       (sum(cd3b,cd4b)>2 or sum(cd3b1,cd4b1)>2) and
                       ((0<cd3c_1<13) or (0<cd4c_1<13)))) then cdd4a15=3;
                  else if ((cd3a=5 and (cd3b>2 or cd3b1 in (3,4,5)) and
                            (0<cd3c_1<13)) or
                           (cd4a=5 and (cd4b>2 or cd4b1 in (3,4,5)) and
                            (0<cd4c_1<13)) or
                           ((cd3a=5 and cd4a=5) and 
                            (sum(cd3b,cd4b)>2 or sum(cd3b1,cd4b1)>2) and
                            ((0<cd3c_1<13) or (0<cd4c_1<13)))) then cdd4a15=5;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of cdd4a1-cdd4a15)=0 then do;
                  cdd4a3x=.;
                  cdd4a5x=.;
               end;
               else do;
                  cdd4a3x=0;
                  cdd4a5x=0;
                  array one cdd4a1-cdd4a15;
                  do over one;
                     if one=3 then cdd4a3x+1;
                     if one=5 then cdd4a5x+1;
                  end;
               end;

            **************************************;
            * three or more in past 12 months    *;
            *                                    *;
            * NOTE:  behaviors need to cluster   *;
            *        within a 12mo period AND    *;
            *        at least one behavior must  *;
            *        occur in later half of 12mo *;
            *        period -- this code handles *;
            *        a specific case of this     *;
            *        criteria by using recency   *;
            *        codes -- a generalized      *;
            *        algorithm is still needed   *;
            *                                    *;
            **************************************;

               sumre=0;
               sum4s=0;
               sum3s=0;
               if sum(cd17b not in (1,5),
                      cd17d_4 not in (1,2,3,4,5))=2 then cd17tmp=.;
               else cd17tmp=(cd17d_4*(cd17d_4 in (1,2,3,4,5) and cd17b=5));
               array two cd3c_4 cd4c_4 cd5c_4 cd6e_4 cd7f_4 cd8d_4 cd9c_4 
                         cd10b_4 cd11b_4 cd15c_4 cd16c_4 cd17tmp cd18d_4 
                         cd19c_4 cd20c_4 cd21e_4 cd21h_4 cd22c_4 cd23d_4;
               do over two;
                  if two in (1,2,3,4,5) then sumre+1;
                  if 1<two<5 then sum4s+1;
                  if 1<two<4 then sum3s+1;
               end;
               if cd26 not in (1,5) and sumre=0 then cdd4a16=.;
               else cdd4a16=(4*(cd26=5 or sum4s>2))+1;

            *********************************;
            * at least one in past 6 months *;
            *********************************;

               if sumre=0 then cdd4a17=.;
               else cdd4a17=(4*(sum3s>0))+1;
 
            *******************;
            * criteria for A  *;
            *******************;

               if n(cdd4a3x,cdd4a5x,cdd4a16,cdd4a17)=0 then cdd4a=.;
               else do;
                  cdd4a=1;
                  if (cdd4a3x>0 and sum(cdd4a3x,cdd4a5x)>2 and
                      cdd4a16=5 and cdd4a17=5) then cdd4a=3;
                  else if (cdd4a3x<1 and cdd4a5x>2 and 
                           cdd4a16=5 and cdd4a17=5) then cdd4a=5;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(cd12a not in (1,5),cd12b not in (1,5),cd12c not in (1,5),
                   cd12d not in (1,5),cd12e not in (1,5),cd12f not in (1,5),
                   cd12g not in (1,5),cd12h not in (1,5),cd12i not in (1,5),
                   cd12j not in (1,5),cd12k not in (1,5))=11 then cdd4b=.;
            else cdd4b=(4*(sum(cd12a=5,cd12b=5,cd12c=5,cd12d=5,cd12e=5,cd12f=5,
                               cd12g=5,cd12h=5,cd12i=5,cd12j=5,cd12k=5)>0))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if dm3a<0 then cdd4c=.;
            else cdd4c=(4*(dm3a>17))+1;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(cdd4a,cdd4b,cdd4c)=0 then cdd4dx=.;
            else do;
               cdd4dx=1;
               if (cdd4a=3 and cdd4b=5 and cdd4c=1) then cdd4dx=3;
               else if (cdd4a=5 and cdd4b=5 and cdd4c=1) then cdd4dx=5;
            end;

      *******************;
      * variable labels *;
      *******************;

         label cdd4a1  = "DSM4 CD (Lifetime) Crit. A1"
               cdd4a2  = "DSM4 CD (Lifetime) Crit. A2"
               cdd4a3  = "DSM4 CD (Lifetime) Crit. A3"
               cdd4a4  = "DSM4 CD (Lifetime) Crit. A4"
               cdd4a5  = "DSM4 CD (Lifetime) Crit. A5"
               cdd4a6  = "DSM4 CD (Lifetime) Crit. A6"
               cdd4a7  = "DSM4 CD (Lifetime) Crit. A7"
               cdd4a8  = "DSM4 CD (Lifetime) Crit. A8"
               cdd4a9  = "DSM4 CD (Lifetime) Crit. A9"
               cdd4a10 = "DSM4 CD (Lifetime) Crit. A10"
               cdd4a11 = "DSM4 CD (Lifetime) Crit. A11"
               cdd4a12 = "DSM4 CD (Lifetime) Crit. A12"
               cdd4a13 = "DSM4 CD (Lifetime) Crit. A13"
               cdd4a14 = "DSM4 CD (Lifetime) Crit. A14"
               cdd4a15 = "DSM4 CD (Lifetime) Crit. A15"
               cdd4a3x = "DSM4 CD (Lifetime) Sum of As=3"
               cdd4a5x = "DSM4 CD (Lifetime) Sum of As=5"
               cdd4a16 = "DSM4 CD (Lifetime) Crit. A16"
               cdd4a17 = "DSM4 CD (Lifetime) Crit. A17"
               cdd4a   = "DSM4 CD (Lifetime) Crit. A"
               cdd4b   = "DSM4 CD (Lifetime) Crit. B"
               cdd4c   = "DSM4 CD (Lifetime) Crit. C"
               cdd4dx  = "DSM4 CD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    cdd4a1-cdd4a17 cdd4a3x cdd4a5x cdd4a cdd4b cdd4c cdd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_cd_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cdd4,cd20a cd20b cdd4a1);
                     %multi(cdd4,cd21a cd21b cd21b1 cd21f cd21g cd21g1 cdd4a2);
                     %multi(cdd4,cd22a cdd4a3);
                     %multi(cdd4,cd23a cd23a1 cdd4a4);
                     %multi(cdd4,cd19a cdd4a5);
                     %multi(cdd4,cd15a cdd4a6);
                     %multi(cdd4,cd24 cdd4a7);
                     %multi(cdd4,cd17a cd17b cdd4a8);
                     %multi(cdd4,cd18a cdd4a9);
                     %multi(cdd4,cd16a cdd4a10);
                     %multi(cdd4,cd8a cd8b cd8c_1-cd8c_3 cd9a cd9b cd9b1 cd10a 
                                 cdd4a11);
                     %multi(cdd4,cd14a cd14b cd14c cd14c1 cd14d cd14e cd14e1
                                 cdd4a12);

                     %multi(cdd4,time1 time2 timed);
                     %multi(cdd4,cd5a cd5b cd5b1 cd5c_1 
                                 cd6a timed cd6d cd6d1 cd6e_1 cdd4a13);

                     %multi(cdd4,cd7a cd7b cd7c cd7c1 cd7d cdd4a14);
                     %multi(cdd4,cd3a cd3b cd3b1 cd3c_1 cd4a cd4b cd4b1 cd4c_1 
                                 cdd4a15);
                     %multi(cdd4,cdd4a1-cdd4a15 cdd4a3x cdd4a5x);

                     %multi(cdd4,cd17b cd17d_4 cd17tmp);
                     %multi(cdd4,cd3c_4 cd4c_4 cd5c_4 cd6e_4 cd7f_4 cd8d_4 
                                 cd9c_4 cd10b_4 cd11b_4 cd15c_4 cd16c_4 cd17tmp 
                                 cd18d_4 cd19c_4 cd20c_4 cd21e_4 cd21h_4 
                                 cd22c_4 cd23d_4 sumre sum3s sum4s);

                     %multi(cdd4,cd26 sum4s cdd4a16);
                     %multi(cdd4,sum3s cdd4a17);
                     %multi(cdd4,cdd4a3x cdd4a5x cdd4a16 cdd4a17 cdd4a);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cdd4,cd12a cd12b cd12c cd12d cd12e cd12f
                                 cd12g cd12h cd12i cd12j cd12k cdd4b);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cdd4,dm3a cdd4c);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cdd4,cdd4a cdd4b cdd4c cdd4dx);

            ******************;
            * keep variables *;
            ******************;

               data cdd4;
               set cdd4;
               keep ind_id  
                    cdd4a1-cdd4a17 cdd4a3x cdd4a5x cdd4a cdd4b cdd4c cdd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=cdd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend cdd4;

********;
* test *;
********;
/*
   %cdd4(coga.csaga2,1);
*/
