****************************************************************;
* CSSAGA-II Diagnostic Algorithm -- DSM-IV -- Major Depression *;
****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %dep4(dsn,chk)                                            *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro depd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DEPD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DEPD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data depd4;
      set &dsn.;

      attrib dcd4sk dpd4sk mdd4sk
             dcd4a1-dcd4a9 dcd4ax dcd4a
             dpd4a1-dpd4a9 dpd4ax dpd4a
             dcd4c dpd4c 
             dcd4d dpd4d
             dcd4e dpd4e
             dcd4dx dpd4dx 
             mdd4a mdd4c mdd4d mdd4e mdd4dx length=3;

      *********************;
      * test for skip out *;
      *********************;

         *******************;
         * current episode *;
         *******************;

            if sum(dp1c_1 not in (1,5),
                   dp1c_2 not in (1,5),
                   dp1c_3 not in (1,5),
                   dp1c_4 not in (1,5),
                   dp1c_5 not in (1,5))=5 then dcd4sk=5;
            else do;
               if sum(dp1c_1=5,dp1c_2=5,dp1c_3=5,dp1c_4=5,dp1c_5=5)>0 then 
                  dcd4sk=1;
               else dcd4sk=5;
            end;

            if dcd4sk=5 then do;
                array skp1 dcd4a1-dcd4a9 dcd4ax dcd4a
                           dcd4c dcd4d dcd4e dcd4dx;
                do over skp1;
                  skp1=.S;
                end;
            end;

         ****************************;
         * most severe past episode *;
         ****************************;

            if sum(dp1c_1 not in (1,5),
                   dp1c_2 not in (1,5),
                   dp1c_3 not in (1,5),
                   dp1c_4 not in (1,5),
                   dp1c_5 not in (1,5))=5 then dpd4sk=5;
            else do;
               if sum(dp7c_1=5,dp7c_2=5,dp7c_3=5,dp7c_4=5,dp7c_5=5)>0 then 
                  dpd4sk=1;
               else dpd4sk=5;
            end;

            if dpd4sk=5 then do;
                array skp2 dpd4a1-dpd4a9 dpd4ax dpd4a
                           dpd4c dpd4d dpd4e dpd4dx;
                do over skp2;
                  skp2=.S;
                end;
            end;
 
         *****************;
         * lifetime ever *;
         *****************;

            if (dcd4sk=5 and dpd4sk=5) then mdd4sk=5;
            else mdd4sk=1;

            if mdd4sk=5 then do;
               array skp3 mdd4a mdd4c mdd4d mdd4e mdd4dx;
               do over skp3;
                  skp3=.S;
               end;
            end;

            if mdd4sk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         if dcd4sk=1 then do;

         *******************;
         * current episode *;
         *******************;

            *******************************;
            * depressed mood/irritability *;
            *******************************;

               if sum((dp1c_1 not in (1,5)),
                      (dp1c_2 not in (1,5)),
                      (dp1c_5 not in (1,5)))=3 then dcd4a1=.;
               else dcd4a1=sum((dp1c_1=5),(dp1c_2=5),(dp1c_5=5))>0;

            *************;
            * anhedonia *;
            *************;

               if sum((dp1c_3 not in (1,5)),
                      (dp1c_4 not in (1,5)))=2 then dcd4a2=.;
               else dcd4a2=sum((dp1c_3=5),(dp1c_4=5))>0;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if sum((dp19a_1 not in (1,5)),
                      (dp19a1_1 not in (1,5)),
                      (dp20a_1 not in (1,5)),
                      (dp20a1_1 not in (1,5)))=4 then dcd4a3=.;
               else dcd4a3=sum((dp19a_1=5),(dp19a1_1=5),
                               (dp20a_1=5),(dp20a1_1=5))>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dp21a_1 not in (1,5)),
                      (dp21b_1 not in (1,5)),
                      (dp21c_1 not in (1,5)),
                      (dp21d_1 not in (1,5)))=4 then dcd4a4=.;
               else dcd4a4=sum((dp21a_1=5),(dp21b_1=5),
                               (dp21c_1=5),(dp21d_1=5))>0;

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;

               if sum((dp22_1 not in (1,5)),
                      (dp23_1 not in (1,5)))=2 then dcd4a5=.;
               else dcd4a5=sum((dp22_1=5),(dp23_1=5))>0;
   
            **************************;
            * fatigue/loss of energy *;
            **************************;

               if (dp1c_6 not in (1,5)) then dcd4a6=.;
               else dcd4a6=(dp1c_6=5);

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if sum((dp24_1 not in (1,5)),
                      (dp25_1 not in (1,5)),
                      (dp25a_1 not in (1,5)))=3 then dcd4a7=.;
               else dcd4a7=sum((dp24_1=5),(dp25_1=5),(dp25a_1=5))>0;

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dp26a_1 not in (1,5)),
                      (dp26b_1 not in (1,5)),
                      (dp27a_1 not in (1,5)),
                      (dp27b_1 not in (1,5)))=4 then dcd4a8=.;
               else dcd4a8=sum((dp26a_1=5),(dp26b_1=5),
                               (dp27a_1=5),(dp27b_1=5))>0;

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if sum((dp28a_1 not in (1,5)),
                      (dp28b_1 not in (1,5)),
                      (dp28c_1 not in (1,5)),
                      (dp28d_1 not in (1,5)))=4 then dcd4a9=.;
               else dcd4a9=sum((dp28a_1=5),(dp28b_1=5),
                               (dp28c_1=5),(dp28d_1=5))>0;

            ***********************************************************;
            * at least 5 from a1..a9 including at least 1 from a1..a2 *;
            ***********************************************************;

               if n(of dcd4a1-dcd4a9)=0 then dcd4ax=.;
               else dcd4ax=(n(of dcd4a1-dcd4a9)>0 and 
                            (dcd4a1 or dcd4a2) and 
                            sum(of dcd4a1-dcd4a9)>=5);

            ***************************************;
            * sxs occured in same two-week period *;
            ***************************************;

               if (dp30a_1 not in (1,5)) then dcd4a=.;
               else if (dp30a_1=1) then dcd4a=0;
               else if (dp30a_1=5) then do;
                  if sum((dcd4ax not in (0,1)),
                         (dp30a1_1 not in (1,5)),
                         (dp30b_1 not in (1,5)))>0 then dcd4a=.;
                  else dcd4a=((dcd4ax=1) and
                              (sum((dp30a1_1=5),(dp30b_1=5))=2));
               end;

         end;
         if dpd4sk=1 then do;

         ****************************;
         * most severe past episode *;
         ****************************;

            *******************************;
            * depressed mood/irritability *;
            *******************************;

               if sum((dp7c_1 not in (1,5)),
                      (dp7c_2 not in (1,5)),
                      (dp7c_5 not in (1,5)),
                      (dp14b not in (1,5)))=4 then dpd4a1=.;
               else dpd4a1=sum((dp7c_1=5),(dp7c_2=5),(dp7c_5=5),(dp14b=5))>0;

            *************;
            * anhedonia *;
            *************;

               if sum((dp7c_3 not in (1,5)),
                      (dp7c_4 not in (1,5)),
                      (dp14c not in (1,5)))=3 then dpd4a2=.;
               else dpd4a2=sum((dp7c_3=5),(dp7c_4=5),(dp14c=5))>0;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;
  
               if sum((dp19a_2 not in (1,5)),
                      (dp19a1_2 not in (1,5)),
                      (dp20a_2 not in (1,5)),
                      (dp20a1_2 not in (1,5)))=4 then dpd4a3=.;
               else dpd4a3=sum((dp19a_2=5),(dp19a1_2=5),
                               (dp20a_2=5),(dp20a1_2=5))>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dp21a_2 not in (1,5)),
                      (dp21b_2 not in (1,5)),
                      (dp21c_2 not in (1,5)),
                      (dp21d_2 not in (1,5)))=4 then dpd4a4=.;
               else dpd4a4=sum((dp21a_2=5),(dp21b_2=5),
                               (dp21c_2=5),(dp21d_2=5))>0;

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;
  
               if sum((dp22_2 not in (1,5)),
                      (dp23_2 not in (1,5)))=2 then dpd4a5=.;
               else dpd4a5=sum((dp22_2=5),(dp23_2=5))>0;

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if sum((dp7c_6 not in (1,5)),
                      (dp14d not in (1,5)))=2 then dpd4a6=.;
               else dpd4a6=sum((dp7c_6=5),(dp14d=5))>0;

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if sum((dp24_2 not in (1,5)),
                      (dp25_2 not in (1,5)),
                      (dp25a_2 not in (1,5)))=3 then dpd4a7=.;
               else dpd4a7=sum((dp24_2=5),(dp25_2=5),(dp25a_2=5))>0;

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dp26a_2 not in (1,5)),
                      (dp26b_2 not in (1,5)),
                      (dp27a_2 not in (1,5)),
                      (dp27b_2 not in (1,5)))=4 then dpd4a8=.;
               else dpd4a8=sum((dp26a_2=5),(dp26b_2=5),
                               (dp27a_2=5),(dp27b_2=5))>0;

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if sum((dp28a_2 not in (1,5)),
                      (dp28b_2 not in (1,5)),
                      (dp28c_2 not in (1,5)),
                      (dp28d_2 not in (1,5)))=4 then dpd4a9=.;
               else dpd4a9=sum((dp28a_2=5),(dp28b_2=5),
                               (dp28c_2=5),(dp28d_2=5))>0;

            ***********************************************************;
            * at least 5 from a1..a9 including at least 1 from a1..a2 *;
            ***********************************************************;

               if n(of dpd4a1-dpd4a9)=0 then dpd4ax=.;
               else dpd4ax=(n(of dpd4a1-dpd4a9)>0 and 
                            (dpd4a1 or dpd4a2) and 
                            sum(of dpd4a1-dpd4a9)>=5);

            ***************************************;
            * sxs occured in same two-week period *;
            ***************************************;

               if (dp30a_2 not in (1,5)) then dpd4a=.;
               else if (dp30a_2=1) then dpd4a=0;
               else if (dp30a_2=5) then do;
                  if sum((dpd4ax not in (0,1)),
                         (dp30a1_2 not in (1,5)),
                         (dp30b_2 not in (1,5)))>0 then dpd4a=.;
                  else dpd4a=((dpd4ax=1) and 
                              (sum((dp30a1_2=5),(dp30b_2=5))=2));
               end;

         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dcd4a,dpd4a)=0 then mdd4a=.;
            else mdd4a=(dcd4a or dpd4a);

         ***************************;
         * rescale from 0,1 to 1,5 *;
         ***************************;

            array cea dcd4a1-dcd4a9 dcd4ax dcd4a;
            array msa dpd4a1-dpd4a9 dpd4ax dpd4a;
            do over cea;
               if (cea ne . and cea ne .S) then cea=((4*cea)+1);
               if (msa ne . and msa ne .S) then msa=((4*msa)+1);
            end;
            mdd4a=((4*mdd4a)+1);

      **************************************;
      * Criterion B -- Skip in this module *;
      **************************************;

      ****************;
      * Criterion C  *;
      ****************;

         if dcd4sk=1 then do;

            *******************;
            * current episode *;
            *******************;

               if sum((dp32a1_1 not in (1,2,3)),
                      (dp32a2_1 not in (1,2,3)),
                      (dp32a3_1 not in (1,2,3)),
                      (dp32a4_1 not in (1,2,3)))=4 then
                  dcd4c=.;
               else dcd4c=sum((dp32a1_1=3),(dp32a2_1=3),
                              (dp32a3_1=3),(dp32a4_1=3))>0;

         end;
         if dpd4sk=1 then do;

            ****************************;
            * most severe past episode *;
            ****************************;

               if sum((dp32a1_2 not in (1,2,3)),
                      (dp32a2_2 not in (1,2,3)),
                      (dp32a3_2 not in (1,2,3)),
                      (dp32a4_2 not in (1,2,3)))=4 then
                  dpd4c=.;
               else dpd4c=sum((dp32a1_2=3),(dp32a2_2=3),
                              (dp32a3_2=3),(dp32a4_2=3))>0;
 
         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dcd4c,dpd4c)=0 then mdd4c=.;
            else mdd4c=(dcd4c or dpd4c);

         ***************************;
         * rescale from 0,1 to 1,5 *;
         ***************************;

            if dcd4sk=1 then dcd4c=((4*dcd4c)+1);
            if dpd4sk=1 then dpd4c=((4*dpd4c)+1);
            mdd4c=((4*mdd4c)+1);

      ***************;
      * Criterion D *;
      ***************;

         if dcd4sk=1 then do;

            *******************;
            * current episode *;
            *******************;

               if sum((dp4d not in (1,5)),(dp4g not in (1,5)),
                      (sum((dp5a_1 not in (1,5)),(dp5a_2 not in (1,5)),
                           (dp5a_3 not in (1,5)),(dp5a_4 not in (1,5)),
                           (dp5a_5 not in (1,5)),(dp5a_6 not in (1,5)))=6 or
                       sum((dp5b_1 not in (1,5)),(dp5b_2 not in (1,5)),
                           (dp5b_3 not in (1,5)),(dp5b_4 not in (1,5)), 
                           (dp5b_5 not in (1,5)),(dp5b_6 not in (1,5)))=6),
                       (dp6a not in (1,5)),(dp31b_1 not in (1,5)),
                       (dp31c_1 not in (1,5)))=6 then dcd4d=.;
               else dcd4d=((dp4d ne 5 and dp4g ne 5) and 
                           (min((dp5a_1 ne 5),(dp5a_2 ne 5),(dp5a_3 ne 5), 
                                (dp5a_4 ne 5),(dp5a_5 ne 5),(dp5a_6 ne 5))=1 or
                            min((dp5b_1 ne 5),(dp5b_2 ne 5),(dp5b_3 ne 5), 
                                (dp5b_4 ne 5),(dp5b_5 ne 5),(dp5b_6 ne 5))=1) and
                           (dp6a=1) and (dp31b_1=1) and (dp31c_1=1));

         end;
         if dpd4sk=1 then do;

            ****************************;
            * most severe past episode *;
            ****************************;

               if sum((dp11d not in (1,5)),(dp11g not in (1,5)),
                      (sum((dp12a_1 not in (1,5)),(dp12a_2 not in (1,5)),
                           (dp12a_3 not in (1,5)),(dp12a_4 not in (1,5)),
                           (dp12a_5 not in (1,5)),(dp12a_6 not in (1,5)))=6 or
                       sum((dp12b_1 not in (1,5)),(dp12b_2 not in (1,5)),
                           (dp12b_3 not in (1,5)),(dp12b_4 not in (1,5)), 
                           (dp12b_5 not in (1,5)),(dp12b_6 not in (1,5)))=6),
                       (dp13a not in (1,5)),(dp31b_2 not in (1,5)),
                       (dp31c_2 not in (1,5)))=6 then dpd4d=.;
               else dpd4d=((dp11d ne 5 and dp11g ne 5) and 
                           (min((dp12a_1 ne 5),(dp12a_2 ne 5),
                                (dp12a_3 ne 5),(dp12a_4 ne 5),
                                (dp12a_5 ne 5),(dp12a_6 ne 5))=1 or
                            min((dp12b_1 ne 5),(dp12b_2 ne 5),
                                (dp12b_3 ne 5),(dp12b_4 ne 5),
                                (dp12b_5 ne 5),(dp12b_6 ne 5))=1) and
                           (dp13a=1) and (dp31b_2=1) and (dp31c_2=1));

         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dcd4d,dpd4d)=0 then mdd4d=.;
            else mdd4d=(dcd4d or dpd4d);

         ***************************;
         * rescale from 0,1 to 1,5 *;
         ***************************;

            if dcd4sk=1 then dcd4d=((4*dcd4d)+1);
            if dpd4sk=1 then dpd4d=((4*dpd4d)+1);
            mdd4d=((4*mdd4d)+1);

      ***************;
      * Criterion E *;
      ***************;

         if dcd4sk=1 then do;

            *******************;
            * current episode *;
            *******************;

               if sum((dp31a_1 not in (1,5)),
                      (dp31a1_1 not in (1,5)),
                      (dp30d_1<0 or dp30d_1=99),
                      (dp23_1 not in (1,5)),
                      (dp24_1 not in (1,5)),
                      (dp25_1 not in (1,5)),
                      (dp28a_1 not in (1,5)),
                      (dp28b_1 not in (1,5)),
                      (dp28c_1 not in (1,5)),
                      (dp28d_1 not in (1,5)))=10 then dcd4e=.;
               else dcd4e=(((dp31a_1=1) or (dp31a1_1=1)) or
                           ((dp31a_1=5) and (0<dp30d_1<9) and (dp23_1=1) and
                            (dp24_1=1) and (dp25_1=1) and (dp28a_1=1) and 
                            (dp28b_1=1) and (dp28c_1=1) and (dp28d_1=1)));

         end;
         if dpd4sk=1 then do;

            ****************************;
            * most severe past episode *;
            ****************************;

               if sum((dp31a_2 not in (1,5)),
                      (dp31a1_2 not in (1,5)),
                      (dp30d_2<0 or dp30d_2=99),
                      (dp23_2 not in (1,5)),
                      (dp24_2 not in (1,5)),
                      (dp25_2 not in (1,5)),
                      (dp28a_2 not in (1,5)),
                      (dp28b_2 not in (1,5)),
                      (dp28c_2 not in (1,5)),
                      (dp28d_2 not in (1,5)))=10 then dpd4e=.;
               else dpd4e=(((dp31a_2=1) or (dp31a1_2=1)) or
                           ((dp31a_2=5) and (0<dp30d_2<9) and (dp23_2=1) and
                            (dp24_2=1) and (dp25_2=1) and (dp28a_2=1) and 
                            (dp28b_2=1) and (dp28c_2=1) and (dp28d_2=1)));

         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dcd4e,dpd4e)=0 then mdd4e=.;
            else mdd4e=(dcd4e or dpd4e);

         ***************************;
         * rescale from 0,1 to 1,5 *;
         ***************************;

            if dcd4sk=1 then dcd4e=((4*dcd4e)+1);
            if dpd4sk=1 then dpd4e=((4*dpd4e)+1);
            mdd4e=((4*mdd4e)+1);

      *************;
      * Diagnosis *;
      *************;

         if dcd4sk=1 then do;

            *******************;
            * current episode *;
            *******************;

               if (dcd4a not in (1,5)) then dcd4dx=.;
               else do;
                  if sum((dcd4c not in (1,5)),(dcd4d not in (1,5)),
                         (dcd4e not in (1,5)))=3 then dcd4dx=.;
                  else if dcd4a=1 then dcd4dx=1;
                  else if dcd4a=5 then do;
                     if dcd4c=1 then dcd4dx=1;
                     else if dcd4c=5 then do;
                        if (dcd4d=5 and dcd4e=5) then dcd4dx=5;
                        else if (dcd4d=1 or dcd4e=1) then dcd4dx=3;
                     end;
                  end;
               end;

         end;
         if dpd4sk=1 then do;

            ****************************;
            * most severe past episode *;
            ****************************;

               if (dpd4a not in (1,5)) then dpd4dx=.;
               else do;
                  if sum((dpd4c not in (1,5)),(dpd4d not in (1,5)),
                         (dpd4e not in (1,5)))=3 then dpd4dx=.;
                  else if dpd4a=1 then dpd4dx=1;
                  else if dpd4a=5 then do;
                     if dpd4c=1 then dpd4dx=1;
                     else if dpd4c=5 then do;
                        if (dpd4d=5 and dpd4e=5) then dpd4dx=5;
                        else if (dpd4d=1 or dpd4e=1) then dpd4dx=3;
                     end;
                  end;
               end;

         end;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(dcd4dx,dpd4dx)=0 then mdd4dx=.;
            else do;
               if (dcd4dx=3 or dpd4dx=3) then mdd4dx=3;
               else mdd4dx=max(dcd4dx,dpd4dx);
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label dcd4sk = "DSM4 MDD (Current Episode) Skip Out"
               dpd4sk = "DSM4 MDD (Worst Episode) Skip Out"
               mdd4sk = "DSM4 MDD (Lifetime) Skip Out"
               dcd4a1 = "DSM4 MDD (Current Episode) Crit. A1"
               dcd4a2 = "DSM4 MDD (Current Episode) Crit. A2"
               dcd4a3 = "DSM4 MDD (Current Episode) Crit. A3"
               dcd4a4 = "DSM4 MDD (Current Episode) Crit. A4"
               dcd4a5 = "DSM4 MDD (Current Episode) Crit. A5"
               dcd4a6 = "DSM4 MDD (Current Episode) Crit. A6"
               dcd4a7 = "DSM4 MDD (Current Episode) Crit. A7"
               dcd4a8 = "DSM4 MDD (Current Episode) Crit. A8"
               dcd4a9 = "DSM4 MDD (Current Episode) Crit. A9"
               dcd4ax = "DSM4 MDD (Current) 5+ A (inc A1 or A2)"
               dcd4a  = "DSM4 MDD (Current Episode) Criterion A"
               dpd4a1 = "DSM4 MDD (Worst Past Episode) Crit. A1"
               dpd4a2 = "DSM4 MDD (Worst Past Episode) Crit. A2"
               dpd4a3 = "DSM4 MDD (Worst Past Episode) Crit. A3"
               dpd4a4 = "DSM4 MDD (Worst Past Episode) Crit. A4"
               dpd4a5 = "DSM4 MDD (Worst Past Episode) Crit. A5"
               dpd4a6 = "DSM4 MDD (Worst Past Episode) Crit. A6"
               dpd4a7 = "DSM4 MDD (Worst Past Episode) Crit. A7"
               dpd4a8 = "DSM4 MDD (Worst Past Episode) Crit. A8"
               dpd4a9 = "DSM4 MDD (Worst Past Episode) Crit. A9"
               dpd4ax = "DSM4 MDD (Wst Past) 5+ A (inc A1 or A2)"
               dpd4a  = "DSM4 MDD (Worst Past Episode) Crit. A"
               dcd4c  = "DSM4 MDD (Current Episode) Criterion C"
               dpd4c  = "DSM4 MDD (Worst Past Episode) Crit. C"
               dcd4d  = "DSM4 MDD (Current Episode) Criterion D"
               dpd4d  = "DSM4 MDD (Worst Past Episode) Crit. D"
               dcd4e  = "DSM4 MDD (Current Episode) Criterion E"
               dpd4e  = "DSM4 MDD (Worst Past Episode) Crit. E"
               dcd4dx = "DSM4 MDD (Current Episode) Diagnosis"
               dpd4dx = "DSM4 MDD (Worst Past Episode) Diagnosis"
               mdd4a  = "DSM4 MDD (Lifetime) Criterion A"
               mdd4c  = "DSM4 MDD (Lifetime) Criterion C"
               mdd4d  = "DSM4 MDD (Lifetime) Criterion D"
               mdd4e  = "DSM4 MDD (Lifetime) Criterion E"
               mdd4dx = "DSM4 MDD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    dcd4sk dpd4sk mdd4sk
                    dcd4a1-dcd4a9 dcd4ax dcd4a
                    dpd4a1-dpd4a9 dpd4ax dpd4a
                    dcd4c dpd4c 
                    dcd4d dpd4d
                    dcd4e dpd4e
                    dcd4dx dpd4dx 
                    mdd4a mdd4c mdd4d mdd4e mdd4dx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_dep_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depd4,dp1c_1 dp1c_2 dp1c_3 dp1c_4 dp1c_5
                                  dcd4sk);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                    %multi(depd4,dp7c_1 dp7c_2 dp7c_3 dp7c_4 dp7c_5
                                 dpd4sk);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depd4,dcd4sk dpd4sk mdd4sk);

               ***************;
               * Criterion A *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depd4,dcd4sk dp1c_1 dp1c_2 dp1c_5 dcd4a1);
                     %multi(depd4,dcd4sk dp1c_3 dp1c_4 dcd4a2);
                     %multi(depd4,dcd4sk dp19a_1 dp19a1_1 dp20a_1 dp20a1_1 
                                  dcd4a3);
                     %multi(depd4,dcd4sk dp21a_1 dp21b_1 dp21c_1 dp21d_1 dcd4a4);
                     %multi(depd4,dcd4sk dp22_1 dp23_1 dcd4a5);
                     %multi(depd4,dcd4sk dp1c_6 dcd4a6);
                     %multi(depd4,dcd4sk dp24_1 dp25_1 dp25a_1 dcd4a7);
                     %multi(depd4,dcd4sk dp26a_1 dp26b_1 dp27a_1 dp27b_1 dcd4a8);
                     %multi(depd4,dcd4sk dp28a_1 dp28b_1 dp28c_1 dp28d_1 dcd4a9);
                     %multi(depd4,dcd4sk dcd4a1-dcd4a9 dcd4ax);
                     %multi(depd4,dcd4sk dcd4ax dp30a_1 dp30a1_1 dp30b_1 dcd4a);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depd4,dpd4sk dp7c_1 dp7c_2 dp7c_5 dp14b dpd4a1);
                     %multi(depd4,dpd4sk dp7c_3 dp7c_4 dp14c dpd4a2);
                     %multi(depd4,dpd4sk dp19a_2 dp19a1_2 dp20a_2 dp20a1_2 
                                  dpd4a3);
                     %multi(depd4,dpd4sk dp21a_2 dp21b_2 dp21c_2 dp21d_2 dpd4a4);
                     %multi(depd4,dpd4sk dp22_2 dp23_2 dpd4a5);
                     %multi(depd4,dpd4sk dp7c_6 dp14d dpd4a6);
                     %multi(depd4,dpd4sk dp24_2 dp25_2 dp25a_2 dpd4a7);
                     %multi(depd4,dpd4sk dp26a_2 dp26b_2 dp27a_2 dp27b_2 dpd4a8);
                     %multi(depd4,dpd4sk dp28a_2 dp28b_2 dp28c_2 dp28d_2 dpd4a9);
                     %multi(depd4,dpd4sk dpd4a1-dpd4a9 dpd4ax);
                     %multi(depd4,dpd4sk dpd4ax dp30a_2 dp30a1_2 dp30b_2 dpd4a);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depd4,mdd4sk dcd4a dpd4a mdd4a);

               ***************;
               * Criterion C *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depd4,dcd4sk dp32a1_1 dp32a2_1 dp32a3_1 dp32a4_1 
                                  dcd4c);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depd4,dpd4sk dp32a1_2 dp32a2_2 dp32a3_2 dp32a4_2 
                                  dpd4c);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depd4,mdd4sk dcd4c dpd4c mdd4c);

               ***************;
               * Criterion D *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depd4,dcd4sk dp4d dp4g dp5a_1-dp5a_6 dp5b_1-dp5b_6
                                  dp6a dp31b_1 dp31c_1 dcd4d);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depd4,dpd4sk dp11d dp11g dp12a_1-dp12a_6 dp12b_1-dp12b_6 
                                  dp13a dp31b_2 dp31c_2 dp14a dp14a1 dp14a2 
                                  dpd4d);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depd4,mdd4sk dcd4d dpd4d mdd4d);

               ***************;
               * Criterion E *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depd4,dcd4sk dp31a_1 dp31a1_1 dp30d_1 dp23_1 dp24_1 
                                  dp25_1 dp28a_1 dp28b_1 dp28c_1 dp28d_1 dcd4e);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depd4,dpd4sk dp31a_2 dp31a1_2 dp30d_2 dp23_2 dp24_2 
                                  dp25_2 dp28a_2 dp28b_2 dp28c_2 dp28d_2 dpd4e);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depd4,mdd4sk dcd4e dpd4e mdd4e);

               *************;
               * Diagnosis *;
               *************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depd4,dcd4sk dcd4a dcd4c dcd4d dcd4e dcd4dx);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depd4,dpd4sk dpd4a dpd4c dpd4d dpd4e dpd4dx);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depd4,mdd4sk mdd4a mdd4c dpd4d dpd4e mdd4dx);
                     %multi(depd4,mdd4sk dcd4dx dpd4dx mdd4dx);

            ******************;
            * keep variables *;
            ******************;

               data depd4;
               set depd4;
               keep ind_id  
                    dcd4sk dpd4sk mdd4sk
                    dcd4a1-dcd4a9 dcd4ax dcd4a
                    dpd4a1-dpd4a9 dpd4ax dpd4a
                    dcd4c dpd4c 
                    dcd4d dpd4d
                    dcd4e dpd4e
                    dcd4dx dpd4dx 
                    mdd4a mdd4c mdd4d mdd4e mdd4dx;

               run;

            ************;
            * contents *;
            ************;

               proc contents data=depd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend depd4;

********;
* test *;
********;
/*
   %depd4(coga.csaga2,1);
*/
