****************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Substance Abuse and Dependence *;
****************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %drgd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro drgd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DRGD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DRGD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      %let drglst=co st op ha se ot;
      %let wrdlst=COC AMP OPI HAL SED OTH;

      data drgd4;
      set &dsn.;

      %do i=1 %to 6;

         %let drg=%scan(&drglst.,&i.);
         %let wrd=%scan(&wrdlst.,&i.);

         attrib &drg.d4sk
                &drg.4da1-&drg.4da7 &drg.4dax &drg.4da &drg.4db &drg.d4dp
                &drg.4aa1-&drg.4aa4 &drg.4aax &drg.4aa &drg.4ab &drg.d4ab 
                length=3;
 
         *********************;
         * test for skip out *;
         *********************;

            %if &study.=irpg or &who.=c %then %do;
               if sum(dr1a_&i. not in (1,5),
                      dr1e1_&i. not in (1,5),
                      dr5a_&i.<0,
                      dr5a1_&i. not in (0,1,2,3,4))=4 then &drg.d4sk=5;
               else do;
                  if (dr1a_&i.=5 and dr1e1_&i.=5 and 
                      (dr5a_&i.>6 or dr5a1_&i. in (1,2,3,4))) then &drg.d4sk=1;
                  else &drg.d4sk=5;
               end;
            %end;
            %if &study.=coga and &who.=p %then %do;
               if sum(dr1e1_&i. not in (1,5),
                      dr5a_&i.<0,
                      dr5a1_&i. not in (0,1,2,3,4))=4 then &drg.d4sk=5;
               else do;
                  if (dr1e1_&i.=5 and 
                      (dr5a_&i.>6 or dr5a1_&i. in (1,2,3,4))) then &drg.d4sk=1;
                  else &drg.d4sk=5;
               end;
            %end;

            if &drg.d4sk=5 then do;
               array skp&i. &drg.4da1-&drg.4da7 &drg.4dax &drg.4da &drg.4db 
                            &drg.d4dp &drg.4aa1-&drg.4aa4 &drg.4aax &drg.4aa 
                            &drg.4ab &drg.d4ab ;
               do over skp&i.;
                  skp&i.=.S;
               end;
            end;

         **********************************;
         * test for coga parent interview *;
         **********************************;
   
            %if &study.=coga %then %do;
               %if &who.=p %then %do;
                  if &drg.d4sk=1 then do;
                     array nas&i. &drg.4da1-&drg.4da5 &drg.4da7 &drg.4dax 
                                  &drg.4da &drg.4db &drg.d4dp;
                     do over nas&i.;
                        nas&i.=.N;
                     end;
                  end;
                  if &drg.d4sk=5 then go to skipout&i.;
                  else go to parhop1&i.;
               %end;
            %end;

         if &drg.d4sk=5 then go to skipout&i.;

         **************;
         * Dependence *;
         **************;

            ***************;
            * Criterion A *;
            ***************;

               ********************;
               * marked tolerance *;
               ********************;

                  if sum(dr11a_&i. not in (1,5),
                         dr11b_&i. not in (1,5))=2 then &drg.4da1=.;
                  else &drg.4da1=(4*(dr11a_&i.=5 or dr11b_&i.=5))+1;

               ********************************************;
               * characteristic withdrawal symptoms or    *;
               * use to relieve/avoid withdrawal symptoms *;
               ********************************************;

                  if 0<dm3a<13 then do;
                     &drg.4da2=.N;
                  end;
                  else do;
                     if sum(dr33b1_&i. not in (1,5),
                            dr33d2_&i. not in (1,5))=2 then &drg.4da2=.;
                     else &drg.4da2=(4*(dr33b1_&i.=5 or dr33d2_&i.=5))+1;
                  end;

               *********************************************************;
               * used larger amount or for longer period than intended *;
               *********************************************************;

                  if sum(dr13b_&i. not in (1,5),
                         dr13d_&i. not in (1,5))=2 then &drg.4da3=.;
                  else &drg.4da3=(4*(dr13b_&i.=5 or dr13d_&i.=5))+1;
   
               ***************************************************************;
               * persistent desire or 1+ unsuccessful efforts to control use *;
               ***************************************************************;

                  if sum(dr12a_&i. not in (1,5),
                         dr12c_&i. not in (1,5))=2 then &drg.4da4=.;
                  else &drg.4da4=(4*(dr12a_&i.=5 or dr12c_&i.=5))+1;

               ****************************************************************;
               * great deal of time spent obtaining/using/recovering from use *;
               ****************************************************************;

                  if dr14_&i. not in (1,5) then &drg.4da5=.;
                  else &drg.4da5=(4*(dr14_&i.=5))+1;

               **************************************;
               * jump here if coga parent interview *;
               **************************************;

                  parhop1&i.:

               ***********************************************************;
               * use reduced social/occupational/recreational activities *;
               ***********************************************************;

                  if dr15c_&i. not in (1,5) then &drg.4da6=.;
                  else &drg.4da6=(4*(dr15c_&i.=5))+1;

               **********************************;
               * test for coga parent interview *;
               **********************************;

                  %if &study.=coga %then %do;
                     %if &who.=p %then %do;
                        go to parhop2&i.;
                     %end;
                  %end;

               **********************************************************;
               * use despite use-derived social/pysch/physical problems *;
               **********************************************************;

                  if 0<dm3a<13 then do;
                     &drg.4da7=.N;
                  end;
                  else do;
                     if dr28e_&i. not in (1,5) then &drg.4da7=.;
                     else &drg.4da7=(4*(dr28e_&i.=5))+1;
                  end;

               ****************;
               * sum of items *;
               ****************;

                  if n(of &drg.4da1-&drg.4da7)=0 then &drg.4dax=.;
                  else do;
                     &drg.4dax=0;
                     array sxs&i. &drg.4da1-&drg.4da7;
                     do over sxs&i.;
                        if sxs&i.=5 then &drg.4dax+1;
                     end;
                  end;

               ************************;
               * at least three items *;
               ************************;

                  if &drg.4dax=. then &drg.4da=.;
                  else &drg.4da=(4*(&drg.4dax>2))+1;

            ***************;
            * Criterion B *;
            ***************;

               if 0<dm3a<13 then do;
                  &drg.4db=.N;
               end;
               else do;
                  if dr35a_&i. not in (1,5) then &drg.4db=.;
                  else &drg.4db=(4*(dr35a_&i.=5))+1;
               end;
                      
            *************;
            * Diagnosis *;
            *************;
   
               if n(&drg.4da,&drg.4db)=0 then &drg.4dp=.;
               else do;
                  if 0<dm3a<13 then &drg.d4dp=(4*(&drg.4da=5))+1;
                  else &drg.d4dp=(4*(&drg.4da=5 and &drg.4db=5))+1;
               end;

         **************************************;
         * jump here if coga parent interview *;
         **************************************;

            parhop2&i.:

         *********;
         * Abuse *;
         *********;

            ***************;
            * Criterion A *;
            ***************;

               *******************************************************;
               * intoxication/withdrawal interferes with obligations *;
               *******************************************************;

                  if dr24g_&i. not in (1,5) then &drg.4aa1=.;
                  else &drg.4aa1=(4*(dr24g_&i.=5))+1;

               ****************************************************;
               * recurrent use in physically dangerous situations *;
               ****************************************************;

                  if sum(dr18d1_&i. not in (1,5),
                         dr18f1_&i. not in (1,5),
                         dr27c_&i. not in (1,5))=3 then &drg.4aa2=.;
                  else &drg.4aa2=(4*(sum(dr18d1_&i.=5,dr18f1_&i.=5,
                                         dr27c_&i.=5)>0))+1;

               ****************************************;
               * recurrent use-related legal problems *;
               ****************************************;

                  if dr26b1_&i. not in (1,5) then &drg.4aa3=.;
                  else &drg.4aa3=(4*(dr26b1_&i.=5))+1;

               **********************************************************;
               * use despite use-derived social/pysch/physical problems *;
               **********************************************************;

                  %if &who.=p %then %do;
                     &drg.4aa4=.N;
                  %end;
                  %else %do;
                     if dr25f_&i. not in (1,5) then &drg.4aa4=.;
                     else &drg.4aa4=(4*(dr25f_&i.=5))+1;
                  %end;

               ****************;
               * sum of items *;
               ****************;

                  if n(of &drg.4aa1-&drg.4aa4)=0 then &drg.4aax=.;
                  else do;
                     &drg.4aax=0;
                     array axs&i. &drg.4aa1-&drg.4aa4;
                     do over axs&i.;
                        if axs&i.=5 then &drg.4aax+1;
                     end;
                  end;

               *********************;
               * at least one item *;
               *********************;

                  if &drg.4aax=. then &drg.4aa=.;
                  else &drg.4aa=(4*(&drg.4aax>0))+1;

            ***************;
            * Criterion B *;
            ***************;

               if &drg.d4dp=. then &drg.4ab=.;
               else &drg.4ab=&drg.d4dp;

            *************;
            * Diagnosis *;
            *************;

               if n(&drg.4aa,&drg.4ab)=0 then &drg.d4ab=.;
               %if &who.=p %then %do;
                  else &drg.d4ab=(4*(&drg.4aa=5))+1;
               %end;
               %else %do;
                  else &drg.d4ab=(4*(&drg.4aa=5 and &drg.4ab=1))+1;
               %end;

         ****************************;
         * jump to here if skip out *;
         ****************************;

            skipout&i.:

         *******************;
         * variable labels *;
         *******************;

            label &drg.d4sk = "DSM4 &wrd. Skip Out"
                  &drg.4da1 = "DSM4 &wrd. Dependence Crit. A1"
                  &drg.4da2 = "DSM4 &wrd. Dependence Crit. A2"
                  &drg.4da3 = "DSM4 &wrd. Dependence Crit. A3"
                  &drg.4da4 = "DSM4 &wrd. Dependence Crit. A4" 
                  &drg.4da5 = "DSM4 &wrd. Dependence Crit. A5" 
                  &drg.4da6 = "DSM4 &wrd. Dependence Crit. A6" 
                  &drg.4da7 = "DSM4 &wrd. Dependence Crit. A7" 
                  &drg.4dax = "DSM4 &wrd. Dependence Sum of As" 
                  &drg.4da  = "DSM4 &wrd. Dependence 3+ As" 
                  &drg.4db  = "DSM4 &wrd. Dependence Crit. B" 
                  &drg.d4dp = "DSM4 &wrd. Dependence Diagnosis"
                  &drg.4aa1 = "DSM4 &wrd. Abuse Crit. A1"
                  &drg.4aa2 = "DSM4 &wrd. Abuse Crit. A2"
                  &drg.4aa3 = "DSM4 &wrd. Abuse Crit. A3"
                  &drg.4aa4 = "DSM4 &wrd. Abuse Crit. A4"
                  &drg.4aax = "DSM4 &wrd. Abuse Sum of As"
                  &drg.4aa  = "DSM4 &wrd. Abuse 1+ As"
                  &drg.4ab  = "DSM4 &wrd. Abuse Crit. B"
                  &drg.d4ab = "DSM4 &wrd. Abuse Diagnosis";

      %end;

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  &drg.d4sk
                  &drg.4da1-&drg.4da7 &drg.4dax &drg.4da &drg.4db &drg.d4dp
                  &drg.4aa1-&drg.4aa4 &drg.4aax &drg.4aa &drg.4ab &drg.d4ab 

               %end;
               ;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_drg_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  ************;
                  * skip out *;
                  ************;

                     %if &study.=irpg or &who.=c %then %do;
                        %multi(drgd4,dr1a_&i. dr1e1_&i. dr5a_&i. dr5a1_&i.
                                     &drg.d4sk);
                     %end;
                     %if &study.=coga and &who.=p %then %do;
                        %multi(drgd4,dr1e1_&i. dr5a_&i. dr5a1_&i. &drg.d4sk);
                     %end;

                  **************;
                  * Dependence *;
                  **************;

                     ***************;
                     * Criterion A *;
                     ***************;

                        %multi(drgd4,&drg.d4sk dr11a_&i. dr11b_&i. &drg.4da1);
                        %multi(drgd4,&drg.d4sk dr33b1_&i. dr33d2_&i. &drg.4da2);
                        %multi(drgd4,&drg.d4sk dr13b_&i. dr13d_&i. &drg.4da3);
                        %multi(drgd4,&drg.d4sk dr12a_&i. dr12c_&i. &drg.4da4);
                        %multi(drgd4,&drg.d4sk dr14_&i. &drg.4da5);
                        %multi(drgd4,&drg.d4sk dr15c_&i. &drg.4da6);
                        %multi(drgd4,&drg.d4sk dr28e_&i. &drg.4da7);
                        %multi(drgd4,&drg.d4sk &drg.4da1-&drg.4da7 &drg.4dax);
                        %multi(drgd4,&drg.d4sk &drg.4dax &drg.4da);

                     ***************;
                     * Criterion B *;
                     ***************;

                        %multi(drgd4,&drg.d4sk dr35a_&i. &drg.4db);

                     *************;
                     * Diagnosis *;
                     *************;

                        %multi(drgd4,&drg.d4sk &drg.4da &drg.4db &drg.d4dp);

                  *********;
                  * Abuse *;
                  *********;

                     ***************;
                     * Criterion A *;
                     ***************;

                        %multi(drgd4,&drg.d4sk dr24g_&i. &drg.4aa1);
                        %multi(drgd4,&drg.d4sk dr18d1_&i. dr18f1_&i. dr27c_&i. 
                                     &drg.4aa2);
                        %multi(drgd4,&drg.d4sk dr26b1_&i. &drg.4aa3);
                        %multi(drgd4,&drg.d4sk dr25f_&i. &drg.4aa4);
                        %multi(drgd4,&drg.d4sk &drg.4aa1-&drg.4aa4 &drg.4aax);
                        %multi(drgd4,&drg.d4sk &drg.4aax &drg.4aa);

                     ***************;
                     * Criterion B *;
                     ***************;

                        %multi(drgd4,&drg.d4sk &drg.d4dp &drg.4ab);

                     *************;
                     * Diagnosis *;
                     *************;

                        %multi(drgd4,&drg.d4sk &drg.4aa &drg.4ab &drg.d4ab);

               %end;

            ******************;
            * keep variables *;
            ******************;

               data drgd4;
               set drgd4;
               keep ind_id  
               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  &drg.d4sk
                  &drg.4da1-&drg.4da7 &drg.4dax &drg.4da &drg.4db &drg.d4dp
                  &drg.4aa1-&drg.4aa4 &drg.4aax &drg.4aa &drg.4ab &drg.d4ab 

               %end;
               ;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=drgd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend drgd4;

********;
* test *;
********;
/*
   %drgd4(coga.csaga2,1);

   proc freq data=drgd4;
   tables cod4dp std4dp opd4dp had4dp sed4dp otd4dp / list missing;
   tables cod4ab std4ab opd4ab had4ab sed4ab otd4ab / list missing;
   run;
*/
