*********************************************************;
* CSSAGA-II Diagnostic Algorithm -- DSM-IV -- Dysthymia *;
*********************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*********************************************************;
* uncomment next four lines to run as standalone module *;
*********************************************************;
/*
   %include "./dsm4_dep.sas" / nosource2;
   %include "./dsm4_man.sas" / nosource2;

   %depd4(coga.csaga2,0);
   %mand4(coga.csaga2,0);
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %dysd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro dysd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DYSD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DYSD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      proc contents data=&dsn. noprint out=test;
      run;

      data test;
      set test;
      if upcase(substr(name,1,2))="DY";
      run;

      data test;
      set test nobs=count;
      call symput("var"||left(_n_),name);
      call symput("nvar",left(put(count,8.)));
      run;

      data dysd4;
      merge &dsn.(keep=ind_id dm3a dm3b_1-dm3b_3 
                       dp2b_2 dp9_2 dp16_2
                       dp30c1_1 dp30c1_2 dp30c2_1 dp30c2_2 dp30d_1 dp30d_2
                       %do i=1 %to &nvar.;
                          &&var&i.
                       %end;
                       ) 
            depd4(keep=ind_id dcd4sk dpd4sk dcd4dx dpd4dx)
            mand4(keep=ind_id mnd4dx);
      by ind_id;

      attrib dyd4sk1 dyd4sk2 dyd4sk 
             dyd4a dyd4b1-dyd4b6 dyd4bx dyd4b
             dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h dyd4dx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas dyd4sk1 dyd4sk2 dyd4sk 
                      dyd4a dyd4b1-dyd4b6 dyd4bx dyd4b
                      dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h dyd4dx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(dcd4sk not in (1,5),
                dpd4sk not in (1,5),
                sum(dp2b_2 not in (1,2,3,4),
                    dp9_2 not in (1,2,3,4),
                    dp16_2 not in (1,2,3,4))=3)=3 then dyd4sk1=5;
         else do;
            if sum((dcd4sk=1 or dpd4sk=1),
                   sum(dp2b_2=4,dp9_2=4,dp16_2=4)=0)=2 then dyd4sk1=1;
            else dyd4sk1=5;
         end;

         if dyd4sk1=5 then do;
            array skp1 dyd4a dyd4b1-dyd4b6 dyd4bx dyd4b
                       dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h dyd4dx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if dyd4sk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum((dy1 not in (1,5)),
                   (dy3 not in (1,5)))=2 then dyd4a=.;
            else dyd4a=sum(dy1=5,dy3=5)>0;


      *********************;
      * test for skip out *;
      *********************;
 
         if sum(dy1 not in (1,5),
                dy2 not in (1,5),
                dy3 not in (1,5))=3 then dyd4sk2=5;
         else do;
            if sum(dy1=5,dy2=5,dy3=5)>0 then dyd4sk2=1;
            else dyd4sk2=5;
         end;

         if dyd4sk2=5 then do;
            array skp2 dyd4b1-dyd4b6 dyd4bx dyd4b
                       dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h dyd4dx;
            do over skp2;
              skp2=.S;
            end;
         end;

         if dyd4sk2=5 then go to skipout;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if sum((dy5a not in (1,5)),(dy5b not in (1,5)))=2 then dyd4b1=.;
               else dyd4b1=sum(dy5a=5,dy5b=5)>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dy4a not in (1,5)),(dy4b not in (1,5)),
                      (dy4c not in (1,5)),(dy4d not in (1,5)))=4 then dyd4b2=.;
               else dyd4b2=sum(dy4a=5,dy4b=5,dy4c=5,dy4d=5)>0;

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if (dy6 not in (1,5)) then dyd4b3=.;
               else dyd4b3=(dy6=5);

            *******************;
            * low self-esteem *;
            *******************;

               if (dy7 not in (1,5)) then dyd4b4=.;
               else dyd4b4=(dy7=5);

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dy9a not in (1,5)),(dy9b not in (1,5)))=2 then dyd4b5=.;
               else dyd4b5=sum(dy9a=5,dy9b=5)>0;

            ****************************;
            * feelings of hopelessness *;
            ****************************;

               if (dy8 not in (1,5)) then dyd4b6=.;
               else dyd4b6=(dy8=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of dyd4b1-dyd4b6)=0 then dyd4bx=.;
               else dyd4bx=sum(of dyd4b1-dyd4b6);

            ************************;
            * at least two present *;
            ************************;

               if dyd4bx=. then dyd4b=.;
               else dyd4b=(dyd4bx>1);

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(dy12a not in (1,5),dy12b not in (1,2,3,4))=2 then dyd4c=.;
            else dyd4c= sum((dy12a=1),(dy12a=5 and dy12b in (1,2,3)))>0;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum((n(dcd4dx,dpd4dx)=0),
                   (dm3a<0 and min(of dm3b_1-dm3b_3)<0),
                   (sum(dp30c1_1<0,dp30c1_2<0,dp30c2_1<0,dp30c2_2<0)=4 and
                    sum(dp30d_1<0,dp30d_2<0)=2),
                   (n(dy11a_1,dy11a_2)=0))=4 then dyd4d=.;
            else do;
               if max(dcd4dx,dpd4dx)=1 then dyd4d=1;
               else do;
                  if n(dy11a_1,dy11a_2)=0 then dyd4d=.;
                  else do;
                     if sum(dp30c1_1<0,dp30c2_1<0,
                            dp30c1_2<0,dp30c2_2<0)>0 then dyd4d=.;
                     else do;
                        if min(of dm3b_1-dm3b_3)<0 then dyd4d=.;
                        else do;
                           if (dp30c2_2<1 and dp30c2_1>0) then 
                              dyd4d=(int((mdy(dp30c1_1,1,dp30c2_1)-
                                           mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                      dy11a_1)>1 or
                                     (dy11a_1>int(((mdy(dp30c1_1,1,dp30c2_1)-
                                                    mdy(dm3b_1,dm3b_2,dm3b_3))+
                                                   (dp30d_1*7))/365.25));       
                           else if (dp30c2_2>0 and dp30c2_1<1) then
                              dyd4d=(int((mdy(dp30c1_2,1,dp30c2_2)-
                                           mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                      dy11a_1)>1 or
                                     (dy11a_1>int(((mdy(dp30c1_2,1,dp30c2_2)-
                                                    mdy(dm3b_1,dm3b_2,dm3b_3))+
                                                   (dp30d_1*7))/365.25));
                           else do;
                              dyd4d=(int((mdy(dp30c1_2,1,dp30c2_2)-
                                           mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                      dy11a_1)>1 or
                                     ((dy11a_1>int(((mdy(dp30c1_2,1,dp30c2_2)-
                                                     mdy(dm3b_1,dm3b_2,dm3b_3))+
                                                    (dp30d_1*7))/365.25)) and
                                      (int((mdy(dp30c1_1,1,dp30c2_1)-
                                            mdy(dm3b_1,dm3b_2,dm3b_3))/365.25)-
                                       dy11a_1)>1);
                           end;
                        end;
                     end;
                  end;
               end;
            end;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            dyd4e=(mnd4dx not in (3,5));

      ***************;
      * Criterion F *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            dyd4f=.;

      ***************;
      * Criterion G *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum((dy10a not in (1,5)),
                   (dy10b not in (1,5)))=2 then dyd4g=.;
            else dyd4g=max(dy10a,dy10b)=1;

      ***************;
      * Criterion H *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(of dy13b_1-dy13b_7)=0 then dyd4h=.;
            else do;
               sum2s=0;
               sum3s=0;
               array two dy13b_1-dy13b_7;
               do over two;
                  if two=2 then sum2s+1;
                  if two=3 then sum3s+1;
               end;
               dyd4h=(sum2s>1 or sum3s>0);
            end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

           if n(dyd4a,dyd4b,dyd4c,dyd4d,dyd4e,dyd4f,dyd4g,dyd4h)=0 then
              dyd4dx=.;
           else do;
              dyd4dx=1;
              if sum(dyd4a,dyd4b,dyd4c,dyd4d,dyd4e,dyd4h)=6 then do;
                 if dyd4g=0 then dyd4dx=3;
                 else if dyd4g=1 then dyd4dx=5;
              end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         dyd4sk=max(dyd4sk1,dyd4sk2);

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         if dyd4sk1=5 then do;
            array one dyd4a dyd4b1-dyd4b6 dyd4b
                      dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h;
            do over one;
               if one=1 then one=5;
               if one=0 then one=1;
            end;
         end;
         else do;
            if dyd4a=1 then dyd4a=5;
            if dyd4a=0 then dyd4a=1;
         end;

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label dyd4sk1 = "DSM4 Dysthymia Skip Out #1"
               dyd4sk2 = "DSM4 Dsythymia Skip Out #2"
               dyd4sk = "DSM4 Dsythymia Skip Out"
               dyd4a  = "DSM4 Dysthymia Crit. A"
               dyd4b1 = "DSM4 Dysthymia Crit. B1"
               dyd4b2 = "DSM4 Dysthymia Crit. B2"
               dyd4b3 = "DSM4 Dysthymia Crit. B3"
               dyd4b4 = "DSM4 Dysthymia Crit. B4"
               dyd4b5 = "DSM4 Dysthymia Crit. B5"
               dyd4b6 = "DSM4 Dysthymia Crit. B6"
               dyd4bx = "DSM4 Dysthymia Sum + Bs"
               dyd4b  = "DSM4 Dysthymia Crit. B"
               dyd4c  = "DSM4 Dysthymia Crit. C"
               dyd4d  = "DSM4 Dysthymia Crit. D"
               dyd4e  = "DSM4 Dysthymia Crit. E"
               dyd4f  = "DSM4 Dysthymia Crit. F"
               dyd4g  = "DSM4 Dysthymia Crit. G"
               dyd4h  = "DSM4 Dysthymia Crit. H"
               dyd4dx = "DSM4 Dysthymia Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    dyd4sk1 dyd4sk2 dyd4sk
                    dyd4a dyd4b1-dyd4b6 dyd4bx dyd4b
                    dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h dyd4dx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_dys_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(dysd4,dcd4sk dpd4sk dp2b_2 dp9_2 dp16_2 dyd4sk1);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dy1 dy3 dyd4a);

               ************;
               * skip out *;
               ************;

                  %multi(dysd4,dyd4sk1 dy1 dy2 dy3 dyd4sk2);
                  %multi(dysd4,dyd4sk1 dyd4sk2 dyd4sk);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 dy5a dy5b dyd4b1);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dy4a dy4b dy4c dy4d dyd4b2);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dy6 dyd4b3);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dy7 dyd4b4);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dy9a dy9b dyd4b5);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dy8 dyd4b6);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dyd4b1-dyd4b6 dyd4bx);
                     %multi(dysd4,dyd4sk1 dyd4sk2 dyd4bx dyd4b);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 dy12a dy12b dyd4c);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 dcd4dx dpd4dx dy11a_1 dy11a_2
                                  dp30c1_1 dp30c2_1 dp30c1_2 dp30c2_2 
                                  dm3a dm3b_1 dm3b_2 dm3b_3
                                  dyd4d);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 mnd4dx dyd4e);

               ***************;
               * Criterion G *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 dy10a dy10b dyd4g);

               ***************;
               * Criterion H *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 dy13b_1-dy13b_7 dyd4h);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(dysd4,dyd4sk1 dyd4sk2 dyd4a dyd4b dyd4c dyd4d 
                                  dyd4e dyd4g dyd4h dyd4dx);

            ******************;
            * keep variables *;
            ******************;

               data dysd4;
               set dysd4;
               keep ind_id  
                    dyd4sk1 dyd4sk2 dyd4sk
                    dyd4a dyd4b1-dyd4b6 dyd4bx dyd4b
                    dyd4c dyd4d dyd4e dyd4f dyd4g dyd4h dyd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=dysd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend dysd4;

********;
* test *;
********;
/*
   %dysd4(coga.csaga2,1);
*/
