**************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Generalized Anxiety Disorder *;
**************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %gadd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro gadd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in GADD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in GADD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data gadd4;
      set &dsn.;

      attrib gad4sk 
             gad4a gad4b gad4c1-gad4c6 gad4c3x gad4c5x gad4cx gad4c
             gad4e gad4dx length=3;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(ov2a not in (1,5),
                ov2b not in (1,3,5),
                ov3a not in (1,5),
                ov3b not in (1,3,5))=4 then gad4sk=5;
         else do;
            if sum((ov2a ne 5 and ov3a ne 5), 
                   (ov2b not in (3,5) and ov3b not in (3,5)))=2 then gad4sk=5;
            else gad4sk=1;
         end;

         if gad4sk=5 then do;
            array skp1 gad4a gad4b gad4c1-gad4c6 gad4c3x gad4c5x gad4cx gad4c
                       gad4e gad4dx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if gad4sk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ********************************;
            * excessive, unrealistic worry *;
            ********************************;

               if sum(ov2b not in (1,3,5),
                      ov3b not in (1,3,5),
                      ov4a not in (1,3,5),
                      ov4b not in (1,3,5),
                      sum(ov17b not in (1,5),
                          ov19b not in (1,5))>0)=5 then gad4a=.;
               else do;
                  gad4a=1;
                  if ((ov2b=3 or ov3b=3 or ov4a=3 or ov4b=3) and
                      ov19b=5) then gad4a=3;
                  else if ((ov2b=5 or ov3b=5 or ov4a=5 or ov4b=5) and
                           ov17b=5) then gad4a=5;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ******************************;
            * difficult to control worry *;
            ******************************;

               if ov9 not in (1,3,5) then gad4b=.;
               else gad4b=(4*(ov9=5))+(2*(ov9=3))+1; 
 
      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * restlessness, keyed-up, on edge *;
            ***********************************;

               if ov10 not in (1,3,5) then gad4c1=.;
               else gad4c1=(4*(ov10=5))+(2*(ov10=3))+1;

            *******************;
            * easily fatigued *;
            *******************;

               if ov11 not in (1,3,5) then gad4c2=.;
               else gad4c2=(4*(ov11=5))+(2*(ov11=3))+1;

            ****************************;
            * difficulty concentrating *;
            ****************************;

               if sum(ov12a not in (1,3,5),
                      ov12b not in (1,3,5))=2 then gad4c3=.;
               else do;
                  gad4c3=1;
                  if (ov12a=3 or ov12b=3) then gad4c3=3;
                  else if (ov12a=5 or ov12b=5) then gad4c3=5;
               end;

            ****************;
            * irritability *;
            ****************;

               if ov13 not in (1,3,5) then gad4c4=.;
               else gad4c4=(4*(ov13=5))+(2*(ov13=3))+1;

            ******************;
            * muscle tension *;
            ******************;

               if ov14 not in (1,3,5) then gad4c5=.;
               else gad4c5=(4*(ov14=5))+(2*(ov14=3))+1;

            *********************;
            * sleep disturbance *;
            *********************;

               if sum(ov15a not in (1,3,5),
                      ov15b not in (1,3,5),
                      ov15c not in (1,3,5))=3 then gad4c6=.;
               else do;
                  gad4c6=1;
                  if (ov15a=3 or ov15b=3 or ov15c=3) then gad4c6=3;
                  else if (ov15a=5 or ov15b=5 or ov15c=5) then gad4c6=5;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of gad4c1-gad4c6)=0 then gad4cx=.;
               else do;
                  gad4c3x=0;
                  gad4c5x=0;
                  array sxx gad4c1-gad4c6;
                  do over sxx;
                     if sxx=3 then gad4c3x=gad4c3x+1;
                     if sxx=5 then gad4c5x=gad4c5x+1;
                  end;
                  gad4cx=sum(gad4c3x,gad4c5x);
               end;

            *********************;
            * at least one item *;
            *********************;

               if gad4cx=. then gad4c=.;
               else gad4c=(4*(gad4cx>0))+1;

      *******************************;
      * Criterion D -- Not assessed *;
      *******************************;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * clinically significant distress *;
            ***********************************;

               if sum(ov21a_1a not in (1,3,5),
                      ov21a_2a not in (1,3,5),
                      ov21a_3a not in (1,3,5),
                      ov21a_4a not in (1,3,5),
                      ov21a_5a not in (1,3,5))=5 then gad4e=.;
               else do;
                  gad4e=1;
                  opt1_a=0;
                  opt1_b=0;
                  opt1_c=0;
                  opt2_a=0;
                  opt2_b=0;
                  opt2_c=0;
                  array col1{*} ov21a_1a ov21a_2a ov21a_3a ov21a_4a ov21a_5a;
                  array col2{*} ov21a_1b ov21a_2b ov21a_3b ov21a_4b ov21a_5b;
                  do i=1 to dim(col1);
                     if col1(i)=3 then do;
                        opt2_a+1;
                        if col2(i)=3 then opt2_b+1;
                        if col2(i)=2 then opt2_c+1;
                     end;
                     else if col1(i)=5 then do;
                        opt1_a+1;
                        if col2(i)=3 then opt1_b+1;
                        if col2(i)=2 then opt1_c+1;
                     end;
                  end;
                  if ((opt2_a>0 and opt2_b>0) or
                      (opt2_a>1 and opt2_c>1)) then gad4e=3;
                  else if ((opt1_a>0 and opt1_b>0) or
                           (opt1_a>1 and opt1_c>1)) then gad4e=5;
               end;

      ***************************************************;
      * Criterion F -- Assessed within individual items *;
      ***************************************************;
 
      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(gad4a,gad4b,gad4c,gad4e)=0 then gad4dx=.;
            else do;
               gad4dx=1;
               sum3s=0;
               sum5s=0;
               array sxs gad4a gad4b gad4e;
               do over sxs;
                  if sxs=3 then sum3s=sum3s+1;
                  if sxs=5 then sum5s=sum5s+1;
               end;
               if sum5s=3 and gad4c5x>0 then gad4dx=5;
               else if (sum3s-sum5s)>0 and gad4cx>0 then gad4dx=3;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label gad4sk  = "DSM4 GAD Skip Out"
               gad4a   = "DSM4 GAD (Lifetime) Crit. A"
               gad4b   = "DSM4 GAD (Lifetime) Crit. B"
               gad4c1  = "DSM4 GAD (Lifetime) Crit. C1"
               gad4c2  = "DSM4 GAD (Lifetime) Crit. C2"
               gad4c3  = "DSM4 GAD (Lifetime) Crit. C3"
               gad4c4  = "DSM4 GAD (Lifetime) Crit. C4"
               gad4c5  = "DSM4 GAD (Lifetime) Crit. C5"
               gad4c6  = "DSM4 GAD (Lifetime) Crit. C6"
               gad4c3x = "DSM4 GAD (Lifetime) Sum of 3s"
               gad4c5x = "DSM4 GAD (Lifetime) Sum of 5s"
               gad4cx  = "DSM4 GAD (Lifetime) Sum of Cs"
               gad4c   = "DSM4 GAD (Lifetime) Crit. C"
               gad4e   = "DSM4 GAD (Lifetime) Crit. E"
               gad4dx  = "DSM4 GAD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id   
                    gad4sk
                    gad4a gad4b gad4c1-gad4c6 gad4c3x gad4c5x gad4cx gad4c
                    gad4e gad4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_gad_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(gadd4,ov2a ov2b ov3a ov3b gad4sk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadd4,gad4sk ov2b ov3b ov4a ov4b ov17b ov19b gad4a);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadd4,gad4sk ov9 gad4b);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadd4,gad4sk ov10 gad4c1);
                     %multi(gadd4,gad4sk ov11 gad4c2);
                     %multi(gadd4,gad4sk ov12a ov12b gad4c3);
                     %multi(gadd4,gad4sk ov13 gad4c4);
                     %multi(gadd4,gad4sk ov14 gad4c5);
                     %multi(gadd4,gad4sk ov15a ov15b ov15c gad4c6);
                     %multi(gadd4,gad4sk gad4c1-gad4c6 gad4c3x gad4c5x);
                     %multi(gadd4,gad4sk gad4c3x gad4c5x gad4cx);
                     %multi(gadd4,gad4sk gad4cx gad4c);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadd4,gad4sk ov21a_1a ov21a_1b ov21a_2a ov21a_2b 
                                  ov21a_3a ov21a_3b ov21a_4a ov21a_4b
                                  ov21a_5a ov21a_5b gad4e);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(gadd4,gad4sk gad4a gad4b gad4c gad4e gad4dx);

            ******************;
            * keep variables *;
            ******************;

               data gadd4;
               set gadd4;
               keep ind_id  
                    gad4sk
                    gad4a gad4b gad4c1-gad4c6 gad4c3x gad4c5x gad4cx gad4c
                    gad4e gad4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=gadd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend gadd4;

********;
* test *;
********;
/*
   %gadd4(coga.csaga2,1);
*/
