*****************************************************;
* CSSAGA-II Diagnostic Algorithm -- DSM-IV -- Mania *;
*****************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %mand4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro mand4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in MAND4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in MAND4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data mand4;
      set &dsn.;

      attrib mnd4sk
             mnd4a 
             mnd4b1-mnd4b7 mnd4bx mnd4b
             mnd4d mnd4e mnd4dx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas mnd4sk mnd4a 
                      mnd4b1-mnd4b7 mnd4bx mnd4b
                      mnd4d mnd4e mnd4dx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to skipout;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(mn1 not in (1,5),
                mn3a not in (1,5),
                sum(mn8a not in (1,5),mn8b not in (1,5),
                    mn9a not in (1,5),mn9b not in (1,5),mn9c not in (1,5),
                    mn10a not in (1,5),mn10b not in (1,5),
                    mn11 not in (1,5),mn12 not in (1,5),
                    mn13 not in (1,5),mn14 not in (1,5),
                    mn15a not in (1,5),mn15b not in (1,5),mn15c not in (1,5),
                    mn16a not in (1,5),mn16b not in (1,5))=16)=3 then 
            mnd4sk=5;
         else do;
            if sum(mn1=5,mn3a=5,
                   sum(mn8a=5,mn8b=5,
                       mn9a=5,mn9b=5,mn9c=5,
                       mn10a=5,mn10b=5,
                       mn11=5,mn12=5,mn13=5,mn14=5,
                       mn15a=5,mn15b=5,mn15c=5,
                       mn16a=5,mn16b=5)>0)=3 then mnd4sk=1;
            else mnd4sk=5;
         end;

         if mnd4sk=5 then do;
            array skp1 mnd4a mnd4b1-mnd4b7 mnd4bx mnd4b
                       mnd4d mnd4e mnd4dx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if mnd4sk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(mn2a not in (1,5),
                   mn3a not in (1,5),
                   mn18a not in (1,2,3,4,5,6,7),
                   mn17f not in (1,5))=4 then mnd4a=.;
            else mnd4a=((mn2a=5 or mn3a=5) and 
                        (mn18a in (3,4,5,6,7) or mn17f=5));

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***************************************;
            * inflated self-esteem or grandiosity *;
            ***************************************;

               if sum(mn9a not in (1,5),
                      mn9b not in (1,5),
                      mn9c not in (1,5))=3 then mnd4b1=.;
               else mnd4b1=max(mn9a,mn9b,mn9c)=5;

            ****************************;
            * decreased need for sleep *;
            ****************************;

               if (mn8b not in (1,5)) then mnd4b2=.;
               else mnd4b2=(mn8b=5);

            ********************************;
            * talkative/pressure of speech *;
            ********************************;

               if (mn12 not in (1,5)) then mnd4b3=.;
               else mnd4b3=(mn12=5);

            ***********************************;
            * flight of ideas/racing thoughts *;
            ***********************************;

               if (mn13 not in (1,5)) then mnd4b4=.;
               else mnd4b4=(mn13=5);

            *******************;
            * distractibility *;
            *******************;

               if (mn14 not in (1,5)) then mnd4b5=.;
               else mnd4b5=(mn14=5);

            **************************************;
            * increase in goal-directed activity *;
            **************************************;

               if sum(mn10a not in (1,5),
                      mn10b not in (1,5),
                      mn11 not in (1,5))=3 then mnd4b6=.;
               else mnd4b6=max(mn10a,mn10b,mn11)=5;

            **********************************************;
            * excessive involvement in hi-risk activities*;
            **********************************************;

               if sum(mn15a not in (1,5),
                      mn15b not in (1,5),
                      mn15c not in (1,5))=3 then mnd4b7=.;
               else mnd4b7=max(mn15a,mn15b,mn15c)=5;

            ****************;
            * sum of items *;
            ****************;

               if n(of mnd4b1-mnd4b7)=0 then mnd4bx=.;
               else mnd4bx=sum(of mnd4b1-mnd4b7);
 
            **************************;
            * at least three present *;
            **************************;

               if mnd4bx=. then mnd4b=.;
               else mnd4b=(mnd4bx>2);

      *******************************;
      * Criterion C -- Not Assessed *;
      *******************************;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(mn17f not in (1,5),
                   mn20b_1 not in (1,2,3),mn20b_2 not in (1,2,3),
                   mn20b_3 not in (1,2,3),mn20b_4 not in (1,2,3),
                   mn20b_5 not in (1,2,3),mn20b_6 not in (1,2,3),
                   mn20b_7 not in (1,2,3),mn20b_8 not in (1,2,3))=9 then
               mnd4d=.;
            else do;
               sum2s=0;
               sum3s=0;
               array one mn20b_1-mn20b_8;
               do over one;
                  if one=2 then sum2s+1;
                  else if one=3 then sum3s+1;
               end;
               mnd4d=(mn17f=5 or sum2s>3 or sum3s>1 or (sum3s>0 and sum2s>1));
            end;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(mn5c<0,mn5e not in (1,2,3,4,5,6,7),
                   mn6a not in (1,5),mn6b not in (1,5),mn6e not in (1,5),
                   mn7a not in (1,5),mn7c not in (1,5),mn7e not in (1,5),
                   mn18a not in (1,2,3,4,5,6,7),mn17f not in (1,5))=10 then
               mnd4e=.;
            else do;
               mnd4e=((mn5c<3 and mn5e<2) and (mn6a=1 or mn6b=1) and mn6e=1) or
                     ((mn7a=5 or mn7c=5 or mn7e=5) and 
                      (mn18a in (3,4,5,6,7) or mn17f=5));
            end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(mnd4a,mnd4b,mnd4d,mnd4e)=0 then mnd4dx=.;
            else do;
               if (mn7a=1 and mn7c=1 and mn7e=5 and 
                   (mn18a in (3,4,5,6,7) or mn17f=5)) then do;
                  mnd4dx=(5*(mnd4a and mnd4bx>3 and mnd4d and mnd4e))+
                         (3*(mnd4a and mnd4bx>3 and mnd4d and mnd4e=0));
                  if mnd4dx=0 then mnd4dx=1;
               end;
               else do;
                  mnd4dx=(5*(mnd4a and mnd4b and mnd4d and mnd4e))+
                         (3*(mnd4a and mnd4b and mnd4d and mnd4e=0));
                  if mnd4dx=0 then mnd4dx=1;
               end;
            end;

      ***************************;
      * rescale from 0,1 to 1,5 *;
      ***************************;

         array two mnd4a mnd4b1-mnd4b7 mnd4b mnd4d mnd4e;
         do over two;
            if two=1 then two=5;
            if two=0 then two=1;
         end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label mnd4sk = "DSM4 Mania Skip Out"
               mnd4a  = "DSM4 Mania (Lifetime) Crit. A"
               mnd4b1 = "DSM4 Mania (Lifetime) Crit. B1"
               mnd4b2 = "DSM4 Mania (Lifetime) Crit. B2"
               mnd4b3 = "DSM4 Mania (Lifetime) Crit. B3"
               mnd4b4 = "DSM4 Mania (Lifetime) Crit. B4"
               mnd4b5 = "DSM4 Mania (Lifetime) Crit. B5"
               mnd4b6 = "DSM4 Mania (Lifetime) Crit. B6"
               mnd4b7 = "DSM4 Mania (Lifetime) Crit. B7"
               mnd4bx = "DSM4 Mania (Lifetime) Sum of Bs"
               mnd4b  = "DSM4 Mania (Lifetime) Crit. B"
               mnd4d  = "DSM4 Mania (Lifetime) Crit. D"
               mnd4e  = "DSM4 Mania (Lifetime) Crit. E"
               mnd4dx = "DSM4 Mania (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    mnd4sk
                    mnd4a 
                    mnd4b1-mnd4b7 mnd4bx mnd4b
                    mnd4d mnd4e mnd4dx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_man_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(mand4,mn1 mn3a mn8a mn8b mn9a mn9b mn9c
                               mn10a mn10b mn11 mn12 mn13 mn14 
                               mn15a mn15b mn15c mn16a mn16b mnd4sk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(mand4,mnd4sk mn2a mn3a mn18a mn17f mnd4a);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(mand4,mnd4sk mn9a mn9b mn9c mnd4b1);
                     %multi(mand4,mnd4sk mn8b mnd4b2);
                     %multi(mand4,mnd4sk mn12 mnd4b3);
                     %multi(mand4,mnd4sk mn13 mnd4b4);
                     %multi(mand4,mnd4sk mn14 mnd4b5);
                     %multi(mand4,mnd4sk mn10a mn10b mn11 mnd4b6);
                     %multi(mand4,mnd4sk mn15a mn15b mn15c mnd4b7);
                     %multi(mand4,mnd4sk mnd4b1-mnd4b7 mnd4bx);
                     %multi(mand4,mnd4sk mnd4bx mnd4b);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(mand4,mnd4sk mn17f mn20b_1-mn20b_8 mnd4d);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(mand4,mnd4sk mn5c mn5e mn6a mn6b mn6e mn7a mn7c 
                                  mn7e mn18a mn17f mnd4e);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(mand4,mnd4sk mn7a mn7c mn7e mn18a mn17f mnd4bx 
                                  mnd4a mnd4b mnd4d mnd4e mnd4dx);

            ******************;
            * keep variables *;
            ******************;

               data mand4;
               set mand4;
               keep ind_id
                    mnd4sk  
                    mnd4a 
                    mnd4b1-mnd4b7 mnd4bx mnd4b
                    mnd4d mnd4e mnd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=mand4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend mand4;

********;
* test *;
********;
/*
   %mand4(coga.csaga2,1);
*/
