***************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Obsessive/Compulsive Disorder *;
***************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %ocdd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro ocdd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in OCDD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in OCDD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data ocdd4;
      set &dsn.;

      attrib obd4sk cpd4sk ocd4sk 
             obd4a1-obd4a4 obd4a3x obd4a5x obd4ax obd4a obd4c obd4e obd4dx
             cpd4a1 cpd4a2 cpd4a cpd4c cpd4e cpd4dx ocd4dx length=3;

      **************;
      * Obsessions *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            *************************************************;
            * recurrent, persistent ideas/thoughts/impulses *;
            *************************************************;

               if sum(ob1 not in (1,3,5),
                      ob2b not in (1,3,5),
                      ob3c not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob5c not in (1,3,5),
                      ob6b not in (1,3,5))=6 then obd4a1=.;
               else do;
                  obd4a1=1;
                  if sum(ob1=3,ob2b=3,ob3c=3,ob4b=3,ob5c=3,ob6b=3)>0 then 
                     obd4a1=3;
                  else if sum(ob1=5,ob2b=5,ob3c=5,ob4b=5,ob5c=5,ob6b=5)>0 then
                     obd4a1=5;
               end;

            ******************************************************;
            * ideas/thoughts/impulses not simply excessive worry *;
            ******************************************************;

               if sum(ob1 not in (1,3,5),
                      ob2b not in (1,3,5),
                      ob4a not in (1,5),
                      ob4a_2 not in (1,5))=4 then obd4a2=.;
               else do;
                  obd4a2=1;
                  if sum(ob1=3,ob2b=3)>0 then obd4a2=3;
                  else if sum(ob1=5,ob2b=5,ob4a=5,ob4a_2=5)>0 then obd4a2=5;
               end;

            *************************************************;
            * attempts to ignore/suppress thoughts/impulses *;
            *************************************************;

               if sum(ob2b not in (1,3,5),
                      ob3b not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob7 not in (1,3,5))=4 then obd4a3=.;
               else do;
                  obd4a3=1;
                  if sum(ob2b=3,ob3b=3,ob4b=3,ob7=3)>0 then 
                     obd4a3=3;
                  else if sum(ob2b=5,ob3b=5,ob4b=5,ob7=5)>0 then
                     obd4a3=5;
               end;

            *********************;
            * test for skip out *;
            *********************;

               if sum(ob1 not in (1,3,5),
                      ob2b not in (1,3,5),
                      ob3c not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob5c not in (1,3,5),
                      ob6b not in (1,3,5))=6 then obd4sk=5;
               else do;
                  if sum(ob1 in (3,5),
                         ob2b in (3,5),
                         ob3c in (3,5),
                         ob4b in (3,5),
                         ob5c in (3,5),
                         ob6b in (3,5))>0 then obd4sk=1;
                  else obd4sk=5;
               end;

               if obd4sk=5 then do;
                  array skp1 obd4a4 obd4a3x obd4a5x obd4ax obd4a obd4c 
                             obd4e obd4dx;
                  do over skp1;
                     skp1=.S;
                  end;
               end;

               if obd4sk=5 then go to compuls;

            ***************************************************;
            * obsession recognized as from within not without *;
            ***************************************************;

               if ob8a not in (1,5) then obd4a4=.;
               else obd4a4=(4*(ob8a=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of obd4a1-obd4a4)=0 then obd4ax=.;
               else do;
                  obd4a3x=0;
                  obd4a5x=0;
                  array sxs obd4a1-obd4a4;
                  do over sxs;
                     if sxs=3 then obd4a3x=obd4a3x+1;
                     if sxs=5 then obd4a5x=obd4a5x+1;
                  end;
                  obd4ax=sum(obd4a3x,obd4a5x);
                  obd4a=(4*(obd4a3x=0 and obd4a5x=3))+
                        (2*(obd4a3x>0 and obd4ax=3))+1;
               end;

         ******************************;
         * Criterion B - Not assessed *;
         ******************************;

         ***************;
         * Criterion C *;
         ***************;

            ************************************;
            * obsessions cause marked distress *;
            ************************************;

               if sum(ob9b not in (1,3,5),
                      sum(ob14a_1a not in (1,3,5),
                          ob14a_2a not in (1,3,5),
                          ob14a_3a not in (1,3,5),
                          ob14a_4a not in (1,3,5),
                          ob14a_5a not in (1,3,5),
                          ob14a_6a not in (1,3,5))=6)=2 then obd4c=.;
               else do;
                  obd4c=1;
                  sum3as=0;
                  sum5as=0;
                  sum243=0;
                  sum343=0;
                  sum245=0;
                  sum345=0;
                  array oxa ob14a_1a ob14a_2a ob14a_3a 
                            ob14a_4a ob14a_5a ob14a_6a;
                  array oxb ob14a_1b ob14a_2b ob14a_3b 
                            ob14a_4b ob14a_5b ob14a_6b;
                  do over oxa;
                     if oxa=3 then do;
                        sum3as=sum3as+1;
                        if oxb=2 then sum243=sum243+1;
                        if oxb=3 then sum343=sum343+1;
                     end;
                     if oxa=5 then do;
                        sum5as=sum5as+1;
                        if oxb=2 then sum245=sum245+1;
                        if oxb=3 then sum345=sum345+1;
                     end;
                  end;
                  if sum3as>0 then do;
                     if ((sum3as>0 and sum343>0) or
                         (sum3as>1 and sum243>1)) then obd4c=3;
                  end;
                  else if sum5as>0 then do;
                     if sum(((sum5as>0 and sum345>0) or
                             (sum5as>1 and sum245>1)),
                            ob9b=5)=2 then obd4c=5;
                  end;
               end;

         ***************;
         * Criterion E *;
         ***************;

            *******************************;
            * not due to drugs/medication *;
            *******************************;

               if ob12 not in (1,5) then obd4e=.;
               else obd4e=(4*(ob12=1))+1;

         *************;
         * Diagnosis *;
         *************;

            if n(obd4a1,obd4a2,obd4a3,obd4a4,obd4c,obd4e)=0 then obd4dx=.;
            else do;
               obd4dx=1;
               if (obd4a4=5 and 
                   sum(obd4a1 in (3,5),obd4a2 in (3,5),
                       obd4a3 in (3,5),obd4c in (3,5))=4 and
                   sum(obd4a1=3,obd4a2=3,obd4a3=3,obd4c=3)>0) then obd4dx=3;
               else if (sum(obd4a1=3,obd4a2=3,obd4a3=3,obd4c=3)=0 and
                        sum(obd4a1=5,obd4a2=5,obd4a3=5,obd4a4=5,
                            obd4c=5,obd4e=1)=6) then obd4dx=3;
               else if (sum(obd4a1=3,obd4a2=3,obd4a3=3,obd4c=3)=0 and
                        sum(obd4a1=5,obd4a2=5,obd4a3=5,obd4a4=5,
                            obd4c=5,obd4e=5)=6) then obd4dx=5;
            end;

      *************************;
      * jump here if skip out *;
      *************************;

         compuls:

      ***************;
      * Compulsions *;
      ***************;

         ***************;
         * Criterion A *;
         ***************;

            **************************************************************;
            * repititive, intentional behaviors in response to obsession *;
            **************************************************************;

               if sum(cp1a_1 not in (1,5),
                      cp1a_2 not in (1,5),
                      cp1a_3 not in (1,5),
                      cp1a_4 not in (1,5),
                      cp1a_5 not in (1,5),
                      cp2a_1 not in (1,5),
                      cp2a_2 not in (1,5),
                      cp2a_3 not in (1,5),
                      cp3a not in (1,5))=9 then cpd4a1=.;
               else cpd4a1=(4*(sum(cp1a_1=5,cp1a_2=5,cp1a_3=5,
                                   cp1a_4=5,cp1a_5=5,cp2a_1=5,
                                   cp2a_2=5,cp2a_3=5,cp3a=5)>0))+1;

            *********************;
            * test for skip out *;
            *********************;

               if sum(cp1a_1 not in (1,5),
                      cp1a_2 not in (1,5),
                      cp1a_3 not in (1,5),
                      cp1a_4 not in (1,5),
                      cp1a_5 not in (1,5),
                      cp1b not in (1,3,5),
                      cp2a_1 not in (1,5),
                      cp2a_2 not in (1,5),
                      cp2a_3 not in (1,5),
                      cp2b not in (1,3,5),
                      cp3a not in (1,5),
                      cp3b not in (1,3,5))=12 then cpd4sk=5;
               else do;
                  if sum(cp1a_1=5,
                         cp1a_2=5,
                         cp1a_3=5,
                         cp1a_4=5,
                         cp1a_5=5,
                         cp1b in (3,5),
                         cp2a_1=5,
                         cp2a_2=5,
                         cp2a_3=5,
                         cp2b in (3,5),
                         cp3a=5,
                         cp3b in (3,5))>0 then cpd4sk=1;
                  else cpd4sk=5;
               end;

               if cpd4sk=5 then do;
                  array skp2 cpd4a2 cpd4a cpd4c cpd4e cpd4dx;
                  do over skp2;
                     skp2=.S;
                  end;
               end;

               if cpd4sk=5 then go to skipout;

            ***********************************************************;
            * behavior intended to prevent discomfort/event/situation *;
            ***********************************************************;

               if sum(sum(cp4a not in (1,3,5),
                          cp4b not in (1,3,5))=2,
                      cp4c not in (1,3,5))=2 then cpd4a2=.;
               else do;
                  cpd4a2=1;
                  if sum(sum(cp4a in (3,5),cp4b in (3,5))=2,
                         sum(cp4a=3,cp4b=3)>0,
                         cp4c in (3,5))=3 then cpd4a2=3;
                  else if sum(sum(cp4a=3,cp4b=3)=0,
                              sum(cp4a=5,cp4b=5)>0,
                              cp4c=5)=3 then cpd4a2=5;
               end;

            **************;
            * both items *;
            **************;

               if n(cpd4a1,cpd4a2)=0 then cpd4a=.;
               else do;
                  if sum(cpd4a1=5,cpd4a2 in (3,5))=2 then cpd4a=cpd4a2;
                  else cpd4a=1;
               end;

         ******************************;
         * Criterion B - Not assessed *;
         ******************************;

         ***************;
         * Criterion C *;
         ***************;

            *************************************;
            * compulsions cause marked distress *;
            *************************************;

               if sum(cp1b not in (1,3,5),
                      cp2b not in (1,3,5),
                      cp3b not in (1,3,5),
                      sum(cp7b not in (1,5),
                          cp9b not in (1,5))=2,
                      sum(cp11a_1a not in (1,3,5),
                          cp11a_2a not in (1,3,5),
                          cp11a_3a not in (1,3,5),
                          cp11a_4a not in (1,3,5),
                          cp11a_5a not in (1,3,5),
                          cp11a_6a not in (1,3,5))=6)=5 then cpd4c=.;
               else do;
                  cpd4c=1;
                  sum3as=0;
                  sum5as=0;
                  sum243=0;
                  sum343=0;
                  sum245=0;
                  sum345=0;
                  array cxa cp11a_1a cp11a_2a cp11a_3a 
                            cp11a_4a cp11a_5a cp11a_6a;
                  array cxb cp11a_1b cp11a_2b cp11a_3b 
                            cp11a_4b cp11a_5b cp11a_6b;
                  do over cxa;
                     if cxa=3 then do;
                        sum3as=sum3as+1;
                        if cxb=2 then sum243=sum243+1;
                        if cxb=3 then sum343=sum343+1;
                     end;
                     if cxa=5 then do;
                        sum5as=sum5as+1;
                        if cxb=2 then sum245=sum245+1;
                        if cxb=3 then sum345=sum345+1;
                     end;
                  end;
                  if (sum(cp1b=3,cp2b=3,cp3b=3,cp9b=5)>0 or
                      (sum3as>0 and sum343>0) or
                      (sum3as>1 and sum243>1)) then cpd4c=3;
                  else do;
                     if ((sum(cp1b=3,cp2b=3,cp3b=3)=0 and sum3as=0) and
                         (sum(cp1b=5,cp2b=5,cp3b=5,cp7b=5)>0 or
                          (sum5as>0 and sum345>0) or
                          (sum5as>1 and sum245>1))) then cpd4c=5;
                  end;
               end;

         ***************;
         * Criterion E *;
         ***************;

            *******************************;
            * not due to drugs/medication *;
            *******************************;

               if cp6 not in (1,5) then cpd4e=.;
               else cpd4e=(4*(cp6=1))+1;

         *************;
         * Diagnosis *;
         *************;

            if n(cpd4a1,cpd4a2,cpd4c,cpd4e)=0 then cpd4dx=.;
            else do;
               cpd4dx=1;
               if (cpd4a1=5 and 
                   sum(cpd4a2 in (3,5),cpd4c in (3,5))=2 and
                   sum(cpd4a2=3,cpd4c=3)>0) then cpd4dx=3;
               else if sum(cpd4a1=5,cpd4a2=5,cpd4c=5,cpd4e=1)=4 then cpd4dx=3;
               else if sum(cpd4a1=5,cpd4a2=5,cpd4c=5,cpd4e=5)=4 then cpd4dx=5;
            end;

      *************************;
      * jump here if skip out *;
      *************************;

         skipout:

         ocd4sk=(4*(sum(obd4sk=5,cpd4sk=5)=2))+1;

      *************;
      * Diagnosis *;
      *************;

         if ocd4sk=1 then do;
            if n(obd4dx,cpd4dx)=0 then ocd4dx=.;
            else do;
               ocd4dx=1;
               if sum(obd4dx=3,cpd4dx=3) then ocd4dx=3;
               else if sum(obd4dx=3,cpd4dx=3)=0 and 
                       sum(obd4dx=5,cpd4dx=5)>0 then ocd4dx=5;
            end;
         end;
         else do;
            ocd4dx=.S;
         end;

      *******************;
      * variable labels *;
      *******************;

         label obd4sk  = "DSM4 OBSESV Skip Out"
               cpd4sk  = "DSM4 CMPLSV Skip Out"
               ocd4sk  = "DSM4 OCD Skip Out"
               obd4a1  = "DSM4 OBSESV (Lifetime) Crit. A1"
               obd4a2  = "DSM4 OBSESV (Lifetime) Crit. A2"
               obd4a3  = "DSM4 OBSESV (Lifetime) Crit. A3"
               obd4a4  = "DSM4 OBSESV (Lifetime) Crit. A4"
               obd4a3x = "DSM4 OBSESV (Lifetime) Sum of 3s"
               obd4a5x = "DSM4 OBSESV (Lifetime) Sum of 5s"
               obd4ax  = "DSM4 OBSESV (Lifetime) Sum of As"
               obd4a   = "DSM4 OBSESV (Lifetime) Crit. A"
               obd4c   = "DSM4 OBSESV (Lifetime) Crit. C"
               obd4e   = "DSM4 OBSESV (Lifetime) Crit. E"
               obd4dx  = "DSM4 OBSESV (Lifetime) Diagnosis"
               cpd4a1  = "DSM4 CMPLSV (Lifetime) Crit. A1"
               cpd4a2  = "DSM4 CMPLSV (Lifetime) Crit. A2"
               cpd4a   = "DSM4 CMPLSV (Lifetime) Crit. A"
               cpd4c   = "DSM4 CMPLSV (Lifetime) Crit. C"
               cpd4e   = "DSM4 CMPLSV (Lifetime) Crit. E"
               cpd4dx  = "DSM4 CMPLSV (Lifetime) Diagnosis"
               ocd4dx  = "DSM4 OCD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    obd4sk cpd4sk ocd4sk
                    obd4a1-obd4a4 obd4a3x obd4a5x obd4ax obd4a 
                    obd4c obd4e obd4dx
                    cpd4a1 cpd4a2 cpd4a cpd4c cpd4e cpd4dx ocd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_ocd_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               **************;
               * Obsessions *;
               **************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(ocdd4,ob1 ob2b ob3c ob4b ob5c ob6b obd4sk);

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(ocdd4,ob1 ob2b ob3c ob4b ob5c ob6b obd4a1);
                     %multi(ocdd4,ob1 ob2b ob4a ob4a_2 obd4a2);
                     %multi(ocdd4,ob2b ob3b ob4b ob7 obd4a3);
                     %multi(ocdd4,obd4sk ob8a obd4a4);
                     %multi(ocdd4,obd4sk obd4a1-obd4a4 obd4a3x obd4a5x);
                     %multi(ocdd4,obd4sk obd4a3x obd4a5x obd4ax);
                     %multi(ocdd4,obd4sk obd4a3x obd4a5x obd4ax obd4a);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(ocdd4,obd4sk 
                                  ob14a_1a ob14a_1b ob14a_2a ob14a_2b 
                                  ob14a_3a ob14a_3b ob14a_4a ob14a_4b 
                                  ob14a_5a ob14a_5b ob14a_6a ob14a_6b
                                  ob9b obd4c);

                  ***************;
                  * Criterion E *;
                  ***************;

                     %multi(ocdd4,obd4sk ob12 obd4e);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(ocdd4,obd4sk obd4a1 obd4a2 obd4a3 obd4a4 
                                  obd4c obd4e obd4dx);

               ***************;
               * Compulsions *;
               ***************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(ocdd4,cp1a_1 cp1a_2 cp1a_3 cp1a_4 cp1a_5 cp1b
                                  cp2a_1 cp2a_2 cp2a_3 cp2b cp3a cp3b 
                                  cpd4sk);

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(ocdd4,cp1a_1-cp1a_5 cp2a_1-cp2a_3 cp3a cpd4a1);
                     %multi(ocdd4,cpd4sk cp4a cp4b cp4c cpd4a2);
                     %multi(ocdd4,cpd4sk cpd4a1 cpd4a2 cpd4a);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(ocdd4,cpd4sk
                                  cp1b cp2b cp3b cp7b cp9b
                                  cp11a_1a cp11a_1b cp11a_2a cp11a_2b 
                                  cp11a_3a cp11a_3b cp11a_4a cp11a_4b 
                                  cp11a_5a cp11a_5b cp11a_6a cp11a_6b
                                  cpd4c);

                  ***************;
                  * Criterion E *;
                  ***************;

                     %multi(ocdd4,cpd4sk cp6 cpd4e);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(ocdd4,cpd4sk cpd4a1 cpd4a2 cpd4c cpd4e cpd4dx);

               *************;
               * Diagnosis *;
               *************;
 
                  %multi(ocdd4,obd4sk cpd4sk ocd4sk);
                  %multi(ocdd4,ocd4sk obd4dx cpd4dx ocd4dx);

            ******************;
            * keep variables *;
            ******************;

               data ocdd4;
               set ocdd4;
               keep ind_id  
                    obd4sk cpd4sk ocd4sk
                    obd4a1-obd4a4 obd4a3x obd4a5x obd4ax obd4a 
                    obd4c obd4e obd4dx
                    cpd4a1 cpd4a2 cpd4a cpd4c cpd4e cpd4dx ocd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=ocdd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend ocdd4;

********;
* test *;
********;
/*
   %ocdd4(coga.csaga2,1);
*/
