***************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Oppositional Defiant Disorder *;
***************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
**************************************************************;
* to run as standalone module, uncomment the following lines *;
**************************************************************;
/*
   %include "./dsm4_cd.sas" / nosource2;
  
   %cdd4(coga.csaga2,0);
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %oddd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro oddd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ODDD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ODDD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      proc contents data=&dsn. noprint out=test;
      run;

      data test;
      set test;
      if upcase(substr(name,1,2))="OP";
      run;

      data test;
      set test nobs=count;
      call symput("var"||left(_n_),name);
      call symput("nvar",left(put(count,8.)));
      run;

      data oddd4;
      merge &dsn.(keep=ind_id dm3a cd11a 
                       %do i=1 %to &nvar.;
                          &&var&i.
                       %end;
                       ) 
            cdd4(keep=ind_id cdd4dx);
      by ind_id;

      attrib odd4a1-odd4a8 odd4ax odd4a 
             odd4b odd4d1 odd4d2 odd4d odd4dx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **********************;
            * often loses temper *;
            **********************;

               if sum(op1 not in (1,5),
                      cd11a not in (1,3,5))=2 then odd4a1=.;
               else odd4a1=(op1=5 or cd11a=5);

            ****************************;
            * often argues with adults *;
            ****************************;

               if (op2a not in (1,5)) then odd4a2=.;
               else odd4a2=(op2a=5);

            *******************************;
            * often defies requests/rules *;
            *******************************;

               if (op3a not in (1,5)) then odd4a3=.;
               else odd4a3=(op3a=5);

            *******************************;
            * often deliberately annoying *;
            *******************************;

               if (op4 not in (1,5)) then odd4a4=.;
               else odd4a4=(op4=5);

            ***********************;
            * often blames others *;
            ***********************;

               if (op5 not in (1,2,3,4)) then odd4a5=.;
               else odd4a5=(op5=2);

            **********************************;
            * often touchy or easily annoyed *;
            **********************************;

               if (op6 not in (1,5)) then odd4a6=.;
               else odd4a6=(op6=5);

            ****************************;
            * often angry or resentful *;
            ****************************;

               if (op7 not in (1,5)) then odd4a7=.;
               else odd4a7=(op7=5);

            *****************************;
            * often spiteful/vindictive *;
            *****************************;

               if (op8a not in (1,5)) then odd4a8=.;
               else odd4a8=(op8a=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of odd4a1-odd4a8)=0 then odd4ax=.;
               else odd4ax=sum(of odd4a1-odd4a8);

            *************************************;
            * behavior lasted at least 6 months *;
            *************************************;

               if odd4ax=. then odd4a=.;
               else if odd4ax<4 then odd4a=0;
               else do;
                  if (op10b not in (1,5) or op10c not in (1,5)) then odd4a=.;
                  else odd4a=((odd4ax>3) and (op10b=5 and op10c=5));
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(sum(op11a_1a not in (1,5),op11a_2a not in (1,5),
                       op11a_3a not in (1,5),op11a_4a not in (1,5),
                       op11a_5a not in (1,5),op11a_6a not in (1,5))=6,
                   op11b not in (1,5),
                   op11c not in (1,5),
                   op11d not in (1,5))=4 then odd4b=.;
            else odd4b=sum(op11a_1a=5,op11a_2a=5,op11a_3a=5,op11a_4a=5,
                           op11a_5a=5,op11a_6a=5,op11b=5,op11c=5,op11d=5)>0;

      **************************************;
      * Criterion C -- skip in this module *;
      **************************************;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************;
            * does not meet criteria for conduct disorder *;
            ***********************************************;

               odd4d1=1;
               if cdd4dx=5 then odd4d1=0;

            ***********************************;
            * does not meet criteria for ASPD *;
            ***********************************;

               odd4d2=.;
               if 0<dm3a<18 then odd4d2=1;

            *****************;
            * no CD or ASPD *;
            *****************;

               if n(odd4d1,odd4d2)=0 then odd4d=.;
               else odd4d=sum(odd4d1,odd4d2)=2;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(odd4ax,odd4a,odd4b,odd4d)=0 then odd4dx=.;
            else odd4dx=(odd4a=1 and odd4b=1 and odd4d=1);

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         array one odd4a1-odd4a8 odd4a odd4b odd4d1 odd4d2 odd4d odd4dx;
         do over one;
            if one=1 then one=5;
            if one=0 then one=1;
         end;

      *******************;
      * variable labels *;
      *******************;

         label odd4a1 = "DSM4 ODD (Lifetime) Crit. A1"
               odd4a2 = "DSM4 ODD (Lifetime) Crit. A2"
               odd4a3 = "DSM4 ODD (Lifetime) Crit. A3"
               odd4a4 = "DSM4 ODD (Lifetime) Crit. A4"
               odd4a5 = "DSM4 ODD (Lifetime) Crit. A5"
               odd4a6 = "DSM4 ODD (Lifetime) Crit. A6"
               odd4a7 = "DSM4 ODD (Lifetime) Crit. A7"
               odd4a8 = "DSM4 ODD (Lifetime) Crit. A8"
               odd4ax = "DSM4 ODD (Lifetime) Sum of As"
               odd4a  = "DSM4 ODD (Lifetime) 4+ in 6Mo"
               odd4b  = "DSM4 ODD (Lifetime) Crit. B"
               odd4d1 = "DSM4 ODD (Lifetime) Crit. D1"
               odd4d2 = "DSM4 ODD (Lifetime) Crit. D2"
               odd4d  = "DSM4 ODD (Lifetime) Crit. D"
               odd4dx = "DSM4 ODD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    odd4a1-odd4a8 odd4ax odd4a 
                    odd4b odd4d1 odd4d2 odd4d odd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_odd_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddd4,op1 cd11a odd4a1);
                     %multi(oddd4,op2a odd4a2);
                     %multi(oddd4,op3a odd4a3);
                     %multi(oddd4,op4 odd4a4);
                     %multi(oddd4,op5 odd4a5);
                     %multi(oddd4,op6 odd4a6);
                     %multi(oddd4,op7 odd4a7);
                     %multi(oddd4,op8a odd4a8);
                     %multi(oddd4,odd4a1-odd4a8 odd4ax);
                     %multi(oddd4,odd4ax op10b op10c odd4a);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddd4,op11a_1a op11a_2a op11a_3a op11a_4a op11a_5a 
                                  op11a_6a op11b op11c op11d odd4b);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddd4,cdd4dx odd4d1);
                     %multi(oddd4,dm3a odd4d2);
                     %multi(oddd4,odd4d1 odd4d2 odd4d);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddd4,odd4a odd4b odd4d odd4dx);

            ******************;
            * keep variables *;
            ******************;

               data oddd4;
               set oddd4;
               keep ind_id  
                    odd4a1-odd4a8 odd4ax odd4a 
                    odd4b odd4d1 odd4d2 odd4d odd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=oddd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend oddd4;

********;
* test *;
********;
/*
   %oddd4(coga.csaga2,1);
*/
