************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Panic Disorder *;
************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %pand4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro pand4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in PAND4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in PAND4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data pand4;
      set &dsn.;

      attrib pnd4sk1 pnd4sk2 pnd4sk
             pnd4a1a pnd4a101-pnd4a113 pnd4a1x pnd4a1b pnd4a1
             pnd4a2a pnd4a2b pnd4a2c pnd4a2x pnd4a2 pnd4a
             pnd4b pnd4dx length=3;

      *********************;
      * test for skip out *;
      *********************;
 
         if pn1a not in (1,2,3,4,5) then pnd4sk1=5;
         else do;
            if pn1a in (2,3,4,5) then pnd4sk1=1;
            else pnd4sk1=5;
         end;

         if sum(pn2a_1 not in (1,5),pn2a_2 not in (1,5),
                pn2a_3 not in (1,5),pn2a_4 not in (1,5),
                pn2a_5 not in (1,5),pn2a_6 not in (1,5),
                pn2a_7 not in (1,5),pn2a_8 not in (1,5),
                pn2a_9 not in (1,5),pn2a_10 not in (1,5),
                pn2a_11 not in (1,5),pn2a_12 not in (1,5),
                pn2a_13 not in (1,5),pn2a_14 not in (1,5),
                pn2a_15 not in (1,5))=15 then pnd4sk2=5;
         else do;
            if sum(pn2a_1=5,pn2a_2=5,pn2a_3=5,pn2a_4=5,
                   pn2a_5=5,pn2a_6=5,pn2a_7=5,pn2a_8=5,
                   pn2a_9=5,pn2a_10=5,pn2a_11=5,pn2a_12=5,
                   pn2a_13=5,pn2a_14=5,pn2a_15=5)>3 then pnd4sk2=1;
            else pnd4sk2=5;
         end;

         pnd4sk=max(pnd4sk1,pnd4sk2);

         if pnd4sk=5 then do;
            array skp1 pnd4a1a pnd4a101-pnd4a113 pnd4a1x pnd4a1b pnd4a1
                       pnd4a2a pnd4a2b pnd4a2c pnd4a2x pnd4a2 pnd4a
                       pnd4b pnd4dx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if pnd4sk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************;
            * Criterion A1 *;
            ****************;

               ****************************;
               * unexpected panic attacks *;
               ****************************;

                  if sum(pn1a not in (1,2,3,4,5),
                         sum(pn4a<0,pn4a_1 not in (1,2,3,4,5))=2)=2 then 
                     pnd4a1a=.;
                  else pnd4a1a=(4*(pn1a in (3,4,5) and 
                                   sum(pn4a>2,pn4a_1 in (2,3,4,5))>0))+1;

               ****************;
               * palpitations *;
               ****************;

                  if pn2a_1 not in (1,5) then pnd4a101=.;
                  else pnd4a101=(4*(pn2a_1=5))+1;

               ************;
               * sweating *;
               ************;

                  if pn2a_2 not in (1,5) then pnd4a102=.;
                  else pnd4a102=(4*(pn2a_2=5))+1;

               ************************;
               * trembling or shaking *;
               ************************;

                  if pn2a_3 not in (1,5) then pnd4a103=.;
                  else pnd4a103=(4*(pn2a_3=5))+1;

               ***********************;
               * shortness of breath *;
               ***********************;

                  if pn2a_4 not in (1,5) then pnd4a104=.;
                  else pnd4a104=(4*(pn2a_4=5))+1;

               **********************;
               * feeling of choking *;
               **********************;

                  if pn2a_5 not in (1,5) then pnd4a105=.;
                  else pnd4a105=(4*(pn2a_5=5))+1;

               ****************************;
               * chest pain or discomfort *;
               ****************************;

                  if pn2a_6 not in (1,5) then pnd4a106=.;
                  else pnd4a106=(4*(pn2a_6=5))+1;

               ********************************;
               * nausea or abdominal distress *;
               ********************************;

                  if pn2a_7 not in (1,5) then pnd4a107=.;
                  else pnd4a107=(4*(pn2a_7=5))+1;
  
               *****************;
               * feeling dizzy *;
               *****************;

                  if pn2a_8 not in (1,5) then pnd4a108=.;
                  else pnd4a108=(4*(pn2a_8=5))+1;

               ***********************************;
               * derealization/depersonalization *;
               ***********************************;

                  if pn2a_9 not in (1,5) then pnd4a109=.;
                  else pnd4a109=(4*(pn2a_9=5))+1;

               **************************;
               * fear of losing control *;
               **************************;

                  if pn2a_10 not in (1,5) then pnd4a110=.;
                  else pnd4a110=(4*(pn2a_10=5))+1;

               *****************;
               * fear of dying *;
               *****************;

                  if pn2a_15 not in (1,5) then pnd4a111=.;
                  else pnd4a111=(4*(pn2a_15=5))+1;

               ****************;
               * parasthesias *;
               ****************;

                  if pn2a_11 not in (1,5) then pnd4a112=.;
                  else pnd4a112=(4*(pn2a_11=5))+1;

               *************************;
               * chills or hot flashes *;
               *************************;

                  if sum(pn2a_12 not in (1,5),
                         pn2a_13 not in (1,5))=2 then pnd4a113=.;
                  else pnd4a113=(4*(pn2a_12=5 or pn2a_13=5))+1;

               ****************;
               * sum of items *;
               ****************;

                  if n(of pnd4a101-pnd4a113)=0 then pnd4a1x=.;
                  else do;
                     pnd4a1x=0;
                     array sxs pnd4a101-pnd4a113;
                     do over sxs;
                        if sxs=5 then pnd4a1x=pnd4a1x+1;
                     end;
                  end;

               *******************************;
               * symptoms developed suddenly *;
               *******************************;

                  if pn3 not in (1,5) then pnd4a1b=.;
                  else pnd4a1b=(4*(pn3=5))+1;

               *********************;
               * all items present *;
               *********************;

                  if n(pnd4a1a,pnd4a1x,pnd4a1b)=0 then pnd4a1=.;
                  else pnd4a1=(4*(pnd4a1a=5 and pnd4a1x>3 and pnd4a1b=5))+1;

            ****************;
            * Criterion A2 *;
            ****************;

               ***********************************************;
               * persistent concern about additional attacks *;
               ***********************************************;

                  if pn6c not in (1,2,5) then pnd4a2a=.;
                  else pnd4a2a=(4*(pn6c=5))+1;

               ***************************************************;
               * worry about implications/consequences of attack *;
               ***************************************************;

                  if pn7c not in (1,2,5) then pnd4a2b=.;
                  else pnd4a2b=(4*(pn7c=5))+1;

               *****************************************;
               * change in behavior related to attacks *;
               *****************************************;

                  if pn8c not in (1,2,5) then pnd4a2c=.;
                  else pnd4a2c=(4*(pn8c=5))+1;

               ****************;
               * sum of items *;
               ****************;

                  if n(pnd4a2a,pnd4a2b,pnd4a2c)=0 then pnd4a2x=.;
                  else do;
                     pnd4a2x=0;
                     array sxt pnd4a2a pnd4a2b pnd4a2c;
                     do over sxt;
                        if sxt=5 then pnd4a2x=pnd4a2x+1;
                     end;
                     pnd4a2=(4*(pnd4a2x>0))+1;
                  end;

            *************;
            * A1 and A2 *;
            *************;
  
               if n(pnd4a1,pnd4a2)<2 then pnd4a=.;
               else pnd4a=(4*(pnd4a1=5 and pnd4a2=5))+1;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ********************;
            * organic etiology *;
            ********************;

               if pn1a not in (1,2,3,4,5) then pnd4b=.;
               else do;
                  if pn1a in (1,5) then pnd4b=1;
                  if pn1a in (3,4) then pnd4b=pn1a;
               end;

      **************************************;
      * Criterion C -- Skip in this Module *;
      **************************************;

      *******************************;
      * Criterion D -- Not Assessed *;
      *******************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(pnd4a,pnd4b)<2 then pnd4dx=.;
            else do;
               pnd4dx=1;
               if pnd4a=5 then do;
                  if pnd4b=1 then pnd4dx=5;
                  if pnd4b=3 then pnd4dx=3;
                  if pnd4b=4 then pnd4dx=4;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label pnd4sk1  = "DSM4 PANIC Skip Out #1"
               pnd4sk2  = "DSM4 PANIC Skip Out #2"
               pnd4sk   = "DSM4 PANIC Skip Out"
               pnd4a1a  = "DSM4 PANIC (Lifetime) Crit. A1 A"
               pnd4a101 = "DSM4 PANIC (Lifetime) Crit. A1.1"
               pnd4a102 = "DSM4 PANIC (Lifetime) Crit. A1.2"
               pnd4a103 = "DSM4 PANIC (Lifetime) Crit. A1.3"
               pnd4a104 = "DSM4 PANIC (Lifetime) Crit. A1.4"
               pnd4a105 = "DSM4 PANIC (Lifetime) Crit. A1.5"
               pnd4a106 = "DSM4 PANIC (Lifetime) Crit. A1.6"
               pnd4a107 = "DSM4 PANIC (Lifetime) Crit. A1.7"
               pnd4a108 = "DSM4 PANIC (Lifetime) Crit. A1.8"
               pnd4a109 = "DSM4 PANIC (Lifetime) Crit. A1.9"
               pnd4a110 = "DSM4 PANIC (Lifetime) Crit. A1.10"
               pnd4a111 = "DSM4 PANIC (Lifetime) Crit. A1.11"
               pnd4a112 = "DSM4 PANIC (Lifetime) Crit. A1.12"
               pnd4a113 = "DSM4 PANIC (Lifetime) Crit. A1.13"
               pnd4a1x  = "DSM4 PANIC (Lifetime) Sum of A1s"
               pnd4a1b  = "DSM4 PANIC (Lifetime) Crit. A1 B"
               pnd4a1   = "DSM4 PANIC (Lifetime) Crit. A1"
               pnd4a2a  = "DSM4 PANIC (Lifetime) Crit. A2 A"
               pnd4a2b  = "DSM4 PANIC (Lifetime) Crit. A2.B"
               pnd4a2c  = "DSM4 PANIC (Lifetime) Crit. A2.C"
               pnd4a2x  = "DSM4 PANIC (Lifetime) Sum of A2s"
               pnd4a2   = "DSM4 PANIC (Lifetime) Crit. A2"
               pnd4a    = "DSM4 PANIC (Lifetime) A1 and A2"
               pnd4b   = "DSM4 PANIC (Lifetime) Crit. B"
               pnd4dx  = "DSM4 PANIC (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    pnd4sk1 pnd4sk2 pnd4sk
                    pnd4a1a pnd4a101-pnd4a113 pnd4a1x pnd4a1b pnd4a1
                    pnd4a2a pnd4a2b pnd4a2c pnd4a2x pnd4a2 pnd4a
                    pnd4b pnd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_pan_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(pand4,pn1a pnd4sk1);
                  %multi(pand4,pnd4sk1 
                               pn2a_1 pn2a_2 pn2a_3 
                               pn2a_4 pn2a_5 pn2a_6 
                               pn2a_7 pn2a_8 pn2a_9 
                               pn2a_10 pn2a_11 pn2a_12 
                               pn2a_13 pn2a_14 pn2a_15 pnd4sk2); 
                  %multi(pand4,pnd4sk1 pnd4sk2 pnd4sk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     ****************;
                     * Criterion A1 *;
                     ****************;

                        %multi(pand4,pnd4sk pn1a pn4a pn4a_1 pnd4a1a);
                        %multi(pand4,pnd4sk pn2a_1 pnd4a101);
                        %multi(pand4,pnd4sk pn2a_2 pnd4a102);
                        %multi(pand4,pnd4sk pn2a_3 pnd4a103);
                        %multi(pand4,pnd4sk pn2a_4 pnd4a104);
                        %multi(pand4,pnd4sk pn2a_5 pnd4a105);
                        %multi(pand4,pnd4sk pn2a_6 pnd4a106);
                        %multi(pand4,pnd4sk pn2a_7 pnd4a107);
                        %multi(pand4,pnd4sk pn2a_8 pnd4a108);
                        %multi(pand4,pnd4sk pn2a_9 pnd4a109);
                        %multi(pand4,pnd4sk pn2a_10 pnd4a110);
                        %multi(pand4,pnd4sk pn2a_15 pnd4a111);
                        %multi(pand4,pnd4sk pn2a_11 pnd4a112);
                        %multi(pand4,pnd4sk pn2a_12 pn2a_13 pnd4a113);
                        %multi(pand4,pnd4sk pnd4a101-pnd4a113 pnd4a1x);
                        %multi(pand4,pnd4sk pn3 pnd4a1b);
                        %multi(pand4,pnd4sk pnd4a1a pnd4a1x pnd4a1b pnd4a1);

                     ****************;
                     * Criterion A2 *;
                     ****************;

                        %multi(pand4,pnd4sk pn6c pnd4a2a);
                        %multi(pand4,pnd4sk pn7c pnd4a2b);
                        %multi(pand4,pnd4sk pn8c pnd4a2c);
                        %multi(pand4,pnd4sk pnd4a2a pnd4a2b pnd4a2c pnd4a2x);
                        %multi(pand4,pnd4sk pnd4a2x pnd4a2);

                     *************;
                     * A1 and A2 *;
                     *************;

                        %multi(pand4,pnd4sk pnd4a1 pnd4a2 pnd4a);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(pand4,pnd4sk pn1a pnd4b);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(pand4,pnd4sk pnd4a pnd4b pnd4dx);

            ******************;
            * keep variables *;
            ******************;

               data pand4;
               set pand4;
               keep ind_id  
                    pnd4sk1 pnd4sk2 pnd4sk 
                    pnd4a1a pnd4a101-pnd4a113 pnd4a1x pnd4a1b pnd4a1
                    pnd4a2a pnd4a2b pnd4a2c pnd4a2x pnd4a2 pnd4a
                    pnd4b pnd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=pand4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend pand4;

********;
* test *;
********;
/*
   %pand4(coga.csaga2,1);
*/
