*************************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Separation Anxiety Disorder *;
*************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %sepd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro sepd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SEPD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SEPD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data sepd4;
      set &dsn.;

      attrib sad4sk 
             sad4a1-sad4a8 sad4a3x sad4a5x sad4ax sad4a 
             sad4b sad4c sad4d sad4dx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************************************************;
            * persistent worry about harm to/loss of attachment figures *;
            *************************************************************;

               if sa1a not in (1,5) then sad4a2=.;
               else sad4a2=(4*(sa1a=5))+1;

            ********************************************;
            * persistent worry about calamitous events *;
            ********************************************;

               if sa2a not in (1,5) then sad4a3=.;
               else sad4a3=(4*(sa2a=5))+1;

            **************************************;
            * persistent refusal to go to school *;
            **************************************;

               if sum(sa1b not in (1,3,5),
                      sa2b not in (1,3,5),
                      sa3a not in (1,5),
                      sa3b not in (1,3,5),
                      sum(sa3c<0,sa3c_1<0))=5 then sad4a4=.;
               else do;
                  sad4a4=1;
                  if (sa3c>1 or sa3c_1>1) then do;
                     if (sa1b=3 or sa2b=3 or sa3b=3) then sad4a4=3;
                     else if (sa1b=5 or sa2b=5 or 
                              sa3a=5 or sa3b=5) then sad4a4=5;
                  end;
               end;

            *********************;
            * test for skip out *;
            *********************;
 
               if sum(sa3a not in (1,5),
                      sa3b not in (1,3,5),
                      sa3c not in (0,1))=3 then sad4sk=5;
               else do;
                  if sum(sum(sa3a=5,sa3b=5)>0,sa3c>1)>0 then sad4sk=1;
                  else sad4sk=5;
               end;

               if sad4sk=5 then do;
                  array skp1 sad4a1 sad4a5-sad4a8 sad4a3x sad4a5x sad4ax sad4a 
                             sad4b sad4c sad4d sad4dx;
                  do over skp1;
                     skp1=.S;
                  end;
               end;

              if sad4sk=5 then go to skipout;

            ***************************************************************;
            * recurrent excessive distress when separated/to be separated *;
            ***************************************************************;

               if sum(sa7 not in (1,3,5),
                      sa8a not in (1,3,5),
                      sa8b not in (1,3,5),
                      sa11a not in (1,3,5),
                      sa11b not in (1,3,5))=5 then sad4a1=.;
               else do;
                  sad4a9=1;
                  if (sa7=3 or sa8a=3 or sa8b=3 or
                      sa11a=3 or sa11b=3) then sad4a1=3;
                  else if (sa7=5 or sa8a=5 or sa8b=5 or
                           sa11a=5 or sa11b=5) then sad4a1=5;
               end;

            ***************************************;
            * persistent avoidance of being alone *;
            ***************************************;

               if sum(sa6a not in (1,3,5),
                      sa6b not in (1,3,5))=2 then sad4a5=.;
               else do;
                  sad4a5=1;
                  if (sa6a=3 or sa6b=3) then sad4a5=3;
                  else if (sa6a=5 or sa6b=5) then sad4a5=5;
               end;

            *************************************;
            * persistent refusal to go to sleep *;
            *************************************;

               if sum(sa4a not in (1,3,5),
                      sa5a not in (1,3,5),
                      sa5b not in (1,3,5))=3 then sad4a6=.;
               else do;
                  sad4a6=1;
                  if (sa4a=3 or sa5a=3 or sa5b=3) then sad4a6=3;
                  else if (sa4a=5 or sa5a=5 or sa5b=5) then sad4a6=5;
               end;

            ******************************************;
            * repeated separation-related nightmares *;
            ******************************************;

               if sa9 not in (1,3,5) then sad4a7=.;
               else sad4a7=sa9;

            ****************************************************************;
            * complaints of physical symptoms when anticipating separation *;
            ****************************************************************;

               if sa10 not in (1,3,5) then sad4a8=.;
               else sad4a8=sa10;

            ****************;
            * sum of items *;
            ****************;

               if n(of sad4a1-sad4a8)=0 then sad4ax=.;
               else do;
                  sad4a3x=0;
                  sad4a5x=0;
                  array sxs sad4a1-sad4a8;
                  do over sxs;
                     if sxs=3 then sad4a3x+1;
                     if sxs=5 then sad4a5x+1;
                  end;
                  sad4ax=sum(sad4a3x,sad4a5x);
                  sad4a=(4*(sad4ax>2))+1;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ************************************;
            * behavior lasted at least 4 weeks *;
            ************************************;

               if sum(sa12c<0,
                      sa12c_1<0,
                      sa13c<0,
                      sa13c_1<0)=4 then sad4b=.;
               else sad4b=(4*(sum(sa12c>3,sa12c_1=4,
                                  sa13c>3,sa13c_1=4)>0))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************;
            * onset before age 18 *;
            ***********************;

               if sum(sa12a_1<0,sa13a_1<0)=2 then sad4c=.;
               else do;
                  if (sa12a_1 ne . and sa12a_1>0) and 
                     (sa13a_1=. or sa13a_1<0) then
                     sad4c=(4*(sa12a_1<18))+1;
                  if (sa12a_1 ne . and sa12a_1>0) and 
                     (sa13a_1 ne . and sa13a_1>0) then
                     sad4c=(4*(sum(sa12a_1<18,sa13a_1<18)>0))+1;
                  if (sa13a_1 ne . and sa13a_1>0) and 
                     (sa12a_1=. or sa12a_1<0) then
                     sad4c=(4*(sa13a_1<18))+1;
               end;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * clinically significant distress *;
            ***********************************;

               if sum(sa14a_1a not in (1,3,5),
                      sa14a_2a not in (1,3,5),
                      sa14a_3a not in (1,3,5),
                      sa14a_4a not in (1,3,5),
                      sa14a_5a not in (1,3,5))=5 then sa3rd=.;
               else do;
                  sad4d=1;
                  opt1_a=0;
                  opt1_b=0;
                  opt1_c=0;
                  opt2_a=0;
                  opt2_b=0;
                  opt2_c=0;
                  array col1{*} sa14a_1a sa14a_2a sa14a_3a sa14a_4a sa14a_5a;
                  array col2{*} sa14a_1b sa14a_2b sa14a_3b sa14a_4b sa14a_5b;
                  do i=1 to dim(col1);
                     if col1(i)=3 then do;
                        opt2_a+1;
                        if col2(i)=3 then opt2_b+1;
                        if col2(i)=2 then opt2_c+1;
                     end;
                     else if col1(i)=5 then do;
                        opt1_a+1;
                        if col2(i)=3 then opt1_b+1;
                        if col2(i)=2 then opt1_c+1;
                     end;
                  end;
                  if ((opt2_a>0 and opt2_b>0) or
                      (opt2_a>1 and opt2_c>1)) then sad4d=3;
                  else if ((opt1_a>0 and opt1_b>0) or
                           (opt1_a>1 and opt1_c>1)) then sad4d=5;
               end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(sad4a,sad4b,sad4c,sad4d)=0 then sad4dx=.;
            else do;
               sad4dx=1;
               if (sad4ax>2 and sad4b=5 and sad4c=5) then do;
                  if (sad4a3x>0 and sad4d in (3,5)) then sad4dx=3;
                  else if (sad4a3x=0 and sad4d=5) then sad4dx=5;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label sad4sk  = "DSM4 SEPANX Skip Out"
               sad4a1  = "DSM4 SEPANX (Lifetime) Crit. A1"
               sad4a2  = "DSM4 SEPANX (Lifetime) Crit. A2"
               sad4a3  = "DSM4 SEPANX (Lifetime) Crit. A3"
               sad4a4  = "DSM4 SEPANX (Lifetime) Crit. A4"
               sad4a5  = "DSM4 SEPANX (Lifetime) Crit. A5"
               sad4a6  = "DSM4 SEPANX (Lifetime) Crit. A6"
               sad4a7  = "DSM4 SEPANX (Lifetime) Crit. A7"
               sad4a8  = "DSM4 SEPANX (Lifetime) Crit. A8"
               sad4a3x = "DSM4 SEPANX (Lifetime) Sum of 3s"
               sad4a5x = "DSM4 SEPANX (Lifetime) Sum of 5s"
               sad4ax  = "DSM4 SEPANX (Lifetime) Sum of As"
               sad4a   = "DSM4 SEPANX (Lifetime) 8+ in 6Mo"
               sad4b   = "DSM4 SEPANX (Lifetime) Crit. B"
               sad4c   = "DSM4 SEPANX (Lifetime) Crit. C"
               sad4d   = "DSM4 SEPANX (Lifetime) Crit. D"
               sad4dx  = "DSM4 SEPANX (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    sad4sk
                    sad4a1-sad4a8 sad4a3x sad4a5x sad4ax sad4a 
                    sad4b sad4c sad4d sad4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_sep_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(sepd4,sa3a sa3b sa3c sad4sk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepd4,sad4sk sa7 sa8a sa8b sa11a sa11b sad4a1);
                     %multi(sepd4,sa1a sad4a2);
                     %multi(sepd4,sa2a sad4a3);
                     %multi(sepd4,sa1b sa2b sa3a sa3b sa3c sa3c_1 sad4a4);
                     %multi(sepd4,sad4sk sa6a sa6b sad4a5);
                     %multi(sepd4,sad4sk sa4a sa5a sa5b sad4a6);
                     %multi(sepd4,sad4sk sa9 sad4a7);
                     %multi(sepd4,sad4sk sa10 sad4a8);
                     %multi(sepd4,sad4sk sad4a1-sad4a8 sad4a3x sad4a5x);
                     %multi(sepd4,sad4sk sad4a3x sad4a5x sad4ax sad4a);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepd4,sad4sk sa12c sa12c_1 sa13c sa13c_1 sad4b);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepd4,sad4sk sa12a_1 sa13a_1 sad4c);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepd4,sad4sk sa14a_1a sa14a_1b sa14a_2a sa14a_2b
                                  sa14a_3a sa14a_3b sa14a_4a sa14a_4b
                                  sa14a_5a sa14a_5b sad4d);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepd4,sad4sk sad4a3x sad4a5x sad4b sad4c sad4d 
                                  sad4dx);

            ******************;
            * keep variables *;
            ******************;

               data sepd4;
               set sepd4;
               keep ind_id  
                    sad4sk
                    sad4a1-sad4a8 sad4a3x sad4a5x sad4ax sad4a 
                    sad4b sad4c sad4d sad4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=sepd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend sepd4;

********;
* test *;
********;
/*
   %sepd4(coga.csaga2,1);
*/
