********************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Social Phobia Disorder *;
********************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %socd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro socd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SOCD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SOCD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data socd4;
      set &dsn.;

      attrib spd4sk1 spd4sk2 spd4sk
             spd4a spd4b spd4c spd4d spd4e spd4f spd4g spd4dx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas spd4sk1 spd4sk2 spd4sk
                      spd4a spd4b spd4c spd4d spd4e spd4f spd4g spd4dx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      *********************;
      * test for skip out *;
      *********************;
 
         if sp2 not in (1,3,5) then spd4sk1=5;
         else do;
            if sp2 in (3,5) then spd4sk1=1;
            else spd4sk1=5;
         end;

         if spd4sk1=5 then do;
            array skp1 spd4a spd4b spd4c spd4d spd4e spd4f spd4g spd4dx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if spd4sk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************;
            * persistent fear of possible scrutiny *;
            ****************************************;

               if sp2 not in (1,3,5) then spd4a=.;
               else spd4a=(4*(sp2=5))+(2*(sp2=3))+1;

      *********************;
      * test for skip out *;
      *********************;

         if sum(sum(sp3a_1 not in (1,5),
                    sp3a_2 not in (1,5),
                    sp3a_3 not in (1,5),
                    sp3a_4 not in (1,5),
                    sp3a_5 not in (1,5),
                    sp3a_6 not in (1,5),
                    sp3a_7 not in (1,5),
                    sp3a_8 not in (1,5),
                    sp3a_9 not in (1,5),
                    sp3a_10 not in (1,5),
                    sp3a_11 not in (1,5),
                    sp3a_12 not in (1,5),
                    sp3a_13 not in (1,5))=13,
                sum(sp3b_1 not in (1,5),
                    sp3b_2 not in (1,5),
                    sp3b_3 not in (1,5),
                    sp3b_4 not in (1,5),
                    sp3b_5 not in (1,5),
                    sp3b_6 not in (1,5),
                    sp3b_7 not in (1,5),
                    sp3b_8 not in (1,5),
                    sp3b_9 not in (1,5),
                    sp3b_10 not in (1,5),
                    sp3b_11 not in (1,5),
                    sp3b_12 not in (1,5),
                    sp3b_13 not in (1,5))=13,
                sum(sp3c_1 not in (1,5),
                    sp3c_2 not in (1,5),
                    sp3c_3 not in (1,5),
                    sp3c_4 not in (1,5),
                    sp3c_5 not in (1,5),
                    sp3c_6 not in (1,5),
                    sp3c_7 not in (1,5),
                    sp3c_8 not in (1,5),
                    sp3c_9 not in (1,5),
                    sp3c_10 not in (1,5),
                    sp3c_11 not in (1,5),
                    sp3c_12 not in (1,5),
                    sp3c_13 not in (1,5))=13)=3 then spd4sk2=5;
         else do;
            if sum(sum(sp3a_1=5,sp3a_2=5,sp3a_3=5,
                       sp3a_4=5,sp3a_5=5,sp3a_6=5,
                       sp3a_7=5,sp3a_8=5,sp3a_9=5,
                       sp3a_10=5,sp3a_11=5,sp3a_12=5,
                       sp3a_13=5)>0,
                   sum(sp3b_1=5,sp3b_2=5,sp3b_3=5,
                       sp3b_4=5,sp3b_5=5,sp3b_6=5,
                       sp3b_7=5,sp3b_8=5,sp3b_9=5,
                       sp3b_10=5,sp3b_11=5,sp3b_12=5,
                       sp3b_13=5)>0,
                   sum(sp3c_1=5,sp3c_2=5,sp3c_3=5,
                       sp3c_4=5,sp3c_5=5,sp3c_6=5,
                       sp3c_7=5,sp3c_8=5,sp3c_9=5,
                       sp3c_10=5,sp3c_11=5,sp3c_12=5,
                       sp3c_13=5)) then spd4sk2=1;
            else spd4sk2=5;
         end;

         if spd4sk2=5 then do;
            array skp2 spd4b spd4c spd4d spd4e spd4f spd4g spd4dx;
            do over skp2;
              skp2=.S;
            end;
         end;

         if spd4sk2=5 then go to skipout;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************;
            * phobic stimulus invariable provokes anxiety *;
            ***********************************************;

               if sp6 not in (1,5) then spd4b=.;
               else spd4b=(4*(sp6=5))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * fear recognized as unreasonable *;
            ***********************************;

               if sum(sp3c_1 not in (1,5),
                      sp3c_2 not in (1,5),
                      sp3c_3 not in (1,5),
                      sp3c_4 not in (1,5),
                      sp3c_5 not in (1,5),
                      sp3c_6 not in (1,5),
                      sp3c_7 not in (1,5),
                      sp3c_8 not in (1,5),
                      sp3c_9 not in (1,5),
                      sp3c_10 not in (1,5),
                      sp3c_11 not in (1,5),
                      sp3c_12 not in (1,5),
                      sp3c_13 not in (1,5))=13 then spd4c=.;
               else spd4c=(4*(sum(sp3c_1=5,sp3c_2=5,sp3c_3=5,
                                  sp3c_4=5,sp3c_5=5,sp3c_6=5,
                                  sp3c_7=5,sp3c_8=5,sp3c_9=5,
                                  sp3c_10=5,sp3c_11=5,sp3c_12=5,
                                  sp3c_13=5)>0))+1;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************************************;
            * phobic situations avoided or indured w/stress *;
            *************************************************;

               if sp5 not in (1,5) then spd4d=.;
               else spd4d=(4*(sp5=5))+1;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************;
            * avoidant behavior interferes w/ functioning *;
            ***********************************************;

               if sum(sp7 not in (1,5),
                      sp7a not in (1,5),
                      sp7b not in (1,5),
                      sp7c not in (1,5),
                      sp7d not in (1,5))=5 then spd4e=.;
               else spd4e=(4*(sum(sp7=5,sp7a=5,sp7b=5,sp7c=5,sp7d=5)>0))+1;

      ***************;
      * Criterion F *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ******************************;
            * duration at least 6 months *;
            ******************************;

               if sp9 not in (1,5) then spd4f=.;
               else spd4f=(4*(sp9=5))+1;

      ***************;
      * Criterion G *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************;
            * no physiologic/medical/organic basis *;
            ****************************************;

               if sp10 not in (1,2,3) then spd4g=.;
               else spd4g=(4*(sp10=1))+(2*(sp10=3))+1;

      **************************************;
      * Criterion H -- Skip in this module *;
      **************************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(spd4a,spd4b,spd4c,spd4d,spd4e,spd4f,spd4g)=0 then spd4dx=.;
            else do;
               spd4dx=1;
               if (spd4b=5 and spd4c=5 and spd4d=5 and 
                   spd4e=5 and spd4f=5) then do;
                  if (spd4a=3 and spd4g in (3,5)) or
                     (spd4a in (3,5) and spd4g=3) then spd4dx=3;
                  if (spd4a=5 and spd4g=5) then spd4dx=5;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         spd4sk=max(spd4sk1,spd4sk2);

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label spd4sk1 = "DSM4 SOCPHB Skip Out #1"
               spd4sk2 = "DSM4 SOCPHB Skip Out #2"
               spd4sk  = "DSM4 SOCPHB Skip Out"
               spd4a   = "DSM4 SOCPHB (Lifetime) Crit. A"
               spd4b   = "DSM4 SOCPHB (Lifetime) Crit. B"
               spd4c   = "DSM4 SOCPHB (Lifetime) Crit. C"
               spd4d   = "DSM4 SOCPHB (Lifetime) Crit. D"
               spd4e   = "DSM4 SOCPHB (Lifetime) Crit. E"
               spd4f   = "DSM4 SOCPHB (Lifetime) Crit. F"
               spd4g   = "DSM4 SOCPHB (Lifetime) Crit. G"
               spd4dx  = "DSM4 SOCPHB (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id 
                    spd4sk1 spd4sk2 spd4sk 
                    spd4a spd4b spd4c spd4d spd4e spd4f spd4g spd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_soc_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(socd4,sp2 spd4sk1);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 sp2 spd4a);

               ************;
               * skip out *;
               ************;

                  %multi(socd4,spd4sk1 
                               sp3a_1-sp3a_13 sp3b_1-sp3b_13 sp3c_1-sp3c_13
                               spd4sk2);
                  %multi(socd4,spd4sk1 spd4sk2 spd4sk);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 spd4sk2 sp6 spd4b);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 spd4sk2 sp3c_1-sp3c_13 spd4c);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 spd4sk2 sp5 spd4d);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 spd4sk2 sp7 sp7a sp7b sp7c sp7d 
                                  spd4e);

               ***************;
               * Criterion F *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 spd4sk2 sp9 spd4f);

               ***************;
               * Criterion G *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socd4,spd4sk1 spd4sk2 sp10 spd4g);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(socd4,spd4sk1 spd4sk2 
                                  spd4a spd4b spd4c spd4d 
                                  spd4e spd4f spd4g spd4dx);

            ******************;
            * keep variables *;
            ******************;

               data socd4;
               set socd4;
               keep ind_id 
                    spd4sk1 spd4sk2 spd4sk
                    spd4a spd4b spd4c spd4d spd4e spd4f spd4g spd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=socd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend socd4;

********;
* test *;
********;
/*
   %socd4(coga.csaga2,1);
*/
