*******************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Somatization Disorder *;
*******************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %somd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro somd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SOMD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SOMD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data somd4;
      set &dsn.;

      attrib smd4sk
             smd4a 
             smd4b101-smd4b109 smd4b13x smd4b14x smd4b15x smd4b1x smd4b1
             smd4b201-smd4b205 smd4b23x smd4b24x smd4b25x smd4b2x smd4b2
             smd4b303 smd4b304 smd4b33x smd4b34x smd4b35x smd4b3x smd4b3
             smd4b401-smd4b411 smd4b43x smd4b44x smd4b45x smd4b4x smd4b4
             smd4d smd4dx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas smd4sk smd4a 
                      smd4b101-smd4b109 smd4b13x smd4b14x smd4b15x smd4b1x 
                      smd4b1 smd4b201-smd4b205 smd4b23x smd4b24x smd4b25x 
                      smd4b2x smd4b2 smd4b303 smd4b304 smd4b33x smd4b34x 
                      smd4b35x smd4b3x smd4b3 smd4b401-smd4b411 smd4b43x 
                      smd4b44x smd4b45x smd4b4x smd4b4 smd4d smd4dx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to skipout;

      ******************************;
      * rescale somatization items *;
      ******************************;

         array itm sm1 sm2 sm3 sm4a sm4b sm4c sm4d sm4e
                   sm5 sm6a sm6b sm6c sm6d sm6e sm7 
                   sm8a sm8b sm8c sm9 sm10 sm11a sm11b
                   sm11c sm11d sm11e sm11f sm11g sm11h
                   sm11i sm11j sm12a sm12b sm12c;
         do over itm;
            if itm<1 or itm>5 then itm=.;
            if itm=2 then itm=1;
         end;

      *********************;
      * test for skip out *;
      *********************;

         if sum(sum(sm4a not in (1,3,4,5),
                    sm4b not in (1,3,4,5),
                    sm4c not in (1,3,4,5),
                    sm4d not in (1,3,4,5),
                    sm4e not in (1,3,4,5))=5,
                sum(sm6a not in (1,3,4,5),
                    sm6b not in (1,3,4,5),
                    sm6c not in (1,3,4,5),
                    sm6d not in (1,3,4,5),
                    sm6e not in (1,3,4,5))=5)=2 then smd4sk=5;
         else do;
            if sum(max(sm4a,sm4b,sm4c,sm4d,sm4e)=5,
                   sum(sm6a=5,sm6b=5,sm6c=5,sm6d=5,sm6e=5)>3)=2 then 
               smd4sk=1;
            else smd4sk=5;
         end;

         if smd4sk=5 then do;
            array skp smd4a 
                      smd4b101-smd4b109 smd4b13x smd4b14x smd4b15x smd4b1x 
                      smd4b1 smd4b201-smd4b205 smd4b23x smd4b24x smd4b25x 
                      smd4b2x smd4b2 smd4b303 smd4b304 smd4b33x smd4b34x 
                      smd4b35x smd4b3x smd4b3 smd4b401-smd4b411 smd4b43x 
                      smd4b44x smd4b45x smd4b4x smd4b4 smd4d smd4dx;
            do over skp;
               skp=.S;
            end;
         end;

         if smd4sk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         if n(sm1,sm2)=0 then smd4a=.;
         else do;
            if sm1=5 and sm2=5 then smd4a=5;
            else if sm1 in (3,5) or sm2 in (3,5) then smd4a=3;
            else if sm1 in (4,5) or sm2 in (4,5) then smd4a=4;
         end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * pain symptoms *;
         *****************;

            if n(sm4e,sm5,sm6a,sm6b,sm6c,sm6d,sm6e,sm8b,sm12a)=0 then do;
               smd4b13x=.;
               smd4b14x=.;
               smd4b15x=.;
               smd4b1x=.;
               smd4b1=.;
            end;
            else do;
               smd4b1=1;
               smd4b13x=0;
               smd4b14x=0;
               smd4b15x=0;
               array b1i sm4e sm5 sm6a sm6b sm6c sm6d sm6e sm8b sm12a;
               array b1s smd4b101-smd4b109;
               do over b1i;
                  b1s=b1i;
                  if b1i=3 then smd4b13x=smd4b13x+1;
                  if b1i=4 then smd4b14x=smd4b14x+1;
                  if b1i=5 then smd4b15x=smd4b15x+1;
               end;
               smd4b1x=sum(smd4b13x,smd4b14x,smd4b15x);
               if smd4b13x>3 then smd4b1=3;
               else if smd4b14x>3 then smd4b1=4;
               else if smd4b15x>3 then smd4b1=5;
            end;

         *****************************;
         * gastrointestinal symptoms *;
         *****************************;

            if n(sm3,sm4a,sm4b,sm4c,sm4d,sm4e)=0 then do;
               smd4b23x=.;
               smd4b24x=.;
               smd4b25x=.;
               smd4b2x=.;
               smd4b2=.;
            end;
            else do;
               smd4b2=1;
               smd4b23x=0;
               smd4b24x=0;
               smd4b25x=0;
               array b2i sm3 sm4a sm4b sm4c sm4d;
               array b2s smd4b201-smd4b205;
               do over b2i;
                  b2s=b2i;
                  if b2i=3 then smd4b23x=smd4b23x+1;
                  if b2i=4 then smd4b24x=smd4b24x+1;
                  if b2i=5 then smd4b25x=smd4b25x+1;
               end;
               smd4b2x=sum(smd4b23x,smd4b24x,smd4b25x);
               if smd4b23x>1 then smd4b2=3;
               else if smd4b24x>1 then smd4b2=4;
               else if smd4b25x>1 then smd4b2=5;
            end;

         *******************;
         * sexual symptoms *;
         *******************;

            if n(sm12b,sm12c)=0 then do;
               smd4b33x=.;
               smd4b34x=.;
               smd4b35x=.;
               smd4b3x=.;
               smd4b3=.;
            end;
            else do;
               smd4b3=1;
               smd4b33x=0;
               smd4b34x=0;
               smd4b35x=0;
               array b3i sm12b sm12c;
               array b3s smd4b303 smd4b304;
               do over b3i;
                  b3s=b3i;
                  if b3i=3 then smd4b33x=smd4b33x+1;
                  if b3i=4 then smd4b34x=smd4b34x+1;
                  if b3i=5 then smd4b35x=smd4b35x+1;
               end;
               smd4b3x=sum(smd4b33x,smd4b34x,smd4b35x);
               if smd4b33x>0 then smd4b3=3;
               else if smd4b34x>1 then smd4b3=4;
               else if smd4b35x>1 then smd4b3=5;
            end;

         ******************************;
         * pseudoneurologial symptoms *;
         ******************************;

            if n(sm9,sm10,sm11a,sm11b,sm11c,sm11e,
                 sm11f,sm11g,sm11h,sm11i,sm11j)=0 then do;
               smd4b43x=.;
               smd4b44x=.;
               smd4b45x=.;
               smd4b4x=.;
               smd4b4=.;
            end;
            else do;
               smd4b4=1;
               smd4b43x=0;
               smd4b44x=0;
               smd4b45x=0;
               array b4i sm9 sm10 sm11a sm11b sm11c sm11e 
                         sm11f sm11g sm11h sm11i sm11j;
               array b4s smd4b401-smd4b411;
               do over b4i;
                  b4s=b4i;
                  if b4i=3 then smd4b43x=smd4b43x+1;
                  if b4i=4 then smd4b44x=smd4b44x+1;
                  if b4i=5 then smd4b45x=smd4b45x+1;
               end;
               smd4b4x=sum(smd4b43x,smd4b44x,smd4b45x);
               if smd4b43x>1 then smd4b4=3;
               else if smd4b44x>1 then smd4b4=4;
               else if smd4b45x>1 then smd4b4=5;
            end;

      ***************************************************;
      * Criterion C -- Assessed within individual items *;
      ***************************************************;

      ***************;
      * Criterion D *;
      ***************;

         if sum(sm13a not in (1,5),
                sm13b not in (1,5))=2 then smd4d=.;
         else smd4d=(4*(sum(sm13a=1,sm13b=1)>0))+1;

      *************;
      * Diagnosis *;
      *************;

         if n(smd4a,smd4b1,smd4b2,smd4b3,smd4b4,smd4d)=0 then smd4dx=.;
         else do;
            smd4dx=1;
            if (smd4a=5 and smd4b1=5 and smd4b2=5 and 
                sum(dm1=1,dm12a=1,smd4b3)>0 and smd4b4=5 and 
                smd4d=5) then smd4dx=5;
            else if (smd4a=3 and smd4b1 in (3,5) and smd4b2 in (3,5) and
                     sum(dm1=1,dm12a=1,smd4b3 in (3,5))>0 and 
                     smd4b4 in (3,5) and smd4d=5) then smd4dx=3;
            else if (smd4a=4 and smd4b1 in (4,5) and smd4b2 in (4,5) and
                     sum(dm1=1,dm12a=1,smd4b3 in (4,5))>0 and 
                     smd4b4 in (4,5) and smd4d=5) then smd4dx=4;
         end;

      *************************;
      * jump here if skip out *;
      *************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label smd4sk   = "DSM4 SOM Skip Out"
               smd4a    = "DSM4 SOM (Lifetime) Crit. A"
               smd4b101 = "DSM4 SOM (Lifetime) Crit. B1.1"
               smd4b102 = "DSM4 SOM (Lifetime) Crit. B1.2"
               smd4b103 = "DSM4 SOM (Lifetime) Crit. B1.3"
               smd4b104 = "DSM4 SOM (Lifetime) Crit. B1.4"
               smd4b105 = "DSM4 SOM (Lifetime) Crit. B1.5"
               smd4b106 = "DSM4 SOM (Lifetime) Crit. B1.6"
               smd4b107 = "DSM4 SOM (Lifetime) Crit. B1.7"
               smd4b108 = "DSM4 SOM (Lifetime) Crit. B1.8"
               smd4b109 = "DSM4 SOM (Lifetime) Crit. B1.9"
               smd4b13x = "DSM4 SOM (Lifetime) Sum B1s=3"
               smd4b14x = "DSM4 SOM (Lifetime) Sum B1s=4"
               smd4b15x = "DSM4 SOM (Lifetime) Sum B1s=5"
               smd4b1x  = "DSM4 SOM (Lifetime) Sum B1s=3,4,5"
               smd4b1   = "DSM4 SOM (Lifetime) 4+ B1s"
               smd4b201 = "DSM4 SOM (Lifetime) Crit. B2.1"
               smd4b202 = "DSM4 SOM (Lifetime) Crit. B2.2"
               smd4b203 = "DSM4 SOM (Lifetime) Crit. B2.3"
               smd4b204 = "DSM4 SOM (Lifetime) Crit. B2.4"
               smd4b205 = "DSM4 SOM (Lifetime) Crit. B2.5"
               smd4b23x = "DSM4 SOM (Lifetime) Sum B2s=3"
               smd4b24x = "DSM4 SOM (Lifetime) Sum B2s=4"
               smd4b25x = "DSM4 SOM (Lifetime) Sum B2s=5"
               smd4b2x  = "DSM4 SOM (Lifetime) Sum B2s=3,4,5"
               smd4b2   = "DSM4 SOM (Lifetime) 2+ B2s"
               smd4b303 = "DSM4 SOM (Lifetime) Crit. B3.3"
               smd4b304 = "DSM4 SOM (Lifetime) Crit. B3.4"
               smd4b33x = "DSM4 SOM (Lifetime) Sum B3s=3"
               smd4b34x = "DSM4 SOM (Lifetime) Sum B3s=4"
               smd4b35x = "DSM4 SOM (Lifetime) Sum B3s=5"
               smd4b3x  = "DSM4 SOM (Lifetime) Sum B3s=3,4,5"
               smd4b3   = "DSM4 SOM (Lifetime) 1+ B3s"
               smd4b401 = "DSM4 SOM (Lifetime) Crit. B4.1"
               smd4b402 = "DSM4 SOM (Lifetime) Crit. B4.2"
               smd4b403 = "DSM4 SOM (Lifetime) Crit. B4.3"
               smd4b404 = "DSM4 SOM (Lifetime) Crit. B4.4"
               smd4b405 = "DSM4 SOM (Lifetime) Crit. B4.5"
               smd4b406 = "DSM4 SOM (Lifetime) Crit. B4.6"
               smd4b407 = "DSM4 SOM (Lifetime) Crit. B4.7"
               smd4b408 = "DSM4 SOM (Lifetime) Crit. B4.8"
               smd4b409 = "DSM4 SOM (Lifetime) Crit. B4.9"
               smd4b410 = "DSM4 SOM (Lifetime) Crit. B4.10"
               smd4b411 = "DSM4 SOM (Lifetime) Crit. B4.11"
               smd4b43x = "DSM4 SOM (Lifetime) Sum B4s=3"
               smd4b44x = "DSM4 SOM (Lifetime) Sum B4s=4"
               smd4b45x = "DSM4 SOM (Lifetime) Sum B4s=5"
               smd4b4x  = "DSM4 SOM (Lifetime) Sum B4s=3,4,5"
               smd4b4   = "DSM4 SOM (Lifetime) 1+ B4s"
               smd4dx   = "DSM4 SOM (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    smd4sk
                    smd4a 
                    smd4b101-smd4b109 smd4b13x smd4b14x smd4b15x smd4b1x smd4b1
                    smd4b201-smd4b205 smd4b23x smd4b24x smd4b25x smd4b2x smd4b2
                    smd4b303 smd4b304 smd4b33x smd4b34x smd4b35x smd4b3x smd4b3
                    smd4b401-smd4b411 smd4b43x smd4b44x smd4b45x smd4b4x smd4b4
                    smd4d smd4dx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./dsm4_som_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(somd4,sm4a sm4b sm4c sm4d sm4e
                               sm6a sm6b sm6c sm6d sm6e smd4sk);

               ***************;
               * Criterion A *;
               ***************;

                  %multi(somd4,smd4sk sm1 sm2 smd4a);

               ***************;
               * Criterion B *;
               ***************;

                  %multi(somd4,smd4sk sm4e smd4b101);
                  %multi(somd4,smd4sk sm5 smd4b102);
                  %multi(somd4,smd4sk sm6a smd4b103); 
                  %multi(somd4,smd4sk sm6b smd4b104);
                  %multi(somd4,smd4sk sm6c smd4b105);
                  %multi(somd4,smd4sk sm6d smd4b106);
                  %multi(somd4,smd4sk sm6e smd4b107);
                  %multi(somd4,smd4sk sm8b smd4b108);
                  %multi(somd4,smd4sk sm12a smd4b109);
                  %multi(somd4,smd4sk smd4b101-smd4b109
                               smd4b13x smd4b14x smd4b15x 
                               smd4b1x smd4b1);

                  %multi(somd4,smd4sk sm3 smd4b201);
                  %multi(somd4,smd4sk sm4a smd4b202);
                  %multi(somd4,smd4sk sm4b smd4b203);
                  %multi(somd4,smd4sk sm4c smd4b204);
                  %multi(somd4,smd4sk sm4d smd4b205);
                  %multi(somd4,smd4sk smd4b201-smd4b205
                               smd4b23x smd4b24x smd4b25x 
                               smd4b2x smd4b2);

                  %multi(somd4,smd4sk sm12b smd4b303);
                  %multi(somd4,smd4sk sm12c smd4b304);
                  %multi(somd4,smd4sk smd4b303 smd4b304
                               smd4b33x smd4b34x smd4b35x 
                               smd4b3x smd4b3);

                  %multi(somd4,smd4sk sm9 smd4b401);
                  %multi(somd4,smd4sk sm10 smd4b402);
                  %multi(somd4,smd4sk sm11a smd4b403);
                  %multi(somd4,smd4sk sm11b smd4b404);
                  %multi(somd4,smd4sk sm11c smd4b405);
                  %multi(somd4,smd4sk sm11e smd4b406);
                  %multi(somd4,smd4sk sm11f smd4b407);
                  %multi(somd4,smd4sk sm11g smd4b408);
                  %multi(somd4,smd4sk sm11h smd4b409);
                  %multi(somd4,smd4sk sm11i smd4b410);
                  %multi(somd4,smd4sk sm11j smd4b411);
                  %multi(somd4,smd4sk smd4b401-smd4b411
                               smd4b43x smd4b44x smd4b45x 
                               smd4b4x smd4b4);

               ***************;
               * Criterion D *;
               ***************;

                  %multi(somd4,smd4sk sm13a sm13b smd4d);

               *************;
               * Diagnosis *;
               *************;

                  %multi(somd4,smd4sk smd4a smd4b1 smd4b2 
                               dm1 dm12a smd4b3
                               smd4b4 smd4d smd4dx);

            ******************;
            * keep variables *;
            ******************;

               data somd4;
               set somd4;
               keep ind_id  
                    smd4sk
                    smd4a 
                    smd4b101-smd4b109 smd4b13x smd4b14x smd4b15x smd4b1x smd4b1
                    smd4b201-smd4b205 smd4b23x smd4b24x smd4b25x smd4b2x smd4b2
                    smd4b303 smd4b304 smd4b33x smd4b34x smd4b35x smd4b3x smd4b3
                    smd4b401-smd4b411 smd4b43x smd4b44x smd4b45x smd4b4x smd4b4
                    smd4d smd4dx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=somd4;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend somd4;

********;
* test *;
********;
/*
   %somd4(coga.csaga2,1);
*/
