*****************************************************************;
* CSSAGA-II Diagnostic Algorithm - DSM-IV - Nicotine Dependence *;
*****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %tobd4(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro tobd4(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in TOBD4: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in TOBD4: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      *****************************;
      * test for parent interview *;
      *****************************;

         %if &who.=p %then %do;

            data tobd4;
            set &dsn.;
            attrib tdd4sk td4da1-td4da7 td4dax td4da td4db tdd4dp length=3;
            array nas tdd4sk td4da1-td4da7 td4dax td4da td4db tdd4dp;
            do over nas;
               nas=.N;
            end;
            keep ind_id tdd4sk td4da1-td4da7 td4dax td4da td4db tdd4dp;
            run;

            %goto errexit;

         %end;

      data tobd4;
      set &dsn.;

      attrib tdd4sk td4da1-td4da7 td4dax td4da td4db tdd4dp length=3;

      *********************;
      * test for skip out *;
      *********************;

         if sum(td1a not in (1,5),
                td1c not in (1,5),
                td1d not in (1,5),
                td4 not in (1,5))=4 then tdd4sk=5;
         else do;
            if sum(sum(td1a=5,td1c=5,td1d=5)>0,td4=5)=2 then tdd4sk=1;
            else tdd4sk=5;
         end;

         if tdd4sk=5 then do;
            array skp td4da1-td4da7 td4dax td4da td4db tdd4dp;
            do over skp;
               skp=.S;
            end;
         end;

         if tdd4sk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            ********************;
            * marked tolerance *;
            ********************;

               if sum(td20b not in (1,5),
                      td20c not in (1,5))=2 then td4da1=.;
               else td4da1=(4*(td20b=5 or td20c=5))+1;

            ********************************************;
            * characteristic withdrawal symptoms or    *;
            * use to relieve/avoid withdrawal symptoms *;
            ********************************************;

               if sum(td16b not in (1,5),
                      td16e not in (1,5))=2 then td4da2=.;
               else td4da2=(4*(td16b=5 or td16e=5))+1;

            *********************************************************;
            * used larger amount or for longer period than intended *;
            *********************************************************;

               if sum(td13 not in (1,5),
                      td13a not in (1,5))=2 then td4da3=.;
               else td4da3=(4*(td13=5 or td13a=5))+1;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            ***************************************************************;

               if sum(td15 not in (1,5),
                      td15b_1 not in (1,5))=2 then td4da4=.;
               else td4da4=(4*(td15=5 or td15b_1=5))+1;

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            ****************************************************************;

               if sum(sum(td5a_1<0,td5a_2<0,td5b_1,td5b_2)=4,
                      sum(td11a<0,td11a_1 not in (1,5))=2,
                      td11b not in (1,5))=3 then td4da5=.;
               else td4da5=(4*((sum(td5a_1,td5a_2)>1 or 
                                sum(td5b_1,td5b_2)>19) or
                               sum(td11a>2,td11a_1=5)>0 or
                               td11b=5))+1;

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if td12c not in (1,5) then td4da6=.;
               else td4da6=(4*(td12c=5))+1;

            ***********************************************************;
            * use reduced social/occupational/recreational activities *;
            ***********************************************************;

               if sum(td17c not in (1,5),
                      td18b not in (1,5),
                      td19 not in (1,5))=3 then td4da7=.;
               else td4da7=(4*(td17c=5 or td18b=5 or td19=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of td4da1-td4da7)=0 then td4dax=.;
               else do;
                  td4dax=0;
                  array sxs td4da1-td4da7;
                  do over sxs;
                     if sxs=5 then td4dax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if td4dax=. then td4da=.;
               else td4da=(4*(td4dax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if td21 not in (1,5) then td4db=.;
            else td4db=(4*(td21=5))+1;
                                 
         *************;
         * Diagnosis *;
         *************;

            if n(td4da,td4db)=0 then td4dp=.;
            else tdd4dp=(4*(td4da=5 and td4db=5))+1;

      *********************************************;
      * jump here if parent interview or skip out *;
      *********************************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label tdd4sk = "DSM4 TOB Dependence Skip Out"
               td4da1 = "DSM4 TOB Dependence Crit. A1"
               td4da2 = "DSM4 TOB Dependence Crit. A2"
               td4da3 = "DSM4 TOB Dependence Crit. A3"
               td4da4 = "DSM4 TOB Dependence Crit. A4" 
               td4da5 = "DSM4 TOB Dependence Crit. A5" 
               td4da6 = "DSM4 TOB Dependence Crit. A6" 
               td4da7 = "DSM4 TOB Dependence Crit. A7" 
               td4dax = "DSM4 TOB Dependence Sum of As" 
               td4da  = "DSM4 TOB Dependence 3+ As" 
               td4db  = "DSM4 TOB Dependence Crit. B" 
               tdd4dp = "DSM4 TOB Dependence Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    tdd4sk td4da1-td4da7 td4dax td4da td4db tdd4dp;
               run;

         %end;
         %else %do;

            run;

            %if &who.=c %then %do;

               ******************;
               * reroute output *;
               ******************;

                  proc printto new print="./dsm4_tob_&study._&who..lst";
                  run;

               **************;
               * run checks *;
               **************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(tobd4,td1a td1c td1d td4 tdd4sk);

                  **************;
                  * Dependence *;
                  **************;

                     ***************;
                     * Criterion A *;
                     ***************;
    
                        %multi(tobd4,tdd4sk td20b td20c td4da1);
                        %multi(tobd4,tdd4sk td16b td16e td4da2);
                        %multi(tobd4,tdd4sk td13 td13a td4da3);
                        %multi(tobd4,tdd4sk td15 td15b_1 td4da4);
                        %multi(tobd4,tdd4sk td5a_1 td5a_2 td5b_1 td5b_2 
                                     td11a td11a_1 td11b td4da5);
                        %multi(tobd4,tdd4sk td12c td4da6);
                        %multi(tobd4,tdd4sk td17c td18b td19 td4da7);
                        %multi(tobd4,tdd4sk td4da1-td4da7 td4dax);
                        %multi(tobd4,tdd4sk td4dax td4da);

                     ***************;
                     * Criterion B *;
                     ***************;

                        %multi(tobd4,tdd4sk td21 td4db);
   
                     *************;
                     * Diagnosis *;
                     *************;

                        %multi(tobd4,tdd4sk td4da td4db tdd4dp);

               ******************;
               * keep variables *;
               ******************;

                  data tobd4;
                  set tobd4;
                  keep ind_id  
                       tdd4sk td4da1-td4da7 td4dax td4da td4db tdd4dp;
                  run;

               ************;
               * contents *;
               ************;

                  proc contents data=tobd4;
                  run;

               ******************;
               * reroute output *;
               ******************;

                  proc printto;
                  run;

            %end;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend tobd4;

********;
* test *;
********;
/*
   %tobd4(coga.csaga2,1);
*/
