**********************************************************;
* CSSAGA-II Diagnostic Algorithm - Feighner - Alcoholism *;
**********************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %alcfn(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro alcfn(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ALCFN: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ALCFN: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data alcfn;
      set &dsn.;

      attrib alfna1-alfna4 alfnax alfna alfnb1-alfnb4 alfnbx alfnb 
             alfnc1-alfnc4 alfncx alfnc alfnd1-alfnd5 alfndx alfnd 
             alfnx alfndp alfnsk length=3; 

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(al1a not in (1,5),
                   al1f not in (1,5),
                   al1g not in (1,5),
                   al11a<0)=4 then alfnsk=5;
            else do;
               if (al1a=5 and (al1f=5 or al1g=5) and al11a>2) then alfnsk=1;
               else alfnsk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(al1c not in (1,5),
                   al1f not in (1,5),
                   al11a<0)=3 then alfnsk=5;
            else do;
               if ((al1c=5 or al1f=5) and al11a>2) then alfnsk=1;
               else alfnsk=5;
            end;
         %end;

         if alfnsk=5 then do;
            array skp alfna1-alfna4 alfnax alfna alfnb1-alfnb4 alfnbx alfnb 
                      alfnc1-alfnc4 alfncx alfnc alfnd1-alfnd5 alfndx alfnd 
                      alfnx alfndp;
            do over skp;
               skp=.S;
            end;
         end;

         if alfnsk=5 then go to skipout;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &study.=coga %then %do;
            %if &who.=p %then %do;
               array nas alfna1-alfna4 alfnax alfna alfnb1-alfnb4 alfnbx alfnb 
                         alfnd1-alfnd5 alfndx alfnd;
               do over nas;
                  nas=.N;
               end;
               go to par_hop1;
            %end;
         %end;

      **************;
      * Alcoholism *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            ***************************;
            * any withdrawal symptoms *;
            ***************************;

               if 0<dm3a<13 then do;
                  alfna1=.N;
               end;
               else do;
                  if sum(al46a_1 not in (1,5),al46b_1 not in (1,5),
                         al46c_1 not in (1,5),al46d_1 not in (1,5),
                         al46e_1 not in (1,5),al46f_1 not in (1,5),
                         al46g_1 not in (1,5),al46h_1 not in (1,5),
                         al46i_1 not in (1,5),al46j_1 not in (1,5),
                         al46k_1 not in (1,5),al46l_1 not in (1,5),
                         al46m_1 not in (1,5))=13 then alfna1=.;
                  else alfna1=(4*(sum(al46a_1=5,al46b_1=5,al46c_1=5,al46d_1=5,
                                      al46e_1=5,al46f_1=5,al46g_1=5,al46h_1=5,
                                      al46i_1=5,al46j_1=5,al46k_1=5,al46l_1=5,
                                      al46m_1=5)>0))+1;
               end;

            ***************************************************;
            * history of marked complications -- NOT ASSESSED *;
            ***************************************************;

               alfna2=.N;

            ***********************;
            * alcoholic blackouts *;
            ***********************;

               if al21d not in (1,5) then alfna3=.;
               else alfna3=(4*(al21d=5))+1;

            *******************************;
            * alcoholic binges or benders *;
            *******************************;

               if al20d not in (1,5) then alfna4=.;
               else alfna4=(4*(al20d=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of alfna1-alfna4)=0 then alfnax=.;
               else do;
                  alfnax=0;
                  array sxa alfna1-alfna4;
                  do over sxa;
                     if sxa=5 then alfnax+1;
                  end;
               end;

            *********************;
            * at least one item *;
            *********************;

               if alfnax=. then alfna=.;
               else alfna=(4*(alfnax>0))+1;

         ***************;
         * Criterion B *;
         ***************;

            ***************************************;
            * unable to stop drinking when wanted *;
            ***************************************;

               if sum(al13c_1 not in (1,5),
                      al15a not in (1,5),
                      al16a not in (1,5))=3 then alfnb1=.;
               else alfnb1=(4*(al13c_1=1 or al15a=5 or al16a=5))+1;

            ****************************************;
            * tried to control drinking with rules *;
            ****************************************;

               if 0<dm3a<13 then do;
                  alfnb2=.N;
               end;
               else do;
                  if al14a not in (1,5) then alfnb2=.;
                  else alfnb2=(4*(al14a=5))+1;
               end;

            *****************************;
            * drinking before breakfast *;
            *****************************;

               if 0<dm3a<13 then do;
                  alfnb3=.N;
               end;
               else do;
                  if al22a not in (1,5) then alfnb3=.;
                  else alfnb3=(4*(al22a=5))+1;
               end;

            ******************************************;
            * drinking non-beverage forms of alcohol *;
            ******************************************;

               if al24a not in (1,5) then alfnb4=.;
               else alfnb4=(4*(al24a=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of alfnb1-alfnb4)=0 then alfnbx=.;
               else do;
                  alfnbx=0;
                  array sxb alfnb1-alfnb4;
                  do over sxb;
                     if sxb=5 then alfnbx+1;
                  end;
               end;

            *********************;
            * at least one item *;
            *********************;

               if alfnbx=. then alfnb=.;
               else alfnb=(4*(alfnbx>0))+1;

         **************************************;
         * jump here if coga parent interview *;
         **************************************;

            par_hop1:

         ***************;
         * Criterion C *;
         ***************;

            *******************;
            * arrests for dui *;
            *******************;

               if 0<dm3a<13 then do;
                  alfnc1=.N;
               end;
               else do;
                  if al34a not in (1,5) then alfnc1=.;
                  else alfnc1=(4*(al34a=5))+1;
               end;

            *****************************************;
            * drinking-related traffic difficulties *;
            *****************************************;

               if 0<dm3a<13 then do;
                  alfnc2=.N;
               end;
               else do;
                  if sum(al26b not in (1,5),
                         al26c not in (1,5))=2 then alfnc2=.;
                  else alfnc2=(4*(al26b=5 or al26c=5))+1;
               end;

            ************************************;
            * drinking-related trouble at work *;
            ************************************;

               if sum(al31a not in (1,5),
                      al31b not in (1,5),
                      al31c not in (1,5),
                      al31d not in (1,5))=4 then alfnc3=.;
               else alfnc3=(4*(sum(al31a=5,al31b=5,al31c=5,al31d=5)>0))+1;

            ***************************;
            * drinking-related fights *;
            ***************************;

               if al33c_1 not in (1,5) then alfnc4=.;
               else alfnc4=(4*(al33c_1=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of alfnc1-alfnc4)=0 then alfncx=.;
               else do;
                  alfncx=0;
                  array sxc alfnc1-alfnc4;
                  do over sxc;
                     if sxc=5 then alfncx+1;
                  end;
               end;

            *********************;
            * at least one item *;
            *********************;

               if alfncx=. then alfnc=.;
               else alfnc=(4*(alfncx>0))+1;

         **********************************;
         * test for coga parent interview *;
         **********************************;

            %if &study.=coga %then %do;
               %if &who.=p %then %do;
                  go to par_hop2;
               %end;
            %end;

         ***************;
         * Criterion D *;
         ***************;

            ****************************************;
            * self perception of drinking too much *;
            ****************************************;

               if al39a not in (1,5) then alfnd1=.;
               else alfnd1=(4*(al39a=5))+1;

            *********************************************;
            * familial objections to amount of drinking *;
            *********************************************;

               if 0<dm3a<13 then do;
                  alfnd2=.N;
               end;
               else do;
                  if al32b not in (1,2,3,4) then alfnd2=.;
                  else alfnd2=(4*(al32b=2))+1;
               end;

            ***********************************;
            * loss of friends due to drinking *;
            ***********************************;

               if al33d_1 not in (1,5) then alfnd3=.;
               else alfnd3=(4*(al33d_1=5))+1;

            ***************************************;
            * others object to amount of drinking *;
            ***************************************;

               if al33a_1 not in (1,5) then alfnd4=.;
               else alfnd4=(4*(al33a_1=5))+1;

            ********************************;
            * feels guility about drinking *;
            ********************************;

               if al40a not in (1,5) then alfnd5=.;
               else alfnd5=(4*(al40a=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of alfnd1-alfnd5)=0 then alfndx=.;
               else do;
                  alfndx=0;
                  array sxd alfnd1-alfnd5;
                  do over sxd;
                     if sxd=5 then alfndx+1;
                  end;
               end;

            *********************;
            * at least one item *;
            *********************;

               if alfndx=. then alfnd=.;
               else alfnd=(4*(alfndx>0))+1;

         **************************************;
         * jump here if coga parent interview *;
         **************************************;

            par_hop2:

         ****************;
         * sum of items *;
         ****************;

            if n(alfna,alfnb,alfnc,alfnd)=0 then alfnx=.;
            else do;
               alfnx=0;
               array sxs alfna alfnb alfnc alfnd;
               do over sxs;
                  if sxs=5 then alfnx+1;
               end;
            end;

         *************;
         * Diagnosis *;
         *************;

            if alfnx=. then alfndp=.;
            else do;
               alfndp=1;
               if alfnx>0 then alfndp=3;
               if alfnx>2 then alfndp=5;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label alfnsk = "FGN AlC Skip Out"
               alfna1 = "FGN Alcoholism Crit. A1"
               alfna2 = "FGN Alcoholism Crit. A2"
               alfna3 = "FGN Alcoholism Crit. A3"
               alfna4 = "FGN Alcoholism Crit. A4" 
               alfnax = "FGN Alcoholism Sum of As"
               alfna  = "FGN Alcoholism 1+ As"
               alfnb1 = "FGN Alcoholism Crit. B1"
               alfnb2 = "FGN Alcoholism Crit. B2"
               alfnb3 = "FGN Alcoholism Crit. B3"
               alfnb4 = "FGN Alcoholism Crit. B4" 
               alfnbx = "FGN Alcoholism Sum of Bs"
               alfnb  = "FGN Alcoholism 1+ Bs"
               alfnc1 = "FGN Alcoholism Crit. C1"
               alfnc2 = "FGN Alcoholism Crit. C2"
               alfnc3 = "FGN Alcoholism Crit. C3"
               alfnc4 = "FGN Alcoholism Crit. C4" 
               alfncx = "FGN Alcoholism Sum of Cs"
               alfnc  = "FGN Alcoholism 1+ Cs"
               alfnd1 = "FGN Alcoholism Crit. D1"
               alfnd2 = "FGN Alcoholism Crit. D2"
               alfnd3 = "FGN Alcoholism Crit. D3"
               alfnd4 = "FGN Alcoholism Crit. D4" 
               alfnd5 = "FGN Alcoholism Crit. D5" 
               alfndx = "FGN Alcoholism Sum of Ds"
               alfnd  = "FGN Alcoholism 1+ Ds"
               alfnx  = "FGN Alcoholism Sum of A,B,C,D"
               alfndp = "FGN Alcoholism Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id alfnsk 
                    alfna1-alfna4 alfnax alfna alfnb1-alfnb4 alfnbx alfnb 
                    alfnc1-alfnc4 alfncx alfnc alfnd1-alfnd5 alfndx alfnd 
                    alfnx alfndp;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./fgn_alc_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(alcfn,al1a al1f al1g al11a alfnsk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(alcfn,al1c al1f al11a alfnsk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(alcfn,alfnsk al46a_1 al46b_1 al46c_1 al46d_1 
                                  al46e_1 al46f_1 al46g_1 al46h_1
                                  al46i_1 al46j_1 al46k_1 al46l_1
                                  al46m_1 alfna1);
                     %multi(alcfn,alfnsk al21d alfna3);
                     %multi(alcfn,alfnsk al20d alfna4);
                     %multi(alcfn,alfnsk alfna1-alfna4 alfnax);
                     %multi(alcfn,alfnsk alfnax alfna);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(alcfn,alfnsk al13c_1 al15a al16a alfnb1);
                     %multi(alcfn,alfnsk al14a alfnb2);
                     %multi(alcfn,alfnsk al22a alfnb3);
                     %multi(alcfn,alfnsk al24a alfnb4);
                     %multi(alcfn,alfnsk alfnb1-alfnb4 alfnbx);
                     %multi(alcfn,alfnsk alfnbx alfnb);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(alcfn,alfnsk al34a alfnc1);
                     %multi(alcfn,alfnsk al26b al26c alfnc2);
                     %multi(alcfn,alfnsk al31a al31b al31c al31d alfnc3);
                     %multi(alcfn,alfnsk al33c_1 alfnc4);
                     %multi(alcfn,alfnsk alfnc1-alfnc4 alfncx);
                     %multi(alcfn,alfnsk alfncx alfnc);

                  ***************;
                  * Criterion D *;
                  ***************;

                     %multi(alcfn,alfnsk al39a alfnd1);
                     %multi(alcfn,alfnsk al32b alfnd2);
                     %multi(alcfn,alfnsk al33d_1 alfnd3);
                     %multi(alcfn,alfnsk al33a_1 alfnd4);
                     %multi(alcfn,alfnsk al40a alfnd5);
                     %multi(alcfn,alfnsk alfnd1-alfnd5 alfndx);
                     %multi(alcfn,alfnsk alfndx alfnd);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(alcfn,alfnsk alfna alfnb alfnc alfnd alfnx);
                     %multi(alcfn,alfnsk alfnx alfndp);

            ******************;
            * keep variables *;
            ******************;

               data alcfn;
               set alcfn;
               keep ind_id alfnsk
                    alfna1-alfna4 alfnax alfna alfnb1-alfnb4 alfnbx alfnb 
                    alfnc1-alfnc4 alfncx alfnc alfnd1-alfnd5 alfndx alfnd 
                    alfnx alfndp;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=alcfn;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend alcfn;

********;
* test *;
********;
/*
   %alcfn(coga.csaga2,1);
*/
