*******************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Attention Deficit/     *;
*                                          Hyperactivity Disorder *;
*******************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %addic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro addic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ADDIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ADDIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data addic;
      set &dsn.;

      attrib aticg11-aticg19 aticg1x 
             aticg21-aticg25 aticg2x 
             aticg31-aticg34 aticg3x
             aticg1-aticg3 aticg35 
             aticg4 aticg6 aticdx aticsk length=3;

      ****************;
      * Criterion G1 *;
      ****************;

         *****************;
         * lifetime/ever *;
         *****************;

            *****************************************************;
            * often poor attention to details/careless mistakes *;
            *****************************************************;

               if sum(at1a not in (1,5),
                      at1b not in (1,5))=2 then aticg11=.;
               else aticg11=(at1a=5 or at1b=5);

            ***********************************;
            * difficulty sustaining attention *;
            ***********************************;

               if sum(at2a not in (1,5),
                      at2b not in (1,5))=2 then aticg12=.;
               else aticg12=(at2a=5 or at2b=5);

            *****************************;
            * often seems not to listen *;
            *****************************;

               if at3 not in (1,5) then aticg13=.;
               else aticg13=(at3=5);

            ************************************************;
            * difficulty following through on instructions *;
            ************************************************;

               if at4 not in (1,5) then aticg14=.;
               else aticg14=(at4=5);

            **********************************************;
            * difficulty organizing tasks and activities *;
            **********************************************;

               if at5 not in (1,5) then aticg15=.;
               else aticg15=(at5=5);

            ***********************************************************;
            * often avoids tasks that require sustained mental effort *;
            ***********************************************************;

               if at6 not in (1,5) then aticg16=.;
               else aticg16=(at6=5);

            *****************************************************;
            * often loses things necessary for tasks/activities *;
            *****************************************************;

               if at7 not in (1,5) then aticg17=.;
               else aticg17=(at7=5); 

            *******************************************;
            * easily distracted by extraneous stimuli *;
            *******************************************;

               if at8 not in (1,5) then aticg18=.;
               else aticg18=(at8=5);

            ***************************************;
            * often forgetful in daily activities *;
            ***************************************;

               if at9 not in (1,5) then aticg19=.;
               else aticg19=(at9=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of aticg11-aticg19)=0 then aticg1x=.;
               else aticg1x=sum(of aticg11-aticg19);

            **********************;
            * at least 6 present *;
            **********************;

               if aticg1x=. then aticg1=.;
               else aticg1=(aticg1x>5);

      ****************;
      * Criterion G2 *;
      ****************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************;
            * often fidgets/squirms *;
            *************************;

               if at15 not in (1,5) then aticg21=.;
               else aticg21=(at15=5);

            *******************************;
            * difficulty remaining seated *;
            *******************************;

               if at16 not in (1,5) then aticg22=.;
               else aticg22=(at16=5);

            *********************************************;
            * often runs/climbs about or feels restless *;
            *********************************************;

               if at17 not in (1,5) then aticg23=.;
               else aticg23=(at17=5);

            ******************************;
            * difficulty playing queitly *;
            ******************************;

               if at18 not in (1,5) then aticg24=.;
               else aticg24=(at18=5);

            ***********************************;
            * often appears driven by a motor *;
            ***********************************;

               if at19 not in (1,5) then aticg25=.;
               else aticg25=(at19=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of aticg21-aticg25)=0 then aticg2x=.;
               else aticg2x=sum(of aticg21-aticg25);

            **********************;
            * at least 3 present *;
            **********************;

               if aticg2x=. then aticg2=.;
               else aticg2=(aticg2x>2);

      ****************;
      * Criterion G3 *;
      ****************;

         *****************;
         * lifetime/ever *;
         *****************;

            ************************************************;
            * blurts out answers before question completed *;
            ************************************************;

               if at11 not in (1,5) then aticg31=.;
               else aticg31=(at11=5);

            ****************************;
            * difficulty awaiting turn *;
            ****************************;

               if at12 not in (1,5) then aticg32=.;
               else aticg32=(at12=5);

            *******************************;
            * often interupts or intrudes *;
            *******************************;

               if at13 not in (1,5) then aticg33=.;
               else aticg33=(at13=5);

            ***************************;
            * often talks excessively *;
            ***************************;

               if at20 not in (1,5) then aticg34=.;
               else aticg34=(at20=5);

            ****************;
            * sum of items *;
            ****************;

               if n(of aticg31-aticg34)=0 then aticg3x=.;
               else aticg3x=sum(of aticg31-aticg34);

            **********************;
            * at least 1 present *;
            **********************;

               if aticg3x=. then aticg3=.;
               else aticg3=(aticg3x>0);

      *********************;
      * test for skip out *;
      *********************;

         if sum(aticg1x<0,aticg2x<0,aticg3x<0,
                at10 not in (1,5), 
                at14 not in (1,5))=5 then aticsk=1;
         else do;
            if sum(aticg1x,aticg2x,aticg3x,at10=5,at14=5)>3 then aticsk=0;
            else aticsk=1;
         end;

         if aticsk=1 then do;
            array skp aticg35 aticg4 aticg6 aticdx;
            do over skp;
               skp=.S;
            end;
         end;

         if aticsk=1 then go to skipout;

      *******************;
      * Criterion G3(5) *;
      *******************;

         *****************;
         * lifetime/ever *;
         *****************;

            if (at21c not in (1,5)) then aticg35=.;
            else aticg35=(at21c=5);

      ****************;
      * Criterion G4 *;
      ****************;

         *****************;
         * lifetime/ever *;
         *****************;

            if at21a_1<0 then aticg4=.;
            else aticg4=(0<at21a_1<8);

      ********************************;
      * Criterion G5 -- Not assessed *;
      ********************************;

      ****************;
      * Criterion G6 *;
      ****************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(at22a_1a not in (1,5),
                   at22a_2a not in (1,5),
                   at22a_3a not in (1,5),
                   at22a_4a not in (1,5),
                   at22a_5a not in (1,5))=5 then aticg6=.;
            else aticg6=sum(at22a_1a=5,at22a_2a=5,at22a_3a=5,
                            at22a_4a=5,at22a_5a=5)>0;

      ***************************************;
      * Criterion G7 -- Skip in this module *;
      ***************************************;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(aticg1,aticg2,aticg3,aticg35,aticg4,aticg6)=0 then 
               aticdx=.;
            else aticdx=sum(aticg1,aticg2,aticg3,
                            aticg35,aticg4,aticg6)=6;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         array one aticsk
                   aticg11-aticg19 
                   aticg21-aticg25 
                   aticg31-aticg34 
                   aticg1-aticg3 aticg35 
                   aticg4 aticg6 aticdx;
         do over one;
            if one=1 then one=5;
            if one=0 then one=1;
         end;

      *******************;
      * variable labels *;
      *******************;

         label aticsk  = "ICD10 ADHD Skip Out"
               aticg11 = "ICD10 ADHD (Lifetime) Crit. G1(1)"
               aticg12 = "ICD10 ADHD (Lifetime) Crit. G1(2)"
               aticg13 = "ICD10 ADHD (Lifetime) Crit. G1(3)"
               aticg14 = "ICD10 ADHD (Lifetime) Crit. G1(4)"
               aticg15 = "ICD10 ADHD (Lifetime) Crit. G1(5)"
               aticg16 = "ICD10 ADHD (Lifetime) Crit. G1(6)"
               aticg17 = "ICD10 ADHD (Lifetime) Crit. G1(7)"
               aticg18 = "ICD10 ADHD (Lifetime) Crit. G1(8)"
               aticg19 = "ICD10 ADHD (Lifetime) Crit. G1(9)"
               aticg21 = "ICD10 ADHD (Lifetime) Crit. G2(1)"
               aticg22 = "ICD10 ADHD (Lifetime) Crit. G2(2)"
               aticg23 = "ICD10 ADHD (Lifetime) Crit. G2(3)"
               aticg24 = "ICD10 ADHD (Lifetime) Crit. G2(4)"
               aticg25 = "ICD10 ADHD (Lifetime) Crit. G2(5)"
               aticg31 = "ICD10 ADHD (Lifetime) Crit. G3(1)"
               aticg32 = "ICD10 ADHD (Lifetime) Crit. G3(2)"
               aticg33 = "ICD10 ADHD (Lifetime) Crit. G3(3)"
               aticg34 = "ICD10 ADHD (Lifetime) Crit. G3(4)"
               aticg1x = "ICD10 ADHD (Lifetime) Sum of G1s"
               aticg2x = "ICD10 ADHD (Lifetime) Sum of G2s"
               aticg3x = "ICD10 ADHD (Lifetime) Sum of G3s"
               aticg1  = "ICD10 ADHD (Lifetime) Crit. G1"
               aticg2  = "ICD10 ADHD (Lifetime) Crit. G2"
               aticg3  = "ICD10 ADHD (Lifetime) Crit. G3"
               aticg35 = "ICD10 ADHD (Lifetime) Crit. G3(5)"
               aticg4  = "ICD10 ADHD (Lifetime) Crit. G4"
               aticg6  = "ICD10 ADHD (Lifetime) Crit. G6"
               aticdx  = "ICD10 ADHD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id aticsk 
                    aticg11-aticg19 aticg1x 
                    aticg21-aticg25 aticg2x 
                    aticg31-aticg34 aticg3x
                    aticg1-aticg3 aticg35 
                    aticg4 aticg6 aticdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_add_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ****************;
               * Criterion G1 *;
               ****************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addic,at1a at1b aticg11);
                     %multi(addic,at2a at2b aticg12);
                     %multi(addic,at3 aticg13);
                     %multi(addic,at4 aticg14);
                     %multi(addic,at5 aticg15);
                     %multi(addic,at6 aticg16);
                     %multi(addic,at7 aticg17);
                     %multi(addic,at8 aticg18);
                     %multi(addic,at9 aticg19);
                     %multi(addic,aticg11-aticg19 aticg1x);
                     %multi(addic,aticg1x aticg1);

               ****************;
               * Criterion G2 *;
               ****************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addic,at15 aticg21);
                     %multi(addic,at16 aticg22);
                     %multi(addic,at17 aticg23);
                     %multi(addic,at18 aticg24);
                     %multi(addic,at19 aticg25);
                     %multi(addic,aticg21-aticg25 aticg2x);
                     %multi(addic,aticg2x aticg2);

               ****************;
               * Criterion G3 *;
               ****************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(addic,at11 aticg31);
                     %multi(addic,at12 aticg32);
                     %multi(addic,at13 aticg33);
                     %multi(addic,at20 aticg34);
                     %multi(addic,aticg31-aticg34 aticg3x);
                     %multi(addic,aticg3x aticg3);

               ************;
               * skip out *;
               ************;

                  %multi(addic,aticg1x aticg2x aticg3x at10 at14 aticsk);

               *******************;
               * Criterion G3(5) *;
               *******************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addic,aticsk at21c aticg35);

               ****************;
               * Criterion G4 *;
               ****************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addic,aticsk at21a_1 aticg4);

               ****************;
               * Criterion G6 *;
               ****************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addic,aticsk at22a_1a at22a_2a at22a_3a 
                                  at22a_4a at22a_5a aticg6);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(addic,aticsk aticg1 aticg2 aticg3 aticg35 aticg4
                                  aticg6 aticdx);

            ******************;
            * keep variables *;
            ******************;

               data addic;
               set addic;
               keep ind_id aticsk 
                    aticg11-aticg19 aticg1x 
                    aticg21-aticg25 aticg2x 
                    aticg31-aticg34 aticg3x
                    aticg1-aticg3 aticg35 
                    aticg4 aticg6 aticdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=addic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend addic;

********;
* test *;
********;
/*
   %addic(coga.csaga2,1);
*/
