***************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Alcohol Dependence *;
***************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %alcic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro alcic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ALCIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ALCIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data alcic;
      set &dsn.;

      attrib alica1-alica6 alicax alica alicb alicdp alicsk length=3;

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(al1a not in (1,5),
                   al1f not in (1,5),
                   al1g not in (1,5),
                   al11a<0)=4 then alicsk=5;
            else do;
               if (al1a=5 and (al1f=5 or al1g=5) and al11a>2) then alicsk=1;
               else alicsk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(al1c not in (1,5),
                   al1f not in (1,5),
                   al11a<0)=3 then alicsk=5;
            else do;
               if ((al1c=5 or al1f=5) and al11a>2) then alicsk=1;
               else alicsk=5;
            end;
         %end;

         if alicsk=5 then do;
             array skp alica1-alica6 alicax alica alicb alicdp;
             do over skp;
                skp=.S;
             end;
         end;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &who.=p %then %do;
            if alicsk=1 then do;
               array nas alica1-alica4 alicax alica alicb alicdp;
               do over nas;
                  nas=.N;
               end;
            end;
            if alicsk=5 then go to skipout;
            else go to par_hop1;
         %end;

         if alicsk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            ********************************;
            * strong desire to use alcohol *;
            ********************************;

               if 0<dm3a<13 then do;
                  alica1=.N;
               end;
               else do;
                  if al23a not in (1,5) then alica1=.;
                  else alica1=(4*(al23a=5))+1;
               end;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            * or used larger amount or for longer period than intended    *;
            ***************************************************************;

               if sum(al13a not in (1,5),
                      al13c_2 not in (1,5),
                      al15c not in (1,5),
                      al16c not in (1,5))=4 then alica2=.;
               else alica2=(4*(al13a=5 or al13c_2=5 or 
                               al15c=5 or al16c=5))+1;

            **************;
            * withdrawal *;
            **************;

               if 0<dm3a<13 then do;
                  alica3=.N;
               end;
               else do;
                  if sum(sum(al46a_4 not in (1,5),al46b_4 not in (1,5),
                             al46d_4 not in (1,5),al46e_4 not in (1,5),
                             al46f_4 not in (1,5),al46g_4 not in (1,5),
                             al46h_4 not in (1,5),al46i_4 not in (1,5),
                             al46k_4 not in (1,5),al46l_4 not in (1,5))=10,
                         al46t not in (1,5))=2 then alica3=.;
                  else alica3=(4*(sum(al46a_4=5,al46b_4=5,al46d_4=5,al46e_4=5,
                                      al46f_4=5,al46g_4=5,al46h_4=5,al46i_4=5,
                                      al46k_4=5,al46l_4=5)>2 and al46t=5))+1;
               end;

            *************;
            * tolerance *;
            *************;

               if sum(al12e not in (1,5),
                      al12f not in (1,5))=2 then alica4=.;
               else alica4=(4*(al12e=5 or al12f=5))+1;

            **************************************;
            * jump here if coga parent interview *;
            **************************************;

               par_hop1:

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            * or use reduced social/occupational/recreational activities   *;
            ****************************************************************;

               if sum(al18b not in (1,5),
                      al19d not in (1,5),
                      al20d not in (1,5))=3 then alica5=.;
               else alica5=(4*(al18b=5 or al19d=5 or al20d=5))+1;

            ******************************************************;
            * use despite clear evidence of harmful consequences *;
            ******************************************************;

               if sum(al25e not in (1,5),
                      al35c not in (1,5),
                      al36a not in (1,5),
                      al37e not in (1,5))=4 then alica6=.;
               else alica6=(4*(al25e=5 or al35c=5 or al36a=5 or al37e=5))+1;

            **********************************;
            * test for coga parent interview *;
            **********************************;

               %if &study.=coga %then %do;
                  %if &who.=p %then %do;
                     go to skipout;
                  %end;
               %end;

            ****************;
            * sum of items *;
            ****************;

               if n(of alica1-alica6)=0 then alicax=.;
               else do;
                  alicax=0;
                  array sx1 alica1-alica6;
                  do over sx1;
                     if sx1=5 then alicax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if alicax=. then alica=.;
               else alica=(4*(alicax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if 0<dm3a<13 then do;
               alicb=.N;
            end;
            else do;
               if sum(al49b not in (1,5),
                      al49c not in (1,5))=2 then alicb=.;
               else alicb=(4*(al49b=5 or al49c=5))+1;
            end;
                   
         *************;
         * Diagnosis *;
         *************;

            if n(alica,alicb)=0 then alicdp=.;
            else do;
               if 0<dm3a<13 then alicdp=(4*(alica=5))+1;
               else alicdp=(4*(alica=5 and alicb=5))+1;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label alicsk = "ICD10 ALC Skip Out"
               alica1 = "ICD10 ALC Dependence Crit. A1"
               alica2 = "ICD10 ALC Dependence Crit. A2"
               alica3 = "ICD10 ALC Dependence Crit. A3"
               alica4 = "ICD10 ALC Dependence Crit. A4" 
               alica5 = "ICD10 ALC Dependence Crit. A5" 
               alica6 = "ICD10 ALC Dependence Crit. A6" 
               alicax = "ICD10 ALC Dependence Sum of As" 
               alica  = "ICD10 ALC Dependence 3+ As" 
               alicb  = "ICD10 ALC Dependence Crit. B" 
               alicdp = "ICD10 ALC Dependence Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id alicsk
                    alica1-alica6 alicax alica alicb alicdp;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_alc_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(alcic,al1a al1f al1g al11a alicsk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(alcic,al1c al1f al11a alicsk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(alcic,alicsk al23a alica1);
                     %multi(alcic,alicsk al13a al13c_2 al15c al16c alica2);
                     %multi(alcic,alicsk al46a_4 al46b_4 al46d_4 al46e_4 al46f_4
                                  al46g_4 al46h_4 al46i_4 al46k_4 al46l_4 
                                  al46t alica3);
                     %multi(alcic,alicsk al12e al12f alica4);
                     %multi(alcic,alicsk al18b al19d al20d alica5);
                     %multi(alcic,alicsk al25e al35c al36a al37e alica6);
                     %multi(alcic,alicsk alica1-alica6 alicax);
                     %multi(alcic,alicsk alicax alica);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(alcic,alicsk al49b al49c alicb);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(alcic,alicsk alica alicb alicdp);

            ******************;
            * keep variables *;
            ******************;

               data alcic;
               set alcic;
               keep ind_id alicsk
                    alica1-alica6 alicax alica alicb alicdp;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=alcic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend alcic;

********;
* test *;
********;
/*
   %alcic(coga.csaga2,1);
*/
