****************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Cannabis Dependence *;
****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %canic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro canic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in CANIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in CANIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data canic;
      set &dsn.;

      attrib mjicsk mjica1-mjica6 mjicax mjica mjicb mjicdp length=3;

      *********************;
      * test for skip out *;
      *********************;

         %if &study.=irpg or &who.=c %then %do;
            if sum(mj1a not in (1,5),
                   mj1c not in (1,5),
                   mj7a<0,
                   mj7b not in (0,1,2,3,4,5))=4 then mjicsk=5;
            else do;
               if (mj1a=5 and mj1c=5 and (mj7a>20 or mj7b in (1,2,3,4,5))) then
                  mjicsk=1;
               else mjicsk=5;
            end;
         %end;
         %if &study.=coga and &who.=p %then %do;
            if sum(mj1c not in (1,5),
                   mj1c_1 not in (1,5),
                   mj7a<0,
                   mj7b not in (0,1,2,3,4,5))=4 then mjicsk=5;
            else do;
               if ((mj1c=5 or mj1c_1=5) and 
                   (mj7a>20 or mj7b in (1,2,3,4,5))) then mjicsk=1;
               else mjicsk=5;
            end;
         %end;

         if mjicsk=5 then do;
             array skp mjica1-mjica6 mjicax mjica mjicb mjicdp;
             do over skp;
                skp=.S;
             end;
         end;

      **********************************;
      * test for coga parent interview *;
      **********************************;

         %if &study.=coga %then %do;
            %if &who.=p %then %do;
               if mjicsk=1 then do;
                  array nas mjica1-mjica4 mjicax mjica mjicb mjicdp;
                  do over nas;
                     nas=.N;
                  end;
               end;
               if mjicsk=5 then go to skipout;
               else go to par_hop1;
            %end;
         %end;

      if mjicsk=5 then go to skipout;

      **************;
      * Dependence *;
      **************;

         ***************;
         * Criterion A *;
         ***************;

            **********************************;
            * strong desire to use marijuana *;
            **********************************;

               if 0<dm3a<13 then do;
                  mjica1=.N;
               end;
               else do;
                  if mj13a not in (1,5) then mjica1=.;
                  else mjica1=(4*(mj13a=5))+1;
               end;

            ***************************************************************;
            * persistent desire or 1+ unsuccessful efforts to control use *;
            * or used larger amount or for longer period than intended    *;
            ***************************************************************;

               if sum(mj9b not in (1,5),
                      mj9d not in (1,5),
                      mj10b not in (1,5),
                      mj10d not in (1,5))=4 then mjica2=.;
               else mjica2=(4*(mj9b=5 or mj9d=5 or 
                               mj10b=5 or mj10d=5))+1;

            **************;
            * withdrawal *;
            **************;

               if 0<dm3a<13 then do;
                  mjica3=.N;
               end;
               else do;
                  if sum(mj29c not in (1,5),
                         mj29d not in (1,5))=2 then mjica3=.;
                  else mjica3=(4*(mj29c=5 and mj29d=5))+1;
               end;

            *************;
            * tolerance *;
            *************;

               if sum(mj8a not in (1,5),
                      mj8b not in (1,5))=2 then mjica4=.;
               else mjica4=(4*(mj8a=5 or mj8b=5))+1;

            **************************************;
            * jump here if coga parent interview *;
            **************************************;

               par_hop1:

            ****************************************************************;
            * great deal of time spent obtaining/using/recovering from use *;
            * or use reduced social/occupational/recreational activities   *;
            ****************************************************************;

               if sum(mj11 not in (1,5),
                      mj12c not in (1,5))=2 then mjica5=.;
               else mjica5=(4*(mj11=5 or mj12c=5))+1;

            ******************************************************;
            * use despite clear evidence of harmful consequences *;
            ******************************************************;

               if sum(mj23b not in (1,5),
                      mj24e not in (1,5))=2 then mjica6=.;
               else mjica6=(4*(mj23b=5 or mj24e=5))+1;

            **********************************;
            * test for coga parent interview *;
            **********************************;

               %if &study.=coga %then %do;
                  %if &who.=p %then %do;
                     go to skipout;
                  %end;
               %end;

            ****************;
            * sum of items *;
            ****************;

               if n(of mjica1-mjica6)=0 then mjicax=.;
               else do;
                  mjicax=0;
                  array sx1 mjica1-mjica6;
                  do over sx1;
                     if sx1=5 then mjicax+1;
                  end;
               end;

            ************************;
            * at least three items *;
            ************************;

               if mjicax=. then mjica=.;
               else mjica=(4*(mjicax>2))+1;

         ***************;
         * Criterion B *;
         ***************;

            if 0<dm3a<13 then do;
               mjicb=.N;
            end;
            else do;
               if sum(mj32b not in (1,5),
                      mj32c not in (1,5))=2 then mjicb=.;
               else mjicb=(4*(mj32b=5 or mj32c=5))+1;
            end;
                   
         *************;
         * Diagnosis *;
         *************;

            if n(mjica,mjicb)=0 then mjicdp=.;
            else do;
               if 0<dm3a<13 then mjicdp=(4*(mjica=5))+1;
               else mjicdp=(4*(mjica=5 and mjicb=5))+1;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label mjicsk = "ICD10 CAN Skip Out"
               mjica1 = "ICD10 CAN Dependence Crit. A1"
               mjica2 = "ICD10 CAN Dependence Crit. A2"
               mjica3 = "ICD10 CAN Dependence Crit. A3"
               mjica4 = "ICD10 CAN Dependence Crit. A4" 
               mjica5 = "ICD10 CAN Dependence Crit. A5" 
               mjica6 = "ICD10 CAN Dependence Crit. A6" 
               mjicax = "ICD10 CAN Dependence Sum of As" 
               mjica  = "ICD10 CAN Dependence 3+ As" 
               mjicb  = "ICD10 CAN Dependence Crit. B" 
               mjicdp = "ICD10 CAN Dependence Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id mjicsk
                    mjica1-mjica6 mjicax mjica mjicb mjicdp;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_can_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %if &study.=irpg or &who.=c %then %do;
                     %multi(canic,mj1a mj1c mj7a mj7b mjicsk);
                  %end;
                  %if &study.=coga and &who.=p %then %do;
                     %multi(canic,mj1c mj1c_1 mj7a mj7b mjicsk);
                  %end;

               **************;
               * Dependence *;
               **************;

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(canic,mjicsk mj13a mjica1);
                     %multi(canic,mjicsk mj9b mj9d mj10b mj10d mjica2);
                     %multi(canic,mjicsk mj29c mj29d mjica3);
                     %multi(canic,mjicsk mj8a mj8b mjica4);
                     %multi(canic,mjicsk mj11 mj12c mjica5);
                     %multi(canic,mjicsk mj23b mj24e mjica6);
                     %multi(canic,mjicsk mjica1-mjica6 mjicax);
                     %multi(canic,mjicsk mjicax mjica);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(canic,mjicsk mj32b mj32c mjicb);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(canic,mjicsk mjica mjicb mjicdp);

            ******************;
            * keep variables *;
            ******************;

               data canic;
               set canic;
               keep ind_id mjicsk
                    mjica1-mjica6 mjicax mjica mjicb mjicdp;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=canic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend canic;

********;
* test *;
********;
/*
   %canic(coga.csaga2,1);
*/
