*************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Conduct Disorder *;
*************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %cdic(dsn,chk)                                            *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro cdic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in CDIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in CDIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      data cdic;
      set &dsn.;

      attrib cdicg1-cdicg23 cdicg3x cdicg5x cdicdx length=3;

      ***************;
      * Criterion G *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **********************;
            * often loses temper *;
            **********************;

               if sum(op1 not in (1,5),
                      cd11a not in (1,3,5))=2 then cdicg1=.;
               else do;
                  cdicg1=1;
                  if cd11a=3 then cdicg1=3;
                  else if (op1=5 or cd11a=5) then cdicg1=5;
               end;

            ****************************;
            * often argues with adults *;
            ****************************;

               if (op2a not in (1,5)) then cdicg2=.;
               else cdicg2=(4*(op2a=5))+1;

            *******************************;
            * often defies requests/rules *;
            *******************************;

               if (op3a not in (1,5)) then cdicg3=.;
               else cdicg3=(4*(op3a=5))+1;

            *******************************;
            * often deliberately annoying *;
            *******************************;

               if (op4 not in (1,5)) then cdicg4=.;
               else cdicg4=(4*(op4=5))+1;

            ***********************;
            * often blames others *;
            ***********************;

               if (op5 not in (1,2,3,4)) then cdicg5=.;
               else cdicg5=(4*(op5=2))+1;

            **********************************;
            * often touchy or easily annoyed *;
            **********************************;

               if (op6 not in (1,5)) then cdicg6=.;
               else cdicg6=(4*(op6=5))+1;

            ****************************;
            * often angry or resentful *;
            ****************************;

               if (op7 not in (1,5)) then cdicg7=.;
               else cdicg7=(4*(op7=5))+1;

            *****************************;
            * often spiteful/vindictive *;
            *****************************;

               if (op8a not in (1,5)) then cdicg8=.;
               else cdicg8=(4*(op8a=5))+1;

            *****************************************************;
            * often lies/breaks promises to obtain goods/favors *;
            *****************************************************;

               if sum(sum(cd8a not in (1,3,5),cd8b not in (1,3,5))=2,
                      sum(cd8c_1 not in (1,5),cd8c_2 not in (1,5),
                          cd8c_3 not in (1,5))=3,
                      cd9a not in (1,3,5),
                      sum(cd9b<0,cd9b1 not in (1,2,3,4,5))=2,
                      cd10a not in (1,3,5))=5 then cdicg9=.;
               else do;
                  cdicg9=1;
                  if (((cd8a=3 or cd8b=5) and max(of cd8c_1-cd8c_3)=5) or
                      (cd9a=3 and (cd9b>2 or cd9b1 in (3,4,5))) or
                      (cd10a=3)) then cdicg9=3;
                  else if (((cd8a=5 or cd8b=5) and max(of cd8c_1-cd8c_3)=5) or
                           (cd9a=5 and (cd9b>2 or cd9b1 in (3,4,5))) or
                           (cd10a=5)) then cdicg9=5;
               end;

            ***********************************;
            * often initiates physical fights *;
            ***********************************;

               if sum(cd21f not in (1,3,5),
                      cd21g<0,
                      cd21g1 not in (1,2,3,4,5))=3 then cdicg10=.;
               else do;
                  cdicg10=1;
                  if (cd21f=3 and (cd21g>2 or cd21g1 in (3,4,5))) then 
                     cdicg10=3;
                  else if (cd21f=5 and (cd21g>2 or cd21g1 in (3,4,5))) then
                     cdicg10=5;
               end;

            *********************;
            * has used a weapon *;
            *********************;

               if cd22a not in (1,3,5) then cdicg11=.;
               else cdicg11=(4*(cd22a=5))+(2*(cd22a=3))+1;

            ******************************;
            * often stays out after dark *;
            ******************************;

               ************************************;
               * note:  review of data indicate   *;
               *        that not all interviews   *; 
               *        completed with 24hr clock *;
               *        so this coding may not be *;
               *        entirely appropriate      *;
               ************************************;

                  if (cd6b_1 ge 0 and cd6c_1 ge 0) then do;
                     time1=(cd6c_1+(cd6c_2/60));
                     time2=cd6b_1;
                     if time2=24 then time2=0;
                     time2=(time2+(cd6b_2/60));
                     if time2<time1 then time2=time2+24;
                     timed=(time2-time1);
                  end;

               if sum(sum(cd5a not in (1,3,5),
                          sum(cd5b<0,cd5b1 not in (1,2,3,4,5))=2,
                          cd5c_1<0)=3,
                      sum(sum(cd6a not in (1,3,5),timed)=2,
                          sum(cd6d<0,cd6d1 not in (1,2,3,4,5))=2,
                          cd6e_1<0)=3)=2 then cdicg12=.;
               else do;                  
                  cdicg12=1;
                  if ((cd5a=3 and (cd5b>2 or cd5b1 in (3,4,5)) and
                       (0<cd5c_1<13)) or
                      (cd6a=3 and timed ge 2 and 
                       (cd6d>2 or cd6d1 in (3,4,5)) and 
                       (0<cd6e_1<13))) then cdicg12=3;
                  else if ((cd5a=5 and (cd5b>2 or cd5b1 in (3,4,5)) and
                            (0<cd5c_1<13)) or
                           (cd6a=5 and timed ge 2 and 
                            (cd6d>2 or cd6d1 in (3,4,5)) and 
                            (0<cd6e_1<13))) then cdicg12=5;
               end;

            ***************************************;
            * exhibits physical cruelty to others *;
            ***************************************;

               if sum(cd23a not in (1,5),
                      cd23a1 not in (1,3,5))=2 then cdicg13=.;
               else cdicg13=(4*(cd23a=5 and cd23a1=1))+1;

            ****************************************;
            * exhibits physical cruelty to animals *;
            ****************************************;

               if cd19a not in (1,3,5) then cdicg14=.;
               else cdicg14=(4*(cd19a=5))+(2*(cd19a=3))+1;

            *****************************;
            * non-firesetting vandalism *;
            *****************************;

               if cd18a not in (1,3,5) then cdicg15=.;
               else cdicg15=(4*(cd18a=5))+(2*(cd18a=3))+1;

            *********************;
            * firesetting/arson *;
            *********************;

               if sum(cd17a not in (1,3,5),
                      cd17b not in (1,5))=2 then cdicg16=.;
               else cdicg16=(4*(cd17a=5 and cd17b=5))+
                            (2*(cd17a=3 and cd17b=5))+1;

            ****************************************;
            * nonconfrontational, nontrivial theft *;
            ****************************************;

               if sum(sum(cd14a not in (1,3,5),cd14b not in (1,3,5))=2,
                      sum(cd14c<0,cd14c1 not in (1,2,3,4,5))=2,
                      cd14d not in (1,3,5),
                      sum(cd14e<0,cd14e1 not in (1,2,3,4,5))=2)=4 then 
                  cdicg17=.;
               else do;
                  cdicg17=1;
                  if (((cd14a=3 or cd14b=5) and 
                       (cd14c>1 or cd14c1 in (2,3,4,5))) or
                      (cd14d=3 and (cd14e>1 or cd14e1 in (2,3,4,5))) or
                      sum(cd14a=3,cd14b=3,cd14d=3)>1) then cdicg17=3;
                  else if (((cd14a=5 or cd14b=5) and 
                            (cd14c>1 or cd14c1 in (2,3,4,5))) or
                           (cd14d=5 and (cd14e>1 or cd14e1 in (2,3,4,5))) or
                           sum(cd14a=5,cd14b=5,cd14d=5)>1) then cdicg17=5;
               end;

            ********************;
            * frequent truancy *;
            ********************;

               if sum(cd3a not in (1,3,5),
                      sum(cd3b<0,cd3b1 not in (1,2,3,4,5))=2,
                      cd3c_1<0,
                      cd4a not in (1,3,5),
                      sum(cd4b<0,cd4b1 not in (1,2,3,4,5))=2,
                      cd4c_1<0)=6 then cdicg18=.;
               else do;
                  cdicg18=1;
                  if ((cd3a=3 and (cd3b>2 or cd3b1 in (3,4,5)) and 
                       (0<cd3c_1<13)) or
                      (cd4a=3 and (cd4b>2 or cd4b1 in (3,4,5)) and
                       (0<cd4c_1<13)) or
                      ((cd3a=3 and cd4a=3) and 
                       (sum(cd3b,cd4b)>2 or sum(cd3b1,cd4b1)>2) and
                       ((0<cd3c_1<13) or (0<cd4c_1<13)))) then cdicg18=3;
                  else if ((cd3a=5 and (cd3b>2 or cd3b1 in (3,4,5)) and
                            (0<cd3c_1<13)) or
                           (cd4a=5 and (cd4b>2 or cd4b1 in (3,4,5)) and
                            (0<cd4c_1<13)) or
                           ((cd3a=5 and cd4a=5) and 
                            (sum(cd3b,cd4b)>2 or sum(cd3b1,cd4b1)>2) and
                            ((0<cd3c_1<13) or (0<cd4c_1<13)))) then cdicg18=5;
               end;

            **********************;
            * run away from home *;
            **********************;

               if sum(cd7a not in (1,3,5),
                      cd7b not in (1,2,3),
                      sum(cd7c<0,cd7c1 not in (1,2,3,4,5))=2,
                      cd7d<0)=4 then cdicg19=.;
               else do;
                  cdicg19=1;
                  if ((cd7a=3 and cd7b=3) and 
                      ((cd7c>1 or cd17c1 in (2,3,4,5)) or cd7d ge 2)) then
                     cdicg19=3;
                  else if ((cd7a=5 and cd7b=3) and 
                           ((cd7c>1 or cd17c1 in (2,3,4,5)) or cd7d ge 2)) then
                     cdicg19=5;
               end;

            ***********************************;
            * committed confrontational crime *;
            ***********************************;

               if cd15a not in (1,3,5) then cdica20=.;
               else cdicg20=(4*(cd15a=5))+(2*(cd15a=3))+1;

            ****************************;
            * committed sexual assault *;
            ****************************;

               if cd24 not in (1,3,5) then cdica21=.;
               else cdicg21=(4*(cd24=5))+(2*(cd24=3))+1;

            *******************************;
            * frequent bullying of others *;
            *******************************;

               if sum(cd20a not in (1,3,5),
                      cd20b not in (1,3,5))=2 then cdicg22=.;
               else do;
                  cdicg22=1;
                  if (cd20a=3 or cd20b=3) then cdicg22=3;
                  else if (cd20a=5 or cd20b=5) then cdicg22=5;
               end;

            ***********************************;
            * committed breaking and entering *;
            ***********************************;

               if cd16a not in (1,3,5) then cdicg23=.;
               else cdicg23=(4*(cd16a=5))+(2*(cd16a=3))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of cdicg1-cdicg23)=0 then do;
                  cdicg3x=.;
                  cdicg5x=.;
               end;
               else do;
                  cdicg3x=0;
                  cdicg5x=0;
                  array one cdicg1-cdicg23;
                  do over one;
                     if one=3 then cdicg3x+1;
                     if one=5 then cdicg5x+1;
                  end;
               end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if (cd26 not in (1,5) and cdicg3x=. and cdicg5x=.) then cdicdx=.;
            else do;
               if cd26=1 then cdicdx=1;
               else if (cd26=5 and sum(cdicg3x,cdicg5x)<3) then cdicdx=1;
               else if (cd26=5 and cdicg3x>0 and sum(cdicg3x,cdicg5x)>2) then 
                  cdicdx=3;
               else if (cd26=5 and cdicg3x<1 and cdicg5x>2) then cdicdx=5;
            end;

      *******************;
      * variable labels *;
      *******************;

         label cdicg1  = "ICD10 CD (Lifetime) Crit. A1"
               cdicg2  = "ICD10 CD (Lifetime) Crit. A2"
               cdicg3  = "ICD10 CD (Lifetime) Crit. A3"
               cdicg4  = "ICD10 CD (Lifetime) Crit. A4"
               cdicg5  = "ICD10 CD (Lifetime) Crit. A5"
               cdicg6  = "ICD10 CD (Lifetime) Crit. A6"
               cdicg7  = "ICD10 CD (Lifetime) Crit. A7"
               cdicg8  = "ICD10 CD (Lifetime) Crit. A8"
               cdicg9  = "ICD10 CD (Lifetime) Crit. A9"
               cdicg10 = "ICD10 CD (Lifetime) Crit. A10"
               cdicg11 = "ICD10 CD (Lifetime) Crit. A11"
               cdicg12 = "ICD10 CD (Lifetime) Crit. A12"
               cdicg13 = "ICD10 CD (Lifetime) Crit. A13"
               cdicg14 = "ICD10 CD (Lifetime) Crit. A14"
               cdicg15 = "ICD10 CD (Lifetime) Crit. A15"
               cdicg16 = "ICD10 CD (Lifetime) Crit. A16"
               cdicg17 = "ICD10 CD (Lifetime) Crit. A17"
               cdicg18 = "ICD10 CD (Lifetime) Crit. A18"
               cdicg19 = "ICD10 CD (Lifetime) Crit. A19"
               cdicg20 = "ICD10 CD (Lifetime) Crit. A20"
               cdicg21 = "ICD10 CD (Lifetime) Crit. A21"
               cdicg22 = "ICD10 CD (Lifetime) Crit. A22"
               cdicg23 = "ICD10 CD (Lifetime) Crit. A23"
               cdicg3x = "ICD10 CD (Lifetime) Sum of As=3"
               cdicg5x = "ICD10 CD (Lifetime) Sum of As=5"
               cdicdx = "ICD10 CD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id 
                    cdicg1-cdicg23 cdicg3x cdicg5x cdicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_cd_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cdic,op1 cd11a cdicg1);
                     %multi(cdic,op2a cdicg2);
                     %multi(cdic,op3a cdicg3);
                     %multi(cdic,op4 cdicg4);
                     %multi(cdic,op5 cdicg5);
                     %multi(cdic,op6 cdicg6);
                     %multi(cdic,op7 cdicg7);
                     %multi(cdic,op8a cdicg8);
                     %multi(cdic,cd8a cd8b cd8c_1-cd8c_3
                                 cd9a cd9b cd9b1 cd10a cdicg9);
                     %multi(cdic,cd21f cd21g cd21g1 cdicg10);
                     %multi(cdic,cd22a cdicg11);

                     %multi(cdic,time1 time2 timed);
                     %multi(cdic,cd5a cd5b cd5b1 cd5c_1 
                                 cd6a timed cd6d cd6d1 cd6e_1 cdicg12);

                     %multi(cdic,cd23a cd23a1 cdicg13);
                     %multi(cdic,cd19a cdicg14);
                     %multi(cdic,cd18a cdicg15);
                     %multi(cdic,cd17a cd17b cdicg16);
                     %multi(cdic,cd14a cd14b cd14c cd14c1 cd14d cd14e cd14e1
                                 cdicg17);
                     %multi(cdic,cd3a cd3b cd3b1 cd3c_1 cd4a cd4b cd4b1 cd4c_1
                                 cdicg18);
                     %multi(cdic,cd7a cd7b cd7c cd7c1 cd7d cdicg19);
                     %multi(cdic,cd15a cdicg20);
                     %multi(cdic,cd24 cdicg21);
                     %multi(cdic,cd20a cd20b cdicg22);
                     %multi(cdic,cd16a cdicg23);
                     %multi(cdic,cdicg1-cdicg23 cdicg3x cdicg5x);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(cdic,cd26 cdicg3x cdicg5x cdicdx);

            ******************;
            * keep variables *;
            ******************;

               data cdic;
               set cdic;
               keep ind_id  
                    cdicg1-cdicg23 cdicg3x cdicg5x cdicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=cdic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend cdic;

********;
* test *;
********;
/*
   %cdic(coga.csaga2,1);
*/
