****************************************************************;
* CSSAGA-II Diagnostic Algorithm -- ICD-10 -- Major Depression *;
****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %depic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro depic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DEPIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DEPIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      data depic;
      set &dsn.;

      attrib dcicsk dpicsk mdicsk
             dcicg1 dcicg2 dcicg3
             dcica 
             dcicb1-dcicb3 dcicb
             dcicc1-dcicc7 dcicc
             dcicsx
             dcicd1-dcicd3 dcicdx
             dpicg1 dpicg2 dpicg3
             dpica 
             dpicb1-dpicb3 dpicb
             dpicc1-dpicc7 dpicc
             dpicsx
             dpicd1-dpicd3 dpicdx
             mdicg1 mdicg2 mdicg3
             mdica 
             mdicb1-mdicb3 mdicb
             mdicc1-mdicc7 mdicc
             mdicd1-mdicd3 mdicdx length=3;

      *********************;
      * test for skip out *;
      *********************;

         *******************;
         * current episode *;
         *******************;

            if sum(dp1c_1 not in (1,5),
                   dp1c_2 not in (1,5),
                   dp1c_3 not in (1,5),
                   dp1c_4 not in (1,5),
                   dp1c_5 not in (1,5))=5 then dcicsk=5;
            else do;
               if sum(dp1c_1=5,dp1c_2=5,dp1c_3=5,dp1c_4=5,dp1c_5=5)>0 then 
                  dcicsk=1;
               else dcicsk=5;
            end;

            if dcicsk=5 then do;
               array skp1 dcicg1 dcicg2 dcicg3 dcica 
                          dcicb1-dcicb3 dcicb dcicc1-dcicc7 dcicc
                          dcicsx dcicd1-dcicd3 dcicdx;
               do over skp1;
                 skp1=.S;
               end;
            end;

         ****************************;
         * most severe past episode *;
         ****************************;

            if sum(dp7c_1 not in (1,5),
                   dp7c_2 not in (1,5),
                   dp7c_3 not in (1,5),
                   dp7c_4 not in (1,5),
                   dp7c_5 not in (1,5))=5 then dpicsk=5;
            else do;
               if sum(dp7c_1=5,dp7c_2=5,dp7c_3=5,dp7c_4=5,dp7c_5=5)>0 then 
                  dpicsk=1;
               else dpicsk=5;
            end;

            if dpicsk=5 then do;
               array skp2 dpicg1 dpicg2 dpicg3 dpica 
                          dpicb1-dpicb3 dpicb dpicc1-dpicc7 dpicc
                          dpicsx dpicd1-dpicd3 dpicdx; 
               do over skp2;
                 skp2=.S;
               end;
            end;

         *****************;
         * lifetime ever *;
         *****************;

            if (dcicsk=5 and dpicsk=5) then mdicsk=5;
            else mdicsk=1;

            if mdicsk=5 then do;
               array skp3 mdicg1 mdicg2 mdicg3 mdica 
                          mdicb1-mdicb3 mdicb mdicc1-mdicc7 mdicc
                          mdicsx mdicd1-mdicd3 mdicdx; 
               do over skp3;
                 skp3=.S;
               end;
            end;

      **********************;
      * jumps for skip out *;
      **********************;

         if mdicsk=5 then go to skipout;
         if dcicsk=1 then go to current;
         if dcicsk=5 and dpicsk=1 then go to past;

      *******************;
      * current episode *;
      *******************;

         current:

         ****************;
         * Criterion G1 *;
         ****************;

            if (dp30b_1 not in (1,5)) then dcicg1=.;
            else dcicg1=(dp30b_1=5);

         ****************;
         * Criterion G2 *;
         ****************;

            %if &study.=irpg %then %do;
               dcicg2=.;
            %end;
            %else %if &study.=coga %then %do;
               if mn1=. then dcicg2=.;
               else dcicg2=1;
            %end;

         ****************;
         * Criterion G3 *;
         ****************;

            if sum((dp4d not in (1,5)),(dp4g not in (1,5)),
                   (sum((dp5a_1 not in (1,5)),(dp5a_2 not in (1,5)),
                        (dp5a_3 not in (1,5)),(dp5a_4 not in (1,5)),
                        (dp5a_5 not in (1,5)),(dp5a_6 not in (1,5)))=6 or
                    sum((dp5b_1 not in (1,5)),(dp5b_2 not in (1,5)),
                        (dp5b_3 not in (1,5)),(dp5b_4 not in (1,5)), 
                        (dp5b_5 not in (1,5)),(dp5b_6 not in (1,5)))=6),
                    (dp6a not in (1,5)),(dp31b_1 not in (1,5)),
                    (dp31c_1 not in (1,5)))=6 then dcicg3=.;
            else dcicg3=((dp4d=5 or dp4g=5) or
                         (sum((dp5a_1=5),(dp5a_2=5),(dp5a_3=5),
                              (dp5a_4=5),(dp5a_5=5),(dp5a_6=5))>0 or
                          sum((dp5b_1=5),(dp5b_2=5),(dp5b_3=5),
                              (dp5b_4=5),(dp5b_5=5),(dp5b_6=5))>0) or
                         (dp6a=5) or (dp31b_1=5) or (dp31c_1=5));
            if dcicg3 ne . then dcicg3=(-1*(dcicg3-1));

         ***************;
         * Criterion A *;
         ***************;

            if n(of dcicg1-dcicg3)=0 then dcica=.;
            else do;
               dcica=(((n(of dcicg1-dcicg3)=1)*(sum(of dcicg1-dcicg3)=1))+
                      ((n(of dcicg1-dcicg3)=2)*(sum(of dcicg1-dcicg3)=2))+
                      ((n(of dcicg1-dcicg3)=3)*(sum(of dcicg1-dcicg3)>1)));
            end;

         ***************;
         * Criterion B *;
         ***************;

            ******************;
            * depressed mood *;
            ******************;

               if sum((dp1c_1 not in (1,5)),
                      (dp1c_2 not in (1,5)))=2 then dcicb1=.;
               else dcicb1=sum((dp1c_1=5),(dp1c_2=5))>0;

            *************;
            * anhedonia *;
            *************;

               if sum((dp1c_3 not in (1,5)),
                      (dp1c_4 not in (1,5)))=2 then dcicb2=.;
               else dcicb2=sum((dp1c_3=5),(dp1c_4=5))>0;

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if (dp1c_6 not in (1,5)) then dcicb3=.;
               else dcicb3=(dp1c_6=5);

            ************************;
            * at least two present *;
            ************************;

               if n(of dcicb1-dcicb3)=0 then dcicb=.;
               else do;
                  dcicb=(((n(of dcicb1-dcicb3)=1)*
                          (sum(of dcicb1-dcicb3)=1))+
                         ((n(of dcicb1-dcicb3)>1)*
                          (sum(of dcicb1-dcicb3)>1)));
               end;    

         ***************;
         * Criterion C *;
         ***************;

            **********************************;
            * loss of confidence/self esteem *;
            **********************************;

               if sum((dp24_1 not in (1,5)),
                      (dp25a_1 not in (1,5)))=2 then dcicc1=.;
               else dcicc1=sum((dp24_1=5),(dp25a_1=5))>0;

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if (dp25_1 not in (1,5)) then dcicc2=.;
               else dcicc2=(dp25_1=5);

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if sum((dp28a_1 not in (1,5)),
                      (dp28b_1 not in (1,5)),
                      (dp28c_1 not in (1,5)),
                      (dp28d_1 not in (1,5)))=4 then dcicc3=.;
               else dcicc3=sum((dp28a_1=5),(dp28b_1=5),
                               (dp28c_1=5),(dp28d_1=5))>0;

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dp26a_1 not in (1,5)),
                      (dp26b_1 not in (1,5)),
                      (dp27a_1 not in (1,5)),
                      (dp27b_1 not in (1,5)))=4 then dcicc4=.;
               else dcicc4=sum((dp26a_1=5),(dp26b_1=5),
                               (dp27a_1=5),(dp27b_1=5))>0;

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;

               if sum((dp22_1 not in (1,5)),
                      (dp23_1 not in (1,5)))=2 then dcicc5=.;
               else dcicc5=sum((dp22_1=5),(dp23_1=5))>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dp21a_1 not in (1,5)),
                      (dp21b_1 not in (1,5)),
                      (dp21c_1 not in (1,5)),
                      (dp21d_1 not in (1,5)))=4 then dcicc6=.;
               else dcicc6=sum((dp21a_1=5),(dp21b_1=5),
                               (dp21c_1=5),(dp21d_1=5))>0;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if sum((dp19a_1 not in (1,5)),
                      (dp19a1_1 not in (1,5)),
                      (dp20a_1 not in (1,5)),
                      (dp20a1_1 not in (1,5)))=4 then dcicc7=.;
               else dcicc7=sum((dp19a_1=5),(dp19a1_1=5),
                               (dp20a_1=5),(dp20a1_1=5))>0;

            ************************;
            * at least one present *;
            ************************;

               if n(of dcicc1-dcicc7)=0 then dcicc=.;
               else dcicc=sum(of dcicc1-dcicc7)>0;

         **************************************;
         * Criterion D -- Skip in this module *;
         **************************************;

         *************;
         * Diagnosis *;
         *************;

            ******************;
            * sum of B and C *;
            ******************;

               dcicsx=sum(sum(of dcicb1-dcicb3),sum(of dcicc1-dcicc7));

            ********;
            * mild *;
            ********;

               if ((dcica=. or n(of dcicg1-dcicg3)<2) and
                   n(of dcicb1-dcicb3)<2 and
                   sum(n(of dcicb1-dcicb3),
                       n(of dcicc1-dcicc7))<4) then dcicd1=.;
               else do;
                  dcicd1=((5*(sum(dcica=1,dcicb=1,dcicsx>3)=3))+
                          (3*(sum(dcica=0,dcicg1=1,dcicg2=1,dcicg3=0,
                                  dcicb=1,dcicsx>3)=6)));
               end;

            ************;
            * moderate *;
            ************;

               if ((dcica=. or n(of dcicg1-dcicg3)<2) and
                   n(of dcicb1-dcicb3)<2 and
                   sum(n(of dcicb1-dcicb3),
                       n(of dcicc1-dcicc7))<6) then dcicd2=.;
               else do;
                  dcicd2=((5*(sum(dcica=1,dcicb=1,dcicsx>5)=3))+
                          (3*(sum(dcica=0,dcicg1=1,dcicg2=1,dcicg3=0,
                                  dcicb=1,dcicsx>5)=6)));
               end;

            **********;
            * severe *;
            **********;

               if ((dcica=. or n(of dcicg1-dcicg3)<2) and
                   n(of dcicb1-dcicb3)=0 and
                   sum(n(of dcicb1-dcicb3),
                       n(of dcicc1-dcicc7))<7) then dcicd3=.;
               else do;
                  dcicd3=((5*(sum(dcica=1,sum(of dcicb1-dcicb3)=3,
                                  dcicsx>7)=3))+
                          (3*(sum(dcica=0,dcicg1=1,dcicg2=1,dcicg3=0, 
                                  sum(of dcicb1-dcicb3)=3,dcicsx>7)=6)));
               end;

            ****************;
            * any severity *;
            ****************;

               if n(of dcicd1-dcicd3)=0 then dcicdx=.;
               else dcicdx=max(of dcicd1-dcicd3);

      ****************************;
      * most severe past episode *;
      ****************************;

         past:

         if dpicsk=5 then go to skipout;

         ****************;
         * Criterion G1 *;
         ****************;

            if (dp30b_2 not in (1,5)) then dpicg1=.;
            else dpicg1=(dp30b_2=5);

         ****************;
         * Criterion G2 *;
         ****************;

            %if &study.=irpg %then %do;
               dpicg2=.;
            %end;
            %else %if &study.=coga %then %do;
               if mn1=. then dpicg2=.;
               else dpicg2=1;
            %end;

         ****************;
         * Criterion G3 *;
         ****************;

            if sum((dp11d not in (1,5)),(dp11g not in (1,5)),
                   (sum((dp12a_1 not in (1,5)),(dp12a_2 not in (1,5)),
                        (dp12a_3 not in (1,5)),(dp12a_4 not in (1,5)),
                        (dp12a_5 not in (1,5)),(dp12a_6 not in (1,5)))=6 or
                    sum((dp12b_1 not in (1,5)),(dp12b_2 not in (1,5)),
                        (dp12b_3 not in (1,5)),(dp12b_4 not in (1,5)), 
                        (dp12b_5 not in (1,5)),(dp12b_6 not in (1,5)))=6),
                    (dp13a not in (1,5)),(dp31b_2 not in (1,5)),
                    (dp31c_2 not in (1,5)))=6 then dpicg3=.;
            else dpicg3=((dp11d=5 or dp11g=5) or
                         (sum((dp12a_1=5),(dp12a_2=5),(dp12a_3=5),
                              (dp12a_4=5),(dp12a_5=5),(dp12a_6=5))>0 or
                          sum((dp12b_1=5),(dp12b_2=5),(dp12b_3=5),
                              (dp12b_4=5),(dp12b_5=5),(dp12b_6=5))>0) or
                         (dp13a=5) or (dp31b_2=5) or (dp31c_2=5));
            if (dp14a=5 and dp14a1=5 and dp14a2=5 and 
                dp31b_2=1 and dp31c_2=1) then dpicg3=0;
            if dpicg3 ne . then dpicg3=(-1*(dpicg3-1));

         ***************;
         * Criterion A *;
         ***************;

            if n(of dpicg1-dpicg3)=0 then dpica=.;
            else do;
               dpica=(((n(of dpicg1-dpicg3)=1)*(sum(of dpicg1-dpicg3)=1))+
                      ((n(of dpicg1-dpicg3)=2)*(sum(of dpicg1-dpicg3)=2))+
                      ((n(of dpicg1-dpicg3)=3)*(sum(of dpicg1-dpicg3)>1)));
            end;

         ***************;
         * Criterion B *;
         ***************;

            ******************;
            * depressed mood *;
            ******************;

               if sum((dp7c_1 not in (1,5)),
                      (dp7c_2 not in (1,5)),
                      (dp14b not in (1,5)),
                      (dp16_1=. and dp16_2=.))=4 then dpicb1=.;
               else dpicb1=sum((dp7c_1=5),(dp7c_2=5),
                               (dp14b=5 and (dp16_1>1 and dp16_2>1)))>0;

            *************;
            * anhedonia *;
            *************;

               if sum((dp7c_3 not in (1,5)),
                      (dp7c_4 not in (1,5)),
                      (dp14c not in (1,5)))=3 then dpicb2=.;
               else dpicb2=sum((dp7c_3=5),(dp7c_4=5),(dp14c=5))>0;

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if sum((dp7c_6 not in (1,5)),
                      (dp14d not in (1,5)))=2 then dpicb3=.;
               else dpicb3=sum((dp7c_6=5),(dp14d=5))>0;

            ************************;
            * at least two present *;
            ************************;

               if n(of dpicb1-dpicb3)=0 then dpicb=.;
               else do;
                  dpicb=(((n(of dpicb1-dpicb3)=1)*
                          (sum(of dpicb1-dpicb3)=1))+
                         ((n(of dpicb1-dpicb3)>1)*
                          (sum(of dpicb1-dpicb3)>1)));
               end;    

         ***************;
         * Criterion C *;
         ***************;

            **********************************;
            * loss of confidence/self esteem *;
            **********************************;

               if sum((dp24_2 not in (1,5)),
                      (dp25a_2 not in (1,5)))=2 then dpicc1=.;
               else dpicc1=sum((dp24_2=5),(dp25a_2=5))>0;

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if (dp25_2 not in (1,5)) then dpicc2=.;
               else dpicc2=(dp25_2=5);

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if sum((dp28a_2 not in (1,5)),
                      (dp28b_2 not in (1,5)),
                      (dp28c_2 not in (1,5)),
                      (dp28d_2 not in (1,5)))=4 then dpicc3=.;
               else dpicc3=sum((dp28a_2=5),(dp28b_2=5),
                               (dp28c_2=5),(dp28d_2=5))>0;

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if sum((dp26a_2 not in (1,5)),
                      (dp26b_2 not in (1,5)),
                      (dp27a_2 not in (1,5)),
                      (dp27b_2 not in (1,5)))=4 then dpicc4=.;
               else dpicc4=sum((dp26a_2=5),(dp26b_2=5),
                               (dp27a_2=5),(dp27b_2=5))>0;

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;

               if sum((dp22_2 not in (1,5)),
                      (dp23_2 not in (1,5)))=2 then dpicc5=.;
               else dpicc5=sum((dp22_2=5),(dp23_2=5))>0;

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if sum((dp21a_2 not in (1,5)),
                      (dp21b_2 not in (1,5)),
                      (dp21c_2 not in (1,5)),
                      (dp21d_2 not in (1,5)))=4 then dpicc6=.;
               else dpicc6=sum((dp21a_2=5),(dp21b_2=5),
                               (dp21c_2=5),(dp21d_2=5))>0;

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if sum((dp19a_2 not in (1,5)),
                      (dp19a1_2 not in (1,5)),
                      (dp20a_2 not in (1,5)),
                      (dp20a1_2 not in (1,5)))=4 then dpicc7=.;
               else dpicc7=sum((dp19a_2=5),(dp19a1_2=5),
                               (dp20a_2=5),(dp20a1_2=5))>0;

            ************************;
            * at least one present *;
            ************************;

               if n(of dpicc1-dpicc7)=0 then dpicc=.;
               else dpicc=sum(of dpicc1-dpicc7)>0;

         **************************************;
         * Criterion D -- Skip in this module *;
         **************************************;

         *************;
         * Diagnosis *;
         *************;

            ******************;
            * sum of B and C *;
            ******************;

               dpicsx=sum(sum(of dpicb1-dpicb3),sum(of dpicc1-dpicc7));

            ********;
            * mild *;
            ********;

               if ((dpica=. or n(of dpicg1-dpicg3)<2) and
                   n(of dpicb1-dpicb3)<2 and
                   sum(n(of dpicb1-dpicb3),
                       n(of dpicc1-dpicc7))<4) then dpicd1=.;
               else do;
                  dpicd1=((5*(sum(dpica=1,dpicb=1,dpicsx>3)=3))+
                          (3*(sum(dpica=0,dpicg1=1,dpicg2=1,dpicg3=0,
                                  dpicb=1,dpicsx>3)=6)));
               end;

            ************;
            * moderate *;
            ************;

               if ((dpica=. or n(of dpicg1-dpicg3)<2) and
                   n(of dpicb1-dpicb3)<2 and
                   sum(n(of dpicb1-dpicb3),
                       n(of dpicc1-dpicc7))<6) then dpicd2=.;
               else do;
                  dpicd2=((5*(sum(dpica=1,dpicb=1,dpicsx>5)=3))+
                          (3*(sum(dpica=0,dpicg1=1,dpicg2=1,dpicg3=0,
                                  dpicb=1,dpicsx>5)=6)));
               end;

            **********;
            * severe *;
            **********;

               if ((dpica=. or n(of dpicg1-dpicg3)<2) and
                   n(of dpicb1-dpicb3)=0 and
                   sum(n(of dpicb1-dpicb3),
                       n(of dpicc1-dpicc7))<7) then dpicd3=.;
               else do;
                  dpicd3=((5*(sum(dpica=1,sum(of dpicb1-dpicb3)=3,
                                  dpicsx>7)=3))+
                          (3*(sum(dpica=0,dpicg1=1,dpicg2=1,dpicg3=0, 
                                  sum(of dpicb1-dpicb3)=3,dpicsx>7)=6)));
               end;

            ****************;
            * any severity *;
            ****************;

               if n(of dpicd1-dpicd3)=0 then dpicdx=.;
               else dpicdx=max(of dpicd1-dpicd3);

      *****************;
      * lifetime/ever *;
      *****************;

         lifetime:

         if mdicsk=5 then go to skipout;

         ****************;
         * Criterion G1 *;
         ****************;

            if (dcicg1=. and dpicg1=.) then mdicg1=.;
            else mdicg1=(dcicg1 or dpicg1);

         ****************;
         * Criterion G2 *;
         ****************;

            if n(dcicg2,dpicg2)=0 then mdicg2=.;
            else mdicg2=(dcicg2 or dpicg2);

         ****************;
         * Criterion G3 *;
         ****************;

            if n(dcicg3,dpicg3)=0 then mdicg3=.;
            else mdicg3=(dcicg3 or dpicg3);
 
         ***************;
         * Criterion A *;
         ***************;

            if n(dcica,dpica)=0 then mdica=.;
            else mdica=(dcica or dpica);

         ***************;
         * Criterion B *;
         ***************;

            ******************;
            * depressed mood *;
            ******************;

               if n(dcicb1,dpicb1)=0 then mdicb1=.;
               else mdicb1=(dcicb1 or dpicb1);

            *************;
            * anhedonia *;
            *************;

               if n(dcicb2,dpicb2)=0 then mdicb2=.;
               else mdicb2=(dcicb2 or dpicb2);

            **************************;
            * fatigue/loss of energy *;
            **************************;

               if n(dcicb3,dpicb3)=0 then mdicb3=.;
               else mdicb3=(dcicb3 or dpicb3);

            ************************;
            * at least two present *;
            ************************;

               if n(dcicb,dpicb)=0 then mdicb=.;
               else mdicb=(dcicb or dpicb);

         ***************;
         * Criterion C *;
         ***************;

            **********************************;
            * loss of confidence/self esteem *;
            **********************************;

               if n(dcicc1,dpicc1)=0 then mdicc1=.;
               else mdicc1=(dcicc1 or dpicc1);

            ***********************************;
            * feelings of guilt/worthlessness *;
            ***********************************;

               if n(dcicc2,dpicc2)=0 then mdicc2=.;
               else mdicc2=(dcicc2 or dpicc2);

            **********************************;
            * suidical ideation/plan/attempt *;
            **********************************;

               if n(dcicc3,dpicc3)=0 then mdicc3=.;
               else mdicc3=(dcicc3 or dpicc3);

            *******************************************;
            * difficulty concentrating/indecisiveness *;
            *******************************************;

               if n(dcicc4,dpicc4)=0 then mdicc4=.;
               else mdicc4=(dcicc4 or dpicc4);

            *************************************;
            * psychomotor agitation/retardation *;
            *************************************;

               if n(dcicc5,dpicc5)=0 then mdicc5=.;
               else mdicc5=(dcicc5 or dpicc5);

            ************************;
            * insomnia/hypersomnia *;
            ************************;

               if n(dcicc6,dpicc6)=0 then mdicc6=.;
               else mdicc6=(dcicc6 or dpicc6);

            **************************;
            * hypophagia/hyperphagia *;
            **************************;

               if n(dcicc7,dpicc7)=0 then mdicc7=.;
               else mdicc7=(dcicc7 or dpicc7);

            ************************;
            * at least one present *;
            ************************;

               if n(dcicc,dpicc)=0 then mdicc=.;
               else mdicc=(dcicc or dpicc);

         **************************************;
         * Criterion D -- Skip in this module *;
         **************************************;

         *************;
         * Diagnosis *;
         *************;

            ********;
            * mild *;
            ********;

               if n(dcicd1,dpicd1)=0 then mdicd1=.;
               else mdicd1=max(dcicd1,dpicd1);

            ************;
            * moderate *;
            ************;

               if n(dcicd2,dpicd2)=0 then mdicd2=.;
               else mdicd2=max(dcicd2,dpicd2);

            **********;
            * severe *;
            **********;

               if n(dcicd3,dpicd3)=0 then mdicd3=.;
               else mdicd3=max(dcicd3,dpicd3);

            ****************;
            * any severity *;
            ****************;

               if n(of mdicd1-mdicd3)=0 then mdicdx=.;
               else mdicdx=max(of mdicd1-mdicd3);

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *************************************;
      * rescale variables from 0,1 to 1,5 *;
      *************************************;

         array one dcicg1 dcicg2 dcicg3
                   dcica 
                   dcicb1-dcicb3 dcicb
                   dcicc1-dcicc7 dcicc;
         array two dpicg1 dpicg2 dpicg3
                   dpica 
                   dpicb1-dpicb3 dpicb
                   dpicc1-dpicc7 dpicc;
         array three mdicg1 mdicg2 mdicg3
                     mdica 
                     mdicb1-mdicb3 mdicb
                     mdicc1-mdicc7 mdicc;
         do over one;
            if dcicsk=1 then do;
               if one=1 then one=5;
               if one=0 then one=1;
            end;
            if dpicsk=1 then do;
               if two=1 then two=5;
               if two=0 then two=1;
            end;
            if mdicsk=1 then do;
               if three=1 then three=5;
               if three=0 then three=1;
            end;
         end;

         array four dcicd1-dcicd3 dcicdx;
         array five dpicd1-dpicd3 dpicdx;
         array six mdicd1-mdicd3 mdicdx;
         do over four;
            if dcicsk=1 then do;
               if four=0 then four=1;
            end;
            if dpicsk=1 then do;
               if five=0 then five=1;
            end;
            if mdicsk=1 then do;
               if six=0 then six=1;
            end;
         end;

      *******************;
      * variable labels *;
      *******************;

         label dcicsk = "ICD10 MDD (Current Episode) Skip Out"
               dpicsk = "ICD10 MDD (Worst Episode) Skip Out"
               mdicsk = "ICD10 MDD (Lifetime) Skip Out"
               dcicg1 = "ICD10 MDD (Current Episode) Crit. G1"
               dcicg2 = "ICD10 MDD (Current Episode) Crit. G2"
               dcicg3 = "ICD10 MDD (Current Episode) Crit. G3"
               dcica  = "ICD10 MDD (Current Episode) Crit. A"
               dcicb1 = "ICD10 MDD (Current Episode) Crit. B1"
               dcicb2 = "ICD10 MDD (Current Episode) Crit. B2"
               dcicb3 = "ICD10 MDD (Current Episode) Crit. B3"
               dcicb  = "ICD10 MDD (Current Episode) 2+ B"
               dcicc1 = "ICD10 MDD (Current Episode) Crit. C1"
               dcicc2 = "ICD10 MDD (Current Episode) Crit. C2"
               dcicc3 = "ICD10 MDD (Current Episode) Crit. C3"
               dcicc4 = "ICD10 MDD (Current Episode) Crit. C4"
               dcicc5 = "ICD10 MDD (Current Episode) Crit. C5"
               dcicc6 = "ICD10 MDD (Current Episode) Crit. C6"
               dcicc7 = "ICD10 MDD (Current Episode) Crit. C7"
               dcicc  = "ICD10 MDD (Current Episode) 1+ C"
               dcicsx = "ICD10 MDD (Current) Sum of B and C"
               dcicd1 = "ICD10 MDD (Current Episode) Dx+ Mild"
               dcicd2 = "ICD10 MDD (Current Episode) Dx+ Mod."
               dcicd3 = "ICD10 MDD (Current Episode) Dx+ Severe"
               dcicdx = "ICD10 MDD (Current Episode) Dx+ Any"
               dpicg1 = "ICD10 MDD (Worst Past Episode) Crit. G1"
               dpicg2 = "ICD10 MDD (Worst Past Episode) Crit. G2"
               dpicg3 = "ICD10 MDD (Worst Past Episode) Crit. G3"
               dpica  = "ICD10 MDD (Worst Past Episode) Crit. A"
               dpicb1 = "ICD10 MDD (Worst Past Episode) Crit. B1"
               dpicb2 = "ICD10 MDD (Worst Past Episode) Crit. B2"
               dpicb3 = "ICD10 MDD (Worst Past Episode) Crit. B3"
               dpicb  = "ICD10 MDD (Worst Past Episode) 2+ B"
               dpicc1 = "ICD10 MDD (Worst Past Episode) Crit. C1"
               dpicc2 = "ICD10 MDD (Worst Past Episode) Crit. C2"
               dpicc3 = "ICD10 MDD (Worst Past Episode) Crit. C3"
               dpicc4 = "ICD10 MDD (Worst Past Episode) Crit. C4"
               dpicc5 = "ICD10 MDD (Worst Past Episode) Crit. C5"
               dpicc6 = "ICD10 MDD (Worst Past Episode) Crit. C6"
               dpicc7 = "ICD10 MDD (Worst Past Episode) Crit. C7"
               dpicc  = "ICD10 MDD (Worst Past Episode) 1+ C"
               dpicsx = "ICD10 MDD (Worst Past) Sum of B and C"
               dpicd1 = "ICD10 MDD (Worst Past Episode) Dx+ Mild"
               dpicd2 = "ICD10 MDD (Worst Past Episode) Dx+ Mod."
               dpicd3 = "ICD10 MDD (Wst Past Episode) Dx+ Severe"
               dpicdx = "ICD10 MDD (Worst Past Episode) Dx+ Any"
               mdicg1 = "ICD10 MDD (Lifetime Episode) Crit. G1"
               mdicg2 = "ICD10 MDD (Lifetime Episode) Crit. G2"
               mdicg3 = "ICD10 MDD (Lifetime Episode) Crit. G3"
               mdica  = "ICD10 MDD (Lifetime Episode) Crit. A"
               mdicb1 = "ICD10 MDD (Lifetime Episode) Crit. B1"
               mdicb2 = "ICD10 MDD (Lifetime Episode) Crit. B2"
               mdicb3 = "ICD10 MDD (Lifetime Episode) Crit. B3"
               mdicb  = "ICD10 MDD (Lifetime Episode) 2+ B"
               mdicc1 = "ICD10 MDD (Lifetime Episode) Crit. C1"
               mdicc2 = "ICD10 MDD (Lifetime Episode) Crit. C2"
               mdicc3 = "ICD10 MDD (Lifetime Episode) Crit. C3"
               mdicc4 = "ICD10 MDD (Lifetime Episode) Crit. C4"
               mdicc5 = "ICD10 MDD (Lifetime Episode) Crit. C5"
               mdicc6 = "ICD10 MDD (Lifetime Episode) Crit. C6"
               mdicc7 = "ICD10 MDD (Lifetime Episode) Crit. C7"
               mdicc  = "ICD10 MDD (Lifetime Episode) 1+ C"
               mdicd1 = "ICD10 MDD (Lifetime Episode) Dx+ Mild"
               mdicd2 = "ICD10 MDD (Lifetime Episode) Dx+ Mod."
               mdicd3 = "ICD10 MDD (Lifetime Episode) Dx+ Severe"
               mdicdx = "ICD10 MDD (Lifetime Episode) Dx+ Any";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id 
                    dcicsk dpicsk mdicsk
                    dcicg1 dcicg2 dcicg3
                    dcica 
                    dcicb1-dcicb3 dcicb
                    dcicc1-dcicc7 dcicc
                    dcicsx
                    dcicd1-dcicd3 dcicdx
                    dpicg1 dpicg2 dpicg3
                    dpica 
                    dpicb1-dpicb3 dpicb
                    dpicc1-dpicc7 dpicc
                    dpicsx
                    dpicd1-dpicd3 dpicdx
                    mdicg1 mdicg2 mdicg3
                    mdica 
                    mdicb1-mdicb3 mdicb
                    mdicc1-mdicc7 mdicc
                    mdicd1-mdicd3 mdicdx;

               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_dep_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dp1c_1 dp1c_2 dp1c_3 dp1c_4 dp1c_5
                                  dcicsk);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                    %multi(depic,dp7c_1 dp7c_2 dp7c_3 dp7c_4 dp7c_5
                                 dpicsk);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,dcicsk dpicsk mdicsk);

               ****************;
               * Criterion G1 *;
               ****************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dp30b_1 dcicg1);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dp30b_2 dpicg1);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk dcicg1 dpicg1 mdicg1);

               ****************;
               * Criterion G2 *;
               ****************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dcicg2);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dpicg2);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk dcicg2 dpicg2 mdicg2);

               ****************;
               * Criterion G3 *;
               ****************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dp4d dp4g dp5a_1-dp5a_6 dp5b_1-dp5b_6
                                  dp6a dp31b_1 dp31c_1 dcicg3);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dp11d dp11g dp12a_1-dp12a_6 
                                  dp12b_1-dp12b_6 dp13a dp31b_2 dp31c_2 
                                  dp14a dp14a1 dp14a2
                                  dpicg3);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk dcicg3 dpicg3 mdicg3);

               ***************;
               * Criterion B *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dcicg1 dcicg2 dcicg3 dcica);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dpicg1 dpicg2 dpicg3 dpica);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk mdicg1 mdicg2 mdicg3 mdica);
                     %multi(depic,mdicsk dcica dpica mdica);

               ***************;
               * Criterion B *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dp1c_1 dp1c_2 dcicb1);
                     %multi(depic,dcicsk dp1c_3 dp1c_4 dcicb2);
                     %multi(depic,dcicsk dp1c_6 dcicb3);
                     %multi(depic,dcicsk dcicb1 dcicb2 dcicb3 dcicb);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dp7c_1 dp7c_2 dp14b dp16_1 dp16_2 
                                  dpicb1);
                     %multi(depic,dpicsk dp7c_3 dp7c_4 dp14c dpicb2);
                     %multi(depic,dpicsk dp7c_6 dp14d dpicb3);
                     %multi(depic,dpicsk dpicb1 dpicb2 dpicb3 dpicb);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk dcicb1 dpicb1 mdicb1);
                     %multi(depic,mdicsk dcicb2 dpicb2 mdicb2);
                     %multi(depic,mdicsk dcicb3 dpicb3 mdicb3);
                     %multi(depic,mdicsk mdicb1 mdicb2 mdicb3 mdicb);
                     %multi(depic,mdicsk dcicb dpicb mdicb);

               ***************;
               * Criterion C *;
               ***************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dp24_1 dp25a_1 dcicc1);
                     %multi(depic,dcicsk dp25_1 dcicc2);
                     %multi(depic,dcicsk dp28a_1 dp28b_1 dp28c_1 dp28d_1 
                                  dcicc3);
                     %multi(depic,dcicsk dp26a_1 dp26b_1 dp27a_1 dp27b_1 
                                  dcicc4);
                     %multi(depic,dcicsk dp22_1 dp23_1 dcicc5);
                     %multi(depic,dcicsk dp21a_1 dp21b_1 dp21c_1 dp21d_1 
                                  dcicc6);
                     %multi(depic,dcicsk dp19a_1 dp19a1_1 dp20a_1 dp20a1_1 
                                  dcicc7);
                     %multi(depic,dcicsk dcicc1-dcicc7 dcicc);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dp24_2 dp25a_2 dpicc1);
                     %multi(depic,dpicsk dp25_2 dpicc2);
                     %multi(depic,dpicsk dp28a_2 dp28b_2 dp28c_2 dp28d_2 
                                  dpicc3);
                     %multi(depic,dpicsk dp26a_2 dp26b_2 dp27a_2 dp27b_2 
                                  dpicc4);
                     %multi(depic,dpicsk dp22_2 dp23_2 dpicc5);
                     %multi(depic,dpicsk dp21a_2 dp21b_2 dp21c_2 dp21d_2 
                                  dpicc6);
                     %multi(depic,dpicsk dp19a_2 dp19a1_2 dp20a_2 dp20a1_2 
                                  dpicc7);
                     %multi(depic,dpicsk dpicc1-dpicc7 dpicc);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk dcicc1 dpicc1 mdicc1);
                     %multi(depic,mdicsk dcicc2 dpicc2 mdicc2);
                     %multi(depic,mdicsk dcicc3 dpicc3 mdicc3);
                     %multi(depic,mdicsk dcicc4 dpicc4 mdicc4);
                     %multi(depic,mdicsk dcicc5 dpicc5 mdicc5);
                     %multi(depic,mdicsk dcicc6 dpicc6 mdicc6);
                     %multi(depic,mdicsk dcicc7 dpicc7 mdicc7);
                     %multi(depic,mdicsk mdicc1-mdicc7 mdicc);
                     %multi(depic,mdicsk dcicc dpicc mdicc);

               *************;
               * Diagnoses *;
               *************;

                  *******************;
                  * current episode *;
                  *******************;

                     %multi(depic,dcicsk dcicb1-dcicb3 dcicc1-dcicc7 dcicsx);
                     %multi(depic,dcicsk dcica dcicg1 dcicg2 dcicg3 
                                  dcicb dcicsx dcicd1);
                     %multi(depic,dcicsk dcica dcicg1 dcicg2 dcicg3 
                                  dcicb dcicsx dcicd2);
                     %multi(depic,dcicsk dcica dcicg1 dcicg2 dcicg3 
                                  dcicb dcicsx dcicd3);
                     %multi(depic,dcicsk dcicd1-dcicd3 dcicdx);

                  ****************************;
                  * most severe past episode *;
                  ****************************;

                     %multi(depic,dpicsk dpicb1-dpicb3 dpicc1-dpicc7 dpicsx);
                     %multi(depic,dpicsk dpica dpicg1 dpicg2 dpicg3 
                                  dpicb dpicsx dpicd1);
                     %multi(depic,dpicsk dpica dpicg1 dpicg2 dpicg3 
                                  dpicb dpicsx dpicd2);
                     %multi(depic,dpicsk dpica dpicg1 dpicg2 dpicg3 
                                  dpicb dpicsx dpicd3);
                     %multi(depic,dpicsk dpicd1-dpicd3 dpicdx);

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(depic,mdicsk dcicd1 dpicd2 mdicd1);
                     %multi(depic,mdicsk dcicd2 dpicd2 mdicd2);
                     %multi(depic,mdicsk dcicd3 dpicd3 mdicd3);
                     %multi(depic,mdicsk dcicdx dpicdx mdicdx);

            ******************;
            * keep variables *;
            ******************;

               data depic;
               set depic;
               keep ind_id 
                    dcicsk dpicsk mdicsk
                    dcicg1 dcicg2 dcicg3
                    dcica 
                    dcicb1-dcicb3 dcicb
                    dcicc1-dcicc7 dcicc
                    dcicsx
                    dcicd1-dcicd3 dcicdx
                    dpicg1 dpicg2 dpicg3
                    dpica 
                    dpicb1-dpicb3 dpicb
                    dpicc1-dpicc7 dpicc
                    dpicsx
                    dpicd1-dpicd3 dpicdx
                    mdicg1 mdicg2 mdicg3
                    mdica 
                    mdicb1-mdicb3 mdicb
                    mdicc1-mdicc7 mdicc
                    mdicd1-mdicd3 mdicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=depic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend depic;

********;
* test *;
********;
/*
   %depic(coga.csaga2,1);
*/
