*****************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Substance Dependence *;
*****************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %drgic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro drgic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in DRGIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in DRGIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let study=%cogairpg(&dsn.);
      %if &study.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      %let drglst=co st op ha se ot;
      %let wrdlst=COC AMP OPI HAL SED OTH;

      data drgic;
      set &dsn.;

      %do i=1 %to 6;

         %let drg=%scan(&drglst.,&i.);
         %let wrd=%scan(&wrdlst.,&i.);

         attrib &drg.icsk 
                &drg.ica1-&drg.ica6 &drg.icax &drg.ica &drg.icb &drg.icdp
                length=3;

         *********************;
         * test for skip out *;
         *********************;

            %if &study.=irpg or &who.=c %then %do;
               if sum(dr1a_&i. not in (1,5),
                      dr1e1_&i. not in (1,5),
                      dr5a_&i.<0,
                      dr5a1_&i. not in (0,1,2,3,4))=4 then &drg.icsk=5;
               else do;
                  if (dr1a_&i.=5 and dr1e1_&i.=5 and 
                      (dr5a_&i.>6 or dr5a1_&i. in (1,2,3,4))) then &drg.icsk=1;
                  else &drg.icsk=5;
               end;
            %end;
            %if &study.=coga and &who.=p %then %do;
               if sum(dr1e1_&i. not in (1,5),
                      dr5a_&i.<0,
                      dr5a1_&i. not in (0,1,2,3,4))=4 then &drg.icsk=5;
               else do;
                  if (dr1e1_&i.=5 and 
                      (dr5a_&i.>6 or dr5a1_&i. in (1,2,3,4))) then &drg.icsk=1;
                  else &drg.icsk=5;
               end;
            %end;

            if &drg.icsk=5 then do;
               array skp&i. &drg.ica1-&drg.ica6 &drg.icax &drg.ica &drg.icb 
                            &drg.icdp;
               do over skp&i.;
                  skp&i.=.S;
               end;
            end;

         **********************************;
         * test for coga parent interview *;
         **********************************;
   
            %if &study.=coga %then %do;
               %if &who.=p %then %do;
                  if &drg.icsk=1 then do;
                     array nas&i. &drg.ica1-&drg.ica4 &drg.ica6 &drg.icax 
                                  &drg.ica &drg.icb &drg.icdp;
                     do over nas&i.;
                        nas&i.=.N;
                     end;
                  end;
                  if &drg.icsk=5 then go to skipout&i.;
                  else go to parhop1&i.;
               %end;
            %end;

         if &drg.icsk=5 then go to skipout&i.;

         **************;
         * Dependence *;
         **************;

            ***************;
            * Criterion A *;
            ***************;

               ********************************;
               * strong desire to use alcohol *;
               ********************************;

                  if 0<dm3a<13 then do;
                     &drg.ica1=.N;
                  end;
                  else do;
                     if dr16a_&i. not in (1,5) then &drg.ica1=.;
                     else &drg.ica1=(4*(dr16a_&i.=5))+1;
                  end;

               ***************************************************************;
               * persistent desire or 1+ unsuccessful efforts to control use *;
               * or used larger amount or for longer period than intended    *;
               ***************************************************************;

                  if sum(dr12a_&i. not in (1,5),
                         dr12c_&i. not in (1,5),
                         dr13b_&i. not in (1,5),
                         dr13d_&i. not in (1,5))=4 then &drg.ica2=.;
                  else &drg.ica2=(4*(dr12a_&i.=5 or dr12c_&i.=5 or 
                                     dr13b_&i.=5 or dr13d_&i.=5))+1;

               **************;
               * withdrawal *;
               **************;

                  if 0<dm3a<13 then do;
                     &drg.ica3=.N;
                  end;
                  else do;
                     if sum(dr33b_&i. not in (1,5),
                            dr33d2_&i. not in (1,5))=2 then &drg.ica3=.;
                     else &drg.ica3=(4*(dr33b_&i.=5 and dr33d2_&i.=5))+1;
                  end;

               *************;
               * tolerance *;
               *************;
 
                  if sum(dr11a_&i. not in (1,5),
                         dr11b_&i. not in (1,5))=2 then &drg.ica4=.;
                  else &drg.ica4=(4*(dr11a_&i.=5 or dr11b_&i.=5))+1;

               **************************************;
               * jump here if coga parent interview *;
               **************************************;

                  parhop1&i.:

               ****************************************************************;
               * great deal of time spent obtaining/using/recovering from use *;
               * or use reduced social/occupational/recreational activities   *;
               ****************************************************************;

                  if sum(dr14_&i. not in (1,5),
                         dr15c_&i. not in (1,5))=2 then &drg.ica5=.;
                  else &drg.ica5=(4*(dr14_&i.=5 or dr15c_&i.=5))+1;

               **********************************;
               * test for coga parent interview *;
               **********************************;

                  %if &study.=coga %then %do;
                     %if &who.=p %then %do;
                        go to skipout&i.;
                     %end;
                  %end;

               ******************************************************;
               * use despite clear evidence of harmful consequences *;
               ******************************************************;

                  if sum(dr18d_&i. not in (1,5),
                         dr27b_&i. not in (1,5),
                         dr28e_&i. not in (1,5))=3 then &drg.ica6=.;
                  else &drg.ica6=(4*(dr18d_&i.=5 or dr27b_&i.=5 or
                                     dr28e_&i.=5))+1;

               ****************;
               * sum of items *;
               ****************;

                  if n(of &drg.ica1-&drg.ica6)=0 then &drg.icax=.;
                  else do;
                     &drg.icax=0;
                     array sx&i. &drg.ica1-&drg.ica6;
                     do over sx&i.;
                        if sx&i.=5 then &drg.icax+1;
                     end;
                  end;

               ************************;
               * at least three items *;
               ************************;

                  if &drg.icax=. then &drg.ica=.;
                  else &drg.ica=(4*(&drg.icax>2))+1;

            ***************;
            * Criterion B *;
            ***************;

               if 0<dm3a<13 then do;
                  &drg.icb=.N;
               end;
               else do;
                  if sum(dr36b_&i. not in (1,5),
                         dr36c_&i. not in (1,5))=2 then &drg.icb=.;
                  else &drg.icb=(4*(dr36b_&i.=5 or dr36c_&i.=5))+1;
               end;
                   
            *************;
            * Diagnosis *;
            *************;

               if n(&drg.ica,&drg.icb)=0 then &drg.icdp=.;
               else do;
                  if 0<dm3a<13 then &drg.icdp=(4*(&drg.ica=5))+1;
                  else &drg.icdp=(4*(&drg.ica=5 and &drg.icb=5))+1;
               end;

         ****************************;
         * jump to here if skip out *;
         ****************************;

            skipout&i.:

         *******************;
         * variable labels *;
         *******************;

            label &drg.icsk = "ICD10 &wrd. Skip Out" 
                  &drg.ica1 = "ICD10 &wrd. Dependence Crit. A1"
                  &drg.ica2 = "ICD10 &wrd. Dependence Crit. A2"
                  &drg.ica3 = "ICD10 &wrd. Dependence Crit. A3"
                  &drg.ica4 = "ICD10 &wrd. Dependence Crit. A4" 
                  &drg.ica5 = "ICD10 &wrd. Dependence Crit. A5" 
                  &drg.ica6 = "ICD10 &wrd. Dependence Crit. A6" 
                  &drg.icax = "ICD10 &wrd. Dependence Sum of As" 
                  &drg.ica  = "ICD10 &wrd. Dependence 3+ As" 
                  &drg.icb  = "ICD10 &wrd. Dependence Crit. B" 
                  &drg.icdp = "ICD10 &wrd. Dependence Diagnosis";

      %end;

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id
               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                    &drg.icsk
                    &drg.ica1-&drg.ica6 &drg.icax &drg.ica &drg.icb &drg.icdp

               %end;
               ;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_drg_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);

                  ************;
                  * skip out *;
                  ************;

                     %if &study.=irpg or &who.=c %then %do;
                        %multi(drgic,dr1a_&i. dr1e1_&i. dr5a_&i. dr5a1_&i.
                                     &drg.icsk);
                     %end;
                     %if &study.=coga and &who.=p %then %do;
                        %multi(drgic,dr1e1_&i. dr5a_&i. dr5a1_&i. &drg.icsk);
                     %end;

                  **************;
                  * Dependence *;
                  **************;

                     ***************;
                     * Criterion A *;
                     ***************;

                        %multi(drgic,&drg.icsk dr16a_&i. &drg.ica1);
                        %multi(drgic,&drg.icsk dr12a_&i. dr12c_&i. 
                                     dr13b_&i. dr13d_&i. &drg.ica2);
                        %multi(drgic,&drg.icsk dr33b_&i. dr33d2_&i. &drg.ica3);
                        %multi(drgic,&drg.icsk dr11a_&i. dr11b_&i. &drg.ica4);
                        %multi(drgic,&drg.icsk dr14_&i. dr15c_&i. &drg.ica5);
                        %multi(drgic,&drg.icsk dr18d_&i. dr27b_&i. dr28e_&i. 
                                     &drg.ica6);
                        %multi(drgic,&drg.icsk &drg.ica1-&drg.ica6 &drg.icax);
                        %multi(drgic,&drg.icsk &drg.icax &drg.ica);

                     ***************;
                     * Criterion B *;
                     ***************;

                        %multi(drgic,&drg.icsk dr36b_&i. dr36c_&i. &drg.icb);
  
                     *************;
                     * Diagnosis *;
                     *************;

                        %multi(drgic,&drg.icsk &drg.ica &drg.icb &drg.icdp);

               %end;
   
            ******************;
            * keep variables *;
            ******************;

               data drgic;
               set drgic;
               keep ind_id  
               %do i=1 %to 6;

                  %let drg=%scan(&drglst.,&i.);
 
                    &drg.icsk 
                    &drg.ica1-&drg.ica6 &drg.icax &drg.ica &drg.icb &drg.icdp

               %end;
               ;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=drgic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend drgic;

********;
* test *;
********;
/*
   %drgic(coga.csaga2,1);

   proc freq data=drgic;
   tables coicdp sticdp opicdp haicdp seicdp oticdp / list missing;
   run;
*/
