*************************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Generalized Anxiety Disorder *;
*************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %gadic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro gadic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in GADIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in GADIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data gadic;
      set &dsn.;

      attrib gaicsk 
             gaica gaicb gaicc1-gaicc6 gaicc3x gaicc5x gaiccx gaicc
             gaicd gaice gaicf gaicdx length=3;

      *********************;
      * test for skip out *;
      *********************;
 
         if sum(ov2a not in (1,5),
                ov2b not in (1,3,5),
                ov3a not in (1,5),
                ov3b not in (1,3,5))=4 then gaicsk=5;
         else do;
            if sum((ov2a ne 5 and ov3a ne 5), 
                   (ov2b not in (3,5) and ov3b not in (3,5)))=2 then gaicsk=5;
            else gaicsk=1;
         end;

         if gaicsk=5 then do;
            array skp1 gaica gaicb gaicc1-gaicc6 gaicc3x gaicc5x gaiccx gaicc
                       gaicd gaice gaicf gaicdx; 
            do over skp1;
              skp1=.S;
            end;
         end;

         if gaicsk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ********************************;
            * excessive, unrealistic worry *;
            ********************************;

               if sum(ov2b not in (1,3,5),
                      ov3b not in (1,3,5),
                      ov4a not in (1,3,5),
                      ov4b not in (1,3,5),
                      sum(ov17b not in (1,5),
                          ov19b not in (1,5))>0)=5 then gaica=.;
               else do;
                  gaica=1;
                  if ((ov2b=3 or ov3b=3 or ov4a=3 or ov4b=3) and
                      ov19b=5) then gaica=3;
                  else if ((ov2b=5 or ov3b=5 or ov4a=5 or ov4b=5) and
                           ov17b=5) then gaica=5;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ******************************;
            * difficult to control worry *;
            ******************************;

               if ov9 not in (1,3,5) then gaicb=.;
               else gaicb=(4*(ov9=5))+(2*(ov9=3))+1; 
 
      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * restlessness, keyed-up, on edge *;
            ***********************************;

               if ov10 not in (1,3,5) then gaicc1=.;
               else gaicc1=(4*(ov10=5))+(2*(ov10=3))+1;

            *******************;
            * easily fatigued *;
            *******************;

               if ov11 not in (1,3,5) then gaicc2=.;
               else gaicc2=(4*(ov11=5))+(2*(ov11=3))+1;

            ****************************;
            * difficulty concentrating *;
            ****************************;

               if sum(ov12a not in (1,3,5),
                      ov12b not in (1,3,5))=2 then gaicc3=.;
               else do;
                  gaicc3=1;
                  if (ov12a=3 or ov12b=3) then gaicc3=3;
                  else if (ov12a=5 or ov12b=5) then gaicc3=5;
               end;

            ****************;
            * irritability *;
            ****************;

               if ov13 not in (1,3,5) then gaicc4=.;
               else gaicc4=(4*(ov13=5))+(2*(ov13=3))+1;

            ******************;
            * muscle tension *;
            ******************;

               if ov14 not in (1,3,5) then gaicc5=.;
               else gaicc5=(4*(ov14=5))+(2*(ov14=3))+1;

            *********************;
            * sleep disturbance *;
            *********************;

               if sum(ov15a not in (1,3,5),
                      ov15b not in (1,3,5),
                      ov15c not in (1,3,5))=3 then gaicc6=.;
               else do;
                  gaicc6=1;
                  if (ov15a=3 or ov15b=3 or ov15c=3) then gaicc6=3;
                  else if (ov15a=5 or ov15b=5 or ov15c=5) then gaicc6=5;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of gaicc1-gaicc6)=0 then gaiccx=.;
               else do;
                  gaicc3x=0;
                  gaicc5x=0;
                  array sxx gaicc1-gaicc6;
                  do over sxx;
                     if sxx=3 then gaicc3x=gaicc3x+1;
                     if sxx=5 then gaicc5x=gaicc5x+1;
                  end;
                  gaiccx=sum(gaicc3x,gaicc5x);
               end;

            ********************;
            * at least 3 items *;
            ********************;

               if gaiccx=. then gaicc=.;
               else gaicc=(4*(gaiccx>2))+1;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **********************;
            * multiple anxieties *;
            **********************;

               if sum(ov2a not in (1,3,5),
                      ov3a not in (1,3,5),
                      ov4a not in (1,3,5),
                      ov4b not in (1,3,5))=4 then gaicd=.;
               else do;
                  gaicd=1;
                  sub3s=0;
                  sub5s=0;
                  array sxz ov2a ov3a ov4a ov4b;
                  do over sxz;
                     if sxz=3 then sub3s=sub3s+1;
                     if sxz=5 then sub5s=sub5s+1;
                  end;
                  if sub3s>1 then gaicd=3;
                  else if sub5s>1 then gaicd=5;
               end;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************;
            * onset before age 18 *;
            ***********************;

               if sum(ov16a_1<0,ov18a_1<0)=2 then gaice=.;
               else gaice=(4*(sum(0<ov16a_1<18,0<ov18a_1<18)>0))+1;

      ***************;
      * Criterion F *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * clinically significant distress *;
            ***********************************;

               if sum(ov21a_1a not in (1,3,5),
                      ov21a_2a not in (1,3,5),
                      ov21a_3a not in (1,3,5),
                      ov21a_4a not in (1,3,5),
                      ov21a_5a not in (1,3,5))=5 then gaicf=.;
               else do;
                  gaicf=1;
                  opt1_a=0;
                  opt1_b=0;
                  opt1_c=0;
                  opt2_a=0;
                  opt2_b=0;
                  opt2_c=0;
                  array col1{*} ov21a_1a ov21a_2a ov21a_3a ov21a_4a ov21a_5a;
                  array col2{*} ov21a_1b ov21a_2b ov21a_3b ov21a_4b ov21a_5b;
                  do i=1 to dim(col1);
                     if col1(i)=3 then do;
                        opt2_a+1;
                        if col2(i)=3 then opt2_b+1;
                        if col2(i)=2 then opt2_c+1;
                     end;
                     else if col1(i)=5 then do;
                        opt1_a+1;
                        if col2(i)=3 then opt1_b+1;
                        if col2(i)=2 then opt1_c+1;
                     end;
                  end;
                  if ((opt2_a>0 and opt2_b>0) or
                      (opt2_a>1 and opt2_c>1)) then gaicf=3;
                  else if ((opt1_a>0 and opt1_b>0) or
                           (opt1_a>1 and opt1_c>1)) then gaicf=5;
               end;

      ***************************************************;
      * Criterion G -- Assessed within individual items *;
      ***************************************************;
 
      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(gaica,gaicb,gaicc,gaice,gaicf)=0 then gaicdx=.;
            else do;
               gaicdx=1;
               sum3s=0;
               sum5s=0;
               array sxs gaica gaicb gaice gaicf;
               do over sxs;
                  if sxs=3 then sum3s=sum3s+1;
                  if sxs=5 then sum5s=sum5s+1;
               end;
               if sum5s=4 and gaicc5x>2 then gaicdx=5;
               else if (sum3s-sum5s)>0 and (gaicc3x>0 and gaiccx>0) then
                  gaicdx=3;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label gaicsk  = "ICD10 GAD Skip Out"
               gaica   = "ICD10 GAD (Lifetime) Crit. A"
               gaicb   = "ICD10 GAD (Lifetime) Crit. B"
               gaicc1  = "ICD10 GAD (Lifetime) Crit. C1"
               gaicc2  = "ICD10 GAD (Lifetime) Crit. C2"
               gaicc3  = "ICD10 GAD (Lifetime) Crit. C3"
               gaicc4  = "ICD10 GAD (Lifetime) Crit. C4"
               gaicc5  = "ICD10 GAD (Lifetime) Crit. C5"
               gaicc6  = "ICD10 GAD (Lifetime) Crit. C6"
               gaicc3x = "ICD10 GAD (Lifetime) Sum of 3s"
               gaicc5x = "ICD10 GAD (Lifetime) Sum of 5s"
               gaiccx  = "ICD10 GAD (Lifetime) Sum of Cs"
               gaicc   = "ICD10 GAD (Lifetime) Crit. C"
               gaicd   = "ICD10 GAD (Lifetime) Crit. D"
               gaice   = "ICD10 GAD (Lifetime) Crit. E"
               gaicf   = "ICD10 GAD (Lifetime) Crit. F"
               gaicdx  = "ICD10 GAD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    gaicsk  
                    gaica gaicb gaicc1-gaicc6 gaicc3x gaicc5x gaiccx gaicc
                    gaicd gaice gaicf gaicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_gad_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(gadic,ov2a ov2b ov3a ov3b gaicsk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadic,gaicsk ov2b ov3b ov4a ov4b ov17b ov19b gaica);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadic,gaicsk ov9 gaicb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadic,gaicsk ov10 gaicc1);
                     %multi(gadic,gaicsk ov11 gaicc2);
                     %multi(gadic,gaicsk ov12a ov12b gaicc3);
                     %multi(gadic,gaicsk ov13 gaicc4);
                     %multi(gadic,gaicsk ov14 gaicc5);
                     %multi(gadic,gaicsk ov15a ov15b ov15c gaicc6);
                     %multi(gadic,gaicsk gaicc1-gaicc6 gaicc3x gaicc5x);
                     %multi(gadic,gaicsk gaicc3x gaicc5x gaiccx);
                     %multi(gadic,gaicsk gaiccx gaicc);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadic,gaicsk ov2a ov3a ov4a ov4b gaicd);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadic,gaicsk ov16a_1 ov18a_1 gaice);

               ***************;
               * Criterion F *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(gadic,gaicsk ov21a_1a ov21a_1b ov21a_2a ov21a_2b 
                                  ov21a_3a ov21a_3b ov21a_4a ov21a_4b
                                  ov21a_5a ov21a_5b gaicf);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(gadic,gaicsk gaica gaicb gaicc gaice gaicf gaicdx);

            ******************;
            * keep variables *;
            ******************;

               data gadic;
               set gadic;
               keep ind_id  
                    gaicsk 
                    gaica gaicb gaicc1-gaicc6 gaicc3x gaicc5x gaiccx gaicc
                    gaicd gaice gaicf gaicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=gadic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend gadic;

********;
* test *;
********;
/*
   %gadic(coga.csaga2,1);
*/
