**************************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Obsessive/Compulsive Disorder *;
**************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %ocdic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro ocdic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in OCDIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in OCDIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data ocdic;
      set &dsn.;

      attrib obicsk cpicsk ocicsk 
             obica obicb1-obicb3 obicb3x obicb5x obicbx obicb obicc obicdx
             cpica cpicb2 cpicb cpicc cpicdx ocicdx length=3;

      **************;
      * Obsessions *;
      **************;

         *********************;
         * test for skip out *;
         *********************;

            if sum(ob1 not in (1,3,5),
                   ob2b not in (1,3,5),
                   ob3c not in (1,3,5),
                   ob4b not in (1,3,5),
                   ob5c not in (1,3,5),
                   ob6b not in (1,3,5))=6 then obicsk=5;
            else do;
               if sum(ob1 in (3,5),
                      ob2b in (3,5),
                      ob3c in (3,5),
                      ob4b in (3,5),
                      ob5c in (3,5),
                      ob6b in (3,5))>0 then obicsk=1;
               else obicsk=5;
            end;

            if obicsk=5 then do;
               array skp1 obica obicb1-obicb3 obicb3x obicb5x obicbx obicb 
                          obicc obicdx;
               do over skp1;
                  skp1=.S;
               end;
            end;

            if obicsk=5 then go to compuls;

         ***************;
         * Criterion A *;
         ***************;

            ********************************************;
            * present most days for at least two weeks *;
            ********************************************;

               if ob11 not in (1,5) then obica=.;
               else obica=(4*(ob11=5))+1;

         ***************;
         * Criterion B *;
         ***************;

            ***************************************************;
            * obsession recognized as from within not without *;
            ***************************************************;

               if ob8a not in (1,5) then obicb1=.;
               else obicb1=(4*(ob8a=5))+1;

            *************************************************;
            * recurrent, persistent ideas/thoughts/impulses *;
            *************************************************;

               if sum(ob1 not in (1,3,5),
                      ob2b not in (1,3,5),
                      ob3c not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob5c not in (1,3,5),
                      ob6b not in (1,3,5))=6 then obicb2=.;
               else do;
                  obicb2=1;
                  if sum(ob1=3,ob2b=3,ob3c=3,ob4b=3,ob5c=3,ob6b=3)>0 then 
                     obicb2=3;
                  else if sum(ob1=5,ob2b=5,ob3c=5,ob4b=5,ob5c=5,ob6b=5)>0 then
                     obicb2=5;
               end;

            *************************************************;
            * attempts to ignore/suppress thoughts/impulses *;
            *************************************************;

               if sum(ob2b not in (1,3,5),
                      ob3b not in (1,3,5),
                      ob4b not in (1,3,5),
                      ob7 not in (1,3,5))=4 then obicb3=.;
               else do;
                  obicb3=1;
                  if sum(ob2b=3,ob3b=3,ob4b=3,ob7=3)>0 then 
                     obicb3=3;
                  else if sum(ob2b=5,ob3b=5,ob4b=5,ob7=5)>0 then
                     obicb3=5;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of obicb1-obicb3)=0 then obicbx=.;
               else do;
                  obicb3x=0;
                  obicb5x=0;
                  array sxs obicb1-obicb3;
                  do over sxs;
                     if sxs=3 then obicb3x=obicb3x+1;
                     if sxs=5 then obicb5x=obicb5x+1;
                  end;
                  obicbx=sum(obicb3x,obicb5x);
                  obicb=(4*(obicb3x=0 and obicb5x=3))+
                        (2*(obicb3x>0 and obicbx=3))+1;
               end;

         ***************;
         * Criterion C *;
         ***************;

            ************************************;
            * obsessions cause marked distress *;
            ************************************;

               if sum(sum(ob9a not in (1,5),ob9b not in (1,5))=2,
                      sum(ob14a_1a not in (1,3,5),
                          ob14a_2a not in (1,3,5),
                          ob14a_3a not in (1,3,5),
                          ob14a_4a not in (1,3,5),
                          ob14a_5a not in (1,3,5),
                          ob14a_6a not in (1,3,5))=6)=2 then obicc=.;
               else do;
                  obicc=1;
                  sum3as=0;
                  sum5as=0;
                  sum243=0;
                  sum343=0;
                  sum245=0;
                  sum345=0;
                  array oxa ob14a_1a ob14a_2a ob14a_3a 
                            ob14a_4a ob14a_5a ob14a_6a;
                  array oxb ob14a_1b ob14a_2b ob14a_3b 
                            ob14a_4b ob14a_5b ob14a_6b;
                  do over oxa;
                     if oxa=3 then do;
                        sum3as=sum3as+1;
                        if oxb=2 then sum243=sum243+1;
                        if oxb=3 then sum343=sum343+1;
                     end;
                     if oxa=5 then do;
                        sum5as=sum5as+1;
                        if oxb=2 then sum245=sum245+1;
                        if oxb=3 then sum345=sum345+1;
                     end;
                  end;
                  if sum3as>0 then do;
                     if ((sum3as>0 and sum343>0) or
                         (sum3as>1 and sum243>1)) then obicc=3;
                  end;
                  else if sum5as>0 then do;
                     if sum(((sum5as>0 and sum345>0) or
                             (sum5as>1 and sum245>1)),
                            sum(ob9a=5,ob9b=5)>0)=2 then obicc=5;
                  end;
               end;

         *************************************;
         * Criterion D - Skip in this module *;
         *************************************;

         *************;
         * Diagnosis *;
         *************;

            if n(obica,obicb1,obicb2,obicb3,obicc)=0 then obicdx=.;
            else do;
               obicdx=1;
               if (obica=5 and obicb1=5 and
                   sum(obicb2 in (3,5),obicb3 in (3,5),obicc in (3,5))=4 and
                   sum(obicb2=3,obicb3=3,obicc=3)>0) then obicdx=3;
               else if (sum(obicb2=3,obicb3=3,obicc=3)=0 and
                        sum(obica=5,obicb1=5,obicb2=5,obicb3=5,obicc=5)=5) then 
                  obicdx=5;
            end;

      *************************;
      * jump here if skip out *;
      *************************;

         compuls:

      ***************;
      * Compulsions *;
      ***************;

         *********************;
         * test for skip out *;
         *********************;

            if sum(cp1a_1 not in (1,5),
                   cp1a_2 not in (1,5),
                   cp1a_3 not in (1,5),
                   cp1a_4 not in (1,5),
                   cp1a_5 not in (1,5),
                   cp1b not in (1,3,5),
                   cp2a_1 not in (1,5),
                   cp2a_2 not in (1,5),
                   cp2a_3 not in (1,5),
                   cp2b not in (1,3,5),
                   cp3a not in (1,5),
                   cp3b not in (1,3,5))=12 then cpicsk=5;
            else do;
               if sum(cp1a_1=5,
                      cp1a_2=5,
                      cp1a_3=5,
                      cp1a_4=5,
                      cp1a_5=5,
                      cp1b in (3,5),
                      cp2a_1=5,
                      cp2a_2=5,
                      cp2a_3=5,
                      cp2b in (3,5),
                      cp3a=5,
                      cp3b in (3,5))>0 then cpicsk=1;
               else cpicsk=5;
            end;

            if cpicsk=5 then do;
               array skp2 cpica cpicb2 cpicb cpicc cpicdx; 
               do over skp2;
                  skp2=.S;
               end;
            end;

            if cpicsk=5 then go to skipout;

         ***************;
         * Criterion A *;
         ***************;

            ********************************************************;
            * compulsions present most days for at least two weeks *;
            ********************************************************;

               if cp5 not in (1,5) then cpica=.;
               else cpica=(4*(cp5=5))+1;

         ***************;
         * Criterion B *;
         ***************;

            **************************************************************;
            * repititive, intentional behaviors in response to obsession *;
            **************************************************************;

               if sum(cp1a_1 not in (1,5),
                      cp1a_2 not in (1,5),
                      cp1a_3 not in (1,5),
                      cp1a_4 not in (1,5),
                      cp1a_5 not in (1,5),
                      cp2a_1 not in (1,5),
                      cp2a_2 not in (1,5),
                      cp2a_3 not in (1,5),
                      cp3a not in (1,5))=9 then cpicb2=.;
               else cpicb2=(4*(sum(cp1a_1=5,cp1a_2=5,cp1a_3=5,
                                   cp1a_4=5,cp1a_5=5,cp2a_1=5,
                                   cp2a_2=5,cp2a_3=5,cp3a=5)>0))+1;

            *************;
            * all items *;
            *************;

               cpicb=cpicb2;

         ***************;
         * Criterion C *;
         ***************;

            *************************************;
            * compulsions cause marked distress *;
            *************************************;

               if sum(cp1b not in (1,3,5),
                      cp2b not in (1,3,5),
                      cp3b not in (1,3,5),
                      sum(cp7a not in (1,5),
                          cp9a not in (1,5))=2,
                      sum(cp11a_1a not in (1,3,5),
                          cp11a_2a not in (1,3,5),
                          cp11a_3a not in (1,3,5),
                          cp11a_4a not in (1,3,5),
                          cp11a_5a not in (1,3,5),
                          cp11a_6a not in (1,3,5))=6)=5 then cpicc=.;
               else do;
                  cpicc=1;
                  sum3as=0;
                  sum5as=0;
                  sum243=0;
                  sum343=0;
                  sum245=0;
                  sum345=0;
                  array cxa cp11a_1a cp11a_2a cp11a_3a 
                            cp11a_4a cp11a_5a cp11a_6a;
                  array cxb cp11a_1b cp11a_2b cp11a_3b 
                            cp11a_4b cp11a_5b cp11a_6b;
                  do over cxa;
                     if cxa=3 then do;
                        sum3as=sum3as+1;
                        if cxb=2 then sum243=sum243+1;
                        if cxb=3 then sum343=sum343+1;
                     end;
                     if cxa=5 then do;
                        sum5as=sum5as+1;
                        if cxb=2 then sum245=sum245+1;
                        if cxb=3 then sum345=sum345+1;
                     end;
                  end;
                  if (sum(cp1b=3,cp2b=3,cp3b=3,cp9a=5)>0 or
                      (sum3as>0 and sum343>0) or
                      (sum3as>1 and sum243>1)) then cpicc=3;
                  else do;
                     if ((sum(cp1b=3,cp2b=3,cp3b=3)=0 and sum3as=0) and
                         (sum(cp1b=5,cp2b=5,cp3b=5,cp7a=5)>0 or
                          (sum5as>0 and sum345>0) or
                          (sum5as>1 and sum245>1))) then cpicc=5;
                  end;
               end;

         *************************************;
         * Criterion D - Skip in this module *;
         *************************************;

         *************;
         * Diagnosis *;
         *************;

            if n(cpica,cpicb,cpicc)=0 then cpicdx=.;
            else do;
               cpicdx=1;
               if (cpica=5 and cpicb=5) then do;
                  if cpicc=3 then cpicdx=3;
                  if cpicc=5 then cpicdx=5;
               end;
            end;

      *************************;
      * jump here if skip out *;
      *************************;

         skipout:

         ocicsk=(4*(sum(obicsk=5,cpicsk=5)=2))+1;

      *************;
      * Diagnosis *;
      *************;

         if ocicsk=1 then do;
            if n(obicdx,cpicdx)=0 then ocicdx=.;
            else do;
               ocicdx=1;
               if sum(obicdx=3,cpicdx=3) then ocicdx=3;
               else if sum(obicdx=3,cpicdx=3)=0 and 
                       sum(obicdx=5,cpicdx=5)>0 then ocicdx=5;
            end;
        end;
        else do;
           ocicdx=.S;
        end;

      *******************;
      * variable labels *;
      *******************;

         label obicsk  = "ICD10 OBSESV Skip Out"
               cpicsk  = "ICD10 CMPLSV Skip Out"
               ocicsk  = "ICD10 OCD Skip Out"
               obica   = "ICD10 OBSESV (Lifetime) Crit. A"
               obicb1  = "ICD10 OBSESV (Lifetime) Crit. B1"
               obicb2  = "ICD10 OBSESV (Lifetime) Crit. B2"
               obicb3  = "ICD10 OBSESV (Lifetime) Crit. B3"
               obicb3x = "ICD10 OBSESV (Lifetime) Sum of 3s"
               obicb5x = "ICD10 OBSESV (Lifetime) Sum of 5s"
               obicbx  = "ICD10 OBSESV (Lifetime) Sum of As"
               obicb   = "ICD10 OBSESV (Lifetime) Crit. B"
               obicc   = "ICD10 OBSESV (Lifetime) Crit. C"
               obicdx  = "ICD10 OBSESV (Lifetime) Diagnosis"
               cpica   = "ICD10 CMPLSV (Lifetime) Crit. A"
               cpicb2  = "ICD10 CMPLSV (Lifetime) Crit. B2"
               cpicb   = "ICD10 CMPLSV (Lifetime) Crit. B"
               cpicc   = "ICD10 CMPLSV (Lifetime) Crit. C"
               cpicdx  = "ICD10 CMPLSV (Lifetime) Diagnosis"
               ocicdx  = "ICD10 OCD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    obicsk cpicsk ocicsk
                    obica obicb1-obicb3 obicb3x obicb5x obicbx obicb 
                    obicc obicdx
                    cpica cpicb2 cpicb cpicc cpicdx ocicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_ocd_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               **************;
               * Obsessions *;
               **************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(ocdic,ob1 ob2b ob3c ob4b ob5c ob6b obicsk);

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(ocdic,obicsk ob11 obica);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(ocdic,obicsk ob8a obicb1);
                     %multi(ocdic,obicsk ob1 ob2b ob3c ob4b ob5c ob6b obicb2);
                     %multi(ocdic,obicsk ob2b ob3b ob4b ob7 obicb3);
                     %multi(ocdic,obicsk obicb1-obicb3 obicb3x obicb5x);
                     %multi(ocdic,obicsk obicb3x obicb5x obicbx obicb);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(ocdic,obicsk 
                                  ob14a_1a ob14a_1b ob14a_2a ob14a_2b 
                                  ob14a_3a ob14a_3b ob14a_4a ob14a_4b 
                                  ob14a_5a ob14a_5b ob14a_6a ob14a_6b
                                  ob9a ob9b obicc);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(ocdic,obicsk obica obicb1 obicb2 obicb3 obicc 
                                  obicdx);

               ***************;
               * Compulsions *;
               ***************;

                  ************;
                  * skip out *;
                  ************;

                     %multi(ocdic,cp1a_1 cp1a_2 cp1a_3 cp1a_4 cp1a_5 cp1b
                                  cp2a_1 cp2a_2 cp2a_3 cp2b cp3a cp3b 
                                  cpicsk);

                  ***************;
                  * Criterion A *;
                  ***************;

                     %multi(ocdic,cpicsk cp5 cpica);

                  ***************;
                  * Criterion B *;
                  ***************;

                     %multi(ocdic,cpicsk 
                                  cp1a_1-cp1a_5 cp2a_1-cp2a_3 cp3a 
                                  cpicb2 cpicb);

                  ***************;
                  * Criterion C *;
                  ***************;

                     %multi(ocdic,cpicsk cp1b cp2b cp3b cp7a cp9a
                                  cp11a_1a cp11a_1b cp11a_2a cp11a_2b 
                                  cp11a_3a cp11a_3b cp11a_4a cp11a_4b 
                                  cp11a_5a cp11a_5b cp11a_6a cp11a_6b
                                  cpicc);

                  *************;
                  * Diagnosis *;
                  *************;

                     %multi(ocdic,cpicsk cpica cpicb cpicc cpicdx);

               *************;
               * Diagnosis *;
               *************;
 
                  %multi(ocdic,obicsk cpicsk ocicsk);
                  %multi(ocdic,ocicsk obicdx cpicdx ocicdx);

            ******************;
            * keep variables *;
            ******************;

               data ocdic;
               set ocdic;
               keep ind_id  
                    obicsk cpicsk ocicsk
                    obica obicb1-obicb3 obicb3x obicb5x obicbx obicb 
                    obicc obicdx
                    cpica cpicb2 cpicb cpicc cpicdx ocicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=ocdic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend ocdic;

********;
* test *;
********;
/*
   %ocdic(coga.csaga2,1);
*/
