**************************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Oppositional Defiant Disorder *;
**************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*********************************************************;
* uncomment following lines to run as standalone module *;
*********************************************************;
/*
   %include "./icd10_cd.sas" / nosource2;

   %cdic(coga.csaga2,0);
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %oddic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro oddic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in ODDIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in ODDIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      %let who=%whoisit(&dsn.);
      %if &who.=dk %then %do;
         %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
         %goto errexit;
      %end;

      proc contents data=&dsn. noprint out=test;
      run;

      data test;
      set test;
      if upcase(substr(name,1,2))="OP";
      run;

      data test;
      set test nobs=count;
      call symput("var"||left(_n_),name);
      call symput("nvar",left(put(count,8.)));
      run;

      data oddic;
      merge &dsn.(keep=ind_id
                       %do i=1 %to &nvar.;
                          &&var&i.
                       %end;
                       ) 
            cdic;
      by ind_id;

      attrib odicg1-odicg23 odicgx odicb odicd odicdx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            array one cdicg1-cdicg23;
            array two odicg1-odicg23;
            do over one;
               two=one;
               if two=3 then two=1;
            end;

            ****************;
            * sum of items *;
            ****************;

               if n(of odicg1-odicg23)=0 then do;
                  sum5s=.;
                  subsum5s=.;
               end;
               else do;
                  sum5s=0;
                  subsum5s=0;
                  array gs{*} odicg1-odicg23;
                  do i=1 to dim(gs);
                     if gs{i}=5 then sum5s+1;
                     if i>8 then do;
                        if gs{i}=5 then subsum5s+1;
                     end;
                  end;
               end;
               odicgx=sum5s;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if odicgx=. then odicb=.;
            else odicb=(4*(odicgx>3 and subsum5s<3))+1;

      ***************************************************;
      * Criterion C -- Assessed within individual items *;
      ***************************************************;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            if sum(op10b not in (1,5),op10c not in (1,5))=2 then odicd=.;
            else odicd=(4*(op10b=5 and op10c=5))+1;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(odicb,odicd)=0 then odicdx=.;
            else odicdx=(4*(odicb=5 and odicd=5))+1;

      *******************;
      * variable labels *;
      *******************;

         label odicg1  = "ICD10 ODD (Lifetime) Crit. A1"
               odicg2  = "ICD10 ODD (Lifetime) Crit. A2"
               odicg3  = "ICD10 ODD (Lifetime) Crit. A3"
               odicg4  = "ICD10 ODD (Lifetime) Crit. A4"
               odicg5  = "ICD10 ODD (Lifetime) Crit. A5"
               odicg6  = "ICD10 ODD (Lifetime) Crit. A6"
               odicg7  = "ICD10 ODD (Lifetime) Crit. A7"
               odicg8  = "ICD10 ODD (Lifetime) Crit. A8"
               odicg9  = "ICD10 ODD (Lifetime) Crit. A9"
               odicg10 = "ICD10 ODD (Lifetime) Crit. A10"
               odicg11 = "ICD10 ODD (Lifetime) Crit. A11"
               odicg12 = "ICD10 ODD (Lifetime) Crit. A12"
               odicg13 = "ICD10 ODD (Lifetime) Crit. A13"
               odicg14 = "ICD10 ODD (Lifetime) Crit. A14"
               odicg15 = "ICD10 ODD (Lifetime) Crit. A15"
               odicg16 = "ICD10 ODD (Lifetime) Crit. A16"
               odicg17 = "ICD10 ODD (Lifetime) Crit. A17"
               odicg18 = "ICD10 ODD (Lifetime) Crit. A18"
               odicg19 = "ICD10 ODD (Lifetime) Crit. A19"
               odicg20 = "ICD10 ODD (Lifetime) Crit. A20"
               odicg21 = "ICD10 ODD (Lifetime) Crit. A21"
               odicg22 = "ICD10 ODD (Lifetime) Crit. A22"
               odicg23 = "ICD10 ODD (Lifetime) Crit. A23"
               odicgx  = "ICD10 ODD (Lifetime) Sum of As"
               odicb   = "ICD10 ODD (Lifetime) 4+ As"
               odicd   = "ICD10 ODD (Lifetime) Crit. D"
               odicdx  = "ICD10 ODD (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    odicg1-odicg23 odicgx odicb odicd odicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_odd_&study._&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddic,cdicg1 odicg1);
                     %multi(oddic,cdicg2 odicg2);
                     %multi(oddic,cdicg3 odicg3);
                     %multi(oddic,cdicg4 odicg4);
                     %multi(oddic,cdicg5 odicg5);
                     %multi(oddic,cdicg6 odicg6);
                     %multi(oddic,cdicg7 odicg7);
                     %multi(oddic,cdicg8 odicg8);
                     %multi(oddic,cdicg9 odicg9);
                     %multi(oddic,cdicg10 odicg10);
                     %multi(oddic,cdicg11 odicg11);
                     %multi(oddic,cdicg12 odicg12);
                     %multi(oddic,cdicg13 odicg13);
                     %multi(oddic,cdicg14 odicg14);
                     %multi(oddic,cdicg15 odicg15);
                     %multi(oddic,cdicg16 odicg16);
                     %multi(oddic,cdicg17 odicg17);
                     %multi(oddic,cdicg18 odicg18);
                     %multi(oddic,cdicg19 odicg19);
                     %multi(oddic,cdicg20 odicg20);
                     %multi(oddic,cdicg21 odicg21);
                     %multi(oddic,cdicg22 odicg22);
                     %multi(oddic,cdicg23 odicg23);
                     %multi(oddic,odicg1-odicg23 sum5s subsum5s);
                     %multi(oddic,sum5s odicgx);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddic,odicgx subsum5s odicb);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddic,op10b op10c odicd);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(oddic,odicb odicd odicdx);

            ******************;
            * keep variables *;
            ******************;

               data oddic;
               set oddic;
               keep ind_id  
                    odicg1-odicg23 odicgx odicb odicd odicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=oddic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend oddic;

********;
* test *;
********;
/*
   %oddic(coga.csaga2,1);
*/
