************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD-10 - Panic Disorder *;
************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %panic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro panic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in PANIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in PANIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data panic;
      set &dsn.;

      attrib pnicsk1 pnicsk2 pnicsk
             pnica pnicb1 pnicb2 pnicb3 
             pnicb4a pnicb4b pnicb4c pnicb4d pnicb4e 
             pnicb4f pnicb4g pnicb4h pnicb4i pnicb4j
             pnicb4k pnicb4l pnicb4m pnicb4n pnicb4x pnicb4 pnicb
             pnicc pnicdx length=3; 

      *********************;
      * test for skip out *;
      *********************;
 
         if pn1a not in (1,2,3,4,5) then pnicsk1=5;
         else do;
            if pn1a in (2,3,4,5) then pnicsk1=1;
            else pnicsk1=5;
         end;

         if sum(pn2a_1 not in (1,5),pn2a_2 not in (1,5),
                pn2a_3 not in (1,5),pn2a_4 not in (1,5),
                pn2a_5 not in (1,5),pn2a_6 not in (1,5),
                pn2a_7 not in (1,5),pn2a_8 not in (1,5),
                pn2a_9 not in (1,5),pn2a_10 not in (1,5),
                pn2a_11 not in (1,5),pn2a_12 not in (1,5),
                pn2a_13 not in (1,5),pn2a_14 not in (1,5),
                pn2a_15 not in (1,5))=15 then pnicsk2=5;
         else do;
            if sum(pn2a_1=5,pn2a_2=5,pn2a_3=5,pn2a_4=5,
                   pn2a_5=5,pn2a_6=5,pn2a_7=5,pn2a_8=5,
                   pn2a_9=5,pn2a_10=5,pn2a_11=5,pn2a_12=5,
                   pn2a_13=5,pn2a_14=5,pn2a_15=5)>3 then pnicsk2=1;
            else pnicsk2=5;
         end;

         pnicsk=max(pnicsk1,pnicsk2);

         if pnicsk=5 then do;
            array skp1 pnica pnicb1 pnicb2 pnicb3 
                       pnicb4a pnicb4b pnicb4c pnicb4d pnicb4e 
                       pnicb4f pnicb4g pnicb4h pnicb4i pnicb4j
                       pnicb4k pnicb4l pnicb4m pnicb4n pnicb4x pnicb4 pnicb
                       pnicc pnicdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if pnicsk=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************;
            * unexpected panic attacks *;
            ****************************;

               if sum(pn1a not in (1,2,3,4,5),
                      sum(pn4a<0,pn4a_1 not in (1,2,3,4,5))=2)=2 then 
                  pnica=.;
               else pnica=(4*(pn1a in (3,4,5) and 
                              sum(pn4a>2,pn4a_1 in (2,3,4,5))>0))+1;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************;
            * Criterion B1 *;
            ****************;

               if pn1a not in (1,2,3,4,5) then pnicb1=.;
               else pnicb1=(4*(pn1a in (3,4,5)))+1;

            ****************;
            * Criterion B2 *;
            ****************;

               if pn3 not in (1,5) then pnicb2=.;
               else pnicb2=(4*(pn3=5))+1;

            ****************;
            * Criterion B3 *;
            ****************;

               if pn3 not in (1,5) then pnicb3=.;
               else pnicb3=(4*(pn3=5))+1;

            ****************;
            * Criterion B4 *;
            ****************;

               ****************;
               * palpitations *;
               ****************;

                  if pn2a_1 not in (1,5) then pnicb4a=.;
                  else pnicb4a=(4*(pn2a_1=5))+1;

               ************;
               * sweating *;
               ************;

                  if pn2a_2 not in (1,5) then pnicb4b=.;
                  else pnicb4b=(4*(pn2a_2=5))+1;

               ************************;
               * trembling or shaking *;
               ************************;

                  if pn2a_3 not in (1,5) then pnicb4c=.;
                  else pnicb4c=(4*(pn2a_3=5))+1;

               *************;
               * dry mouth *;
               *************;

                  if pn2a_14 not in (1,5) then pnicb4d=.;
                  else pnicb4d=(4*(pn2a_14=5))+1;

               ***********************;
               * shortness of breath *;
               ***********************;

                  if pn2a_4 not in (1,5) then pnicb4e=.;
                  else pnicb4e=(4*(pn2a_4=5))+1;

               **********************;
               * feeling of choking *;
               **********************;

                  if pn2a_5 not in (1,5) then pnicb4f=.;
                  else pnicb4f=(4*(pn2a_5=5))+1;

               ****************************;
               * chest pain or discomfort *;
               ****************************;

                  if pn2a_6 not in (1,5) then pnicb4g=.;
                  else pnicb4g=(4*(pn2a_6=5))+1;

               ********************************;
               * nausea or abdominal distress *;
               ********************************;

                  if pn2a_7 not in (1,5) then pnicb4h=.;
                  else pnicb4h=(4*(pn2a_7=5))+1;
  
               *****************;
               * feeling dizzy *;
               *****************;

                  if pn2a_8 not in (1,5) then pnicb4i=.;
                  else pnicb4i=(4*(pn2a_8=5))+1;

               ***********************************;
               * derealization/depersonalization *;
               ***********************************;

                  if pn2a_9 not in (1,5) then pnicb4j=.;
                  else pnicb4j=(4*(pn2a_9=5))+1;

               **************************;
               * fear of losing control *;
               **************************;

                  if pn2a_10 not in (1,5) then pnicb4k=.;
                  else pnicb4k=(4*(pn2a_10=5))+1;

               *****************;
               * fear of dying *;
               *****************;

                  if pn2a_15 not in (1,5) then pnicb4l=.;
                  else pnicb4l=(4*(pn2a_15=5))+1;

               *************************;
               * chills or hot flashes *;
               *************************;

                  if sum(pn2a_12 not in (1,5),
                         pn2a_13 not in (1,5))=2 then pnicb4m=.;
                  else pnicb4m=(4*(pn2a_12=5 or pn2a_13=5))+1;

               ****************;
               * parasthesias *;
               ****************;

                  if pn2a_11 not in (1,5) then pnicb4n=.;
                  else pnicb4n=(4*(pn2a_11=5))+1;

               ****************;
               * sum of items *;
               ****************;

                  if n(pnicb4a,pnicb4b,pnicb4c,pnicb4d,pnicb4e,
                       pnicb4f,pnicb4g,pnicb4h,pnicb4i,pnicb4j,
                       pnicb4k,pnicb4l,pnicb4m,pnicb4n)=0 then pnicb4x=.;
                  else do;
                     pnicb4x=0;
                     suma2d=0;
                     array sxs{*} pnicb4a pnicb4b pnicb4c pnicb4d pnicb4e 
                                  pnicb4f pnicb4g pnicb4h pnicb4i pnicb4j
                                  pnicb4k pnicb4l pnicb4m pnicb4n;
                     do i=1 to dim(sxs);
                        if sxs{i}=5 then pnicb4x=pnicb4x+1;
                        if (i<=4 and sxs{i}=5) then suma2d=suma2d+1;
                     end;
                  end;

               *****************************************;
               * at least four items inc. one from a-d *;
               *****************************************;

                  if sum(sum(pnicb4a not in (1,5),
                             pnicb4b not in (1,5),
                             pnicb4c not in (1,5),
                             pnicb4d not in (1,5))=4,
                         pnicb4x<0) then pnicb4=.;
                  else pnicb4=(4*(suma2d>0 and pnicb4x>3))+1;

            ****************;
            * all of B1-B4 *;
            ****************;

               if n(of pnicb1-pnicb4)<4 then pnicb=.;
               else pnicb=(4*(pnicb1=5 and pnicb2=5 and 
                              pnicb3=5 and pnicb4=5))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ********************;
            * organic etiology *;
            ********************;

               if pn1a not in (1,2,3,4,5) then pnicc=.;
               else do;
                  if pn1a in (1,5) then pnicc=1;
                  if pn1a in (3,4) then pnicc=pn1a;
               end;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(pnica,pnicb,pnicc)<3 then pnicdx=.;
            else do;
               pnicdx=1;
               if (pnica=5 and pnicb=5) then do;
                  if pnicc=1 then pnicdx=5;
                  if pnicc=3 then pnicdx=3;
                  if pnicc=4 then pnicdx=4;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label pnicsk1 = "ICD10 PANIC Skip Out #1"
               pnicsk2 = "ICD10 PANIC Skip Out #2"
               pnicsk  = "ICD10 PANIC Skip Out"
               pnica   = "ICD10 PANIC (Lifetime) Crit. A"
               pnicb1  = "ICD10 PANIC (Lifetime) Crit. B1"
               pnicb2  = "ICD10 PANIC (Lifetime) Crit. B2"
               pnicb3  = "ICD10 PANIC (Lifetime) Crit. B3"
               pnicb4a = "ICD10 PANIC (Lifetime) Crit. B4a"
               pnicb4b = "ICD10 PANIC (Lifetime) Crit. B4b"
               pnicb4c = "ICD10 PANIC (Lifetime) Crit. B4c"
               pnicb4d = "ICD10 PANIC (Lifetime) Crit. B4d"
               pnicb4e = "ICD10 PANIC (Lifetime) Crit. B4e"
               pnicb4f = "ICD10 PANIC (Lifetime) Crit. B4f"
               pnicb4g = "ICD10 PANIC (Lifetime) Crit. B4g"
               pnicb4h = "ICD10 PANIC (Lifetime) Crit. B4h"
               pnicb4i = "ICD10 PANIC (Lifetime) Crit. B4i"
               pnicb4j = "ICD10 PANIC (Lifetime) Crit. B4j"
               pnicb4k = "ICD10 PANIC (Lifetime) Crit. B4k"
               pnicb4l = "ICD10 PANIC (Lifetime) Crit. B4l"
               pnicb4m = "ICD10 PANIC (Lifetime) Crit. B4m"
               pnicb4n = "ICD10 PANIC (Lifetime) Crit. B4n"
               pnicb4x = "ICD10 PANIC (Lifetime) Sum of B4s"
               pnicb4  = "ICD10 PANIC (Lifetime) Crit. B4"
               pnicb   = "ICD10 PANIC (Lifetime) Crit. B"
               pnicc   = "ICD10 PANIC (Lifetime) Crit. C"
               pnicdx  = "ICD10 PANIC (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    pnicsk1 pnicsk2 pnicsk
                    pnica pnicb1 pnicb2 pnicb3 
                    pnicb4a pnicb4b pnicb4c pnicb4d pnicb4e 
                    pnicb4f pnicb4g pnicb4h pnicb4i pnicb4j
                    pnicb4k pnicb4l pnicb4m pnicb4n pnicb4x pnicb4 pnicb
                    pnicc pnicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_pan_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(panic,pn1a pnicsk1);
                  %multi(panic,pnicsk1 
                               pn2a_1 pn2a_2 pn2a_3 
                               pn2a_4 pn2a_5 pn2a_6 
                               pn2a_7 pn2a_8 pn2a_9 
                               pn2a_10 pn2a_11 pn2a_12 
                               pn2a_13 pn2a_14 pn2a_15 pnicsk2); 
                  %multi(panic,pnicsk1 pnicsk2 pnicsk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(panic,pnicsk pn1a pn4a pn4a_1 pnica);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     ****************;
                     * Criterion B1 *;
                     ****************;

                        %multi(panic,pnicsk pn1a pnicb1);

                     ****************;
                     * Criterion B2 *;
                     ****************;

                        %multi(panic,pnicsk pn3 pnicb2);

                     ****************;
                     * Criterion B3 *;
                     ****************;

                        %multi(panic,pnicsk pn3 pnicb3);

                     ****************;
                     * Criterion B4 *;
                     ****************;

                        %multi(panic,pnicsk pn2a_1 pnicb4a);
                        %multi(panic,pnicsk pn2a_2 pnicb4b);
                        %multi(panic,pnicsk pn2a_3 pnicb4c);
                        %multi(panic,pnicsk pn2a_14 pnicb4d);
                        %multi(panic,pnicsk pn2a_4 pnicb4e);
                        %multi(panic,pnicsk pn2a_5 pnicb4f);
                        %multi(panic,pnicsk pn2a_6 pnicb4g);
                        %multi(panic,pnicsk pn2a_7 pnicb4h);
                        %multi(panic,pnicsk pn2a_8 pnicb4i);
                        %multi(panic,pnicsk pn2a_9 pnicb4j);
                        %multi(panic,pnicsk pn2a_10 pnicb4k);
                        %multi(panic,pnicsk pn2a_15 pnicb4l);
                        %multi(panic,pnicsk pn2a_12 pn2a_13 pnicb4m);
                        %multi(panic,pnicsk pn2a_11 pnicb4n);
                        %multi(panic,pnicsk pnicb4a pnicb4b pnicb4c pnicb4d 
                                     pnicb4e pnicb4f pnicb4g pnicb4h pnicb4i 
                                     pnicb4j pnicb4k pnicb4l pnicb4m pnicb4n 
                                     pnicb4x);
                        %multi(panic,pnicsk pnicb4x pnicb4);

                     **********************;
                     * B1, B2, B3, and B4 *;
                     **********************;

                        %multi(panic,pnicsk pnicb1-pnicb4 pnicb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(panic,pnicsk pn1a pnicc);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(panic,pnicsk pnica pnicb pnicc pnicdx);

            ******************;
            * keep variables *;
            ******************;

               data panic;
               set panic;
               keep ind_id  
                    pnicsk1 pnicsk2 pnicsk
                    pnica pnicb1 pnicb2 pnicb3 
                    pnicb4a pnicb4b pnicb4c pnicb4d pnicb4e 
                    pnicb4f pnicb4g pnicb4h pnicb4i pnicb4j
                    pnicb4k pnicb4l pnicb4m pnicb4n pnicb4x pnicb4 pnicb
                    pnicc pnicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=panic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend panic;

********;
* test *;
********;
/*
   %panic(coga.csaga2,1);
*/
