************************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Separation Anxiety Disorder *;
************************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* uncomment the following lines to run as standalone module *;
*************************************************************;
/*
   %include "./icd10_gad.sas" / nosource2;

   %gadic(coga.csaga2,0);
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %sepic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro sepic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SEPIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SEPIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      proc contents data=&dsn. noprint out=test;
      run;

      data test;
      set test;
      if upcase(substr(name,1,2))="SA";
      run;

      data test;
      set test nobs=count;
      call symput("var"||left(_n_),name);
      call symput("nvar",left(put(count,8.)));
      run;

      data sepic;
      merge &dsn.(keep=ind_id 
                       %do i=1 %to &nvar.;
                          &&var&i.
                       %end;
                       )
            gadic(keep=ind_id gaicsk gaicdx);
      by ind_id;

      attrib saicsk
             saica1-saica8 saica3x saica5x saicax saica 
             saicb saicc saicdx length=3;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            *************************************************************;
            * persistent worry about harm to/loss of attachment figures *;
            *************************************************************;

               if sa1a not in (1,5) then saica1=.;
               else saica1=(4*(sa1a=5))+1;

            ********************************************;
            * persistent worry about calamitous events *;
            ********************************************;

               if sa2a not in (1,5) then saica2=.;
               else saica2=(4*(sa2a=5))+1;

            **************************************;
            * persistent refusal to go to school *;
            **************************************;

               if sum(sa1b not in (1,3,5),
                      sa2b not in (1,3,5),
                      sa3a not in (1,5),
                      sum(sa3c<0,sa3c_1<0)>0)=4 then saica3=.;
               else do;
                  saica3=1;
                  if sum(sa3c>1,sa3c_1>1)>0 then do;
                     if (sa1b=3 or sa2b=3) then saica3=3;
                     else if (sa1b=5 or sa2b=5 or sa3a=5) then saica3=5;
                  end;
               end;

            *********************;
            * test for skip out *;
            *********************;
 
               if sum(sa3a not in (1,5),
                      sa3b not in (1,3,5),
                      sa3c not in (0,1))=3 then saicsk=5;
               else do;
                  if sum(sum(sa3a=5,sa3b=5)>0,sa3c>1)>0 then saicsk=1;
                  else saicsk=5;
               end;

               if saicsk=5 then do;
                  array skp1 saica4-saica8 saica3x saica5x saicax saica 
                             saicb saicc saicdx ;
                  do over skp1;
                     skp1=.S;
                  end;
               end;

              if saicsk=5 then go to skipout;

            *************************************;
            * persistent refusal to go to sleep *;
            *************************************;

               if sum(sa4a not in (1,3,5),
                      sa4b not in (1,3,5),
                      sa5a not in (1,3,5),
                      sa5b not in (1,3,5))=3 then saica4=.;
               else do;
                  saica4=1;
                  if (sa4a=3 or sa4b=3 or (sa5a=3 or sa5b=3)) then saica4=3;
                  else if (sa4a=5 or sa4b=5 or 
                           (sa5a=5 or sa5b=5)) then saica4=5;
               end;

            ***************************************;
            * persistent avoidance of being alone *;
            ***************************************;

               if sum(sa6a not in (1,3,5),
                      sa6b not in (1,3,5))=2 then saica5=.;
               else do;
                  saica5=1;
                  if (sa6a=3 or sa6b=3) then saica5=3;
                  else if (sa6a=5 or sa6b=5) then saica5=5;
               end;

            ******************************************;
            * repeated separation-related nightmares *;
            ******************************************;

               if sa9 not in (1,3,5) then saica6=.;
               else saica6=sa9;

            ****************************************************************;
            * complaints of physical symptoms when anticipating separation *;
            ****************************************************************;

               if sa10 not in (1,3,5) then saica7=.;
               else saica7=sa10;

            ***************************************************************;
            * recurrent excessive distress when separated/to be separated *;
            ***************************************************************;

               if sum(sa7 not in (1,3,5),
                      sa8a not in (1,3,5),
                      sa8b not in (1,3,5),
                      sa11a not in (1,3,5),
                      sa11b not in (1,3,5))=5 then saica8=.;
               else do;
                  saica8=1;
                  if (sa7=3 or sa8a=3 or sa8b=3 or 
                      sa11a=3 or sa11b=3) then saica8=3;
                  else if (sa7=5 or sa8a=5 or sa8b=5 or 
                           sa11a=5 or sa11b=5) then saica8=5;
               end;

            ****************;
            * sum of items *;
            ****************;

               if n(of saica1-saica8)=0 then saicax=.;
               else do;
                  saica3x=0;
                  saica5x=0;
                  array sxs saica1-saica8;
                  do over sxs;
                     if sxs=3 then saica3x=saica3x+1;
                     if sxs=5 then saica5x=saica5x+1;
                  end;
                  saicax=sum(saica3x,saica5x);
                  saica=(4*(saicax>2))+1;
               end;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************************************;
            * does not meet criteria for generalized anxiety disorder *;
            ***********************************************************;

               if (gaicdx=. or gaicdx=.S) then saicb=5;
               else if gaicdx=1 then saicb=5;
               else if gaicdx in (3,5) then saicb=1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            **********************;
            * onset before age 6 *;
            **********************;

               if sum(sa12a_1<0,sa13a_1<0)=2 then saicc=.;
               else do;
                  if (sa12a_1 ne . and sa12a_1>0) and 
                     (sa13a_1=. or sa13a_1<0) then
                     saicc=(4*(sa12a_1<6))+1;
                  if (sa12a_1 ne . and sa12a_1>0) and 
                     (sa13a_1 ne . and sa13a_1>0) then
                     saicc=(4*(sum(sa12a_1<6,sa13a_1<6)>0))+1;
                  if (sa13a_1 ne . and sa13a_1>0) and 
                     (sa12a_1=. or sa12a_1<0) then
                     saicc=(4*(sa13a_1<6))+1;
               end;

      **************************************;
      * Criterion D -- Skip in this module *;
      **************************************;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ************************************;
            * behavior lasted at least 4 weeks *;
            ************************************;

               if sum(sa12c<0,
                      sa12c_1<0,
                      sa13c<0,
                      sa13c_1<0)=4 then saice=.;
               else saice=(4*(sum(sa12c>3,sa12c_1=4,
                                  sa13c>3,sa13c_1=4)>0))+1;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(saica,saicb,saicc,saice)=0 then saicdx=.;
            else do;
               saicdx=1;
               if (saicax>2 and saicb=5 and saicc=5 and saice=5) then do;
                  if saica3x>0 then saicdx=3;
                  else if saica3x=0 then saicdx=5;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

      *******************;
      * variable labels *;
      *******************;

         label saicsk  = "ICD10 SEPANX Skip Out"
               saica1  = "ICD10 SEPANX (Lifetime) Crit. A1"
               saica2  = "ICD10 SEPANX (Lifetime) Crit. A2"
               saica3  = "ICD10 SEPANX (Lifetime) Crit. A3"
               saica4  = "ICD10 SEPANX (Lifetime) Crit. A4"
               saica5  = "ICD10 SEPANX (Lifetime) Crit. A5"
               saica6  = "ICD10 SEPANX (Lifetime) Crit. A6"
               saica7  = "ICD10 SEPANX (Lifetime) Crit. A7"
               saica8  = "ICD10 SEPANX (Lifetime) Crit. A8"
               saica3x = "ICD10 SEPANX (Lifetime) Sum of 3s"
               saica5x = "ICD10 SEPANX (Lifetime) Sum of 5s"
               saicax  = "ICD10 SEPANX (Lifetime) Sum of As"
               saica   = "ICD10 SEPANX (Lifetime) 8+ in 6Mo"
               saicb   = "ICD10 SEPANX (Lifetime) Crit. B"
               saicc   = "ICD10 SEPANX (Lifetime) Crit. C"
               saice   = "ICD10 SEPANX (Lifetime) Crit. E"
               saicdx  = "ICD10 SEPANX (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    saicsk
                    saica1-saica8 saica3x saica5x saicax saica 
                    saicb saicc saice saicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_sep_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(sepic,sa3a sa3b sa3c saicsk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepic,sa1a saica1);
                     %multi(sepic,sa2a saica2);
                     %multi(sepic,sa1b sa2b sa3a sa3c sa3c_1 saica3);
                     %multi(sepic,saicsk sa4a sa4b sa5a sa5b saica4);
                     %multi(sepic,saicsk sa6a sa6b saica5);
                     %multi(sepic,saicsk sa9 saica6);
                     %multi(sepic,saicsk sa10 saica7);
                     %multi(sepic,saicsk sa7 sa8a sa8b sa11a sa11b saica8);
                     %multi(sepic,saicsk saica1-saica8 saica3x saica5x);
                     %multi(sepic,saicsk saica3x saica5x saicax saica);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepic,saicsk gaicdx saicb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepic,saicsk sa12a_1 sa13a_1 saicc);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepic,saicsk sa12c sa12c_1 sa13c sa13c_1 saice);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(sepic,saicsk saica3x saica5x saicb saicc saice 
                                  saicdx);

            ******************;
            * keep variables *;
            ******************;

               data sepic;
               set sepic;
               keep ind_id  
                    saicsk 
                    saica1-saica8 saica3x saica5x saicax saica
                    saicb saicc saice saicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=sepic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend sepic;

********;
* test *;
********;
/*
   %sepic(coga.csaga2,1);
*/
