*******************************************************************;
* CSSAGA-II Diagnostic Algorithm - ICD10 - Social Phobia Disorder *;
*******************************************************************;
options ls=100 ps=55;

*********;
* setup *;
*********;
/*
   %include "./cssaga2.setup" / nosource2;
*/
*************************************************************;
* algorithm macro                                           *;
*                                                           *;
* %socic(dsn,chk)                                           *;
* where dsn = dataset name (e.g., coga.csaga2)              *;
*       chk = 1 to run checks, 0 otherwise (can be omitted) *;
*************************************************************;

   %macro socic(dsn,chk);

      %let err_msg=NONE;

      %if &dsn. ne %then %do;
         %exist(&dsn);
         %if &exist=no %then %do;
            %let err_msg=%str(ERROR in SOCIC: Dataset &dsn. Does Not Exist.);
            %goto errexit;
         %end;
      %end;
      %else %do;
         %let err_msg=%str(ERROR in SOCIC: Need dataset name in macro call.);
         %goto errexit;
      %end;

      data socic;
      set &dsn.;

      attrib spicsk1 spicsk2 spicsk
             spica1 spica2 spica 
             spicb1-spicb16 spicbx spicb 
             spicc spicd spice spicdx length=3;

      *************************************;
      * test for interview of/about child *;
      *************************************;

         if 0<dm3a<13 then do;
            array nas spicsk1 spicsk2 spicsk
                      spica1 spica2 spica 
                      spicb1-spicb16 spicbx spicb 
                      spicc spicd spice spicdx;
            do over nas;
               nas=.N;
            end;
         end;
         if 0<dm3a<13 then go to kid_hop;

      *********************;
      * test for skip out *;
      *********************;
 
         if sp2 not in (1,3,5) then spicsk1=5;
         else do;
            if sp2 in (3,5) then spicsk1=1;
            else spicsk1=5;
         end;

         if spicsk1=5 then do;
            array skp1 spica1 spica2 spica 
                       spicb1-spicb16 spicbx spicb 
                       spicc spicd spice spicdx;
            do over skp1;
              skp1=.S;
            end;
         end;

         if spicsk1=5 then go to skipout;

      ***************;
      * Criterion A *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************;
            * persistent fear of possible scrutiny *;
            ****************************************;

               if sp2 not in (1,3,5) then spica1=.;
               else spica1=(4*(sp2=5))+(2*(sp2=3))+1;

            *********************;
            * test for skip out *;
            *********************;

               if sum(sum(sp3a_1 not in (1,5),
                          sp3a_2 not in (1,5),
                          sp3a_3 not in (1,5),
                          sp3a_4 not in (1,5),
                          sp3a_5 not in (1,5),
                          sp3a_6 not in (1,5),
                          sp3a_7 not in (1,5),
                          sp3a_8 not in (1,5),
                          sp3a_9 not in (1,5),
                          sp3a_10 not in (1,5),
                          sp3a_11 not in (1,5),
                          sp3a_12 not in (1,5),
                          sp3a_13 not in (1,5))=13,
                      sum(sp3b_1 not in (1,5),
                          sp3b_2 not in (1,5),
                          sp3b_3 not in (1,5),
                          sp3b_4 not in (1,5),
                          sp3b_5 not in (1,5),
                          sp3b_6 not in (1,5),
                          sp3b_7 not in (1,5),
                          sp3b_8 not in (1,5),
                          sp3b_9 not in (1,5),
                          sp3b_10 not in (1,5),
                          sp3b_11 not in (1,5),
                          sp3b_12 not in (1,5),
                          sp3b_13 not in (1,5))=13,
                      sum(sp3c_1 not in (1,5),
                          sp3c_2 not in (1,5),
                          sp3c_3 not in (1,5),
                          sp3c_4 not in (1,5),
                          sp3c_5 not in (1,5),
                          sp3c_6 not in (1,5),
                          sp3c_7 not in (1,5),
                          sp3c_8 not in (1,5),
                          sp3c_9 not in (1,5),
                          sp3c_10 not in (1,5),
                          sp3c_11 not in (1,5),
                          sp3c_12 not in (1,5),
                          sp3c_13 not in (1,5))=13)=3 then spicsk2=5;
               else do;
                  if sum(sum(sp3a_1=5,sp3a_2=5,sp3a_3=5,
                             sp3a_4=5,sp3a_5=5,sp3a_6=5,
                             sp3a_7=5,sp3a_8=5,sp3a_9=5,
                             sp3a_10=5,sp3a_11=5,sp3a_12=5,
                             sp3a_13=5)>0,
                         sum(sp3b_1=5,sp3b_2=5,sp3b_3=5,
                             sp3b_4=5,sp3b_5=5,sp3b_6=5,
                             sp3b_7=5,sp3b_8=5,sp3b_9=5,
                             sp3b_10=5,sp3b_11=5,sp3b_12=5,
                             sp3b_13=5)>0,
                         sum(sp3c_1=5,sp3c_2=5,sp3c_3=5,
                             sp3c_4=5,sp3c_5=5,sp3c_6=5,
                             sp3c_7=5,sp3c_8=5,sp3c_9=5,
                             sp3c_10=5,sp3c_11=5,sp3c_12=5,
                             sp3c_13=5)) then spicsk2=1;
                  else spicsk2=5;
               end;
      
               if spicsk2=5 then do;
                  array skp2 spica2 spica 
                             spicb1-spicb16 spicbx spicb 
                             spicc spicd spice spicdx;
                  do over skp2;
                    skp2=.S;
                  end;
               end;
      
               if spicsk2=5 then go to skipout;

            *************************************************;
            * phobic situations avoided or indured w/stress *;
            *************************************************;

               if sp5 not in (1,5) then spica2=.;
               else spica2=(4*(sp5=5))+1;

            ***************;
            * either item *;
            ***************;

               if n(spica1,spica2)=0 then spica=.;
               else spica=(4*(sum(spica1 in (3,5),spica2=5)>0))+1;

      ***************;
      * Criterion B *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************;
            * blushing or shaking *;
            ***********************;

               if sum(sp4a_3 not in (1,5),
                      sp4a_13 not in (1,5))=2 then spicb1=.;
               else spicb1=(4*(sum(sp4a_3=5,sp4a_13=5)>0))+1;

            ********************;
            * fear of vomiting *;
            ********************;

               if sp4a_17 not in (1,5) then spicb2=.;
               else spicb2=(4*(sp4a_17=5))+1;

            ******************************************;
            * urgency/fear of micturition/defecation *;
            ******************************************;

               if sp4a_16 not in (1,5) then spicb3=.;
               else spicb3=(4*(sp4a_16=5))+1;

            ****************;
            * palpitations *;
            ****************;

               if sp4a_1 not in (1,5) then spicb4=.;
               else spicb4=(4*(sp4a_1=5))+1;

            ************;
            * sweating *;
            ************;

               if sp4a_2 not in (1,5) then spicb5=.;
               else spicb5=(4*(sp4a_2=5))+1;

            *************;
            * dry mouth *;
            *************;

               if sp4a_14 not in (1,5) then spicb6=.;
               else spicb6=(4*(sp4a_14=5))+1;

            ************************;
            * difficulty breathing *;
            ************************;

               if sp4a_4 not in (1,5) then spicb7=.;
               else spicb7=(4*(sp4a_4=5))+1;

            **********************;
            * feeling of choking *;
            **********************;

               if sp4a_5 not in (1,5) then spicb8=.;
               else spicb8=(4*(sp4a_5=5))+1;

            **************;
            * chest pain *;
            **************;

               if sp4a_6 not in (1,5) then spicb9=.;
               else spicb9=(4*(sp4a_6=5))+1;

            ********************************;
            * nausea or abdominal distress *;
            ********************************;

               if sp4a_7 not in (1,5) then spicb10=.;
               else spicb10=(4*(sp4a_7=5))+1;

            *****************;
            * feeling dizzy *;
            *****************;

               if sp4a_8 not in (1,5) then spicb11=.;
               else spicb11=(4*(sp4a_8=5))+1;

            ***********************************;
            * derealization/depersonalization *;
            ***********************************;

               if sp4a_9 not in (1,5) then spicb12=.;
               else spicb12=(4*(sp4a_9=5))+1;

            **************************;
            * fear of losing control *;
            **************************;

               if sp4a_10 not in (1,5) then spicb13=.;
               else spicb13=(4*(sp4a_10=5))+1;

            *****************;
            * fear of dying *;
            *****************;

               if sp4a_15 not in (1,5) then spicb14=.;
               else spicb14=(4*(sp4a_15=5))+1;

            ***************************;
            * hot flashes/cold chills *;
            ***************************;

               if sp4a_12 not in (1,5) then spicb15=.;
               else spicb15=(4*(sp4a_12=5))+1;

            ********************************;
            * numbness/tingling sensations *;
            ********************************;

               if sp4a_11 not in (1,5) then spicb16=.;
               else spicb16=(4*(sp4a_11=5))+1;

            ****************;
            * sum of items *;
            ****************;

               if n(of spicb1-spicb16)=0 then spicbx=.;
               else do;
                  spicbx=0;
                  sum1_3=0;
                  sum4_16=0;
                  array sxs{*} spicb1-spicb16;
                  do i=1 to dim(sxs);
                     if sxs{i}=5 then do;
                        spicbx=spicbx+1;
                        if i<4 then sum1_3=sum1_3+1;
                        else if i>3 then sum4_16=sum4_16+1;
                     end;
                  end;
                end;

            ****************************;
            * 1+ in 1-3 and 2+ in 4-16 *;
            ****************************;

               if spicbx=. then spicb=.;
               else spicb=(4*(sum1_3>0 and sum4_16>1))+1;

      ***************;
      * Criterion C *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ***********************************;
            * fear recognized as unreasonable *;
            ***********************************;

               if sum(sum(sp3c_1 not in (1,5),
                          sp3c_2 not in (1,5),
                          sp3c_3 not in (1,5),
                          sp3c_4 not in (1,5),
                          sp3c_5 not in (1,5),
                          sp3c_6 not in (1,5),
                          sp3c_7 not in (1,5),
                          sp3c_8 not in (1,5),
                          sp3c_9 not in (1,5),
                          sp3c_10 not in (1,5),
                          sp3c_11 not in (1,5),
                          sp3c_12 not in (1,5),
                          sp3c_13 not in (1,5))=13,
                      sp7 not in (1,5))=2 then spicc=.;
               else spicc=(4*(sum(sp3c_1=5,sp3c_2=5,sp3c_3=5,
                                  sp3c_4=5,sp3c_5=5,sp3c_6=5,
                                  sp3c_7=5,sp3c_8=5,sp3c_9=5,
                                  sp3c_10=5,sp3c_11=5,sp3c_12=5,
                                  sp3c_13=5)>0 and sp7=5))+1;

      ***************;
      * Criterion D *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ********************************************;
            * symptoms restricted to feared situations *;
            ********************************************;

               if sp4b not in (1,5) then spicd=.;
               else spicd=(4*(sp4b=5))+1;

      ***************;
      * Criterion E *;
      ***************;

         *****************;
         * lifetime/ever *;
         *****************;

            ****************************************;
            * no physiologic/medical/organic basis *;
            ****************************************;

               if sp10 not in (1,2,3) then spice=.;
               else spice=(4*(sp10=1))+(2*(sp10=3))+1;

      *************;
      * Diagnosis *;
      *************;

         *****************;
         * lifetime/ever *;
         *****************;

            if n(spica,spicb,spicc,spicd,spice)=0 then spicdx=.;
            else do;
               spicdx=1;
               if (spicb=5 and spicc=5 and spicd=5) then do;
                  if (spica1=3 and spice=3) then spicdx=3;
                  if (spica1 ne 3 and (spica1=5 or spica2=5) and spice=5) then
                     spicdx=5;
               end;
            end;

      ****************************;
      * jump to here if skip out *;
      ****************************;

         skipout:

         spicsk=max(spicsk1,spicsk2);

      *******************************************;
      * jump here if interview of/about 7-12 yo *;
      *******************************************;

         kid_hop:

      *******************;
      * variable labels *;
      *******************;

         label spicsk1 = "ICD10 SOCPHB Skip Out #1"
               spicsk2 = "ICD10 SOCPHB Skip Out #2"
               spicsk  = "ICD10 SOCPHB Skip Out"
               spica1  = "ICD10 SOCPHB (Lifetime) Crit. A1"
               spica2  = "ICD10 SOCPHB (Lifetime) Crit. A2"
               spica   = "ICD10 SOCPHB (Lifetime) Crit. A" 
               spicb1  = "ICD10 SOCPHB (Lifetime) Crit. B1"
               spicb2  = "ICD10 SOCPHB (Lifetime) Crit. B2"
               spicb3  = "ICD10 SOCPHB (Lifetime) Crit. B3"
               spicb4  = "ICD10 SOCPHB (Lifetime) Crit. B4"
               spicb5  = "ICD10 SOCPHB (Lifetime) Crit. B5"
               spicb6  = "ICD10 SOCPHB (Lifetime) Crit. B6"
               spicb7  = "ICD10 SOCPHB (Lifetime) Crit. B7"
               spicb8  = "ICD10 SOCPHB (Lifetime) Crit. B8"
               spicb9  = "ICD10 SOCPHB (Lifetime) Crit. B9"
               spicb10 = "ICD10 SOCPHB (Lifetime) Crit. B10"
               spicb11 = "ICD10 SOCPHB (Lifetime) Crit. B11"
               spicb12 = "ICD10 SOCPHB (Lifetime) Crit. B12"
               spicb13 = "ICD10 SOCPHB (Lifetime) Crit. B13"
               spicb14 = "ICD10 SOCPHB (Lifetime) Crit. B14"
               spicb15 = "ICD10 SOCPHB (Lifetime) Crit. B15"
               spicb16 = "ICD10 SOCPHB (Lifetime) Crit. B16"
               spicbx  = "ICD10 SOCPHB (Lifetime) Sum of Bs"
               spicb   = "ICD10 SOCPHB (Lifetime) Crit. B"
               spicc   = "ICD10 SOCPHB (Lifetime) Crit. C"
               spicd   = "ICD10 SOCPHB (Lifetime) Crit. D"
               spice   = "ICD10 SOCPHB (Lifetime) Crit. E"
               spicdx  = "ICD10 SOCPHB (Lifetime) Diagnosis";

      **********************;
      * stop or run checks *;
      **********************;

         %if &chk. ne 1 %then %do;

            ******************;
            * keep variables *;
            ******************;

               keep ind_id  
                    spicsk1 spicsk2 spicsk
                    spica1 spica2 spica 
                    spicb1-spicb16 spicbx spicb 
                    spicc spicd spice spicdx;
               run;

         %end;
         %else %do;

            run;

            ******************;
            * reroute output *;
            ******************;

               proc printto new print="./icd10_soc_&who..lst";
               run;

            **************;
            * run checks *;
            **************;

               ************;
               * skip out *;
               ************;

                  %multi(socic,sp2 spicsk1);
                  %multi(socic,spicsk1 
                               sp3a_1-sp3a_13 sp3b_1-sp3b_13 sp3c_1-sp3c_13
                               spicsk2);
                  %multi(socic,spicsk1 spicsk2 spicsk);

               ***************;
               * Criterion A *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socic,spicsk1 sp2 spica1);
                     %multi(socic,spicsk1 spicsk2 sp5 spica2);
                     %multi(socic,spicsk1 spicsk2 spica1 spica2 spica);

               ***************;
               * Criterion B *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socic,spicsk1 spicsk2 sp4a_3 sp4a_13 spicb1);
                     %multi(socic,spicsk1 spicsk2 sp4a_17 spicb2);
                     %multi(socic,spicsk1 spicsk2 sp4a_16 spicb3);
                     %multi(socic,spicsk1 spicsk2 sp4a_1 spicb4);
                     %multi(socic,spicsk1 spicsk2 sp4a_2 spicb5);
                     %multi(socic,spicsk1 spicsk2 sp4a_14 spicb6);
                     %multi(socic,spicsk1 spicsk2 sp4a_4 spicb7);
                     %multi(socic,spicsk1 spicsk2 sp4a_5 spicb8);
                     %multi(socic,spicsk1 spicsk2 sp4a_6 spicb9);
                     %multi(socic,spicsk1 spicsk2 sp4a_7 spicb10);
                     %multi(socic,spicsk1 spicsk2 sp4a_8 spicb11);
                     %multi(socic,spicsk1 spicsk2 sp4a_9 spicb12);
                     %multi(socic,spicsk1 spicsk2 sp4a_10 spicb13);
                     %multi(socic,spicsk1 spicsk2 sp4a_15 spicb14);
                     %multi(socic,spicsk1 spicsk2 sp4a_12 spicb15);
                     %multi(socic,spicsk1 spicsk2 sp4a_11 spicb16);
                     %multi(socic,spicsk1 spicsk2 spicb1-spicb16 spicbx spicb);

               ***************;
               * Criterion C *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socic,spicsk1 spicsk2 sp3c_1-sp3c_13 sp7 spicc);

               ***************;
               * Criterion D *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socic,spicsk1 spicsk2 sp4b spicd);

               ***************;
               * Criterion E *;
               ***************;

                  *****************;
                  * lifetime/ever *;
                  *****************;

                     %multi(socic,spicsk1 spicsk2 sp10 spice);

               *************;
               * Diagnosis *;
               *************;

                  *****************;
                  * lifetime/ever *;
                  *****************;
 
                     %multi(socic,spicsk1 spicsk2 spica1 spica2 spicb 
                                  spicc spicd spice spicdx);

            ******************;
            * keep variables *;
            ******************;

               data socic;
               set socic;
               keep ind_id 
                    spicsk1 spicsk2 spicsk
                    spica1 spica2 spica 
                    spicb1-spicb16 spicbx spicb 
                    spicc spicd spice spicdx;
               run;

            ************;
            * contents *;
            ************;

               proc contents data=socic;
               run;

            ******************;
            * reroute output *;
            ******************;

               proc printto;
               run;

         %end;

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend socic;

********;
* test *;
********;
/*
   %socic(coga.csaga2,1);
*/
