********************************;
* macro to run scoring modules *;
********************************;

   %macro scoreit(mydsn,runchk);

      %let err_msg = NONE;

      **************************;
      * check for coga or irpg *;
      **************************;

         %let study=%cogairpg(&mydsn.);
         %if &study.=dk %then %do;
            %let err_msg=%str(ERROR: Cannot tell if &dsn. is COGA or IRPG.);
            %goto errexit;
         %end;

      *************************************;
      * check for parent or child dataset *;
      *************************************;

         %let who=%whoisit(&mydsn.);
         %if &who.=dk %then %do;
            %let err_msg=%str(ERROR: Cannot tell if &dsn. is parent or child.);
            %goto errexit;
         %end;

      ********************;
      * DSM3R algorithms *;
      ********************;

         ******************;
         * mood disorders *;
         ******************;

            %dep3r(&mydsn.,&runchk.);
            %if &study.=coga %then %do;
               %man3r(&mydsn.,&runchk.);
               %dys3r(&mydsn.,&runchk.);
            %end;

         **********************;
         * behavior disorders *;
         **********************;

            %add3r(&mydsn.,&runchk.);
            %cd3r(&mydsn.,&runchk.);
            %odd3r(&mydsn.,&runchk.);

         ***************************;
         * substance-use disorders *;
         ***************************;

            %alc3r(&mydsn.,&runchk.);
            %can3r(&mydsn.,&runchk.);
            %drg3r(&mydsn.,&runchk.);

         *********************;
         * anxiety disorders *;
         *********************;

            %if &study.=coga %then %do;
               %ovx3r(&mydsn.,&runchk.);
               %sep3r(&mydsn.,&runchk.);
               %pan3r(&mydsn.,&runchk.);
               %soc3r(&mydsn.,&runchk.);
               %ocd3r(&mydsn.,&runchk.);
            %end;

         ********************;
         * eating disorders *;
         ********************;

            %if &study.=coga %then %do;
               %anx3r(&mydsn.,&runchk.);
               %bul3r(&mydsn.,&runchk.);
            %end;

         ******************;
         * misc disorders *;
         ******************;

            %if &study.=coga %then %do;
               %som3r(&mydsn.,&runchk.);
               %psy3r(&mydsn.,&runchk.);
            %end;

      *******************;
      * DSM4 algorithms *;
      *******************;

         ******************;
         * mood disorders *;
         ******************;

            %depd4(&mydsn.,&runchk.);
            %if &study.=coga %then %do;
               %mand4(&mydsn.,&runchk.);
               %dysd4(&mydsn.,&runchk.);
            %end;

         **********************;
         * behavior disorders *;
         **********************;

            %addd4(&mydsn.,&runchk.);
            %cdd4(&mydsn.,&runchk.);
            %oddd4(&mydsn.,&runchk.);

         ***************************;
         * substance-use disorders *;
         ***************************;

            %alcd4(&mydsn.,&runchk.);
            %cand4(&mydsn.,&runchk.);
            %drgd4(&mydsn.,&runchk.);
            %tobd4(&mydsn.,&runchk.);

         *********************;
         * anxiety disorders *;
         *********************;

            %if &study.=coga %then %do;
               %gadd4(&mydsn.,&runchk.);
               %sepd4(&mydsn.,&runchk.);
               %pand4(&mydsn.,&runchk.);
               %socd4(&mydsn.,&runchk.);
               %ocdd4(&mydsn.,&runchk.);
            %end;

         ********************;
         * eating disorders *;
         ********************;

            %if &study.=coga %then %do;
               %anxd4(&mydsn.,&runchk.);
               %buld4(&mydsn.,&runchk.);
            %end;

         ******************;
         * misc disorders *;
         ******************;

            %if &study.=coga %then %do;
               %somd4(&mydsn.,&runchk.);
            %end;

      ********************;
      * ICD10 algorithms *;
      ********************;

         ******************;
         * mood disorders *;
         ******************;

            %depic(&mydsn.,&runchk.);

         **********************;
         * behavior disorders *;
         **********************;

            %addic(&mydsn.,&runchk.);
            %cdic(&mydsn.,&runchk.);
            %oddic(&mydsn.,&runchk.);

         ***************************;
         * substance-use disorders *;
         ***************************;

            %alcic(&mydsn.,&runchk.);
            %canic(&mydsn.,&runchk.);
            %drgic(&mydsn.,&runchk.);

         *********************;
         * anxiety disorders *;
         *********************;

            %if &study.=coga %then %do;
               %gadic(&mydsn.,&runchk.);
               %sepic(&mydsn.,&runchk.);
               %panic(&mydsn.,&runchk.);
               %socic(&mydsn.,&runchk.);
               %ocdic(&mydsn.,&runchk.);
            %end;

      **********************;
      * Feighner algorithm *;
      **********************;

         %alcfn(&mydsn.,&runchk.);

      ******************;
      * emergency exit *;
      ******************;

         %errexit:
            %if &err_msg ne NONE %then %do;
               %put &err_msg;
            %end;

   %mend scoreit;


